(defun html-table
  (x &key 
     (file *standard-output*)
     (how "left")
     (rule 0)
     (form "full")
     size
     (caption "")
     (cellspacing 0)
     (cellpadding 0)
     bgcolor
     name
     row-names
     column-names
     select-rows
     select-columns
     (fmt "~a")
     )
  "
Function: html-table

Args:   
     x ;; a matrix
 &key 
     (file *standard-output*) ;; a filename or stream
     (how \"left\") ;; right, left, or center
     (rule 0) ;; if you want table borders, rule is an integer > 0
     (form \"full\") ;; format: full, upper, lower.
     size ;; positive or negative integer - negative for smaller
     (caption \"\") ;; the caption for the table
     (cellspacing 0)  ;; cellspacing is the width of a border
     (cellpadding 0)  ;; cellpadding is a space buffer around each element
     bgcolor ;; a background color (e.g. \"white\", \"CCCCCC\")
     name ;; if name (a string) is given, then a name tag will be assigned to
           ;; the table
     row-names ;; if you want, add a list of row names
     column-names ;; if you want, add a list of column names
     select-rows ;; use only the select-rows rows (numbered from 0)
     select-columns ;; use only the select-columns columns (numbered from 0)
     (fmt \"~a\") ;; either a string or a list of strings (one for each column).
     )

	Writes a matrix of numbers or strings to a file (default is the
standard output) in an html table. Returns nil.

;; Examples:
 (html-table (id 4) :caption \"The 4x4 identity matrix\" :name \"table1\")

 (html-table (id 4) :form \"upper\" :how \"center\" :rule 3)

 (html-table (id 4) :rule 1 :bgcolor \"blue\" :cellpadding 10)

 (html-table (id 4)
              :select-rows (iseq 0 1)
              :select-columns (iseq 0 1) :cellpadding 10 :rule 2)
 (html-table (id 4)
              :select-rows (iseq 0 1)
              :select-columns (iseq 2 3) :cellspacing 10 :rule 2)
              

 ;; this one will create a file called \"crud\" in your current directory:
 ;; (html-table (id 4) :file \"crud\" :caption \"The 4x4 identity matrix\")

 (html-table (id 4) :row-names (list \"Hi\" \"Lo\" \"Up\" \"Down\"))

 (html-table (id 4)
   :row-names (list \"Hi\" \"Lo\" \"Up\" \"Down\")
   :column-names (list \"Hi\" \"Lo\" \"Up\" \"Down\")
   :select-rows (list 1 2)
   :select-columns (list 1 2)
   )

 (html-table (id 4) :fmt \"~,4g\" :size -1) 
 (html-table (id 4) :fmt (list \"~,3f\" \"~,2f\" \"~,1f\" \"~d\"))
 (html-table (id 4) :fmt (list \"~,3g\" \"~,2g\" \"~,1g\" \"~d\"))
"
  (let*
      (
       (x 
	(if (and row-names column-names)
	    (bind-columns (combine "" row-names) (bind-rows column-names x))
	  (if column-names (bind-rows column-names x)
	    (if row-names (bind-columns row-names x)
	      x))))

       (select-rows (if (and select-rows column-names)
			(combine 0 (+ 1 select-rows))
		      select-rows))
       (select-columns (if (and select-columns row-names)
			   (combine 0 (+ 1 select-columns))
			 select-columns))
       (xsize (size x))
       (n (if select-rows select-rows (iseq (first xsize))))
       (m (if select-columns select-columns (iseq (second xsize))))
       
       (last-n (final n))
       (last-m (final m))
       (s (with-output-to-string
	   (form)
	   (format form "~a" "|")
	   (dolist
	    (i m)
	    (format form "~a~a" "|" how)
	    (if (= i last-m) (format form "~a" "||"))
	    )
	   )
	  )

       (fmt (if (stringp fmt) (repeat fmt (second xsize)) fmt))
       (print fmt)

       (tdval "<td>")
       (tdendval "</td>")
       (stuff
	(if size
	    (if (> size 0)
		(setq 
		 tdval (format nil "<td><font size=+~d>" size)
		 tdendval (format nil "</font></td>")
		 )
	      (setq 
	       tdval (format nil "<td><font size=~d>" size)
	       tdendval (format nil "</font></td>")
	       )
	      )	     
	  )
	)

       (result
	(with-output-to-string
	 (texfile)
	 (if name (format texfile "<a name=\"~a\"></a>~%" name))

	 (if (string= how "center") (format texfile "<center>~%"))

	 (if bgcolor
	     (format texfile "~%<table border=0 bgcolor=~a>~%" bgcolor)
	   (format texfile "~%<table border=0>~%"))

	 (format texfile "<tr>~a~%" tdval)
	 (format
	  texfile 
	  "~%  <center>~%  <table border=~d cellspacing=~d cellpadding=~d>~%"
	  rule cellspacing cellpadding
	  )
	 (dolist
	  (i n)
	  (format texfile "  <tr>~%    ~a " tdval)
	  (dolist
	   (j m)
	   (if (or (string= form "full")
		   (and (string= form "upper") (>= j i))
		   (and (string= form "lower") (<= j i)))
	       (format texfile (elt fmt j) (aref x i j)))
	   (if (= j last-m) (format texfile " ~a~%  </tr>" tdendval)
	     (format texfile " ~a~a " tdendval tdval))
	   )
	  (format texfile "~%")
	  )
	 (format
	  texfile
	  "~%  </table>~%  </center>~%~a~%</tr>~%<tr>~a~a~a</tr>~%</table>~%"
	  tdendval tdval caption tdendval)
	 (if (string= how "center") (format texfile "</center>~%"))
	 )
	)
       )
    (if (stringp file)
	(with-open-file
	 (texfile file :direction :output)
	 (format texfile "~a" result)
	 )
      (format file "~a" result)
      )
    )
  )
