;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
#|
	The next buncha macros came out of my work on the traffic problem for
my modelling course at Ripon College. My first attempts were built on Logo.

	I automagically build the usual suspects (methods) into a prototype,
including a list of instances (under the slot 'active-list) and the total
number of objects defined (under the slot 'number). Associated methods allow
uber-protos to keep track of their kids, their grandkids, etc.

	I also create a number of "accessor" functions, which allow me to send
messages using macros, rather than functions. Thus, to give a specific example,
rather than 

	(send andy :height)

one types

	(ask andy height)
or
	(change andy height "5'11''")

I find this "logo-esque" style more attractive. The downside is that, when
one uses macros, one risks to break things when compiled. Thus, any compiled
functions should probably avoid these macros, sticking with the
sends. Programmers probably don't mind! I'm worried about the casual users.... 

							Andy Long,
							October 30, 1997
|#
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                uber-proto
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmacro uber-proto (a b c &optional d doc)
  "
;; Description: (uber-proto name instance-slots class-slots
		 	&optional parent doc)

	A prototype a is defined, using defproto, and a number of standard
methods are installed:

	:number
	:active-list
	:isnew
	:slots
	:save
	:dispose

	Each proto comes with a new :isnew method, and a :dispose method to go
with it. This permits protos to keep track of their instances, in the slot
'active-list. The slot 'number keeps track of the number of living instances.
To avoid problems, one should avoid playing with these slots! You can easily
trick the protos into thinking that they have more children than they actually
do, for example. This insanity will show up in your work (I hope - I hate to
see people get away with that sort of thing!).

	I rigged up a kludgy :save method for instances of your proto (rather
than the protos themselves). Other methods you may want to add are :describe
and :print. Standard methods for accessing each of the instance slots you gave
are also created.

	You may want to take advantage of the functions dispose, show-slots,
and save-object, and the macros new, ask, show, and change.

	If you create a proto with uber-proto which has a parent proto created
by uber-proto, then by creating and disposing of instances of the sub-proto you
will also be registering their appearance/disappearance with the parent (this
is achieved via the use of the call-next-method command).

;; See also: new, dispose, ask, show, show-slots, change, save-object, and
   add-normal-assessors, call-next-method

;; Examples:
 (uber-proto mammal (activities) (hair milk live-birth))
 (send mammal :hair t)
 (send mammal :milk t)
 (send mammal :live-birth t)

 ;; Create the longs proto, child of *object* by default:
 (uber-proto human (chromosomes directions chest-beating) (bi-pedal) mammal)
 (change human bipedal t)

 (uber-proto man (empathy humor) () human)
 ;; Here are two different (equivalent) ways of changing the slots of man:
 ;; change and send
 (change man chromosomes 'xy)
 (change man directions 'nope)
 (send man :chest-beating 'yep)
 ;; There is no man yet:
 (ask man number)

 ;; Create tchapo, child of men:
 (def tchapo (new man :activities '(basketball soccer rap) :empathy 'high))
 (ask tchapo activities)
 (ask tchapo empathy)
 ;; There is now one man (albeit a little one):
 (ask man number)

 ;; Tchapo can change his humor slot (his is high!):
 (send tchapo :humor 10)
 ;; or his activities slot, as we've just seen;
 ;; And he can examine his hair slot:
 (send tchapo :hair)
 ;; But tchapo can't change mammal's hair slot:
 ;; (send tchapo :hair 'black)
 ;; nor human's bi-pedal slot
 ;; (send tchapo :bi-pedal 'sure)
 (show-slots tchapo)
"
  (let (
	(dispose-documentation 
	 (format nil "	This is the :dispose method for ~a objects.
The argument to :dispose should be an object of type ~a or a list of such
objects. All children of the proto are disposed of, so be nice!" a a))
	(add-documentation 
	 (format nil "	This is the :add method for ~a objects. The argument to
 :add should be an object which can be treated as of type ~a or a list of such
 objects. The object will then be considered as ~a." a a a))
	)
    (eval
     `(let ((cc (append ',c '(number active-list))))
	
	;; Define the proto
	(defproto ,a ',b cc ,d ,doc)
	(send ,a :slot-value 'number 0)
	
	;; create the instance-variable methods:
	(add-normal-assessors ,a)
     
	;; then the class-variable methods:
	(add-normal-assessors ,a ',c)

	(defmeth ,a :number (&rest e)
	  "The method :number returns the number of living instances
of a prototype (those that have not been disposed of - how callous we are)."
	  (length (slot-value 'active-list))
	  )
	
	(defmeth ,a :active-list (&rest e)
	  "The method :active-list returns the list of living instances
of a prototype (those that have not been disposed of - how callous we are)."
	  (slot-value 'active-list)
	  )

	(defmeth ,a :slots ()
	  "Slots method, so that one can use the show-slots function."
	  (mapcar (lambda(x) (slot-value x)) (send self :slot-names))
	  )
    
	;; Provide an isnew, the use of call-next-method having the effect of
	;; updating the active-list and number in parent, and so on up the
	;; family tree:
	(defmeth ,a :isnew (&rest e)
	  (call-next-method)
	  (let* (
		 (sl (if e (transpose (split-list e 2))))
		 (fsl (first sl))
		 (ssl (second sl))
		 )
	    (mapcar (lambda(x y) (send self x y)) fsl ssl)
	    )
	  (send ,a :slot-value 'number (1+ (send ,a :number)))
	  (send ,a :slot-value 'active-list
		(append (send ,a :active-list) (list self)))
	  self
	  )

	(defmeth ,a :dispose (&rest e)
	  ,dispose-documentation
	  (apply #'call-next-method e)
	  (let ((bong (remove-bunch e (send ,a :active-list))))
	    (send ,a :slot-value 'number (length bong))
	    (send ,a :slot-value 'active-list bong)
	    )
	  )
	
	(defmeth ,a :add (&rest e)
	  ,add-documentation
	  (apply #'call-next-method e)
	  (let ((bong (append (send ,a :active-list) e)))
	    (send ,a :slot-value 'number (length bong))
	    (send ,a :slot-value 'active-list bong)
	    )
	  )
	
	(defmeth ,a :save (&optional name)
	  "Needs a real save method: this will save instances of prototypes,
but it's maxx kludgy! It relies on the :save-self method I created for
*objects*, defined in obmeths.lsp in the math subdirectory."
	  (send self :save-self 
		(if name name (send self :slot-value 'proto-name))))
	)
     )
    nil
    )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                new
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmacro new (thingy &rest something)
  "
;; Description: (new thingy arguments)

	Creates a new thingy using the usual instance slots, where arguments
fill the usual slots.

;; Examples:
 (uber-proto longs (body) (aches))
 (uber-proto andy (arms legs) (pains) longs)
 (def andyroo (new andy :arms 2))
 (show andyroo)
 (def andyroo (new andy :arms 1 :legs 2))
 (show andyroo)
"
  (if something 
      `(send ,thingy :new ,@something)
    `(send ,thingy :new)))
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                show
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmacro show (thingy) 
  "
;; Description:

	(show object)

This is just a macro shorthand for (send object :show).

;; Examples:
 (uber-proto longs (body) (aches))
 (uber-proto andy (arms legs) (pains) longs)
 (def andyroo (new andy))
 (show andyroo)
 (show andy)
 (show (list andy andyroo))
"
  (if (listp thingy)
      `(mapcar #'(lambda(x) (send x :show)) ,thingy)
    `(send ,thingy :show)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                ask
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmacro ask (thingy something) 
  "
;; Description:
 (ask object slot-name)

	A macro which essentially functions as send.

;; Examples:
 (uber-proto longs (body) (aches))
 (uber-proto andy (arms legs) (pains) longs)
 (def andyroo (new andy))
 (show andyroo)
 (show andy)
 (ask andy number) ;; how many andys?
"
  `(send ,thingy :slot-value ',something))
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                change
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmacro change (f1 f2 f3)
  "
;; Description: macro
 (change object slot new-value)

;; Notes: based on function change2

;; Examples:
 (uber-proto longs (body) (aches))
 (uber-proto andy (arms legs) (pains) longs)
 (def andyroo (new andy))
 (ask andyroo arms)
 (change andyroo arms 1)
 (ask andyroo arms)
"
  (eval `(change2 ,f1 ',f2 ,f3)))
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                change2
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun change2 (thingy something to-this)
  "
 (change2 object slot new-value)
"
  `(send ,thingy :slot-value ',something ',to-this))
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                show-slots
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun show-slots (obj)
  "
;; Description: Function
	show-slots shows the slots and all their values for an object.

;; Examples:
 (uber-proto longs (body) (aches))
 (uber-proto andy (arms legs) (pains) longs)
 (def andyroo (new andy))
 (ask andyroo arms)
 (change andyroo arms 1)
 (show-slots andyroo)
"
  (let* (
	 (own-slots (reverse (remove-duplicates (send obj :slot-names))))
	 (own-slots (remove-bunch '(number active-list) own-slots))
	 (slot-vals
	  (mapcar (lambda(x) (send obj :slot-value x)) own-slots))
	 )
    (mapcar (lambda(x y) (format t "~t~a: ~s~%" x y)) own-slots slot-vals)
    )
  (values)
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 save-object
;;
(defun save-object(object)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          save-object Doc:
;;
;; Author: Andy Long
;; Created: Sat Nov 1 09:47:13 1997
;;
;; Description/Arguments:

       (save-object object)
  
where object is an object or a list of objects.

	Produces a command sufficient to reconstruct the given instance of an
object, provided a :save method is defined for the function. This is really
just a short-hand for (send object :save).

;; Notes:

;; See Also: the *object* method :save-self

;; Examples:
 (uber-proto longs (hio hoi) (thisn))
 (uber-proto andy (hio hoi) (thisn) longs \"Andy object\")
 (def andyroo (send andy :new :hio 1 :hoi 2))
 ;; uber-proto automagically creates a :save method for an instance of an
 ;; object:
 (save-object andyroo)
"
  (if (listp object) (mapcar #'(lambda(x) (send x :save)) object)
    (send object :save)))
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 dispose
;;
(defun dispose(object instance)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          dispose Doc:
;;
;; Author: Andy Long
;; Created: Sat Nov 1 09:47:13 1997
;;
;; Description/Arguments:

       (dispose object instance)
  
	object - an object or list of objects.
	instance - an instance or a list of instances

	If both object and instance are lists, they should have the same
length.

	Dispose removes an instance from the active-list slot of the object,
provided their is such a slot.

;; Notes:

;; See Also: the *object* method :dispose

;; Examples:
 (uber-proto longs (hio hoi) (thisn))
 (uber-proto andy (hio hoi) (thisn) longs)
 (def andyroo (send andy :new 1 2))
 ;; uber-proto automagically creates a :dispose method for an instance of an
 ;; object:
 (dispose andy andyroo)
 (ask andy number)
 (def andyroo (new andy))
 (def andyroo2 (new andy))
 (ask andy number)
 (dispose andy (list andyroo andyroo2))
 (ask andy number)
"
  (cond
   ((and (listp object) (listp instance))
    (mapcar #'(lambda(x y) (send x :dispose y)) object instance)
    )
   ((listp instance)
    (mapcar #'(lambda(y) (send object :dispose y)) instance)
    )
   ((listp object)
    (mapcar #'(lambda(y) (send x :dispose instance)) object)
    )
   (t
    (send object :dispose instance)
    )
   )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
