;; 				boundary.lsp:
#|
	I went through another procedure for matching boundary conditions, and
I want to compare it to the flesh2.lsp method. It took me awhile to figure out
that this method is related to the SVD method, but uses interpolators of
ordinary rather than singular vectors. I reasoned as follows:

	1) Since A=\u{g}_1 \u{e}_1^T + \u{g}_2 \u{e}_2^T

I can get the two edge functions g_1 and g_2 to work out by interpolating the
vectors \u{g}_1 and \u{g}_2 with the functions, and the unit vectors by linear
functions.

	This creates a linear interpolator across the two functions. If I want
instead the function h_1 at the one edge, I simply add the interpolator to the
interpolator of the zero matrix, with zero edges along the x direction and the
difference of the h functions and the linear interpolator along the other edges
(in the y direction):

	0=\u{0}(\u{h}_1 - \u{l}_1)^T  + \u{0} (\u{h}_2 - \u{l}_2)^T.

	2) Then I average the two types obtained, by following the prescription
above with g first, and then with h first. This led to the example
below. Following this example, I will try to generalize to fit a function mesh.
|#
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;	Start the same way that we do in flesh2.lsp:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(let () ;; we create a boundary:
  (setq x1 0
	x2 2
	y1 0
	y2 2
	xdiff (- x2 x1)
	ydiff (- y2 y1))
  (spin-domain x1 x2 y1 y2)
  (spin-points 21))

(func f(x y)
      ;; This piece is zero at x=0 and y=0
      -ln(x+1)*sin(y)
      +
      ;; a piece that is zero at x=2 and y=2:
      -sin(x2-x)*((y-1)* (y-y2))
      + 
      ;; this last piece which is zero on the entire boundary, as the product
      ;; of the first two pieces:
      -ln(x+1)*sin(y)*sin(x-x2)*(square (y-y2))
      ;; 
      + 1
      )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;	now we'll create the slices for comparison:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun g1(x) (funcall #'f x y1))
(defun g2(x) (funcall #'f x y2))
(defun h1(y) (funcall #'f x1 y))
(defun h2(y) (funcall #'f x2 y))
(defmacro average-function(f start end)
  `(lambda(x)
     (- (funcall #',f x)
	(half (+ (funcall #',f ,start)
		 (* 
		  (/ (- x ,start) (- ,end ,start))
		  (- (funcall #',f ,end) (funcall #',f ,start)))
		 )))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				newskin:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun newskin (x y)
  (+
   (* (/ (- y y1) ydiff) (funcall (average-function g2 y1 y2) x))
   (* (/ (- y2 y) ydiff) (funcall (average-function g1 y1 y2) x))
   (* (/ (- x x1) xdiff) (funcall (average-function h2 x1 x2) y))
   (* (/ (- x2 x) xdiff) (funcall (average-function h1 x1 x2) y))
   ))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			This is the flesh2 procedure:
;;
;; we'll compare it to the "new" procedure.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(title "flesh2-type skin")
(def xs (list x1 x2))
(def ys (list y1 y2))
(def A (outer-product xs ys #'f))
(defun G(x) (mapcar (lambda(y) (f x y)) ys))
(defun H(y) (mapcar (lambda(x) (f x y)) xs))
(def inv (pseudo-inverse A))
(defun skin(x y) (matmult (g x) inv (h y)))

(title "f")
(plot f)
(defun diff(x y) (- (skin x y) (f x y)))
(title "differences:s-f")
(plot diff)

(title "flesh2-type skin")
(plot skin)
(defun diff(x y) (- (newskin x y) (f x y)))
(title "differences:n-f")
(plot diff)

(title "new skin")
(plot newskin)
(defun diff(x y) (- (newskin x y) (skin x y)))
(title "differences:n-s")
(plot diff)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				Plot these together:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(def plots (select (agw) (iseq 6)))
(mapcar (lambda (x) (send x :linked t)) plots)
(tile plots :height 250 :width 250 :yshift 50 :ncol 3)

;; These should be identical, and they are:
;; (plot g1 (lambda(x) (newskin x y1)) x1 x2)
;; (plot g2 (lambda(x) (newskin x y2)) x1 x2)
;; (plot h1 (lambda(y) (newskin x1 y)) y1 y2)
;; (plot h2 (lambda(y) (newskin x2 y)) y1 y2)

#| - ael - Mon Aug 10 11:50:18 1998

	Okay, we have shown that we can succeed for arbitrary boundaries using
this method, but we can also match more general function meshes. The game is
clear now:

	1) use the Euclidean vectors and the rows (resp. columns) to get all
rows (resp. columns) right. In the previous example we used tent functions for
the Euclidean vectors, that are 1 at the non-zero value, fall linearly to
zero at the adjoining neighbors, and are zero elsewhere.

	2) find the interpolator of the zero matrix, with zero edges along the
x (rows) direction (resp. y - columns) and the difference of the column
(resp. row) functions and the interpolators of part 1 (in the previous example
these were linear functions).

	3) Steps 1) and 2) provide one interpolator, but for symmetry sake (if
for no other) I do both steps in both directions, then average the result.

 - lea - |#

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;	now we'll create the slices for comparison (m of them):
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(def m 3)
(title (format nil "Using ~d functions" m))
(def xs (rseq x1 x2 m))
(def ys (rseq y1 y2 m))
(def A (outer-product xs ys #'f))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				flesh procedure:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun G(x) (mapcar (lambda(y) (f x y)) ys))
(defun H(y) (mapcar (lambda(x) (f x y)) xs))
(def inv (pseudo-inverse A))
(mprint inv)
(defun skin(x y) (matmult (g x) inv (h y)))
(plot skin)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				new procedure:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun first-skin(f ys)
  (let (
	(tents
	 (mapcar
	  #'tent
	  (combine (- (first ys) (- (second ys) (first ys))) (butlast ys))
	  (combine (rest ys) (+ (last ys) (- (last ys) (last (butlast ys)))))
	  ))
	)
    (lambda(x y)
      (sum
       (*
	(funcall f x)
	(map-elements #'funcall tents y)
	)
       )
      )
    )
  )
(def fs (first-skin #'g ys))
(plot fs)
(defun second-skin(g h xs ys)
  (let (
	(fs (first-skin #'g ys))
	(tents
	 (mapcar
	  #'tent
	  (combine (- (first xs) (- (second xs) (first xs))) (butlast xs))
	  (combine (rest xs) (+ (last xs) (- (last xs) (last (butlast xs)))))
	  ))
	)
    (lambda(x y)
      (sum
       (*
	(- (funcall h y) (map-elements fs xs y)
	(map-elements #'funcall tents x)
	)
       )
      )
    )
  )
(def fs )
(plot (first-skin #'g ys))

