#| - ael - Tue Sep 8 19:29:47 1998

					cmj.lsp: 

	This code demonstrates a simple method for fitting arbitrary function
meshes. It followed acceptance of our paper in the College Math Journal, which
explains the name of the file.

	Motivation: for full-rank square matrices, our svd approach (as
demonstrated in file flesh.lsp, for example, but demonstrated again here) is
perfectly adequate. If the matrix is not full-rank, however, or is not square,
then we have to go to "plan B". This is the problem we consider here.

	We are given "slice functions" g(x,j) and h(i,y), where i and j are
row and column indices of the m by n matrix A. We want to fit them (interpolate
them) exactly. Here's the algorithm:

	1) Use the (pseudo-)inverse (the SVD method) with your favorite
	   interpolation scheme to generate the first guess, which we call
	   A_1(x,y). This will not fit the mesh functions g and h in general,
	   however. This means that there will be residual curves GG and HH,
	   which fit the zero matrix.

	2) Now we calculate the zero matrix function mesh: these functions
	   are given by
				GG(x,j) = g(x,j)-A_1(x,j)
	   and
				HH(i,y) = h(i,y)-A_1(i,y).

	3) Fit the zero matrix function mesh. To do so, we decompose the zero
	   matrix 0 into an outer-product expansion, as demonstrated here with
	   the 4x3 example:

		   [1 0 0]	   [0 1 0]	   [0 0 1]
		[0]		[0]		[0]
	   0 = 	[0]	   +	[0]	   +	[0]	    +
		[0]		[0]		[0]
		[0]		[0]		[0]

		   [0 0 0]	   [0 0 0]	   [0 0 0]	   [0 0 0]
		[1]		[0]		[0]		[0]
		[0]	   +	[1]	   +	[0]	    +	[0]
		[0]		[0]		[1]		[0]
		[0]		[0]		[0]		[1]

	   Now we interpolate each unit vector, and interpolate each zero
	   vector by the appropriate function (GG or HH) corresponding to the
	   column or row indicated. It's that easy! And yes, the interpolation
	   of each unit vector really IS arbitrary (as I will show in the
	   example by doing it in two different ways - tents and domes).
|#
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;	All that verbiage is longer than the actual code used to generate the
;; skin. Start with a test function (one with a little character):
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(func f(x y)
      -ln(x+1)*sin(y) +
      (sinc (x - 1) + (y - 1)) +
      -sin(end-x)*((y-1)* (y-end)) + 
      -ln(x+1)*sin(y)*sin(x-end)*(square (y-end))
      + 1)
;; (defun f(x y) (+ (* y x) (exp (* y (1- x))) (sin (* y 6 (ln (+ 1 x))))))

;; We'll work on the square from 0 to 2 in each coordinate:
(def start 0)
(def end 2)

;; Consider an mxn matrix:
(def m 5)
(def n 3)
(def xs (rseq start end m))
(def ys (rseq start end n))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;      We create the matrix A by evaluating f on the grid points, and 
;; deduce the function mesh by taking slices of the function f along the grid
;; lines:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(def A (outer-product xs ys #'f))
;; These are VECTOR functions in lisp:
(defun G(x) (mapcar (lambda(y) (f x y)) ys))
(defun H(y) (mapcar (lambda(x) (f x y)) xs))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;	Now we create the interpolating function for the matrix (which fails to
;; interpolate the function mesh) using the traditional SVD approach:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(def inv (pseudo-inverse A))
(defun skin(x y) (matmult (g x) inv (h y)))
(defun diff(x y) (- (skin x y) (f x y)))

(spin-domain start end start end)
(spin-points 17) ;; 17 is pretty good for the example I chose (5x3 matrix):
(title "f and its skin")
(plot f skin)
(title "SVD skin - f")
(plot diff)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; 	Now we'll correct for the error (i.e., interpolate the function mesh):
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
#|	Clearly diff isn't zero along all five functions forming the mesh (it is
zero along the two-function dimension of the g functions, indicating that the
skin gets the g's right, but fails to be zero along the h dimensions (fails to
get any of the h functions correct)).

	Thus we now fit a function to the zero matrix, with the proper function
mesh given by the functions gg and hh. These are essentially the errors, which
we form by subtracting the SVD method slices from the true mesh function: |#

(defun GG(x) (- (g x) (mapcar (lambda(y) (skin x y)) ys)))
(defun HH(y) (- (h y) (mapcar (lambda(x) (skin x y)) xs)))

#|	Each of these functions will multiply an interpolator of a unit
vector. Tent functions will be used by default. I'll also consider a dome
function, which will smooth things out (make things differentiable, at
least). |#
	
(defun trial (&optional (type #'tent))
  (let* (
	 ;; These are the differences between equally spaced points:
	 (xdiff (- (second xs) (first xs)))
	 (ydiff (- (second ys) (first ys)))
	 ;; create tent (or dome) function interpolators for the unit vectors:
	 (xfuns (mapcar type (- xs xdiff) (+ xs xdiff)))
	 (yfuns (mapcar type (- ys ydiff) (+ ys ydiff)))
	 )
    ;; zeroskin is the interpolator of the outer-product expansion of the zero
    ;; matrix: 
    (defun zeroskin(x y)
      (+
       (dot (gg x) (mapcar (lambda(fun) (funcall fun y)) yfuns))
       (dot (hh y) (mapcar (lambda(fun) (funcall fun x)) xfuns))))

    (title "Pretty tight fit! That's two functions.")
    (plot f (lambda(x y) (+ (skin x y) (zeroskin x y))))

    (title "should see zeros along the mesh")
    (plot (lambda(x y) (- (f x y) (+ (skin x y) (zeroskin x y)))))
    ))

(trial) ;; with tent functions, by default
(trial #'dome)

#| - ael - Fri Sep 11 02:28:11 1998

	That's all folks! Nothin' to it.

 - lea - |#
