;;; ::::::::::::::::::::::::::::::
;;; a simple help system
;;; f. udina, jul 99 version 99/7/20.- 1.01beta
;;; ::::::::::::::::::::::::::::::

#| This file implements a simple help system for alling lisp functions
through a dialog window that shows the required (and optional)
arguments.

It looks for the arguments in the function-lambda-expression,
where defun stores it.

For compiled functions, that does'nt have lambda-expression available,
it looks for the args in the doc string.
It requires the documentation string having a particular format. 
The first line must begin with "ARGS: " followed by the lambda list of 
the arguments in the same format as appear in the defun (see below for
a couple of examples).

If the function FOO is defined with a conforming doc string, then calling
(dhelp foo)
will produce a dialog window showing the required arguments for the call
and also key arguments, optional arguments or rest arguments.

After loading dhelp.lsp, try (dhelp 'dhelp)

The current version does'nt support more than one 
keyword of the key, optional, rest.

Examples:
(defun make-kregression (data headers col1 col2 &rest args)
"ARGS: data headers col1 col2 &rest args"
(...))

(defun draw-cross-summary (data headers row col what &key (output t) (col-width 8) (sorted t))
"ARGS: data headers row col what &key (output t) (col-width 8) (sorted t)
Draws a cross-summary of WHAT in DATA with headers using
ROW and COL (ROW values will be used as row headers,
COL values as column headers in the table drawn).
 Keys: output is an output stream, col-width is cell width,
sorted t means sort the values, nil means do not."
(...))

|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||#

(defun dhelp (funname &optional (what 'dialog))
"ARGS: funname &optional (what 'dialog)
 Args need to be quoted.
 Given a funname, if what is template, it simply prints a
template of the call. If what is dialog (default), a dialog window 
appear for the call arguments being filled. An optional or key arg
left empty, will not appear in the call."
(unless (fboundp funname)
  (format t "~a is'nt bound to any function" funname)
  (top-level))
(multiple-value-bind
 (1stline restdoc doc args pos pargs kargs oargs rargs)
 (dhelp-get-args funname)
 (setq doc (documentation funname 'function))
 (unless (stringp 1stline)
   (format t "~a does'nt contain information about args" funname)
   (top-level))
 (setq args (if (and ;;remove surrounding ()
		 (stringp 1stline)
		 (> (length 1stline) 1)
		 (char= #\( (select 1stline 0))
		 (char= #\) (select 1stline (1- (length 1stline )))))
		(subseq 1stline 1 (1- (length 1stline)))
	      1stline))
 (setq pos (position #\& args))
 (when pos
   (setq pargs (if (= 0 pos) nil
		 (subseq args 0 (1- pos))))
   (cond ((string= "&KEY " (string-upcase (subseq args pos (+ pos 5))))
	  (setq kargs (subseq args (+ pos 5))))
	 ((string= "&REST " (string-upcase (subseq args pos (+ pos 6))))
	  (setq rargs (subseq args (+ pos 6))))
	 ((string= "&OPTIONAL " (string-upcase (subseq args pos (+ pos 10))))
	  (setq oargs (subseq args (+ pos 10)))))
   (when (or (and kargs (position #\& kargs))
	     (and oargs (position #\& oargs))
	     (and rargs (position #\& rargs)))
     (format t "more than one keyword not supported: ~a, ~a" funname
	    args)
     (top-level)))
 (unless pos (setq pargs args))
 (ecase what
	(template (format nil "(~a ~a)~%" funname args))
	(dialog (dhelp-dialog-call funname (dhelp-split-args pargs) 
				   (dhelp-split-args kargs t) 
				   (dhelp-split-args oargs)
				   (dhelp-split-args rargs)
				   (if restdoc restdoc doc))))))

(defun dhelp-get-args (funname)
"Args: funname
 Return in a single line with no leading, no trailing, no double spaces
 the arguments for the function FUNNAME.
 Return nil if args not found.
 Second value is the rest of the doc string."
(let ((white-chars (list #\space #\tab #\newline))
      args doc docargs restdoc 1stline posend)
  (setq args (second (function-lambda-expression 
		      (symbol-function funname))))
  (when args (setq args (write-to-string args :case :downcase)))
  ;;look also in doc
  (setq doc (documentation funname 'function))
  (setq restdoc doc)
  (when (and (stringp doc)
	     (string= "ARGS:" (string-upcase (subseq doc 0 5))))
    (setq 1stline (subseq doc 5))
    ;;remove leading white space
    (loop (unless (and (> (length 1stline) 0)
		       (member (select 1stline 0) white-chars))
	    (return))
	  (setq 1stline (subseq 1stline 1)))
    ;;look for surrounding ()
    (if (char= #\( (select 1stline 0))
	(setq posend (dhelp-look-closing 1stline))
      (setq posend (position #\newline 1stline :test #'char=)))
    (when posend
      (setq docargs (dhelp-clean-white (subseq 1stline 0 posend)))
      (when (< (+ 6 posend) (length 1stline))
	(setq restdoc (subseq doc (+ 6 posend)))
	(loop (unless (and (> (length restdoc) 0)
		       (member (select restdoc 0) white-chars))
	    (return))
	  (setq restdoc (subseq restdoc 1)))
	))
    )
  (values (if args args docargs) restdoc)))

(defun dhelp-look-closing (str)
  "Returns position + 1 of the closing parenthesis
that closes the first one (does'nt check for it)"
(let (ch (pos 0) (level 0))
  (loop
   (when (>= pos (length str))
     (return nil))
   (setq ch (select str pos))
   (case ch
	 (#\( (incf level))
	 (#\) 
	  (decf level)
	  (if (= 0 level) (return (1+ pos)))))
   (incf pos))))


(defun dhelp-clean-white (str &aux res white ch len last-was-white)
  "replace any several white-space in a row
by a single space. Delete leading and trailing too."
  (setq res (make-string (length str)))
  (setq white (list #\space #\tab #\newline))
  (setq last-was-white t)
  (setq len 0)
  (dotimes (pos (length str))
	   (setq ch (aref str pos))
	   (cond ((member ch white)
		  (unless last-was-white
		    (setq ch #\space)
		    (setf (aref res len) ch)
		    (incf len)
		    (setq last-was-white t)))
		 (t (setf (aref res len) ch)
		    (incf len)
		    (setq last-was-white nil))))
  (if (= 0 len)
      ""
     (if (char= #\space (aref res (1- len)))
	 (subseq res 0 (1- len))
       (subseq res 0 len))))

(defun dhelp-split-args (args &optional are-keys)
"given a string, returns a list of words,
 a pair (key value) would be a single word"
(let* ((rest args)
       wordlist sppos parpos clparpos)
  (loop
   ;;(format t "rest: ~a~%" rest)
   (loop ;skip spaces
    (when (or (= 0 (length rest))
	      (char/= #\space (select rest 0)))
      (return))
    (setq rest (subseq rest 1)))
   (when (= 0 (length rest)) (return (reverse wordlist)))
   
   (setq sppos (position #\space rest :test #'char=))
   (setq parpos (position #\( rest :test #'char=))
   (cond ((and parpos
	       (< parpos sppos))
	  (setq clparpos (position #\) rest :test #'char=))
	  (unless clparpos 
	    (format t "bad parenthesis: ~a" args)
	    (top-level))
	  (push (format nil (if are-keys ":~a ~a" "~a ~a")
			(subseq rest 1 sppos)
			(subseq rest (1+ sppos) clparpos))
		wordlist)
	  (setq rest (subseq rest (1+ clparpos))))
	 (sppos
	  (push (format nil (if are-keys ":~a" "~a")(subseq rest 0 sppos))
		wordlist)
	  (setq rest (subseq rest (1+ sppos))))
	 (t (push (format nil (if are-keys ":~a" "~a") rest)
		  wordlist)
	    (setq rest ""))))
))




(defun dhelp-dialog-call (funname args keyargs optargs restargs restdoc)
  (let* ((size (cond ((member :MSDOS *features*)
		      '(220 20))
		     ((member :X11 *features*)
		      '(220 25))
		     (t '(220 20))))
	 (restdoc-size (cond ((member :MSDOS *features*)
			      '(450 120 70))
			     ((member :X11 *features*)
			      '(450 120 50))
			     (t '(450 120 50))))
	 (prompt (format nil "Fill the arguments for\n ~a" funname))
	 (args-items
	  (mapcar #'(lambda (arg)
		      (list (send text-item-proto :new arg :size size)
			    (send text-item-proto :new (cond ((string= arg "data")
							      "*data*")
							     ((string= arg "headers")
							      "*headers*")
							     (t ""))
				  :size size :editable t)))
		  args))
	 (kor-prompt (format nil "~a arguments:" (cond (keyargs "Key")
						       (optargs "Optional")
						       (restargs "Rest"))))
	 splout tmp sppos sp2pos
	 (split-key #'(lambda (str)
			;;:key -> (key "") and :key val -> (key val)
			(setq sppos (position #\space str :test #'char=))
			(if sppos
			    (setq sp2pos (position #\space str :test #'char=
						   :start (1+ sppos)))
			  (setq sp2pos nil))
			(cond (sp2pos
			       (list (subseq str 0 sppos)
				     (subseq str (1+ sppos) sp2pos)))
			      (sppos
			       (list (subseq str 0 sppos)
				     (subseq str (1+ sppos))))
			      (t (list str "")))))
	 (other-items (cond
		       ((or keyargs optargs)
			(setq splout (mapcar split-key
					     (if keyargs keyargs optargs)))
			(mapcar #'(lambda (pair)
				    (list (send text-item-proto :new
						(first pair) :size size)
					  (send text-item-proto :new
						(second pair) :size size :editable t)))
				splout))
		       (restargs
			(list (list (send text-item-proto :new
					  (car restargs) :size size)
				    (send text-item-proto :new
					  "" :size size :editable t))))
		       (t nil)))
	 (ok-btn (Send button-item-proto :new "OK"))
	 (cancel-btn (Send button-item-proto :new "Cancel"))
	 (print-btn (Send button-item-proto :new "Print it"))
	 (help-btn (Send button-item-proto :new "Help"))
	 (doc-text (send text-item-proto :new 
			 (dhelp-break-string-lines 
			  (third restdoc-size) restdoc)
			 :size (select restdoc-size '(0 1)) :editable nil))
	 items dlg repl fcall scall)
    (setq items (list prompt
		      (list args-items)
		      ))
    (when other-items
	(setq items (append items
			    (list kor-prompt)
			    (list (list other-items)))))
    (setq items (append items
			(list (list ok-btn print-btn cancel-btn help-btn))
			(list doc-text)))
    (setq dlg (send dialog-proto :new items))
    (loop (setq repl (Send dlg :modal-dialog))
	  (when (member repl (list ok-btn cancel-btn print-btn))
		(if (eq repl cancel-btn)
		    (return)
		  (if (every #'identity
			     (mapcar #'(lambda (text-item)
					 (string/= "" (send text-item
							   :text)))
				     (second (transpose args-items))))
		      (return)
		    (message-dialog "You must fill the mandatory args"))))
	  (when (eql repl help-btn)
		(message-dialog (dhelp-break-string-lines
				 80 (documentation funname 'function)))))
    (cond ((eq repl cancel-btn)
	   (send dlg :dispose)
	   (values))
	  ((member repl (list ok-btn print-btn))
	   (setq fcall (list (symbol-name funname)))
	   (dolist (pair args-items)
		   (push (send (Second pair) :Text) fcall))
	   (when other-items
		 (dolist (pair other-items)
			 (cond (keyargs
				(unless (string= "" (send (second pair) :text))
					(push (send (first pair) :text) fcall)
					(push (send (second pair) :text) fcall)))
				 ((or optargs restargs)
				  (push (send (second pair) :text) fcall)))))
	   (setq fcall (Reverse fcall))
	   (setq scall "(")
	   (dolist (w fcall)
		   (setq scall (concatenate 'string scall w " ")))
	   (setq scall (concatenate 'string scall ")"))
	   (send dlg :dispose)
	   (if (eq print-btn repl)
	       (progn (format t "~a~%" scall) (values))
	     (progn (format t "DHELP is calling ~s~%" scall)
		    (eval (car (eval-string scall)))))))
))

(defun dhelp-break-string-lines (num str-text &key max-lines)
 "args: num str. Changes spaces to newlines in a copy of str to fit the text in
num width. Fails if a single word is larger than num"
(when (member :MSDOS *features*)
  (setq str-text (concatenate 'string str-text "#\newline")))
(let ((str (copy-seq str-text))
      (last-break 0)
      (last-space 0)
      (pos 0)
      (num-lines 1))
  (loop
   (when (and max-lines (> num-lines max-lines))
     (return (trim-string str pos)))
   (when (>= pos (length str))
     (return str))
   (if (char= (select str pos) #\Space)
       (setf last-space pos))
   (when  (char= (Select str pos) #\newline)
     (setf last-break pos)
     (when max-lines (incf num-lines)))
   (when (>= (- pos last-break) num)
	 (setf (select str last-space) #\newline)
	 (when max-lines (incf num-lines))
	 (setf last-break last-space))
   (incf pos))))


(defun eval-string (aString)
  "Returns a list of the expressions obtained 
on evaluation of the characters in aString"  
  (let ((st (make-string-input-stream aString))   
        (result nil)   
        (expr nil))  
    (loop (setq expr (read st nil 'eof))  
          (when (eql expr 'eof)    
                (return (reverse result)))  
          (setf result (cons expr result)))))

;;;;;;;;;;;;;;;;;;PROVIDE and exit
(provide "dhelp")
;;;;;;;;;;;;;;;;;;end of file dhelp.lsp
