;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                lorenz.lsp
#|
	This is a collection of examples and exercises to look at different
aspects of the Lorenz equations.
|#
;;
(setq sigma 10
      r 28
      b 8/3)
(def f
     (list
      (lambda(x y z) (* sigma (- y x)))
      (lambda(x y z) (- (* x (- r z)) y))
      (lambda(x y z) (- (* x y) (* z b))))
     )
(defun lorenz (vals)
  (let (
	(x (first vals))
	(y (second vals))
	(z (third vals))
	)
    (list
     (* sigma (- y x))
     (- (* x (- r z)) y)
     (- (* x y) (* z b)))
    )
  )

;; These are some decent parameter values:
(defun orbits (xin yin zin &optional (start 0) (end 4) (points 500))
  (let ((points (euler-nd #'lorenz (list xin yin zin) start end points)))
    (list
     (mapcar #'first points) (mapcar #'second points) (mapcar #'third points)
     )
    ))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                pulses
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(setq r 28)
(def points (rk-4 #'lorenz (* 10 (rand.n 3)) 0 10 500))
(plot (transpose points))
(def pulses (mapcar #'square (mapcar #'first points)))
(plot pulses)
(setq l (1- (length pulses)))
(setq n 2)
(plot
 (select pulses (iseq 0 (- l n)))
 (select pulses (iseq n l)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; This will show a certain number ntimes of orbits:
(defun lorenz-show (ntimes &optional rvals)
  (let (
	starting
	orbit
	)
    (dotimes (i ntimes)
	     (setq starting (* 10 (uniform-rand 3)))
	     (if rvals (setq r (elt rvals i)))
	     (setq orbit
		   (orbits (first starting) (second starting) (third starting)))
	     (when (= i 0)
		   (def myplot (spin-plot orbit))
		   (send myplot :depth-cuing nil)
		   (send myplot :showing-axes nil)
		   (send myplot :title "Lorenz Equations")
		   (send myplot :add-lines orbit)
		   )
	     (send myplot :add-lines orbit)
	     )
    myplot
    )
  )

(lorenz-show 3)
(lorenz-show 30 (rseq 1 25 30))

(def nextpoints (* 10 (uniform-rand 3)))
(orbitals #'lorenz nextpoints 1 50 50)
(orbitals #'lorenz nextpoints 4 200 10 :randomize 10)

(mprint (jacobian f '(x y z)))
(format t "
from this we see that stability is identical at the non-zero equilibrium
points: if we multiply the jacobian J by the identity matrix with the third
diagonal element changed to -1 (")
(mprint (diagonal-to-full '(1 1 -1)))
(format t "
) both fore and aft, then we've got a similar matrix with the same
eigenvalues.
")

(setq jacky-o (jacobian f '(x y z) :matrix-function t))
(eigenvalues (funcall jacky-o 0 0 0))
(def x1 (sqrt (* b (- r 1))))
(def z1 (/ (square x1) b))
(format t "~%confirmation of the equivalence of the eigenvalues:~%")
(eigenvalues (funcall jacky-o x1 x1 z1))
(eigenvalues (funcall jacky-o (- x1) (- x1) z1))

(digits 6)
(dolist (i
	 (list
	  (eigenvalues (funcall jacky-o x1 x1 z1))
	  (eigenvalues (funcall jacky-o 0 0 0))
	  ))
	(print (mapcar #'realpart i)))

;; analytically, the eigenvalue at zero are:
(combine (- b) (quadratic-roots 1 (+ 1 sigma) (* sigma (- 1 r))))

(title "The origin is unstable for r>1:")
(plot 
 (transpose 
  (mapcar 
   (lambda(r) (quadratic-roots 1 (+ 1 sigma) (* sigma (- 1 r))))
   (rseq 1 25 100))))
(plot 
 (transpose 
  (mapcar 
   (lambda(r) 
     (let* (
	    (x1 (sqrt (* b (- r 1))))
	    (z1 (/ (square x1) b))
	    (roots (mapcar #'realpart (eigenvalues (funcall jacky-o x1 x1 z1)))))
       (list (max roots) (min roots))))
   (rseq 1 25 100))))

;; I want to plot all the eigenvalues. The problem is that they are
;; occasionally switched when they are equal at some point. Thus I need to
;; find some way to keep them straight.

(mapcar 
 (lambda(r) 
   (let* (
	  (x1 (sqrt (* b (- r 1))))
	  (z1 (/ (square x1) b))
	  (roots (eigenvalues (funcall jacky-o x1 x1 z1)))
	  )
     roots))
 (rseq 1 25 25))

(plot
 (transpose
  (mapcar 
   (lambda(r) 
     (let* (
	    (x1 (sqrt (* b (- r 1))))
	    (z1 (/ (square x1) b))
	    (f
	     (list
	      (lambda(x y z) (* sigma (- y x)))
	      (lambda(x y z) (- (* x (- r z)) y))
	      (lambda(x y z) (- (* x y) (* z b))))
	     )
	    (jackyo (jacobian f '(x y z) :matrix-function t))
	    (roots (eigenvalues (funcall jackyo x1 x1 z1)))
	    (is-it-real (which (mapcar #'realp roots)))
	    (is-it-complex (which (mapcar #'complexp roots)))
	    (roots (if (= 3 (length is-it-real))
		       roots
		     (let ((index 
			    (select
			     roots 
			     (combine is-it-real (first is-it-complex)))))
		       (combine index (conjugate (final index))))))
	    )
       (combine (mapcar #'realpart roots) (mapcar #'imagpart roots))))
   (rseq 1 5 100))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(def xs (rseq 1 5 100))
(def oints
  (mapcar 
   (lambda(r) 
     (let* (
	    (x1 (sqrt (* b (- r 1))))
	    (z1 (/ (square x1) b))
	    (f
	     (list
	      (lambda(x y z) (* sigma (- y x)))
	      (lambda(x y z) (- (* x (- r z)) y))
	      (lambda(x y z) (- (* x y) (* z b))))
	     )
	    (jackyo (jacobian f '(x y z) :matrix-function t))
	    (roots (eigenvalues (funcall jackyo x1 x1 z1)))
	    (is-it-real (which (mapcar #'realp roots)))
	    (is-it-complex (which (mapcar #'complexp roots)))
	    (roots (if (= 3 (length is-it-real))
		       roots
		     (let ((index 
			    (select
			     roots 
			     (combine is-it-real (first is-it-complex)))))
		       (combine index (conjugate (final index))))))
	    )
       roots))
   xs))
(plot xs (first (transpose oints)))
(send my-plot :add-plot-item 
      (list 'lines xs (mapcar #'realpart (second (transpose oints))) 'red 2)
      (list 'lines xs (mapcar #'imagpart (second (transpose oints))) 'red 2)
      (list 'lines xs (mapcar #'realpart (third (transpose oints))) 'blue 2)
      (list 'lines xs (mapcar #'imagpart (third (transpose oints))) 'blue 2)
      )
(send my-plot :adjust-to-data)

(setq r 1.1)
(setq jacky-o (jacobian f '(x y z) :matrix-function t))
(eigenvalues (funcall jacky-o 0 0 0))
(def x1 (sqrt (* b (- r 1))))
(def z1 (/ (square x1) b))
(format t "~%confirmation of the equivalence of the eigenvalues:~%")
(eigenvalues (funcall jacky-o x1 x1 z1))
