;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			A function proto:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(uber-proto
  function-proto 
  (function
    domain
    num-vars
    derivative
    anti-derivative
    inverse
    domain-test-function
    plot-domain
    plot-points
    description
    )
  nil
  *object*
  "A Basic Function Proto

Author: Andy Long - April 1998

	This is my function proto, a start on something which
simultaneously serves my mathematical interests and my interests in OOP. I'm
trying to understand the general xlispstat method of prototypes, and developing
my own methods which I want objects to share.

	I foresee at least the following types of child functions:

	- univariate-function-proto
	- bivariate-function-proto
	- distribution-proto		;; for probability distributions

One problem I envision is that something might really be both univariate and a
distribution. It can't be at the same time a child of both prototypes. How to
handle this problem? At the moment, I imagine that I might be able to use
:reparent. I'm going to work on this in obj-dist.lsp.

	Another thing: I want prototypes to keep track of their kids. 
Kids keep track of their kids, and if I want to know about my grandkids I ask
my kids about their kids. If kids want to know about their grand-parents, they
ask their parents.

	To do that we need to add a new :isnew message to kids, to update its
family tree when it is born. It sends a message to its parent. And at its death
it also has to send a message (:dispose), that it is no more.

	There has to be some adjustment when the reparent method is called -
otherwise, too many members of the family will be counted.

"
  )

;; Initialize the associated functions to "nilly", a function which returns
;; nil:
(send function-proto :derivative #'nilly)
(send function-proto :anti-derivative #'nilly)
(send function-proto :inverse #'nilly)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			Some kid protos:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			univariate-function-proto
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(uber-proto
 univariate-function-proto nil nil function-proto
 "A univariate function proto.

;; Examples:
 (setq
  sin
  (send 
   univariate-function-proto
   :new
   :function (lambda(x) (sin x)) 
   :plot-domain (list -pi pi)
   :derivative (lambda(x) (cos x))
   :anti-derivative (lambda(x) (- (cos x)))
   :inverse (lambda(x) (asin x))
   :domain-test-function (lambda(x) t)
   :domain 'analytic
   :num-vars 1
   :description \"The trigonometric function sine.\"
   ))
 (send sin :what)

 (def
  cos
  (send 
   univariate-function-proto
   :new
   :function (lambda(x) (cos x))
   :plot-domain (list -pi pi)
   :derivative (lambda(x) (- (sin x)))
   :anti-derivative (lambda(x) (sin x))
   :inverse (lambda(x) (acos x))
   :domain-test-function (lambda(x) t)
   :domain 'analytic
   :num-vars 1
   :description \"The trigonometric function cosine.\"))
 (send cos :what)
")

(send univariate-function-proto :help :help)
(send univariate-function-proto :num-vars 1)
(send univariate-function-proto :plot-domain '(-1 1))
(send univariate-function-proto :plot-points 100)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			bivariate-function-proto
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(uber-proto bivariate-function-proto nil nil function-proto
	    "A bivariate function proto.

;; Examples:
 (def
  sin-x+y
  (send 
   bivariate-function-proto
   :new
   :function (lambda(x y) (sin (+ x y)) )
   :num-vars 2
   :domain 'analytic
   :plot-points 20
   :plot-domain (list -pi pi -pi pi)
   :description \"The trigonometric function sine(x+y).\"
   ))
 (send sin-x+y :what)
")

(send bivariate-function-proto :help)
(send bivariate-function-proto :num-vars 2)
(send bivariate-function-proto :plot-domain '(-1 1 -1 1))
(send bivariate-function-proto :plot-points 20)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				METHODS
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;	:try-out
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth function-proto :try-out ()
  (helplass (send self :documentation 'proto))
  (values))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;	:plot
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth function-proto :plot ()
  "
	Collects the information that any specific function will need to plot
properly. It is for each sub-type to actually plot.
;; Examples:
 (send sin :plot)
"
  (mapcar
   (lambda(x) (send self :slot-value x))
   '(num-vars plot-domain plot-points)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;	:new-parent
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth function-proto :new-parent (new-parent)
  (let ((old-parent (first (send self :parents))))
    (send old-parent :dispose self)
    (send new-parent :add self)
    (send self :reparent new-parent)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;	:what
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth function-proto :what (&optional (stream t))
  "
;; Description: describes a function instance.

;; Examples:
 (send sin-x+y :what)
"
  (let (
	(description (send self :description))
	(slots (send self :slot-value 'instance-slots))
	)
    (if description (format stream "~%~a~%" description))
    (mapcar
     (lambda(x)
       (let ((y (send self :slot-value x)))
	 (format
	  stream "~%~s:~30t~s"
	  x
	  (if (functionp y) (back-parse (third (get-lambda-expression y))) y))
	 ))
     slots))
  (format stream "~2%")
  (values)
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;		:plot (univariate-function-proto method)
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth univariate-function-proto :plot ()
  "
Plots the function on its standard plot-domain.
;; Examples:
 (send sin :plot)
"
  (let ((plot-info (call-next-method)))
    (points (third plot-info))
    (plotter (combine (send self :function) (second plot-info)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;		:plot-all (univariate-function-proto method)
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth univariate-function-proto :plot-all ()
  "
;; Description: Plots all available functions: the function itself;
   derivative, anti-derivative, and inverse.

;; Examples:
 (send cos :plot-all)
"
  (points (send self :plot-points))
  (plotter (combine
	    (send self :function)
	    (send self :derivative)
	    (send self :anti-derivative)
	    (send self :inverse)
	    (send self :plot-domain))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;		:plot (bivariate-function-proto method)
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth bivariate-function-proto :plot ()
  "
Plots the function on its standard plot-domain.
;; Examples:
 (send sin-x+y :plot)
"
  (let (
	(plot-info (call-next-method))
	(tmp-dom *spin-domain*)
	(nmpnts *spin-points*)
	my-plot
	)
    (apply #'spin-domain (second plot-info))
    (spin-points (third plot-info))
    (setq my-plot (plotter (send self :function)))
    (spin-points nmpnts)
    (apply #'spin-domain tmp-dom)
    my-plot
    ))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			Look at some examples:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(send univariate-function-proto :try-out)
(send bivariate-function-proto :try-out)

(try-out :plot :proto univariate-function-proto)
(try-out :plot-all :proto univariate-function-proto)
(try-out :plot :proto bivariate-function-proto)
