(defun euler-delay-plot (f ynot start end tao &key axes title)
  "
;; Description:

	euler-delay-plot (f ynot start end tao &key axes title)

For delay equations, you'll need a tao, and the length of the initial
list ynot will determine the number of steps in one tao unit.

;; Examples:
;;
;; This is just a quick example that I thought might be interesting:
;;
 (def yknot 1)
 (def num 200)
 (def tao 2)
 (def ynot 1)
 (def tmin 0)
 (def tmax 16)
 (def ynot (repeat yknot (ceiling (* num (/  tao (- tmax tmin))))))

;; This is not a delay equation: it should settle down to sinusoidal variation:
 (def alpha -.5)
 (defun f(x time xd) (+ (* alpha x) (sin (* 2 time))))
 (euler-delay-plot #'f ynot tmin tmax tao :title \"No Delay\")

 (defun f(x time xd) (+ (* alpha xd) (sin (* 2 time))))
 (euler-delay-plot #'f ynot tmin tmax tao :title \"Delay\")
 ;;
 ;; From the exercises at the end of chapter 4, 'Understanding Non-linear
 ;; Dynamics', by Kaplan and Glass, for one-d diffeqs:
 ;;
 ;; This example is suggested by the authors, in computer project 3:
 ;;
 (def num 400)
 (def tao 2)
 (def tmin 0)
 (def tmax 32)
 (defun f(x time xd) (- (/ (* 2 xd) (+ 1 (^ xd alpha))) x))

 (def alpha .25)
 (def yknot 1.3)
 (def ynot (repeat yknot (ceiling (* num (/  tao (- tmax tmin))))))
 (euler-delay-plot #'f ynot tmin tmax tao :title \"Repeated high: .25\")

 (def yknot .3)
 (def ynot (repeat yknot (ceiling (* num (/  tao (- tmax tmin))))))
 (euler-delay-plot #'f ynot tmin tmax tao :title \"Repeated low: .25\")

 (def ynot (uniform-rand (ceiling (* num (/  tao (- tmax tmin))))))
 (euler-delay-plot #'f ynot tmin tmax tao :title \"Uniform Rand: .25\")

 (def alpha 1)

 (def yknot .3)
 (def ynot (repeat yknot (ceiling (* num (/  tao (- tmax tmin))))))
 (euler-delay-plot #'f ynot tmin tmax tao :title \"Repeated low: 1\")

 (def yknot 1.3)
 (def ynot (repeat yknot (ceiling (* num (/  tao (- tmax tmin))))))
 (euler-delay-plot #'f ynot tmin tmax tao :title \"Repeated high: 1\")

 (def ynot (normal-rand (ceiling (* num (/  tao (- tmax tmin))))))
 (euler-delay-plot #'f ynot tmin tmax tao :title \"Normal: 1\")

 (def ynot (uniform-rand (ceiling (* num (/  tao (- tmax tmin))))))
 (euler-delay-plot #'f ynot tmin tmax tao :title \"Uniform: 1\")

 (def alpha 2)

 (def yknot 1.3)
 (def ynot (repeat yknot (ceiling (* num (/  tao (- tmax tmin))))))
 (euler-delay-plot #'f ynot tmin tmax tao :title \"Repeated high: 2\")

 (def ynot (normal-rand (ceiling (* num (/  tao (- tmax tmin))))))
 (euler-delay-plot #'f ynot tmin tmax tao :title \"Normal: 2\")

 (def ynot (uniform-rand (ceiling (* num (/  tao (- tmax tmin))))))
 (euler-delay-plot #'f ynot tmin tmax tao :title \"Uniform: 2\")

 (def alpha 4)

 (def yknot 1.3)
 (def ynot (repeat yknot (ceiling (* num (/  tao (- tmax tmin))))))
 (euler-delay-plot #'f ynot tmin tmax tao :title \"Repeated high: 4\")

 (def ynot (normal-rand (ceiling (* num (/  tao (- tmax tmin))))))
 (euler-delay-plot #'f ynot tmin tmax tao :title \"Normal: 4\")

 (def ynot (uniform-rand (ceiling (* num (/  tao (- tmax tmin))))))
 (euler-delay-plot #'f ynot tmin tmax tao :title \"Uniform: 4\")
"
  (let* (
	 (hsize (/ tao (length ynot)))
	 (delay-index (length ynot))
	 (yvalues ynot)
	 (num (ceiling (/ (- end start) hsize)))
	 (timesteps (rseq start end num))
	 j
	 )
    (dotimes
     (i (- num delay-index))
     (setq j (1- (+ i delay-index)))
     (setq yvalues
	   (append yvalues 
		   (+ (last yvalues)
		      (* hsize 
			 (funcall 
			  f 
			  (select yvalues j)
			  (select timesteps j)
			  (select yvalues i)
			  )
			 )
		      )
		   )
	   )
     )
    (def my-plot (plot-lines timesteps yvalues))
    (if axes (let 
		 (
		  (xmin (select axes 0))
		  (xmax (select axes 1))
		  (ymin (select axes 2))
		  (ymax (select axes 3))
		  )
	       (send my-plot :range 0 xmin xmax)
	       (send my-plot :range 1 ymin ymax)
	       )
      )
    (if title (send my-plot :title title))
    )
  my-plot
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                        euler-delay-plot-continued
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun euler-delay-plot-continued (f ynot start end tao my-plot &key axes title)
  "
;; Description:

   (euler-delay-plot-continued f ynot start end tao my-plot &key axes title)

	Picks up where Euler-delay-plot left off - and keeps on going.

;; Examples:
 (def yknot 1)
 (def num 200)
 (def tao 2)
 (def ynot 1)
 (def tmin 0)
 (def tmax 16)
 (def ynot (repeat yknot (ceiling (* num (/  tao (- tmax tmin))))))

;; This is not a delay equation: it should settle down to sinusoidal variation:
 (def alpha -.5)
 (defun f(x time xd) (+ (* alpha x) (sin (* 2 time))))
 (euler-delay-plot #'f ynot tmin tmax tao :title \"No Delay\")
 (euler-delay-plot-continued #'f ynot tmin tmax tao my-plot)
"
  (let* (
	 (hsize (/ tao (length ynot)))
	 (delay-index (length ynot))
	 (yvalues ynot)
	 (num (ceiling (/ (- end start) hsize)))
	 (timesteps (rseq start end num))
	 j nynot nstart nend yvalindex
	 )
    (dotimes
     (i (- num delay-index))
     (setq j (1- (+ i delay-index)))
     (setq yvalues
	   (append yvalues 
		  (+ (last yvalues)
		     (* hsize 
			(funcall 
			 f 
			 (select yvalues j)
			 (select timesteps j)
			 (select yvalues i)
			 )
			)
		     )
		  )
	  )
     )
;;    (send my-plot :close)
    (send my-plot :clear-lines)
    (def my-plot (plot-lines timesteps yvalues))
    (if axes (let 
		 (
		  (xmin (select axes 0))
		  (xmax (select axes 1))
		  (ymin (select axes 2))
		  (ymax (select axes 3))
		  )
	       (send my-plot :range 0 xmin xmax)
	       (send my-plot :range 1 ymin ymax)
	       )
      )
    (if title (send my-plot :title title))
    (setq nstart (- end tao))
    (setq nend (+ nstart (- end start)))
    (setq yvalindex (1- (length yvalues)))
    (setq nynot (select yvalues (iseq (- yvalindex delay-index) yvalindex)))
    (if (ok-or-cancel-dialog "continue?")
	(euler-delay-plot-continued
	 #'f nynot nstart nend tao my-plot :title title))
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
