(defun ipopprep(file &key (prog "spacestat"))
  "
    ipopprep(file)

	Turns a spacestat .Gal file into a Stat .con file.

"
  (let ()
    (with-open-file
     (f file :direction :input)
     (with-open-file
      (g (strcat file ".out") :direction :output)
      (cond 
       ((string= prog "spacestat")
	(read-line f nil nil)
	(while
	    (setq
	     a (string-to-list (read-line f nil nil)))
	  (setq
	   c (string-to-list (read-line f nil nil))
	   )
	  (format g "~d 1 ~d ~{~d ~}~%" (first a) (length c) c)
	  )
	)
       ((string= prog "mapstat")
	(while
	    (setq
	     a (string-to-list (read-line f nil nil)))
	  (format g "~d 1 ~d ~{~d ~}~%" (first a) (length (rest a)) (rest a))
	  )
	)
       (t (format t "Don't know that software!"))
       )
      )
     )
    )
  )

(defun ipopprepmapstat(file)
  "
    ipopprepmapstat(file)

	Turns a spacestat .Gal file into a Stat .con file.

"
  (let ()
    (with-open-file
     (f file :direction :input)
     (with-open-file
      (g (strcat file ".out") :direction :output)
      (while
	  (setq
	   a (string-to-list (read-line f nil nil)))
	(format g "~d 1 ~d ~{~d ~}~%" (first a) (length (rest a)) (rest a))
	)
      )
     )
    )
  )

(defun statbna(file outfile &key verbose)
  "

	statbna(file outfile &key verbose)

	Turns a grass bna file into a stat bna file.

	The convolutions below are due to the fact that the file comes in with
commas in it. They have to be stripped off....
"
  (with-open-file
   (f file :direction :input)
   (with-open-file
    (g outfile :direction :output)
    (while 
	(setq a (read-line f nil nil))
      (let* (
	     (data (string-to-list a))
	     (data (mapcar (lambda(x) (if (listp x) (second x) x)) data))
	     (n (final data))
	     (num (first data))
	     )
	(if verbose (print a))
	(format g "\"~a\",\"NOT_USED\",\"NOT_USED\",~d~%" num n)
	(dotimes
	 (i n)
	 (let ((line (string-to-list (read-line f nil nil))))
	   (format g "~7,4f,~7,4f~%" (first line) (second (second line)))
	   )
	 )
	)
      )
    )
   )
  )

(defun morans-I-sims(file file2 &key (nruns 99))
  "
;; Notes:

	morans-I-sims(file file2 &key (nruns 99))

	This is for running Moran's I on a Stat data set, with a stat con
file. The data should be a single column of data (not the data/label format
allowed by stat).

	file2: either a string (filename) or a matrix, of which the first
column will be used.

;; Examples:
; (def lymeout (morans-I-sims \"LYME.CON\" \"lyme.dat\"))
; (def oakout (morans-I-sims \"bg_26125.gal.out\" \"bgoak_nr.txt\"))
; (def wayout (morans-I-sims \"bg_26163.gal.out\" \"bgway_nr.txt\"))
"
  (let* (
	 (rates
	  (column 1
		  (cond
		   ((stringp file2)
		    (read-matrix-data file2))
		   ((matrixp file2)
		    file2)
		   (t
		    (bind-columns file2))
		   )
		  )
	  )
	 (rates (- rates (mean rates)))
	 (n (length rates))
	 pairs a
	 )
    (with-open-file
     (f file :direction :input)
     (while 
	 (setq a (read-line f nil nil))
       (let* (
	      (data (string-to-list a))
	      (i (1- (first data)))
	      (num (third data))
	      (js (1- (select (reverse data) (iseq num))))
	      )
	 (setq pairs (append pairs (mapcar (lambda(x) (list i x)) js)))
	 )
       )
     )
    (let* (
	   (np (size pairs))
	   (bottom (dot rates rates))
	   (firsts (mapcar #'first pairs))
	   (seconds (mapcar #'second pairs))
	   (top (sum 
		 (*
		  (select rates firsts)
		  (select rates seconds)
		  )
		 )
		)
	   (factor (/ n (* np bottom)))
	   (run1 (* factor top))
	   (runs (mapcar
		  (lambda(i)
		    (let ((newrates (sample rates n)))
		      (* factor
			 (sum 
			  (*
			   (select newrates firsts)
			   (select newrates seconds)
			   )
			  ))))
		  (iseq nruns)
		  )
		 )
	   (p (/ (size (where runs run1 #'<)) (1+ nruns)))
	   )
      (list
       run1
       runs
       p
       )
      )
    )
  )
(defun showout(x &key (height 30) (title "plot"))
  "
;; for showing the results of morans-i-sims

;; Examples:
; (showout oakout :title \"Oakland County\")
; (showout wayout :height 30 :title \"Wayne County\")
"
  (let (
	(plot (histo (second x)))
	(val (first x))
	)
    (make-title (strcat title (format nil ": p=~4,3f" (- 1 (third x)))))
    (send plot :add-plot-item 
	  (list 'lines (list val val) (list 0 height))))
  )


(defun morans-I-val(file file2 &key (nruns 99))
  "
;; Notes:

	morans-I-val(file file2 &key (nruns 99))

	This is for running Moran's I on a Stat data set, with a stat con
file. The data should be a single column of data (not the data/label format
allowed by stat).

	file2: either a string (filename) or a matrix, of which the first
column will be used.

;; Examples:
; (def lymeout (morans-I-val \"LYME.CON\" \"lyme.dat\"))
; (def oakout (morans-I-val \"bg_26125.gal.out\" \"bgoak_nr.txt\"))
; (def wayout (morans-I-val \"bg_26163.gal.out\" \"bgway_nr.txt\"))
"
  (let* (
	 (rates
	  (column 1
		  (cond
		   ((stringp file2)
		    (read-matrix-data file2))
		   ((matrixp file2)
		    file2)
		   (t
		    (bind-columns file2))
		   )
		  )
	  )
	 (rates (- rates (mean rates)))
	 (n (length rates))
	 pairs a
	 )
    (with-open-file
     (f file :direction :input)
     (while 
	 (setq a (read-line f nil nil))
       (let* (
	      (data (string-to-list a))
	      (i (1- (first data)))
	      (num (third data))
	      (js (1- (select (reverse data) (iseq num))))
	      )
	 (setq pairs (append pairs (mapcar (lambda(x) (list i x)) js)))
	 )
       )
     )
    (let* (
	   (np (size pairs))
	   (bottom (dot rates rates))
	   (firsts (mapcar #'first pairs))
	   (seconds (mapcar #'second pairs))
	   (top (sum 
		 (*
		  (select rates firsts)
		  (select rates seconds)
		  )
		 )
		)
	   (factor (/ n (* np bottom)))
	   (run1 (* factor top))
	   )
      run1
      )
    )
  )

