;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;      An application of Object-Oriented Programming to tensors
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
#|
 A tensor is given by an array. If we're considering an mxnxp tensor, then it
would appear to be a list of m lists of n lists of p elements.

 Thus,
  an nxp matrix is a tensor, which would be given by 
	dimension: (list n p)
	elements:  list of n lists of p elements.

  An n vector is a tensor, with dimension (list n), given by a list of n values.
  A real number is a tensor, with dimension nil, given by a list of a single
value.

And so on.

				List of functions:

t+(&rest a) 
t*(&rest a) 
t-(&rest a) 
t/(&rest a) 
dimensions-check(a)
vectors-to-tensor(&rest a)
recursive-outer-product(&rest e)
3-tensor-to-matrices(tensor)
diagonalize-3-tensor(tensor &key (eps .001))
diagonalize-tensor(tensor &key (eps .001))
tensor-test()
optimize-rank-one-over-tensor(tensor)
round-x(x)
2x2x2(x y z)

				List of methods:

tensor :elements (&optional (new nil set))
tensor :dim (&optional new)
tensor :number (&optional new)
tensor :active-list (&optional new)
tensor :normalize ()
tensor :norm ()
tensor :isnew (&rest e)
tensor :rank-one-approximation (&optional verbose)

|#
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                         Proto tensor:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(uber-proto tensor (dim elements) nil)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				:normalize
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth tensor :normalize ()
  (let ((elements (send self :elements)))
    (when elements
	  (setf (slot-value 'elements) 
		(/ elements (sqrt (sum (square elements))))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				:norm
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth tensor :norm ()
  (let ((elements (send self :elements)))
    (when elements (sqrt (sum (square elements))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				:rank-one-approximation
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth tensor :rank-one-approximation (&optional verbose)
  "
Description: calls the function optimize-rank-one-over-tensor on itself, to
provide a set of normalized vectors which, combined as a rank-one tensor, best
appoximate the tensor itself.

Returns: the vectors and the singular value.

"
  (optimize-rank-one-over-tensor self))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				:isnew
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth tensor :isnew (&rest e)
  "Method :isnew is for creating new tensors. The useful arguments are the
dimensions (given as a list), and (optionally) the elements given as a
correctly-sized array.

	If the dimensions given and the dimensions of the elements don't agree,
the dimensions of the elements take precedence, and an error message will be
printed.

;; Examples:
 ;; The following give the same result:
 (def a (send tensor :new :dim (list 2 2 3)))
 (def a (send tensor :new :elements (make-array (list 2 2 3))))

 ;; This one gives an error, as the dimensions of the two disagree, but
 ;; proceeds using the dimensions of the elements:
 (def c (send tensor :new :dim '(2 3) :elements 1))

 ;; And here they agree, but why not let us deduce the dimensions?
 (def c
   (send tensor :new :dim '(1 1 1) 
		     :elements (make-array '(1 1 1) :initial-contents '(1))))
"
  (let* (
	 (sl (if e (transpose (split-list e 2))))
	 (fsl (first sl))
	 (ssl (second sl))
	 )
    (mapcar (lambda(x y) (send self x y)) fsl ssl)
    )
  (send tensor :slot-value 'number (1+ (send tensor :number)))
  (send tensor :slot-value 'active-list
		(append (send tensor :active-list) (list self)))
  (let* (
	 (dimensions (send self :dim))
	 (elements (send self :elements))
	 (elements (if (numberp elements) (vector elements) elements))
	 )
    (setf (slot-value 'elements)
	  (if elements
	      (if (arrayp elements)
		  (let ((dim (setf (slot-value 'dim) (array-dimensions elements))))
		    (if (and dimensions (not (equalp dimensions dim)))
			(format t
				"
Dimensions don't match. Using dimensions ~s of the elements given...~%" dim))
		    elements
		    )
		(error "Initial elements must be given as an array.")
		)
	    (funcall #'make-array dimensions))) 
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                  The Usual Arithmetic Operations:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun dimensions-check(a)
  "Dimension check makes sure that a group of tensors (given by a list) all
have the same dimensions. I need this one for the operations I'm going to
define for tensors, like t+, t-, t* and t/
"
  (let* (
	 (dims (mapcar (lambda(x) (array-dimensions x)) a))
	 (ndim (first dims))
	 (newdims (mapcar (lambda(x) (- x ndim)) dims))
	 )
    (if (= (sum newdims) 0) t nil)
    ))

(defun tensorp(x)
  "Checks whether an object is a tensor or not.
;; Examples:
 (tensorp (new tensor))
"
  (if (objectp x)
      (if (eq 'tensor (send x :slot-value 'proto-name)) t)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				turn-it-into-a-tensor
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun turn-it-into-a-tensor(x)
  "
;; Description: turn-it-into-a-tensor(x) turns matrices, vectors, lists, and
arrays into tensors (oddly enough).

;; Examples:
 (turn-it-into-a-tensor (id 4))
 (turn-it-into-a-tensor #3a(((1))))
 (turn-it-into-a-tensor #(1 2))
 (turn-it-into-a-tensor 1)
"
  (cond
   ((or (vectorp x) (listp x))
    (let ((size (list (size x))))
      (new tensor size (make-array size :initial-contents (combine x)))
      )
    )
   ((or (matrixp x) (arrayp x))
    (let ((size (size x)))
      (new tensor size (make-array size :initial-contents (combine x)))
      )
    )
   ((numberp x)
    (new tensor '(1) (make-array '(1) :initial-contents (list x)))
    )
   (t
    (format t "Sorry: I'm not sure what to do with this object.~%")
    nil
    )
   )
  )


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				t+
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun t+(&rest a) 
  "Tensor (array) addition"
  (if (dimensions-check a) (apply #'+ a) 
    (error (format t "Dimensions don't add up: ~s" dims))
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				t*
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun t*(&rest a) 
  "Tensor (array) multiplication"
  (if (dimensions-check a) (apply #'* a) 
    (error (format t "Dimensions don't add up: ~s" dims))
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				t-
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun t-(&rest a) 
  "Tensor (array) subtraction"
  (if (dimensions-check a) (apply #'- a) 
    (error (format t "Dimensions don't add up: ~s" dims))
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				t/
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun t/(&rest a) 
  "Tensor (array) division"
  (if (dimensions-check a) (apply #'/ a) 
    (error (format t "Dimensions don't add up: ~s" dims))
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                     Now to some useful functions:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          recursive-outer-product
;;
(defun recursive-outer-product(&rest e)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          recursive-outer-product Doc:
;;
;; Author: Andy Long
;; Created: 10/11/1997
;;
;; Description/Arguments:

       (recursive-outer-product &rest vectors)
  
	I created function recursive-outer-product as a helper to
vectors-to-tensor.

;; Notes:

;; See Also: vectors-to-tensor

;; Examples:
 (recursive-outer-product  '(1 2) '(3 4 5) '(6 7 8 9))
"
  (if (= (length e) 2)
      (apply #'outer-product e)
    (outer-product (apply #'recursive-outer-product (butlast e))
		   (final e))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 vectors-to-tensor
;;
(defun vectors-to-tensor(&rest a)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          vectors-to-tensor Doc:
;;
;; Author: Andy Long
;; Created: 10/11/1997
;;
;; Description/Arguments:

       (vectors-to-tensor &rest a)
  
	Vector-to-tensor takes as its argument any number of vectors, and
 returns the tensor created by multiplying them component-wise. Thus the
 (i,j,k)th element of a 3-tensor would be obtained by multiplying the ith
 component of the first, the jth component of the second, and the kth component
 of the third.

;; Notes: this identity map gave me the idea:
 (- 
  (combine (make-array '(2 3 4) :initial-contents (iseq 24)))
  (iseq 24))

;; See Also: I created function recursive-outer-product as a helper.
 
;; Examples:
 (vectors-to-tensor '(1 2) '(3 4 5))

 (def v1 (sample (iseq 10) 2)) 
 (def v2 (sample (iseq 10) 3)) 
 (def v3 (sample (iseq 10) 4)) 
 (def a (vectors-to-tensor v1 v2 v3))
 ;; Should print 24 zeros:
 (dotimes (i 2)
   (dotimes (j 3)
     (dotimes (k 4)
       (princ (- (* (elt v1 i) (elt v2 j) (elt v3 k)) (aref a i j k))))))
"
  (let* (
	 (lists (mapcar (lambda(x) (coerce x 'list)) a))
	 (dims (mapcar #'length lists))
	 (new (funcall #'make-array dims))
	 )
    (if (= (length a) 1)
	(coerce (first a) 'vector)
      (make-array
       dims
       :initial-contents (combine (apply #'recursive-outer-product a))
       )
      )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 3-tensor-to-matrices
;;
(defun 3-tensor-to-matrices(tensor)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          3-tensor-to-matrices Doc:
;;
;; Author: Andy Long
;; Created: 10/11/1997
;;
;; Description/Arguments:

       (3-tensor-to-matrices tensor)
  
;; Notes:

;; See Also:

;; Examples:
 (def a (3-tensor-to-matrices (vectors-to-tensor '(1 2) '(2 3 4) '(1 3))))
 (mapcar #'size a)
 (def svds (mapcar #'svd a))
 (mapcar #'first svds)
 (mapcar #'second svds)
"
  (let* (
	 (dims (array-dimensions tensor))
	 (xi (first dims))
	 (yi (second dims))
	 (zi (third dims))
	 (xi-seq (iseq xi))
	 (yi-seq (iseq yi))
	 (zi-seq (iseq zi))
	 )
    (list
     (apply #'bind-columns
	    (mapcar
	     (lambda(z)
	       (apply #'bind-columns
		      (mapcar
		       (lambda(y)
			 (let ((val (select tensor xi-seq y z)))
			   (mapcar (lambda(x) (aref val x 0 0)) xi-seq)
			   )
			 )
		       yi-seq
		       )
		      )
	       )
	     zi-seq
	     )
	    )
     (apply #'bind-columns
	    (mapcar
	     (lambda(x)
	       (apply #'bind-columns
		      (mapcar
		       (lambda(z)
			 (let ((val (select tensor x yi-seq z)))
			   (mapcar (lambda(y) (aref val 0 y 0)) yi-seq)
			   )
			 )
		       zi-seq
		       )
		      )
	       )
	     xi-seq
	     )
	    )
     (apply #'bind-columns
	    (mapcar
	     (lambda(y)
	       (apply #'bind-columns
		      (mapcar
		       (lambda(x)
			 (let ((val (select tensor x y zi-seq)))
			   (mapcar (lambda(z) (aref val 0 0 z)) zi-seq)
			   )
			 )
		       xi-seq
		       )
		      )
	       )
	     yi-seq
	     )
	    )
     )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 diagonalize-3-tensor
;;
(defun diagonalize-3-tensor(te &key (eps .001))
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          diagonalize-3-tensor Doc:
;;
;; Author: Andy Long
;; Created: Sat Oct 11 05:14:29 CDT 1997
;;
;; Description/Arguments:

       (diagonalize-3-tensor)
  
;; Notes:

;; See Also:

;; Examples:
 (def ten 
   (t+
     (vectors-to-tensor '(1  2) '(2 3 4) '(1 3))
     (vectors-to-tensor '(2 -1) '(2 3 -13/4) '(3 -1))
     )
   )
 (mvsq '(v1s v2s v3s ts ss val) (diagonalize-3-tensor ten :eps 1e-12))
 ss
 val
 (mapcar
   (lambda(x) (sum (apply #'t* x)))
   (select (all-subsets ts) (which (= (mapcar #'length (all-subsets ts)) 2)))
   )

 (def ten (make-array '(2 3 2) :initial-contents (rand.u 12)))
 (mvsq '(v1s v2s v3s ts ss val) (diagonalize-3-tensor ten :eps 1e-12))
 ss
 val
 (mapcar
   (lambda(x) (sum (apply #'t* x)))
   (select (all-subsets ts) (which (= (mapcar #'length (all-subsets ts)) 2)))
   )
"
  (let (
	(tensor te)
	a v1 v2 v3 svds sing-vals sing-tensor
	v1s v2s v3s sing-val ts ss
	)
    (while (> (sum (t* tensor tensor)) eps)
      (setq a (3-tensor-to-matrices tensor)
	    svds (mapcar #'svd a)
	    sing-vals (mapcar #'second svds)
	    v1 (column 1 (first (first svds)))
	    v2 (column 1 (first (second svds)))
	    v3 (column 1 (first (third svds)))
	    sing-tensor (vectors-to-tensor v1 v2 v3)
	    sing-val (sum (t* tensor sing-tensor))
	    v1s (append v1s (list v1))
	    v2s (append v2s (list v2))
	    v3s (append v3s (list v3))
	    ts (append ts (list sing-tensor))
	    ss (append ss (list sing-val))
	    tensor (t- tensor (* sing-val sing-tensor))
	    )
      )
    (list v1s v2s v3s ts ss (sum (t* tensor tensor)))
    )
  )
;;
(defun diagonalize-tensor(tensor &key (eps .001))
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          diagonalize-tensor Doc:
;;
;; Author: Andy Long
;; Created: Sat Oct 11 05:14:29 CDT 1997
;;
;; Description/Arguments:

       (diagonalize-tensor tensor &key (eps .001))
  
;; Notes: This doesn't work, as we don't have any constraint on the tensors
   which requires orthogonality of the rank-one tensors obtained. Thus we
   quickly introduce back components of once removed tensors.

	I'm currently using optimize-rank-one-over-tensor, to which I added a
	switch so that I can hand it a bunch of orthogonal tensors to
	orthogonalize against. Still isn't working right, but maybe we're
	getting closer.

		I'm requiring the orthogonality in a curious way: I maximize
	the function <uxvxw,T>/(+ 1 (abs (sum <uxvxw,O_i>)))

;; See Also: optimize-rank-one-over-tensor

;; Examples:
 (def ten 
   (t+
     (vectors-to-tensor '(1  2) '(2 3 4) '(1 3))
     (vectors-to-tensor '(2 -1) '(2 3 -13/4) '(3 -1))
     )
   )
 (apply #'* (mapcar #'norm (list '(1  2) '(2 3 4) '(1 3))))
 (apply #'* (mapcar #'norm (list '(2 -1) '(2 3 -13/4) '(3 -1))))
 (diagonalize-tensor ten)

 (def ten (make-array '(2 3 2) :initial-contents (rand.u 12)))
 (def ts (diagonalize-tensor ten))
 (mapcar
   (lambda(x)
    (sum (apply #'t* (mapcar (lambda(x) (apply #'vectors-to-tensor x)) x))))
   (all-pairs (second ts))
   )
"
  (do*
   (
    (tensor
     (if (tensorp tensor) (send tensor :elements) tensor)
     (- tensor r1))
    
    (rank1tensor
     (optimize-rank-one-over-tensor tensor :verbose 1)
     (optimize-rank-one-over-tensor tensor :verbose 1 :orthogonals r1s))
    
    (svecs (first rank1tensor) (first rank1tensor))
    (sv (second rank1tensor) (second rank1tensor))
    (svs (list sv) (combine svs sv))
    (vs (list svecs) (append vs (list svecs)))
    (r1
     (* sv (apply #'vectors-to-tensor svecs))
     (* sv (apply #'vectors-to-tensor svecs))
     )
    (r1s (list r1) (append r1s (list r1)))
    )     
   ((< (sum (t* tensor tensor)) eps)
    (list svs vs)
    )
   (printall rank1tensor svecs sv svs vs)
   )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
(defun tensor-test()
  (def a (send tensor :new :dim (list 2 2 3)))
  (def b (send tensor :new :dim (list 2 2 3)))
  
  (def a (send tensor :new :dim (list 2 2 3) :elements (make-array (list 2 2 3)
						    :initial-element 4)))
  (def b (send tensor :new :dim (list 2 2 3) :elements (make-array (list 2 2 3)
						    :initial-element 8)))
  (def c (send tensor :new :dim (list 2 2 3) :elements (make-array (list 2 2 3)
						    :initial-element 1)))
  
  (def aa (send a :elements))
  (def ba (send b :elements))
  (def ca (send c :elements))
  
  (t+ aa ba ca)
  (t* aa ba ca)
  (t/ aa ba ca)
  (t- aa ba ca)
  
  (def v1 (sample (iseq 10) 2)) 
  (def v2 (sample (iseq 10) 3)) 
  (def v3 (sample (iseq 10) 4)) 
  (def a (vectors-to-tensor v1 v2 v3))
  ;; Should print 24 zeros:
  (dotimes
   (i 2)
   (dotimes
    (j 3)
    (dotimes
     (k 4)
     (princ (- (* (elt v1 i) (elt v2 j) (elt v3 k)) (aref a i j k))))))
  
  (def ten (vectors-to-tensor '(1 2) '(2 3 4) '(1 3)))
  (def a (3-tensor-to-matrices ten))
  (mapcar #'size a)
  (def svds (mapcar #'svd a))
  (mapcar #'first svds)
  (def sing-vals (mapcar #'second svds))
  (def sing-tensor
       (vectors-to-tensor
	(column 1 (first (first svds)))
	(column 1 (first (second svds)))
	(column 1 (first (third svds)))
	))
  
  ;; verify that I understand the svd representation:
  (def svd (first svds))
  (matmult (first svd) (diagonal-to-matrix (second svd)) (transpose (third svd)))
  
  (def new-tensor (t-
		   ten
		   (*
		    (sum (t* ten sing-tensor))
		    sing-tensor
		    )))
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 optimize-rank-one-over-tensor
;;
(defun optimize-rank-one-over-tensor
  (tensor &key
	  (eps 1e-6)
	  verbose
	  x0
	  orthogonals
	  )
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          optimize-rank-one-over-tensor Doc:
;;
;; Author: Andy Long
;; Created: Wed Mar 4 09:18:52 1998
;;
;; Description/Arguments:

 (optimize-rank-one-over-tensor
   tensor &key
	  (eps 1e-6)
	  verbose
	  orthogonals
	  x0
	  )
  
	tensor is a tensor or array;
	eps is a small positive real, used if orthogonal tensors are present;
	verbose decides whether you want to be talkative or not;
	orthogonals is a list of orthogonal components;
	x0 is an initial guess, if desired (a list of vectors).

;; Notes:

;; See Also: newtonmax

;; Examples:
 (def a (vectors-to-tensor '(1 2) '(3 4) '(-1 1 1)))
 (optimize-rank-one-over-tensor a :verbose 2)

 (def a (new tensor '(2 2 2) (make-array '(2 2 2))))
 (def v1 (normalize (rand.u 2)))
 (def v2 (normalize (rand.u 2)))
 (def v3 (normalize (rand.u 2)))
 (change a elements (vectors-to-tensor v1 v2 v3))
 (send a :normalize)
 (optimize-rank-one-over-tensor a :verbose 2)
 (pprint (list v1 v2 v3))
"
  (let* (
	 (tensor (if (tensorp tensor) (send tensor :elements) tensor))
	 (size (size tensor))
	 (subsets (mapcar #'iseq size))
	 (subsets (+ subsets (combine 0 (butlast (cumsum size)))))
	 (ran (if x0 x0 (rand.u (sum size))))
	 (result
	  (newtonmax
	   (if orthogonals
	       (lambda(z)
		 (let ((trial (apply #'vectors-to-tensor
				     (mapcar
				      (lambda(x) (normalize (select z x)))
				      subsets))))
		   (/
		    (sum (t* trial tensor))
		    (+ 1 (sum (abs (mapcar (lambda(x) (sum (t* trial x)))
					   orthogonals)))))))
	     (lambda(z)
	       (sum
		(t*
		 (apply #'vectors-to-tensor
			(mapcar
			 (lambda(x) (normalize (select z x)))
			 subsets))
		 tensor))))
	   ran
	   :verbose verbose
	   :return-derivs t
	   )
	  )
	 (lambda (second result))
	 (result
	  (mapcar (lambda(x) (normalize (select (first result) x))) subsets)
	  )
	 )
    (if verbose (pprint result))
    (list result lambda)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				t-mult
;;
(defun t-mult(A B seqA seqB)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			t-mult Doc:
;;
;; Author: Andy Long
;; Created: Mon Mar 2 20:11:12 1998
;;
;; Description/Arguments: t-mult
  
	We have tensors A_{mxnxp} and B_{pxnxq} for example, and want to create
  a tensor C_{mxq} by annihilating the last two dimensions of A against the
  first two of B. This is a generalization of the vector inner-product.

;; Arguments: A B seqA seqB

	where

	A	- a tensor
	B	- a tensor
	seqA	- a list of the dimensions of A to be annihilated
	seqB	- a list of the dimensions of B to be annihilated

;; Returns: a tensor obtained by multiplying out the dimensions of A and B
;; specified in the lists seqA and seqB.

;; Notes:

;; See Also: make-all-tuples

;; Examples:
 \"We'll start by checking some vector/matrix results (they're easier!):\"
 (t-mult #(1 2) #(1 2) (iseq 1) (iseq 1))
 (t-mult '(1 2) #(1 2) (iseq 1) (iseq 1))

 (def 2x2 #2A((1 1) (2 2)))
 \"this should be the sum of the element-wise product, i.e. 10:\"
 (t-mult 2x2 2x2 (iseq 2) (iseq 2))
 \"this should be 9:\"
 (t-mult (transpose 2x2) 2x2 (iseq 2) (iseq 2))

 (def 2x3 #2A((1 1 1) (2 2 2)))
 \"The next two matrices should be equal:\"
 ;; This is the same as matrix multiplication:
 (t-mult 2x2 2x3 (list 1) (list 0))
 (matmult 2x2 2x3)
 \"We can transpose by simply changing the indices we use:\"
 \"The next two matrices should be equal:\"
 (t-mult 2x2 2x3 (list 0) (list 0))
 (matmult (transpose 2x2) 2x3)

 \"Now we get into the 'real' tensors - no longer matrices.\"
 \"Let's check that the identity works to give the same tensor back.\"
 \"The next two tensors should be equal:\"
 (setq 2x2x2-tensor #3A(((1 1) (2 2)) ((-1 -1) (-1 -1))))
 (t-mult (id 2) 2x2x2-tensor (list 1) (list 0))

 (setq 2x2x3-tensor #3A(((1 1 1) (2 2 2)) ((-1 -1 -1) (3 3 3))))
 \"Gives a vector, as we annihilate both dimensions of the identity matrix:\"
 (t-mult (id 2) 2x2x3-tensor (list 0 1) (list 0 1))

 \"This should return a 2x3 matrix:\"
 (t-mult 2x2x2-tensor 2x2x3-tensor (list 1 2) (list 0 1))

 \"Here's a four tensor!\"
 (pprint (t-mult 2x2x2-tensor 2x2x3-tensor (list 2) (list 0)))
"
  (let* (
	 ;; turn a and b into arrays, if they aren't already:
	 (a (cond
	     ((listp a) (coerce a 'vector))
	     ((arrayp a) a)
	     (t (send a :elements))))
	 (b (cond
	     ((listp b) (coerce b 'vector))
	     ((arrayp b) b)
	     (t (send b :elements))))

	 ;; pick off their sizes:
	 (adims (size a))
	 (bdims (size b))
	 (adims (if (numberp adims) (list adims) adims))
	 (bdims (if (numberp bdims) (list bdims) bdims))

	 ;; How many dimensions are there?
	 (a-all-dims (iseq (length adims)))
	 (b-all-dims (iseq (length bdims)))

	 ;; Which of those dimensions will we be keeping?
	 (a-keepers (sort-data (set-difference a-all-dims seqA)))
	 (b-keepers (sort-data (set-difference b-all-dims seqB)))

	 ;; Their dimensions:
	 (a-keepers-dims (select adims a-keepers))
	 (b-keepers-dims (select bdims b-keepers))
	 (a-pitch-dims   (select adims seqA))
	 (b-pitch-dims   (select bdims seqB))
	    
	 ;; We need sequences for those we're keeping and pitching:
	 (a-keepers-sizes (mapcar #'iseq a-keepers-dims))
	 (b-keepers-sizes (mapcar #'iseq b-keepers-dims))
	 (a-pitch-sizes (mapcar #'iseq a-pitch-dims))
	 (b-pitch-sizes (mapcar #'iseq b-pitch-dims))
	 
	 ;; Let's make all tuples:
	 (a-indices (apply #'make-all-tuples a-keepers-sizes))
	 (b-indices (apply #'make-all-tuples b-keepers-sizes))

	 ;; Now we're going to create some "choice" variables, which will
	 ;; change index as we go along:
	 (achoice (iseq (length adims)))
	 (bchoice (iseq (length bdims)))

	 ;; We'll store the result in - what else? - the result array!
	 (result
	  (cond
	   ((and a-keepers-dims b-keepers-dims)
	    (make-array (combine a-keepers-dims b-keepers-dims)))
	   (a-keepers-dims
	    (make-array a-keepers-dims))
	   (b-keepers-dims
	    (make-array b-keepers-dims))
	   ))
	 )

    ;; The dimensions to be pitched will always we equal to sequences:
    (setf (select achoice seqA) a-pitch-sizes)
    (setf (select bchoice seqB) b-pitch-sizes)

    ;; Now we check to see if either or both of our tensors will be
    ;; completely annihilated: 
    (cond
     ;; neither
     ((and a-indices b-indices)
      (dolist
       (a-ind a-indices)
       (dolist
	(b-ind b-indices)
	(setf
	 (select achoice a-keepers) a-ind
	 (select bchoice b-keepers) b-ind
	 (apply #'aref result (combine a-ind b-ind))
	 (sum (* (apply #'select a achoice)
		 (apply #'select b bchoice)))))))
     ;; b
     (a-indices
      (dolist
       (a-ind a-indices)
       (setf
	(select achoice a-keepers) a-ind
	(apply #'aref result a-ind)
	(sum (* (apply #'select a achoice) b)))))
     ;; a
     (b-indices
      (dolist
       (b-ind b-indices)
       (setf
	(select bchoice b-keepers) b-ind
	(apply #'aref result b-ind)
	(sum (* a (apply #'select b bchoice))))))
     ;; both
     (t ;; otherwise we have a scalar:
      (setq result (sum (* a b)))
      )
     )
    result
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
#| - Tue Mar 3 01:07:09 1998
;; Old t-mult code:
  (let* (
	 ;; turn a and b into arrays, if they aren't already:
	 (a (if (arrayp a) a (send a :elements)))
	 (b (if (arrayp b) b (send b :elements)))

	 ;; pick off their sizes:
	 (adims (size a))
	 (bdims (size b))

	 ;; How many dimensions are there?
	 (a-all-dims (iseq (length adims)))
	 (b-all-dims (iseq (length bdims)))

	 ;; Which of those dimensions will we be keeping?
	 (a-keepers (sort-data (set-difference a-all-dims seqA)))
	 (b-keepers (sort-data (set-difference b-all-dims seqB)))

	 ;; Their dimensions:
	 (a-keepers-dims (select adims a-keepers))
	 (b-keepers-dims (select bdims b-keepers))
	 (a-pitch-dims   (select adims seqA))
	 (b-pitch-dims   (select bdims seqB))
	    
	 ;; We need sequences for those we're keeping and pitching:
	 (a-keepers-sizes (mapcar #'iseq a-keepers-dims))
	 (b-keepers-sizes (mapcar #'iseq b-keepers-dims))
	 (a-pitch-sizes (mapcar #'iseq a-pitch-dims))
	 (b-pitch-sizes (mapcar #'iseq b-pitch-dims))
	 
	 ;; Let's make all tuples:
	 (a-indices (apply #'make-all-tuples a-keepers-sizes))
	 (b-indices (apply #'make-all-tuples b-keepers-sizes))

	 ;; We'll store the result in - what else? - result:
	 (result
	  (cond
	   ((and a-keepers-dims b-keepers-dims)
	    (make-array (combine a-keepers-dims b-keepers-dims)))
	   (a-keepers-dims
	    (make-array a-keepers-dims))
	   (b-keepers-dims
	    (make-array b-keepers-dims))
	   (t nil)))
	 )
    (if (not result) ;; then we have a scalar:
	(setq result (sum (* a b)))
      (dolist
       (a-ind (if a-indices a-indices (list nil)))
       (dolist
	(b-ind (if b-indices b-indices (list nil)))
	(setf (apply #'aref result 
		     (if (and a-ind b-ind) (combine a-ind b-ind)
		       (if a-ind a-ind b-ind)
		       ))
	      (sum (* 
		    (apply #'select a (smoosh (list a-ind a-pitch-sizes)))
		    (apply #'select b (smoosh (list b-pitch-sizes b-ind)))
		    ))))))
    result
    )
  )

|#
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
