#|
	Peter Norvig has a bunch of interesting, even fantastic, examples in
his book "Paradigms of Artificial Intelligence Programming". I hope he won't
mind that I've run away with some of them! Thanks Peter.

							Andy Long
							November, 1997
|#
(defmacro define-class (class inst-vars class-vars &body methods)
  "
;; Description: 

	macro define-class (class inst-vars class-vars &body methods)

	Define a class for object-oriented programming, and some functions
automatically to work with those objects.

;; Source:
 Code from Paradigms of Artificial Intelligence Programming
 Copyright (c) 1991 Peter Norvig
	 ael: from file clos.lsp: Object-oriented programming examples

;; Examples:

;;; ==============================
  (define-class account (name &optional (balance 0.00)) ((interest-rate .06))
    (withdraw (amt) (if (<= amt balance) (decf balance amt)
  			'insufficient-funds)) 
    (deposit  (amt) (incf balance amt))
    (balance  ()    balance)
    (name     ()    name)
    (interest ()    (incf balance (* interest-rate balance))))

  (setf acct2 (account \"A. User\" 2000.00)) ;; -> #<CLOSURE 24003064>
  (deposit acct2 42.00) ;; -> 2042.0
  (interest acct2) ;; -> 2164.52
  (balance acct2) ;; -> 2164.52
  (name acct2) ;; -> \"A. User\"

;;; ==============================
  (define-class password-account (password acct) ()
    (change-password (pass new-pass)
		     (if (equal pass password) (setf password new-pass)
                       'wrong-password))
    (otherwise (pass &rest args)
	       (if (equal pass password) (apply message acct args)
                 'wrong-password)))
  (setf acct3 (password-account \"secret\" acct2)) ;; -> #<CLOSURE 33427277>
  (name acct3 \"secret\") ;; -> \"A. User\"
  (balance acct3 \"secret\") ;; -> 2164.52
  (withdraw acct3 \"guess\" 2000.00) ;; -> WRONG-PASSWORD
  (withdraw acct3 \"secret\" 4000.00) ;; -> 'INSUFFICIENT-FUNDS
  (withdraw acct3 \"secret\" 2000.00) ;; -> 164.52

;;; ==============================
  (define-class limited-account (limit acct) ()
    (withdraw (amt) (if (> amt limit) 'over-limit (withdraw acct amt)))
    (otherwise (&rest args) (apply message acct args)))
	       
  (setf acct4 (password-account \"pass\"
				(limited-account 100.00
						 (account \"A. Thrifty Spender\"
							  500.00)))) ;; -> 
  (withdraw acct4 \"pass\" 200.00) ;; -> OVER-LIMIT
  (withdraw acct4 \"pass\" 20.00) ;; -> 480.0
  (withdraw acct4 \"guess\" 20.00) ;; -> WRONG-PASSWORD
"
  ;; Define constructor and generic functions for methods
  `(let ,class-vars
     (mapcar #'ensure-generic-fn ',(mapcar #'first methods))
     (defun ,class ,inst-vars
       #'(lambda (message)
           (case message
             ,@(mapcar #'make-clause methods))))))

(defun get-method (object message)
  "Return the method that implements message for this object. Defined in
  norvig.lsp as a helper for define-class."
  (funcall object message))

(defun make-clause (clause)
  "Translate a message from define-class into a case clause.  Defined in
  norvig.lsp as a helper for define-class."
  `(,(first clause) #'(lambda ,(second clause) .,(rest2 clause))))

(defun ensure-generic-fn (message)
  "Define an object-oriented dispatch function for a message,
  unless it has already been defined as one. Defined in
  norvig.lsp as a helper for define-class."
  (unless (generic-fn-p message)
    (let ((fn #'(lambda (object &rest args)
                  (apply (get-method object message) args))))
      (setf (symbol-function message) fn)
      (setf (get message 'generic-fn) fn))))

(defun generic-fn-p (fn-name)
  "Is this a generic function? Defined in norvig.lsp as a helper for
 define-class."
  (and (fboundp fn-name) 
       (eq (get fn-name 'generic-fn) (symbol-function fn-name))))

