;;
			;general n-dimensional first order de's
;;
(defun euler-nd (f ynot start end num)
  "
Description: euler-nd (f ynot start end num)

	f (the derivative function) should take a list and return a same-sized
list. If that is the case, then this function will return the Euler solution
for start to end with step-size (end-start)/num, as a list of lists.

;; Examples:

 (defun f (vals)
  (let (
	(x (first vals))
	(y (second vals))
	)
    (list (- x (^ x 2) (* x y))
	  (- (* x y) (/ y 2) (^ y 2)))
    )
  )

 (def points (euler-nd #'f '(.1 .2) 0 100 200))
 (def orbit1 (list (mapcar #'first points) (mapcar #'second points)))
 (plot-lines orbit1)

;; 			Lorenz equations: beautiful!
;; These are the equations:
 (defun f (vals)
  (let (
	(x (first vals))
	(y (second vals))
	(z (third vals))
	)
    (list
	(* 10 (- y x))
	(- (* x (- 28 z)) y)
	(- (* x y) (* z 2.6666666666666665)))
    )
  )

;; These are some decent parameter values:
 (defun orbits (xin yin zin)
   (let ((points (euler-nd #'f (list xin yin zin) 0 4 500)))
    (list
      (mapcar #'first points) (mapcar #'second points) (mapcar #'third points))
  ))

;; This will show a certain number ntimes of orbits:
 (defun lorenz-show (ntimes)
   (let* (
	  (starting (* 10 (uniform-rand 3)))
	  (orbit (orbits (first starting) (second starting) (third starting)))
	  )
    (def myplot (spin-plot orbit))
    (send myplot :depth-cuing nil)
    (send myplot :showing-axes nil)
    (send myplot :title \"Lorenz Equations\")
    (send myplot :add-lines orbit)
    (dotimes (i (1- ntimes))
	     (setq starting (* 10 (uniform-rand 3)))
	     (setq orbit
		   (orbits (first starting) (second starting) (third starting)))
	     (send myplot :add-lines orbit)
	     )
    myplot
    )
  )

 (lorenz-show 3)

 (def nextpoints (* 10 (uniform-rand 3)))
 (orbitals #'f nextpoints 1 50 50)
 (orbitals #'f nextpoints 4 200 10 :randomize 10)

 (dotimes (i 5)
	 (def step-size (+ .4 (* i .4)))
	 (def end-size (+ 40 (* i 40)))
	 (orbitals #'f nextpoints step-size end-size 20 :randomize 10)
	 )
"
  (let (
	(hsize (/ (- end start) num))
	(timesteps (rseq start end num))
	(yvalues (list ynot))
	)
    (dotimes
     (i num)
     (setq
      nextvals (select yvalues i)
      yvalues
      (append
       yvalues (list (+ (final yvalues) (* hsize (funcall f nextvals)))))
      )
     )
    yvalues
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                eulerplot
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun eulerplot (f ynot start end num &key axes title)
  "
       eulerplot (f ynot start end num &key axes title)

;; Examples:
 (eulerplot (lambda(x y) (- x (^ x 2))) .1 0 8 100 :axes '(0 8 0 2))
 (eulerplot (lambda(x y) (- x (^ x 2))) .1 0 8 100 :axes '(-2 10 -1 3))
 ;;
 ;; f(x,t) = k*e^(alpha t)*x: Gompertz growth (decreasing exponential constant)
 ;;
 (def alpha -.7)
 (def kappa 2)
 (eulerplot #'(lambda(x tt)(* kappa (exp (* alpha tt)) x))
   .1 0 8 100 :axes '(0 8 0 2))
"
  (let (
	(hsize (/ (- end start) num))
	(timesteps (rseq start end num))
	(yvalues (list ynot))
	)
    (dotimes
     (i (- num 1))
     (setq yvalues
	   (append yvalues 
		   (+ (last yvalues)
		      (* hsize 
			 (funcall f 
				  (select yvalues i)
				  (select timesteps i)
				  )
			 )
		      )
		   )
	   )
     )
    (def my-plot (send andyplot-proto :new 2 :show nil))
    (send my-plot :add-lines timesteps yvalues)
    (send my-plot :adjust-to-data)
    (send my-plot :plot-item (list (list 'lines timesteps yvalues 'black)))
    (if axes (let 
		 (
		  (xmin (select axes 0))
		  (xmax (select axes 1))
		  (ymin (select axes 2))
		  (ymax (select axes 3))
		  )
	       (send my-plot :range 0 xmin xmax)
	       (send my-plot :range 1 ymin ymax)
	       )
      )
    (if title (send my-plot :title title))
    (send my-plot :show-window)
    my-plot
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;             euler-delay-plot, for general first order de's
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun euler-delay-plot (f ynot start end tao &key axes title)
  "
;; Description:

	euler-delay-plot (f ynot start end tao &key axes title)

For delay equations, you'll need a tao, and the length of the initial
list ynot will determine the number of steps in one tao unit.

;; Examples:
;;
;; This is just a quick example that I thought might be interesting:
;;
 (def yknot 1)
 (def num 200)
 (def tao 2)
 (def ynot 1)
 (def tmin 0)
 (def tmax 16)
 (def ynot (repeat yknot (ceiling (* num (/  tao (- tmax tmin))))))

;; This is not a delay equation: it should settle down to sinusoidal variation:
 (def alpha -.5)
 (defun f(x time xd) (+ (* alpha x) (sin (* 2 time))))
 (euler-delay-plot #'f ynot tmin tmax tao :title \"No Delay\")

 (defun f(x time xd) (+ (* alpha xd) (sin (* 2 time))))
 (euler-delay-plot #'f ynot tmin tmax tao :title \"Delay\")
 ;;
 ;; From the exercises at the end of chapter 4, 'Understanding Non-linear
 ;; Dynamics', by Kaplan and Glass, for one-d diffeqs:
 ;;
 ;; This example is suggested by the authors, in computer project 3:
 ;;
 (def num 400)
 (def tao 2)
 (def tmin 0)
 (def tmax 32)
 (defun f(x time xd) (- (/ (* 2 xd) (+ 1 (^ xd alpha))) x))

 (def alpha .25)
 (def yknot 1.3)
 (def ynot (repeat yknot (ceiling (* num (/  tao (- tmax tmin))))))
 (euler-delay-plot #'f ynot tmin tmax tao :title \"Repeated high: .25\")

 (def yknot .3)
 (def ynot (repeat yknot (ceiling (* num (/  tao (- tmax tmin))))))
 (euler-delay-plot #'f ynot tmin tmax tao :title \"Repeated low: .25\")

 (def ynot (uniform-rand (ceiling (* num (/  tao (- tmax tmin))))))
 (euler-delay-plot #'f ynot tmin tmax tao :title \"Uniform Rand: .25\")

 (def alpha 1)

 (def yknot .3)
 (def ynot (repeat yknot (ceiling (* num (/  tao (- tmax tmin))))))
 (euler-delay-plot #'f ynot tmin tmax tao :title \"Repeated low: 1\")

 (def yknot 1.3)
 (def ynot (repeat yknot (ceiling (* num (/  tao (- tmax tmin))))))
 (euler-delay-plot #'f ynot tmin tmax tao :title \"Repeated high: 1\")

 (def ynot (normal-rand (ceiling (* num (/  tao (- tmax tmin))))))
 (euler-delay-plot #'f ynot tmin tmax tao :title \"Normal: 1\")

 (def ynot (uniform-rand (ceiling (* num (/  tao (- tmax tmin))))))
 (euler-delay-plot #'f ynot tmin tmax tao :title \"Uniform: 1\")

 (def alpha 2)

 (def yknot 1.3)
 (def ynot (repeat yknot (ceiling (* num (/  tao (- tmax tmin))))))
 (euler-delay-plot #'f ynot tmin tmax tao :title \"Repeated high: 2\")

 (def ynot (normal-rand (ceiling (* num (/  tao (- tmax tmin))))))
 (euler-delay-plot #'f ynot tmin tmax tao :title \"Normal: 2\")

 (def ynot (uniform-rand (ceiling (* num (/  tao (- tmax tmin))))))
 (euler-delay-plot #'f ynot tmin tmax tao :title \"Uniform: 2\")

 (def alpha 4)

 (def yknot 1.3)
 (def ynot (repeat yknot (ceiling (* num (/  tao (- tmax tmin))))))
 (euler-delay-plot #'f ynot tmin tmax tao :title \"Repeated high: 4\")

 (def ynot (normal-rand (ceiling (* num (/  tao (- tmax tmin))))))
 (euler-delay-plot #'f ynot tmin tmax tao :title \"Normal: 4\")

 (def ynot (uniform-rand (ceiling (* num (/  tao (- tmax tmin))))))
 (euler-delay-plot #'f ynot tmin tmax tao :title \"Uniform: 4\")
"
  (let* (
	 (hsize (/ tao (length ynot)))
	 (delay-index (length ynot))
	 (yvalues ynot)
	 (num (ceiling (/ (- end start) hsize)))
	 (timesteps (rseq start end num))
	 j my-plot
	 )
    (dotimes
     (i (- num delay-index))
     (setq j (1- (+ i delay-index)))
     (setq yvalues
	   (append yvalues 
		   (+ (last yvalues)
		      (* hsize 
			 (funcall 
			  f 
			  (select yvalues j)
			  (select timesteps j)
			  (select yvalues i)
			  )
			 )
		      )
		   )
	   )
     )
    (setq my-plot (plotit (list timesteps yvalues)))
    (if axes (let ()
	       (send my-plot :range 0 (select axes 0) (select axes 1))
	       (send my-plot :range 1 (select axes 2) (select axes 3))
	       )
      )
    (if title (send my-plot :title title))
    )
  my-plot
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                        euler-delay-plot-continued
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun euler-delay-plot-continued (f ynot start end tao my-plot &key axes title)
  "
;; Description:

   (euler-delay-plot-continued f ynot start end tao my-plot &key axes title)

	Picks up where Euler-delay-plot left off - and keeps on going.

;; Examples:
 (def yknot 1)
 (def num 200)
 (def tao 2)
 (def ynot 1)
 (def tmin 0)
 (def tmax 16)
 (def ynot (repeat yknot (ceiling (* num (/  tao (- tmax tmin))))))

;; This is not a delay equation: it should settle down to sinusoidal variation:
 (def alpha -.5)
 (defun f(x time xd) (+ (* alpha x) (sin (* 2 time))))
 (euler-delay-plot #'f ynot tmin tmax tao :title \"No Delay\")
 (euler-delay-plot-continued #'f ynot tmin tmax tao my-plot)
"
  (let* (
	 (hsize (/ tao (length ynot)))
	 (delay-index (length ynot))
	 (yvalues ynot)
	 (num (ceiling (/ (- end start) hsize)))
	 (timesteps (rseq start end num))
	 j nynot nstart nend yvalindex
	 )
    (dotimes
     (i (- num delay-index))
     (setq j (1- (+ i delay-index)))
     (setq yvalues
	   (append yvalues 
		  (+ (last yvalues)
		     (* hsize 
			(funcall 
			 f 
			 (select yvalues j)
			 (select timesteps j)
			 (select yvalues i)
			 )
			)
		     )
		  )
	  )
     )
    (send my-plot :clear-lines)
    (send my-plot :add-lines timesteps yvalues)
    (send my-plot :adjust-to-data)
    (if axes (let ()
	       (send my-plot :range 0 (select axes 0) (select axes 1))
	       (send my-plot :range 1 (select axes 2) (select axes 3))
	       )
      )
    (if title (send my-plot :title title))
    (send my-plot :show-window)
    (if (ok-or-cancel-dialog "continue?")
	(euler-delay-plot-continued
	 f
	 (let ((yvalindex (1- (length yvalues))))
	   (select yvalues (iseq (- yvalindex delay-index) yvalindex)))
	 (- end tao)
	 (+ (- end tao) (- end start))
	 tao
	 my-plot
	 :title title)
      )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
