;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; fourier Doc:    Creation Date:4/24/96        Author: Andy Long
;;
;;
(defmacro fourier (f n &optional (approx-name 'fourier-approx) (partition 40))
  "
;; Description/Arguments:

       (fourier f n &optional (approx-name 'fourier-approx) (partition 40))
  
	Computes the first n coefficients of the fourier series expansion for a
function f, and returns the approximating function as fourier-approx (unless
another name is specified as approx-name). Partition is the number of
subdivisions of the interval [-pi,pi] that the integrations will use (more the
better, but the default is probably okay unless your function is outrageous!).

;; Notes: uses qguass to compute the integrals.

;; See Also: qgauss, fourier2

;; Examples:
 (new-plot)
 (title \"f is a saw tooth function\")
 (func f(x) x * (bump x (- pi) pi))
 (fourier f 7)
 (plot f fourier-approx -pi pi)

 (new-plot)
 (func f(x) (bump x 0 pi))
 (title \"f is a bump on a log\")
 (fourier f 3 hippo)
 (plot f hippo -pi pi)
 (fourier f 7)
 (plot f fourier-approx -pi pi)
 (fourier f 15)
 (plot f fourier-approx -pi pi)
"
  `(fourier2 (turn-it-into-a-function ,f) ,n ',approx-name ,partition)
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; fourier2 Doc:    Creation Date:4/24/96        Author: Andy Long
;;
;;
(defun fourier2 (f n &optional (approx-name 'fourier-approx) (partition 40))
  "
;; Description/Arguments:

       (fourier2 f n &optional (approx-name 'fourier-approx) (partition 40))
  
	Computes the first n coefficients of the fourier series expansion for a
function f, and returns the approximating function as fourier-approx (unless
another name is specified as approx-name). Partition is the number of
subdivisions of the interval [-pi,pi] that the integrations will use (more the
better, but the default is probably okay unless your function is outrageous!).

;; Notes: uses qguass to compute the integrals.

;; See Also: qgauss

;; Examples:
 (new-plot)
 (title \"f is a saw tooth function\")
 (func f(x) x*(bump x (- pi) pi))
 (fourier2 #'f 7)
 (plot f fourier-approx -pi pi)

 (new-plot)
 (func f(x) (bump x 0 pi))
 (title \"f is a bump on a log\")
 (fourier2 #'f 3 'hippo)
 (plot f hippo -pi pi)
 (fourier2 #'f 7)
 (plot f fourier-approx -pi pi)
 (fourier2 #'f 15)
 (plot f fourier-approx -pi pi)
"
  (let (
	(ao (eval `(qgauss (lambda (x) (/ (,f x) 2pi)) -pi pi ,partition)))
	(a-index (iseq 1 n))
	(b-index (iseq 1 n))
	(eps 1e-6)
	a b j
	)

    ;; Calculate the coefficients by numerical integration:
    (dotimes
     (i n)
     (setq j (1+ i))
     (setq
      a
      (append
       a (list
	  (eval `(qgauss (lambda (x) (/ (* (,f x) (cos (* ,j x))) pi))
			 -pi pi ,partition))
	  ))
      )
     (setq
      b
      (append
       b (list
	  (eval `(qgauss (lambda (x) (/ (* (,f x) (sin (* ,j x))) pi))
			 -pi pi ,partition))
	  ))
      )
     )
    
    ;; Let's remove all the coefficients that don't really contribute:
    (dotimes
     (i n)
     (let ((energy (sqrt (+ (^ ao 2) (sum (^ a 2)) (sum (^ b 2))))))
       (if (< (/ (abs (nth i a)) energy) eps)
	   (list
	    (setf (select a i) 0)
	    (setf (select a-index i) 0)
	    )
	 )
       (if (< (/ (abs (nth i b)) energy) eps)
	   (list
	    (setf (select b i) 0)
	    (setf (select b-index i) 0)
	    )
	 )
       )
     )

    (format t "~2%                  Fourier Coefficients: ~% ~%")
    (format t "constant term: ~g ~% ~%" ao)
    (format t "coefficients of cosine terms: ~g ~% ~%" a)
    (format t "coefficients of sine terms: ~g ~% ~%" b)
    
    (setq a (remove 0 a)
	  a-index (remove 0 a-index)
	  b (remove 0 b)
	  b-index (remove 0 b-index))
    
    (eval 
     (cond
      ((/= 0 (* (length b) (length a)))
       `(defun ,approx-name
	  (x) (+ ,ao (sum
		      (* ',a (cos (* ',a-index x)))
		      (* ',b (sin (* ',b-index x))))))
       )		      
      ((= 0 (length b))
       `(defun ,approx-name (x) (+ ,ao (sum (* ',a (cos (* ',a-index x))))))
       )
      (t
       `(defun ,approx-name (x) (+ ,ao (sum (* ',b (sin (* ',b-index x))))))
       )
      )
     )
    )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; dft Doc:    Creation Date:4/26/96        Author: Andy Long
;;
;;
(defun dft(x &key (approx-name 'fft-approx) (print nil))
  "
;; Description: (dft x &key (approx-name 'fft-approx) (print nil))

	The discrete fourier transform takes a vector (or list) x, and
expresses it in a basis of trigonometric functions. Function dft returns the
function which approximates the given data, and optionally prints the
coefficients of the discrete approximation.

;; Examples:
 ;; this brings in standard data sets, including precipitation.
 (load-data \"tutorial\") 
 (dft precipitation)

 (new-plot)
 (title \"precipitation data\")
 (plot (iseq (length precipitation)) precipitation) 
 (title \"Fourier approximation to data\")
 (plot fft-approx 0 30)
 (send my-plot :add-points (iseq (length precipitation)) precipitation)
"
  (let* (
	 (lenx (length x))
	 (1/lenx (/ 1 lenx))
	 (fft-x (coerce (fft x) 'list))
	 (ao (* 1/lenx (realpart (first fft-x))))
	 )
    (if print (print fft-x))
    (if (evenp lenx)
	(setq nfreq (/ lenx 2)
	      two-parts (split-list fft-x nfreq)
	      one-part (* 1/lenx 
			  (combine (rest (first two-parts))
				   (/ (first (second two-parts)) 2))))
      (setq nfreq (/ (1- lenx) 2)
	    two-parts (split-list (rest fft-x) nfreq)
	    one-part (* 1/lenx (first two-parts))
	    )
      )
    (eval
     `(defun ,approx-name (x)
	(+
	 ,ao
	 (* 2
	    (sum (realpart
		  (* ',one-part (exp (* #c(0 1) x 2pi (iseq 1 ,nfreq) ,1/lenx)))
		  )
		 )
	    )
	 )
	)
     )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
