#|

Iterative hidden-line / ray-tracing program, with edge detection

Idea:
	1) set an eye position, and a view position (defines an axis)

	2) world edge (defined by a cone (angle from axis),
			with rounded base determined by near-sightedness)

	3) define probe directions in the world view: set up some radii, so
		many directions per radius.

	4) Iterative: okay everyone, take a step.

		a. If a probe hits, it sends out a flash, then dies.
		b. a flash is a single step in this procedure, defined from the
			point where the probe hit: this is the edge detection
			part: if there's an edge, a single step won't find it
			(we hope!).

	The size of the flash should be dictated by the distance at which it
occurs: farther flashes should be bigger in spatial extent, but less dense. So
use a constant number of points in a flash. In a way, this is a reflection.

	5) goto 4

Notes:

	As we proceed from eye position, the surface will be illuminated from
the near to the far, in order. This seems ideal. If only it could happen at the
speed of light!

|#
(defun hid-ray-recursion
  (f eye-position view-point up angle angle-divy near-sight
     &key (max-probes 1024) (niterates 1))
  "
;; Description: function
  (hid-ray-recursion
    f eye-position view-point up angle angle-divy near-sight
      &key (max-probes 1024) (niterates 1))

	f - the function;
	eye-position - the point at which the eyeball lives;
	view-point - the center of what the eyeball is lookin' at;
	up - which way is up
	angle - in degrees, how much of the world the eyeball can take in;
	angle-divy - how fine to divide that world, from the angle's standpoint;
	near-sight - beyond this, the world is dark and foreboding;
		max-probes - the maximum number of little excursion rays; and
		niterates - how many times to do this: I'm going to 'burst'
			when I hit, to see if I've got an edge. A burst would
			have niterates of 1 (just take a single step and see if
			you see anything).

;; Examples: 
 (func f(x y) x * y)
 (data eye-position 0 0 3)
 (data view-position 0 0 0)
 (data up 1 0 0)
 (hid-ray-recursion #'f eye-position view-position up 50 20 6 :niterates 50)

 (func f(x y) 1)
 (data eye-position 0 0 3)
 (data view-position 0 0 0)
 (data up 1 0 0)
 (hid-ray-recursion #'f eye-position view-position up 30 10 20 :niterates 100)
"
  (let* (
	 (eyex (first eye-position))
	 (eyey (second eye-position))
	 (eyez (third eye-position))
	 (xmin 1e31)
	 (xmax -1e31)
	 (ymin 1e31)
	 (ymax -1e31)
	 ;; This is a vector of the proper length in the line of sight:
	 (center-ray (* near-sight (normalize (- view-point eye-position))))
	 ;; These relative vectors are in the front plane: we'll follow the
	 ;; center ray, and add combinations of these to get our other ray
	 ;; points:
	 (relative-direction-1 (normalize (cross center-ray up)))
	 (relative-direction-2 (normalize (cross center-ray
						 relative-direction-1)))
	 (n angle-divy) ; just write a short form of angle-divy
	 (nangle (rseq 0 angle n))
	 ;; number of points at a radius should be proportional to radius:
	 (radial-num-pnts (combine 1 (* 4 (iseq 1 (1- n)))))
	 (step-length (/ near-sight niterates))
	 (num-probes (sum radial-num-pnts))
	 ;; every probe gets a location
	 (locations (make-array num-probes))
	 ;; we'll rely on a change in sign to tell us that something
	 ;; interesting has happened:
	 (signs (eval `(- ,eyez (,f ,eyex ,eyey))))
	 (signs (make-array (length locations) :initial-element (signum signs)))
	 (probe-locations nil)
	 (angle-bit (/ angle n))
	 (2-pi-i (* 2 pi #C(0.0 1.0)))
	 (probes-so-far -1) ;; we'll increment this before doing anything; this
			    ;; will always be one less than the actual number
			    ;; of probes.

	 (my-plot (apply #'send andyspin-proto :new 3 :show nil nil))
	 )

    (print (sum radial-num-pnts))
    (print (aref signs 1))
					; Form the locations of the rays:
    (dotimes
     (i n) ;; step over the n angles
     (let (
	   ;; This is a list of the number of points at each radius
	   (tmp-angle (* i angle-bit))
	   (probes-at-this-angle (nth i radial-num-pnts))
	   )
       (dotimes
	(j probes-at-this-angle) ;; for each probe-at-this-angle
	(setq probes-so-far (+ probes-so-far 1))
	(if (< probes-so-far max-probes)
	    (setf
	     (aref locations probes-so-far)
	     (* tmp-angle (exp (/ (* j 2-pi-i) probes-at-this-angle))))
	  )
	)
       )
     )
    ;; locations are now made into vectors:
    (dotimes (i (length locations))
	     (let (
		   (present-val (aref locations i))
		   )
	       (setf (aref locations i)
		     (+ 
		      (* (realpart present-val) relative-direction-1)
		      (* (imagpart present-val) relative-direction-2)
		      center-ray)
		     )
	       )
	     )
					; iterate - step and check:
    (dotimes
     (ii niterates)
     (let (
	   (nscale (/ ii niterates))
	   )
       (dotimes
	(i (length locations))
	(let (
	      (sign (aref signs i))
	      )
	  (if (/= sign 0)
	      (let* (
		     (point (coerce (* nscale (aref locations i)) 'list))
		     (x (+ eyex (first point)))
		     (y (+ eyey (second point)))
		     (z (+ eyez (third point)))
		     (fval (f x y))
		     )
		(if (< (* sign (- z fval)) 0)
		    (list 
		     (if (> x xmax) (setq xmax x))
		     (if (< x xmin) (setq xmin x))
		     (if (> y ymax) (setq ymax y))
		     (if (< y ymin) (setq ymin y))
		     (send my-plot :add-points
			   (list (list x) (list y) (list fval))
			   )
		     (setf (aref signs i) 0)
		     )
		  )
		)
	    )
	  )
	)
       )
     )
    (send my-plot :add-points (list (list eyex) (list eyey) (list eyez)))
    (send my-plot :adjust-to-data :draw nil)
    (setq xrange (send my-plot :range 0))
    (setq yrange (send my-plot :range 1))
    (send my-plot :new-menu)
    (send my-plot :real-axes)
    (send my-plot :showing-axes nil)
    (send my-plot :mouse-mode 'hand-rotate)
    (if (and
	 (< (abs (- xmax xmin)) 1e31)
	 (< (abs (- ymax ymin)) 1e31)
	 )
	(send my-plot :add-function f xmin xmax ymin ymax :draw nil)
      )
    (send my-plot :show-window)
;;    (send my-plot :frame-rect box)
#|
    (print xrange)
    (send my-plot :range 0 (first xrange) (second xrange))
    (print xrange)
    (send my-plot :range 1 (first yrange) (second yrange))
|#
    my-plot
    )
  )(help HID-RAY-RECURSION)
