;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;       Jan's histo stuff, in one file: I had to make changes
;;                        to two calls to fit-normal
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; This is inspired by Brown and Hwang, American Statistician, 1993,
;; 251-255, see also, 1994, 353-354. It is far more general, however.
;; It takes a andyplot-proto, extracts all the linestarts, then
;; reconstructs from this all the line segments in the plot, eliminating
;; the vertical segments in the process. These segments can result from
;; a (normalized) histogram, or from a function plot, or just from
;; an arbitrary bunch of linestarts. Some lines can be horizontal,
;; some can run from right to left. Segments are coded by their
;; endpoints on the horizontal axes, intercepts, and slopes.
;;
;; The program then minimizes
;;
;; sum_{i=1}^n int_{x_i}^{y_i}(a_i+b_i z-cf(z,m,s))^2dz
;;
;; where x_i,y_i are the endpoints of segment i, a_i is the
;; intercept of the correspoding line and b_i is its slope,
;; and f(z,m,s) is the density of a normal with mean m and
;; standard-deviation s.
;;
;; We minimize either over m,s,c (default), i.e. we also
;; scale the fitted density, or we minimize over m,s fixing
;; c at 1. This can be used to fit a normal to a histogram,
;; but also to fit a normal to the plot of a gamma density,
;; and so on.
;;
;; Version 1.0 ** March 23 1995 ** Jan de Leeuw
;; Version 1.1 ** March 26 1995 **
;;                The draw parameter is handled differently.
;;                Made a defun from the defmeth
;;                Returns two lists for :add-lines
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Couple of macros to construct the :isnew method and all assessor
;; methods for the slots of a prototype. After defining the prototype
;; the slots are either argslots or keyslots. Argslots are mandatory
;; arguments for the :isnew method, keyslots are keyword arguments.
;; The macros create the :isnew method, and a :set-foo assessor
;; method for each slot (which returns the contents if called without
;; argement, and sets the content if called with an argument.)
;;
;; Jan de Leeuw, 03-10-95
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defmacro assessor (key slot prototype)
`(defmeth ,prototype ,key (&optional (content nil set))
   (if set (setf (slot-value ',slot) content))
   (slot-value ',slot)))

(defmacro is-new-method (prototype argslots keyslots)
  `(defmeth ,prototype :isnew 
     (,@(mapcar #'(lambda (x)
		    (intern (concatenate 'string "THE-"
					 (symbol-name x)))) argslots) 
	&key 
	,@(mapcar #'(lambda (x)
		      (intern (concatenate 'string "THE-"
					   (symbol-name x)))) keyslots))
     ,@(mapcar #'(lambda (x y) `(send self ,x ,y)) 
	       (mapcar #'(lambda (x)
			   (intern (concatenate 'string "SET-"
						(symbol-name x)) 'keyword)) argslots)
	       (mapcar #'(lambda (y)
			   (intern (concatenate 'string "THE-"
						(symbol-name y)))) argslots))
     ,@(mapcar #'(lambda (x y z) `(send self ,x (if ,y ,y (send self ,z)))) 
	       (mapcar #'(lambda (x)
			   (intern (concatenate 'string "SET-"
						(symbol-name x)) 'keyword)) keyslots)
	       (mapcar #'(lambda (y)
			   (intern (concatenate 'string "THE-"
						(symbol-name y)))) keyslots)
	       (mapcar #'(lambda (z)
			   (intern (concatenate 'string "MAKE-"
						(symbol-name z)) 'keyword)) keyslots)))
  )

(defmacro make-assessors (prototype argslots keyslots)
  `(progn 
     ,@(mapcar #'(lambda (x y) `(assessor ,x ,y ,prototype))
	       (mapcar #'(lambda (x)
			   (intern (concatenate 'string "SET-"
						(symbol-name x)) 'keyword)) (append argslots keyslots))
	       (mapcar #'(lambda (x)
			   (intern (symbol-name x))) (append argslots keyslots))))
  )

(defun fit-normal
  (lst &key (mean 0) (sigma 1) (scale t) (draw t) (verbose nil))
  "Args: x
Given a list of linestarts, which define a number of
segments, this fits a normal curve to the segments by minimizing mean
square error."
  (let ((arg (list mean sigma)))
    (setf arg (newtonmax #'(lambda (arg)
			     (- (normal-fit-loss-function
				 lst (first arg) (second arg) scale)))
			 arg
			 :verbose verbose))
    (if verbose 
	(format t "~,10f ~%"
		(normal-fit-loss-function lst (first arg) (second arg) scale)))
    (let* ((ccc (normal-fit-collect-coefs lst (first arg) (second arg)))
	   (c11 (second ccc))
	   (c22 (third ccc))
	   (srt (mapcar #'first lst))
	   (end (mapcar #'second lst))
	   (xes (rseq (min srt) (max end) 50)))
      (format t
	      "~%Mean: ~g; Standard Deviation: ~g.~%" (first arg) (second arg))
      (list xes (mapcar
		 #'(lambda (x)
		     (let ((fac (if scale (/ c11 c22) 1))
			   (mn (first arg))
			   (sg (second arg)))
		       (* fac (/ (normal-dens (/ (- x mn) sg)) sg))
		       )) xes)))
    ))

(defmeth andyplot-proto :convert-linestarts ()
  "Args: lines
The list of linestarts is converted to a list of endpoints,
slopes, and intercepts."
  (let* ((nn (send self :num-lines))
	 (sg nil))
    (dotimes (i nn)
	     (let ((j (send self :linestart-next i)))
	       (if j (let ((x0 (send self :linestart-coordinate 0 i))
			   (y0 (send self :linestart-coordinate 1 i))
			   (x1 (send self :linestart-coordinate 0 j))
			   (y1 (send self :linestart-coordinate 1 j)))
		       (if (/= x0 x1)
			   (let ((a (/ (- (* y0 x1) (* y1 x0))
				       (- x1 x0)))
				 (b (/ (- y1 y0) (- x1 x0))))
			     (setf sg (append sg (list x0 x1 a b)))))))))
    (split-list sg 4)
    ))

(defun normal-fit-collect-coefs (lst mn sg)
  (let* ((srt (mapcar #'first lst))
         (end (mapcar #'second lst))
         (int (mapcar #'third lst))
         (slp (mapcar #'fourth lst))
	 
         (c00 (sum (+ (* (^ slp 2) (/ (- (^ end 3) (^ srt 3)) 3))
                      (* slp int (- (^ end 2) (^ srt 2)))
                      (* (^ int 2) (- end srt)))))
         (df1 (/ (- srt mn) sg))
         (df2 (/ (- end mn) sg))

         (c11 (sum (- (* (+ int (* mn slp))
                         (- (normal-cdf df2)
                            (normal-cdf df1)))
                      (* sg slp
                         (- (normal-dens df2)
                            (normal-dens df1))))))

         (c22 (sum (/ (- (normal-cdf (* df2 (sqrt 2)))
                         (normal-cdf (* df1 (sqrt 2))))
                      (* 2 sg (sqrt pi))))))
    (list c00 c11 c22)
    ))

(defun normal-fit-loss-function (lst mn sg sc)
  (let* ((ccc (normal-fit-collect-coefs lst mn sg))
         (c00 (first ccc))
         (c11 (second ccc))
         (c22 (third ccc)))
    ;; ael - caused zero divide in newtonmax
    ;; (if sc (- c00 (/ (^ c11 2) c22)) 
    ;; lea
    (if sc (- c00 (/ (^ c11 2) (if (= 0 c22) 1e-6 c22)))
      (- (+ c00 c22) (+ c11 c11)))
    ))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; This module has a complicated history. The first version was written
;; by Jan de Leeuw, mid 1994, to provide an Xlisp-Stat version of the
;; S-routines in Haerdle's book, and to provide a histogram-proto that
;; inherits from the andyplot-proto.
;;
;; The second version was done begin 1995 by Jason Bond, who added a
;; slider for the binwidth.
;;
;; Version 3.0 was done by Jan de Leeuw in March 1995. It involved a
;; complete overhaul of the code. XOR drawing was eliminated, menu
;; support improved, slider support improved, code made more modular,
;; my-histogram-proto and my-polygon-proto separated.
;;
;; 3.0     ** March 22 1995 ** Still no support for warping and fitting
;;         a normal.
;; 3.1     ** March 23 1995 ** Support for fitting a normal.
;;
;; 3.2     ** March 26 1995 ** Make sure the menu-items toggle in the
;;         correct way. Improved updating of plots, much greater
;;         efficiency. Created menu items for options. Wrote my-histogram
;;         function. Foundations put in for warping and overlays.
;;         Optimized normal-density fitting.
;;
;; 3.3     ** May 22, 1995 ** Support for different bin-widths.  Function
;;         call for different bin-widths is:
;;         (my-histogram data :my-cutpoints cut-points) where cutpoints
;;         is a list of cutoff points.  Of course, changing the bin-widths
;;         using the slider makes everything go back to a single bin-width.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defproto my-histogram-proto
  '(data            ;; the data
    binwidth        ;; the binwidth
    origin          ;; the origin
    histo           ;; is there a histogram in the plot ?
    poly            ;; is there a polygon in the plot ?
    normal          ;; is there a normal in the plot ?
    prob            ;; counts or probabilities
    full-hist       ;; is the histogram full or hollow ?
    with-points     ;; does the polygon have midpoints on it ?
    full-overlay    ;; histogram plot overlay
    point-overlay   ;; polygon plot overlay
    counts          ;; bin-counts
    cut-points      ;; cut-points for bins
    mid-points      ;; mid-points of bins
    slider)         ;; slot for slider
  ()
  andyplot-proto)

(make-assessors my-histogram-proto
		(data)
		(binwidth
		 origin
		 histo
		 poly
		 normal
		 full-hist
		 with-points
		 full-overlay
		 point-overlay
		 prob
		 counts
		 cut-points
		 mid-points
		 slider))

(defmeth my-histogram-proto :isnew 
  (the-data &key the-binwidth the-origin my-cutpoints)
  (call-method graph-proto :isnew 2 :title (if *title* *title* "Histogram"))
  (send self :set-data (sort-data the-data))
  (send self :set-histo t)
  #| - ael - Fri May 1 19:02:58 1998
  (send self :set-binwidth
	(if my-cutpoints my-cutpoints
	  (if the-binwidth the-binwidth (send self :make-binwidth))))
 - lea - |#
  (send self :set-binwidth
	  (if the-binwidth the-binwidth 
	    (if my-cutpoints my-cutpoints
	      (send self :make-binwidth))))

  (if the-origin (send self :set-origin the-origin) (send self :make-origin))
  ;;ael - moved to below: (send self :set-counts (send self :make-counts))
  (send self :set-cut-points
	(if my-cutpoints my-cutpoints (send self :make-cut-points)))

  (send self :set-counts (send self :make-counts))
  (send self :set-mid-points (send self :make-mid-points))
  (send self :set-full-overlay (send self :make-full-overlay))
  (send self :set-point-overlay (send self :make-point-overlay))
  (send self :set-slider nil)
  (send self :set-normal nil)
  (send self :set-full-hist t)
  (send self :set-with-points t)
  (send self :set-prob nil)
  (send self :menu (send self :make-menu))
  (send self :make-plot)
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			my-histogram-proto methods
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth my-histogram-proto :close ()
  (if (send self :set-slider)
      (send (send self :set-slider) :close))
  (call-next-method)
  )
(defmeth my-histogram-proto :num-bins () (length (send self :set-counts)))
(defmeth my-histogram-proto :num-cuts ()
  (length (send self :set-cut-points)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  Menu Support
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defmeth my-histogram-proto :make-menu ()
  (let* (
         (slider (send menu-item-proto :new "Change-Bins"
		       :action #'(lambda () 
				   (send self :make-slider))))
#|
         (histo (send menu-item-proto :new "Show Histogram"
                      :action #'(lambda () 
                                  (send self :toggle-histo-poly)
                                  (send self :toggle-plot)
                                  (send self :refit-normal))))
|#
         (full (send menu-item-proto :new "Histogram Only"
		     :action #'(lambda ()
;;ael				 (send self :toggle-full-option)
;;ael				 (send self :toggle-plot)
				 (send self :make-plot)
				 )))
         (poly (send menu-item-proto :new "Show Polygon"
		     :action #'(lambda () 
				 (send self :poly-lines)
				 (send self :adjust-to-data)
;;ael				 (send self :toggle-histo-poly)
;;ael				 (send self :toggle-plot)
;;ael				 (send self :refit-normal)
				 )))
         (points (send menu-item-proto :new "Show Midpoints"
                       :action #'(lambda ()
;;ael                                   (send self :toggle-midpoint-option)
;;ael                                   (send self :toggle-plot)
				   (send self :poly-points)
				   (send self :adjust-to-data)
				   )))
         (normal (send menu-item-proto :new "Show Normal" 
		       :action #'(lambda ()
				   (send self :normal-lines)
				   (send self :adjust-to-data)
;;ael				   (send self :toggle-normal-item)
;;ael				   (send self :toggle-normal-plot)
				   )))
         (dash (send dash-item-proto :new))
         (menu (send menu-proto :new "Histogram"))
         )
    (send menu :append-items
	  slider dash ;; histo
	  full ;; dash
	  poly points ;; dash
	  normal)
    ;;ael (send histo :mark t)
    ;;ael (send full :mark t)
    ;;ael (send points :mark t)
    ;;ael (send points :enabled nil)
    menu))

(defmeth my-histogram-proto :toggle-histo-poly ()
  (let ((mm (send (send self :menu) :find-item "Show Histogram"))
        (mn (send (send self :menu) :find-item "Show Polygon"))
        (mo (send (send self :menu) :find-item "Full Histogram"))
        (mp (send (send self :menu) :find-item "Show Midpoints")))
    (send self :set-histo
          (not (send self :set-histo)))
    (send self :set-poly
          (not (send self :set-poly)))
    (send mn :mark
          (if (send self :set-poly) t nil))
    (send mm :mark
          (if (send self :set-histo) t nil))
    (cond ((send self :set-poly)
           (send mo :enabled nil)
           (send mp :enabled t))
          ((send self :set-histo)
           (send mo :enabled t)
           (send mp :enabled nil)))
    ))

(defmeth my-histogram-proto :toggle-full-option ()
  (let ((mo (send (send self :menu) :find-item "Full Histogram")))
    (send self :set-full-hist
	  (not (send self :set-full-hist)))
    (send mo :mark
	  (if (send self :set-full-hist) t nil))
    ))

(defmeth my-histogram-proto :toggle-midpoint-option ()
  (let ((mp (send (send self :menu) :find-item "Show Midpoints")))
    (send self :set-with-points
          (not (send self :set-with-points)))
    (send mp :mark
          (if (send self :set-with-points) t nil))
    ))

(defmeth my-histogram-proto :toggle-normal-item ()
  (let ((mm (send (send self :menu) :find-item "Show Normal")))
    (send self :set-normal
          (not (send self :set-normal)))
    (send mm :mark
          (if (send self :set-normal) t nil))
    ))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  Slider Support
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defmeth my-histogram-proto :make-slider ()
  (let* ((data (send self :set-data))
         (sdat (data-spacing data))
         (rdat (data-range data))
	 (change-bins (send my-change-hist-bins-item-proto :new self))
	 (bins (send self :num-bins))
	 )
    (setq change-bins (send change-bins :do-action))
    (if (/= bins (if change-bins change-bins bins))
	(let ()
	  (send self :set-binwidth
		(max
		 (/ (- (max data) (min data)) (1- change-bins))
		 sdat)
		)
	  (send self :recompute)
	  (send self :make-plot)
	  ))
    ))

;; my-CHANGE-HIST-BINS-ITEM-PROTO. Opens new integer dialog.
(defproto my-change-hist-bins-item-proto '(graph) () menu-item-proto)

(defmeth my-change-hist-bins-item-proto :isnew (h)
  (setf (slot-value 'graph) h)
  (call-next-method "Change Bins"))

(defmeth my-change-hist-bins-item-proto :do-action ()
  (let ((bins (get-new-integer "Number of bins"
                               1
                               100
                               (send (slot-value 'graph) :num-bins))))
    bins 
    ))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  Plot Methods
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defmeth my-histogram-proto :refit-normal ()
  ;;ael  (let* ((aa (fit-normal (send self :convert-linestarts)))
  (let* ((data (send self :set-data))
	 (aa (fit-normal
	      (send self :convert-linestarts)
	      :mean (mean data)
	      :sigma (standard-deviation data)
	      ))
	 ;;ael
	 (nn (send self :num-lines))
         (ll (+ (- nn 50) (iseq 50))))
    (send self :linestart-coordinate 0 ll (first aa))
    (send self :linestart-coordinate 1 ll (second aa))
    (if (send self :set-normal)
        (send self :toggle-normal-plot))
    ))

(defmeth my-histogram-proto :make-plot (&key normal poly)
  (send self :clear)
  (send self :horizontal-lines)
  (send self :hollow-vertical-lines)
  (send self :full-vertical-lines)
  (if poly (send self :poly-lines))
  (if normal (send self :normal-lines))
  (send self :adjust-to-data)
  )

(defmeth my-histogram-proto :re-plot (&key normal poly) 
  (send self :clear)
  (send self :horizontal-lines)
  (send self :hollow-vertical-lines)
  (send self :full-vertical-lines)
  (if poly (send self :poly-lines))
  (if normal (send self :normal-lines))
  (send self :adjust-to-data)
  )

(defmeth my-histogram-proto :toggle-plot ()
  (let* ((nn (send self :num-bins))
         (ll (* 2 (iseq (1- (* 4 nn)))))
         (lh (select ll (iseq nn)))
         (lo (select ll (+ nn (iseq (1+ nn)))))
         (lf (select ll (+ (+ nn nn 1) (iseq (1- nn)))))
         (lp (select ll (+ (* 3 nn) (iseq (1- nn))))))
    (cond ((send self :set-histo)
           (send self :linestart-next lh (1+ lh))
           (send self :linestart-next lo (1+ lo))
           (if (send self :set-full-hist)
               (send self :linestart-next lf (1+ lf))
             (send self :linestart-next lf nil))
           (send self :linestart-next lp nil)
           (send self :clear-points))
          ((send self :set-poly)
           (send self :linestart-next lh nil)
           (send self :linestart-next lo nil)
           (send self :linestart-next lf nil)
           (send self :linestart-next lp (1+ lp))
           (if (send self :set-with-points)
               (send self :poly-points)
             (send self :clear-points))))
    (send self :adjust-to-data)
    ))

(defmeth my-histogram-proto :toggle-normal-plot ()
  (let* ((nn (send self :num-lines))
         (ll (+ (- nn 50) (iseq 49))))
    (if (send self :set-normal)
        (send self :linestart-next ll (1+ ll))
      (send self :linestart-next ll nil))
    (send self :adjust-to-data)
    ))

(defmeth my-histogram-proto :horizontal-lines ()
  (andy-print ":horizontal-lines ()")
  (let* ((cp (send self :set-cut-points))
         (cn (send self :set-counts))
         (nn (length cn))
	 )
    (andy-print cp cn nn)
    (dotimes (i nn)
      (let ((ci (elt cp i))
            (cj (elt cn i))
            (ck (elt cp (1+ i))))
        (send self :add-lines (list ci ck) (list cj cj) :draw nil)))
    ))

(defmeth my-histogram-proto :poly-lines ()
  (andy-print ":poly-lines ()")
  (let* ((mp (send self :set-mid-points))
         (cn (send self :set-counts))
         (nn (1- (length cn))))
    (dotimes (i nn)
      (let ((ci (elt mp i))
            (cj (elt mp (1+ i)))
            (ck (elt cn i))
            (cl (elt cn (1+ i))))
	(andy-print (list ci cj) (list ck cl))
        (send self :add-lines (list ci cj) (list ck cl) :draw nil)))
    ))

(defmeth my-histogram-proto :poly-points ()
  (let* ((mp (send self :set-mid-points))
         (cn (send self :set-counts)))
    (send self :add-points mp cn :draw nil))
  )

(defmeth my-histogram-proto :hollow-vertical-lines ()
  (andy-print ":hollow-vertical-lines ()")
  (let* ((cp (send self :set-cut-points))
         (cn (send self :set-counts))
         (nn (length cn)))
    (dolist
     (i (iseq (1- nn)))
     (let* ((ci (elt cp (1+ i)))
	    (cj (elt cn i))
	    (ck (elt cn (1+ i))))
       (andy-print ci (min cj ck) (max cj ck) cj ck)
       (send self :add-lines (list ci ci)
	     (list (min cj ck) (max cj ck)) :draw nil)
       ))))

(defmeth my-histogram-proto :full-vertical-lines ()
  (andy-print ":full-vertical-lines ()")
  (let* ((cp (send self :set-cut-points))
         (cn (send self :set-counts))
	 (nn (length cn)))
    (send self :add-lines (list (first cp) (first cp)) 
	  (list 0 (first cn)) :draw nil)
    (dolist
     (i (rest (iseq (1- nn))))
     (let* ((ci (elt cp i))
	    (cj (elt cn i))
	    ;; ael - changed from 1+: (ck (elt cn (1+ i)))
	    (ck (elt cn (1- i)))
	    ;; lea
	    )
       (andy-print 'first ci (min cj ck) cj ck)
       (send self :add-lines (list ci ci) (list 0 (min cj ck)) :draw nil)
       (setq ci (elt cp (1+ i)))
       (andy-print 'second ci 0 (min cj ck) cj ck)
       (send self :add-lines (list ci ci) (list 0 (min cj ck)) :draw nil)))
    (if (= 1 (- (length cp) nn))
	(send self :add-lines (list (final cp) (final cp)) 
	      (list 0 (final cn)) :draw nil)
      (send self :add-lines (list (final (butlast cp)) (final (butlast cp))) 
	    (list 0 (final cn)) :draw nil)
      )))
    

(defmeth my-histogram-proto :normal-lines ()
  ;;ael  (let ((ll (fit-normal (send self :convert-linestarts))))
  (let* ((data (send self :set-data))
	 (ll (fit-normal
	      (send self :convert-linestarts)
	      :mean (mean data)
	      :sigma (standard-deviation data)
	      )))
    ;;ael
    (send self :add-lines (first ll) (second ll) :draw nil)
    ))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  Compute Methods
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defmeth my-histogram-proto :recompute ()
  (send self :set-cut-points (send self :make-cut-points))
  (send self :set-mid-points (send self :make-mid-points))
  (send self :set-counts (send self :make-counts))
  )

(defmeth my-histogram-proto :make-counts ()
  (let ((data (send self :set-data))
        (binwidth (send self :set-binwidth))
        (origin (send self :set-origin)))
    ;; ael    (count-in-bins data binwidth origin
    (count-in-bins data (send self :set-cut-points) origin)
    ))

(defmeth my-histogram-proto :make-cut-points ()
  (let* ((data (send self :set-data))
         (binwidth (send self :set-binwidth))
         (origin (send self :set-origin))
         (minx (floor (/ (- (min data) origin) binwidth)))
         (maxx (ceiling (/ (- (max data) origin) binwidth))))
    (+ origin (* binwidth (iseq minx maxx)))
))

(defmeth my-histogram-proto :make-mid-points ()
  (let ((cp (send self :set-cut-points)))
    (/ (+ (butlast cp) (rest cp)) 2)
    ))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  Initial Constructors
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defmeth my-histogram-proto :make-origin ()
  (send self :set-origin 
        (mean (send self :set-data))))

(defmeth my-histogram-proto :make-binwidth ()
  (send self :set-binwidth
        (if (send self :set-histo)
            (send self :opt-histo-binwidth)
          (send self :opt-poly-binwidth)))
  )

(defmeth my-histogram-proto :opt-histo-binwidth ()
  "See Scott, Biometrika, 66, 1979, 605-610"
  (let (
	(data (send self :set-data))
	)
    (* (standard-deviation data) 
       (^ (* 24 (sqrt pi) (/ (length data))) (/ 3)))
    ))

(defmeth my-histogram-proto :opt-poly-binwidth ()
  "See Scott, JASA, 80, 1985, 348-454"
  (let (
	(data (send self :set-data))
	)
    (* 2 (standard-deviation data) 
       (^ (* (/ 40 49) (sqrt pi) 
	     (/ (length data))) (/ 5)))
    ))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  Overlay support
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(defmeth my-histogram-proto :make-full-overlay ()
  )

(defmeth my-histogram-proto :make-point-overlay ()
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  Warp support
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun warp-histogram (data binwidth origin m &key (kernel 0))
  (let* (
	 (n (length data))
	 (d (/ binwidth m))
	 (c (count-in-bins data d origin))
	 (w (kernel-weights binwidth n m kernel))
	 )
    ))

(defun kernel-weights (binwidth n m kernel)
  (let* (
	 (cm 0)
	 (wm (repeat 0 m))
	 (mm (iseq m))
	 (nh (* n binwidth))
	 (im (/ mm m))
	 )
    (case kernel 
	  (0 (setf cm (/ m (* (1- (* 2 m)) nh)))
	     (setf wm (repeat cm m)))
	  (1 (setf cm (/ (* nh)))
	     (setf wm (* cm (- 1 im))))
	  (2 (setf cm (/ (* 3 m m) (* nh (1- (* 4 m m)))))
	     (setf wm (* cm (- 1 (^ im 2)))))
	  (3 (setf cm (/ (* 15 (^ m 4)) (* nh (1- (* 16 (^ m 4))))))
	     (setf wm (* cm (^ (- 1 (^ im 2)) 2))))
	  )))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Miscellaneous
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun count-in-bins (data binwidth origin)
  (andy-print ":count-in-bins ()")
  (andy-print data binwidth origin)
  (if (listp binwidth)
      (progn 
	(let (
	      (beg-bin (butlast binwidth))
	      (end-bin (rest binwidth))
	      (bcnt (repeat 0 (1- (length binwidth))))
	      )
	  (andy-print 'a-list bcnt)
	  (mapcar #'(lambda (x) 
		      (let ((posn
			     (first
			      ;; The fact that I'm using <= means that anybody
			      ;; sitting right on the line goes into the
			      ;; preceding bin:
			      (which (if-else (<= beg-bin x end-bin) t nil)))))
			(if posn
			    (setf (elt bcnt posn) (1+ (elt bcnt posn))))))
		  data)
	  (andy-print bcnt)
	  bcnt))
    (progn (let* (
                  (indx (floor (/ data binwidth)))
                  (jndx (- indx (min indx)))
                  (nbin (1+ (max jndx)))
                  ;; ael (bcnt (repeat 0 nbin))
		  (bcnt (repeat 0 (if points points nbin)))
		  )
	     (andy-print 'c-i-b indx jndx nbin bcnt)
	     (mapcar #'(lambda (x) (setf (elt bcnt x) (1+ (elt bcnt x)))) jndx)
	     (andy-print 'not-a-list bcnt)
	     bcnt
	     ))))

(defun data-spacing (data) (min (abs (difference (sort-data data)))))
(defun data-range (data) (- (max data) (min data)))  

(defmeth menu-proto :find-item (str)
  "Method args: (str)
Finds and returns menu item with title STR."
  (dolist (item (send self :items))
	  (if (string-equal str (send item :title))
	      (return item))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				histo
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun histo (data &key 
		   (binwidth nil have-bin)
		   (origin nil have-origin)
		   (my-cutpoints nil points))
  "
;; Description: histo (data &key 
				(binwidth nil have-bin)
				(origin nil have-origin)
				(my-cutpoints nil points))

;; Author: Jan de Leeuw, with modifications by Andy Long.

;; Returns: plot

;; Notes: 
;; This is inspired by Brown and Hwang, American Statistician, 1993,
;; 251-255, see also, 1994, 353-354.

;; Examples:
 (histo (rand.n 100))
 (histo (geometric-rand 100 .09) :my-cutpoints (* 2 (iseq 0 20)))
"
  (cond ((and have-bin have-origin)
         (send my-histogram-proto :new data :the-binwidth binwidth
               :the-origin origin))
        (have-bin
         (send my-histogram-proto :new data :the-binwidth binwidth))
        (have-origin
         (send my-histogram-proto :new data :the-origin origin))
        (points 
         (send my-histogram-proto :new data :my-cutpoints my-cutpoints))
        (t (send my-histogram-proto :new data)))
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 points-to-histogram
;;
(defun points-to-histogram(x y
			     &key
			     (fill-color 'blue)
			     (line-color 'black)
			     (squeeze 2)
			     )
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; points-to-histogram Doc:    Creation Date:3/13/97        Author: Andy Long
;;
;; Description/Arguments:

 (points-to-histogram(x y
			     &key
			     (fill-color 'blue)
			     (line-color 'black)
			     (squeeze 2)
			     )
;; Examples:
 (points-to-histogram (iseq 10) (rand.u 10))
"
  (def my-plot (plotter))
  (let (
	(xdiff (/ (- (second x) (first x)) squeeze))
	(xmin (min x))
	(xmax (max x))
	(ymax 0)
	)
    (mapcar
     (lambda(x y)
       (let (
	     (x1 (- x xdiff))
	     (x2 (+ x xdiff))
	     )
	 (setq ymax (max ymax y))
	 (send my-plot :add-plot-item
	       (list
		'patch
		(list (list x1 0)(list x2 0)(list x2 y)(list x1 y)(list x1 0))
		fill-color
		line-color
		)
	       :draw nil
	       )
	 )
       )
     x y
     )
    (send my-plot :range 0 (- xmin (* 2 xdiff)) (+ xmax (* 2 xdiff)))
    (send my-plot :range 1 0 (* 1.1 ymax))
    (send my-plot :plot-item)
    )
  my-plot
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
