;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 2-by-2-eigen
;;
(defun 2-by-2-eigen(matrix)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; 2-by-2-eigs Doc:    Creation Date:3/13/97        Author: Andy Long
;;
;; Description/Arguments:

       (2-by-2-eigen matrix)
  
;; Notes: Written in response to the fact that eigen only works for symmetric
;; matrices - yikes!

;; See Also: eigen

;; Examples:
 (2-by-2-eigen (matrix '(2 2) (list 0 1 -1 0)))
 (eigen (matrix '(2 2) (list 0 1 -1 0)))
 (2-by-2-eigen (id 2))
 (eigen (id 2))
"
  (let* (
	 (a (aref matrix 0 0))
	 (b (aref matrix 0 1))
	 (c (aref matrix 1 0))
	 (d (aref matrix 1 1))
	 (eigvals (quadratic 1 (- (+ a d)) (- (* a d) (* b c))))
	 (ev1 (list b (- (first eigvals) a)))
	 (ev2 (list b (- (second eigvals) a)))
	 )
    (list eigvals
	  (bind-columns
	   (/ ev1 (if (= 0 (norm ev1)) 1 (norm ev1)))
	   (/ ev2 (if (= 0 (norm ev2)) 1 (norm ev2)))
	   ))
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 2-by-2-matrix-action
;;
(defun 2-by-2-matrix-action(matrix)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; 2-by-2-matrix-action Doc:    Creation Date:3/18/97        Author: Andy Long
;;
;; Description/Arguments:

       (2-by-2-matrix-action matrix)
  
	Demonstrates that the image of the unit circle under matrix
multiplication is an ellipse. The plot is of the points, and of their
images, which are joined by lines.

;; Examples:
 (title \"Spirograph?!\")
 ;; Try this one several times, to get a feeling for the range possible.
 (2-by-2-matrix-action (matrix (list 2 2) (rand.u 4)))

 ;; Here's a really pretty one:
 (2-by-2-matrix-action #2a((1 2) (3 4)))
 (2-by-2-matrix-action (inverse #2a((1 2) (3 4))))
"
  (let* (
	 (theta (rseq 0 2pi 100))
	 (coss (cos theta))
	 (sins (sin theta))
	 (stuff (row-list (matprod matrix (bind-rows coss sins))))
	 (x (coerce (first stuff) 'list))
	 (y (coerce (second stuff) 'list))
	 domain range
	 )
    (plotter)
    (send my-plot :add-plot-item (list 'points x y 'blue))
    (send my-plot :add-plot-item (list 'points coss sins 'red)) 
    (mapcar
     (lambda(x y u v)
       (send my-plot :add-plot-item
	     (list 'lines (list x u) (list y v) 'black)
	     )
       )
     x y coss sins
     )	   
    (send my-plot :update-plot)

    (setq
     domain (send my-plot :range 0)
     range (send my-plot :range 1)
     domain (list (min (first domain) (first range))
		  (max (second domain) (second range)))
     )     
    (apply #'send my-plot :range 0 domain)
    (apply #'send my-plot :range 1 domain)

    nil
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 2-by-2-peak
;;
(defun 2-by-2-peak(matrix)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; 2-by-2-peak Doc:    Creation Date:3/18/97        Author: Andy Long
;;
;; Description/Arguments:

       (2-by-2-peak matrix)
	
;; Notes: I'm trying to identify one of the eigenvectors, evidently.

;; See Also: 2-by-2-matrix-action

;; Examples:
 (2-by-2-peak #2a((1 2) (3 4)))
"
  (let* (
	 (a (aref matrix 0 0))
	 (b (aref matrix 0 1))
	 (c (aref matrix 1 0))
	 (d (aref matrix 1 1))
	 (tmp1 (+ (* a b) (* c d)))
	 (tmp2 (+ (* a a) (* c c)))
	 (tmp3 (+ (* b b) (* d d)))
	 (x (sqrt (/ tmp3 (- (* tmp2 tmp3) (* tmp1 tmp1)))))
	 (y (* (- x) (/ tmp1 tmp3)))
	 )
    (2-by-2-matrix-action matrix)
    (send my-plot :add-plot-item
	  (list 'lines (list (- x) x) (list (- y) y) 'red))
    (send my-plot :update-plot)
    (norm (matprod matrix (list x y)))
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 vector-in-span-of-vectors-p
;;
(defun vector-in-span-of-vectors-p(v M &key (eps 1e-10))
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          vector-in-span-of-vectors-p Doc:
;;
;; Author: Andy Long
;; Created: Fri Feb 6 11:16:04 1998
;;
;; Description/Arguments:

       (vector-in-span-of-vectors-p v M &key (eps 1e-10))
  
;; Returns: T or NIL, depending on whether vector-in-span-of-vectors...

	The test is as follows: we project v onto the span of the M vectors,
then check the norm of the difference of the vector v and its projection.

;; Notes: created as a helper for generalized-eigenvectors.

;; See Also: generalized-eigenvectors

;; Examples:
 (vector-in-span-of-vectors-p #(1 0) (id 2))
 (vector-in-span-of-vectors-p #(1 0) #2a((0 0)(0 1)))

 (vector-in-span-of-vectors-p
  #(0 0 .5 -.5)
  #2a(
     ( -1.00000      0.000000    )
     ( 0.000000      0.000000    )
     ( 0.000000       1.00000    )
     ( 0.000000      0.000000    )
     )
  )
 (vector-in-span-of-vectors-p
  #(1 0 1 0)
  #2a(
     ( -1.00000      0.000000    )
     ( 0.000000      0.000000    )
     ( 0.000000       1.00000    )
     ( 0.000000      0.000000    )
     )
  )
"
  ;; Make M into a matrix, if it's given as a list of vectors:
  (setq M (apply #'bind-columns (span-vectors M)))
  (let* (
	 ;; Project v onto the columns of M
	 (projection (matmult v M))
	 ;; Then, in the "basis" of M, v can be written as
	 (expansion (matmult M projection))
	 )
    ;; We now have a look at the difference between them:
    ;; if it's very small, we'll consider that it was reproduced in the basis
    (zero-vector-p (- v expansion) :eps eps)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 span-vectors
;;
(defun span-vectors(M &key (eps 1e-12))
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          span-vectors Doc:
;;
;; Author: Andy Long
;; Created: Fri Feb 6 17:06:34 1998
;;
;; Description/Arguments:

       (span-vectors M &key (eps 1e-12))

where
	M	- a matrix of column vectors, or list of vectors
	eps	- a value which will signal when the dimension of the space is
		  achieved
  
;; Returns: a list of orthogonal vectors which span the space generated by the
list of vectors (or matrix).

;; Notes: Relies on the svd for the calculation of the orthogonal vectors.

;; See Also: svd

;; Examples:
 ;; The three following commands all give the same response,
 (span-vectors #2a((.2 .3) (1 1)))
 (span-vectors #2a((.2 .3 0) (1 1 0)))
 (span-vectors #2a((.2 .3 .5) (1 1 2)))
 ;; and this fourth one will give the same answer with zeros tagged onto the
 ;; end:
 (span-vectors #2a((.2 .3) (1 1)(0 0)))
"
  ;; Make M into a matrix, if it's given as a list of vectors:
  (if (listp M) (setq M (apply #'bind-columns M)))
  (when
   (> (sum (square (combine M))) 0)
   (let* (
	  (svd (svd M))
	  (sings (second svd))
	  (sum-sings (sum sings))
	  )
     (do*
      (
       (sing sings (rest sing))
       (u (column-lists (first svd)) (rest u))
       (sing-vecs (list (first u)) (append sing-vecs (list (first u))))
       (total (first sing) (+ total (first sing)))
       )
      ((< (- sum-sings total) eps) sing-vecs)
      )      
     )
   )
  )
;;
;; reduce-to-basis has been replaced by an alias to span-vectors
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 orthogonal-p
;;
(defun orthogonal-p(v1 v2 &key (eps 1e-6) (test #'v-and))
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          orthogonal-p Doc:
;;
;; Author: Andy Long
;; Created: Fri Feb 6 18:55:10 1998
;;
;; Description/Arguments:

       (orthogonal-p v1 v2 &key (eps 1e-6) (test #'v-and))
  
where
	v1 is a vector/list, and 
	v2 is a vector or matrix;
	eps is the precision to which the test is being carried out.
	test is v-and (vector-and: only used if v2 is a matrix)

;; Returns: T or Nil, depending whether vectors v1 and v2 are orthogonal to eps
   precision (or, in the event that v2 is a matrix, if every column of v2 is
   orthogonal to v1).

;; See Also: dot, norm

;; Examples:
 (orthogonal-p #(1 0) #(1 0))
 (orthogonal-p #(1 0) #(0 1))
 (orthogonal-p #(1 0) #(1e-6 1))
 (orthogonal-p #(1 0) #(1e-13 1))
 (orthogonal-p #(1 0) #2a((1e-13 1)(1 1)))
 (orthogonal-p #(1 0) #2a((0 0)(1 1)))
"
  (if (matrixp v2)
      (funcall
       test
       (mapcar #'(lambda (v) (orthogonal-p v1 v :eps eps)) (column-list v2)))
    (< (dot v1 v2) eps)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 colinear-p
;;
(defun colinear-p(v1 v2 &key (eps 1e-6) (test #'v-and))
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          colinear-p Doc:
;;
;; Author: Andy Long
;; Created: Fri Feb 6 18:55:10 1998
;;
;; Description/Arguments:

       (colinear-p v1 v2 &key (eps 1e-6) (test #'v-and))
  
where
	v1 is a vector/list, and 
	v2 is a vector or matrix. 
	eps is the precision to which the test is being carried out.
	test is v-and (vector-and: only used if v2 is a matrix)

;; Returns: T or Nil, depending whether vectors v1 and v2 are colinear to eps
   precision (or, in the event that v2 is a matrix, if every column of v2 is
   colinear to v1).

;; See Also: dot, norm

;; Examples:
 (colinear-p #(1 0) #(1 0))
 (colinear-p #(1 0) #(11 0))
 (colinear-p #(1 0) #(0 1))
 (colinear-p #(1 2) #(2 4))
 (colinear-p #(1 0) #2a((1e-13 1)(1 1)))
 (colinear-p #(1 0) #2a((2 0)(0 1)))
 (colinear-p #(1 0) #2a((2 0)(0 1)) :test #'v-or)

 (colinear-p '(1 3 2)
	    #2a(
		( -8.748178E-2 -0.252538    )
		(-0.262445     -0.757614    )
		(-0.174964     -0.505076    )
		))
"
  (if (matrixp v2)
      (funcall
       test
       (mapcar #'(lambda (v) (colinear-p v1 v :eps eps)) (column-list v2)))
    (let ((pnorms (* (norm v1) (norm v2))))
      (if (/= 0 pnorms) 
	  (< (abs (- (abs (dot v1 v2)) pnorms)) eps))
      )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 projection
;;
(defun projection(v B)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          projection Doc:
;;
;; Author: Andy Long
;; Created: Fri Feb 6 19:27:00 1998
;;
;; Description/Arguments:

       (projection v B)
  
	v - a vector
	B - a matrix or list of vectors of the same dimension as v.

;; Returns: a vector, the projection of vector v onto the columns of B.
   (This is really just (matmult v B)!)

;; Examples:
 (projection #(1 2) (id 2))
 (projection #(1 2) (column-list (id 2)))
"
  (matmult v (if (matrixp B) B (apply #'bind-columns B)))
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 orthogonal-complement
;;
(defun orthogonal-complement(v B)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          orthogonal-complement Doc:
;;
;; Author: Andy Long
;; Created: Fri Feb 6 19:30:01 1998
;;
;; Description/Arguments:

       (orthogonal-complement v B)
  
	v - a vector
	B - a list of same-sized vectors, or a matrix with the vectors in the
	    columns.

	The orthogonal complement of a vector v with respect to a set B of
  vectors is that part of v not in the span of the others. We can consider v as
  the sum of two vectors, one in the span and the other orthogonal to the span.
  It is the latter which is the orthogonal complement of v to B.

;; Returns: the component of v orthogonal to the column space of B, i.e. BB'v.

;; See Also:

;; Examples:
 (orthogonal-complement #(1 2) (list #(1 0) #(0 1)))
 (orthogonal-complement #(1 2) (id 2))
 ;; the second component of vector #(1 2) has no effect on the image v'B:
 (orthogonal-complement #(1 2) #2a((1 0)(0 0)))
 (orthogonal-complement #(1 3) #2a((1 0)(0 0)))

 ;; also works for rectangular matrices, or incomplete sets:
 (orthogonal-complement #(1 3) (list #(1 0)))
 (orthogonal-complement #(1 3 0) (list #(1 0 0) #(0 0 1)))
"
  (if (listp B) (setq B (apply #'bind-columns B)))
  (- v (matmult B (matmult v B)))
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				zero-vector-p
;;
(defun zero-vector-p(v &key (eps 1e-12) (test #'<))
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			zero-vector-p Doc:
;;
;; Author: Andy Long
;; Created: Sat Feb 7 10:01:38 1998
;;
;; Description/Arguments:

	(zero-vector-p v &key (eps 1e-12) (test #'<))
  
;; Arguments: a vector v, and optionally eps (a vector is declared a zero
  vector if its norm is less than eps). To 

;; Returns: T or NIL

;; Examples:
 (zero-vector-p #(0 0))
 (zero-vector-p #(1 0))
 (zero-vector-p #(0 1e-6))
 (zero-vector-p #(0 1e-6) :eps 1e-3)
 ;; by altering test to #'>=, for example, we can ask whether the norm of p is
 ;; bigger than eps:
 (zero-vector-p #(0 1) :test #'>=)
 ;; which should be the same as
 (not (zero-vector-p #(0 1)))
"
  (funcall test (norm v) eps)
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 complete-A-from-subspace-B
;;
(defun complete-A-from-subspace-B(A B &key (eps 1e-12))
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          complete-A-from-subspace-B Doc:
;;
;; Author: Andy Long
;; Created: Fri Feb 6 19:19:08 1998
;;
;; Description/Arguments:

       (complete-A-from-subspace-B A B &key (eps 1e-12))
  
where

	A	- is a matrix or list of vectors
	B	- is an orthogonal set of vectors (either as a list or a
		  matrix) from the span of A.
optionally
 	eps	- is used to determine whether a vector is a zero vector
		  or not.

;; Returns: a basis for A using the vectors from B (assuming that they're in
   the span of A!).

;; Notes: written to help solve the generalized-eigenvector problem.

;; See Also: 

;; Examples:
 (complete-A-from-subspace-B #2a((2 0)(0 3)) #2a((1 0)(0 1)))
 (complete-A-from-subspace-B #2a((1 0)(0 0)) #2a((0 0)(1 0)))
 (complete-A-from-subspace-B 
  #2a(
     (-0.816497     -1.000072E-16)
     (-0.408248     -0.707107    )
     ( 0.408248     -0.707107    )
     )
  #2a(
     ( -8.748178E-2 -0.252538    )
     (-0.262445     -0.757614    )
     (-0.174964     -0.505076    )
     )
  :eps 1e-4
  )
"
  ;; turn them into matrices if given as lists:
  (if (matrixp A) (setq A (column-lists A)))
  (if (matrixp B) (setq B (column-lists B)))
  (let* (
	 ;; We start by checking that B is in the span of A:
	 (check 
	  (mapcar
	   #'(lambda(x) (vector-in-span-of-vectors-p x A :eps eps)) B))
	 ;; We then eliminate those which are not in the span:
	 (B (select B (which check)))
	 ;; We also eliminate any zero vectors from B:
	 (B (select
	     B
	     (which
	      (mapcar
	       #'not
	       (mapcar
		#'(lambda(x) (zero-vector-p x :eps eps))
		B)))))
	 ;; These we reduce to a basis (taking out colinear vectors, etc.):
	 (B (if B (span-vectors B :eps eps)))
	 )
    (dolist
     (a A)
     (cond
      ;; if B is nil, then we'll just turn A into a basis, starting with the
      ;; first non-zero vector::
      ((not B)
       (if (not (zero-vector-p a :eps eps))
	   (setq B (list (normalize a)))))
      ;; if a is the zero vector, or colinear with any other vector, or in the
      ;; span of B, we do nothing:
      ((or (zero-vector-p a :eps eps) 
	   (colinear-p a (apply #'bind-columns B) :eps eps :test #'v-or)
	   (vector-in-span-of-vectors-p a B :eps eps)
	   )
       nil
       )
      ;; in the event that a is orthogonal, we add it to the B vectors:
      ((orthogonal-p a (apply #'bind-columns B) :eps eps)
       (setq B (append B (list a)))
       )
      (t
       ;; otherwise we do gram-schmidt, introducing the orthogonal complement:
       (setq
	B
	(append B (list (normalize (orthogonal-complement a B)))))
       )
      )
     )
    B
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				mixed-product
;;
(defun mixed-product(a b c)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			mixed-product Doc:
;;
;; Author: Andy Long
;; Created: Wed Feb 25 11:19:23 1998
;;
;; Description/Arguments: mixed-product
  
;; Arguments: a b c

;; Returns: Volume of the parallelpiped formed of the three vectors a, b, and c.

;; Notes: Thanks to Oussou, student at IMSP, for bringing its absence to light.

;; See Also: outer-product, inner-product, cross

;; Examples:
 (mixed-product #(1 0 0) #(0 1 0) #(0 0 1))
 (mixed-product #(1 0 0) #(0 3 0) #(0 0 2))
 (mixed-product #(-1 0 0) #(0 3 0) #(0 0 2))
 (mixed-product #(-1 0 0) #(3 0 0) #(0 0 2))
"
  (abs (dot (cross a b) c))
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				rowspace
;;
(defun rowspace(A)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			rowspace Doc:
;;
;; Author: Andy Long
;; Created: Sat Apr 18 09:03:02 1998
;;
;; Description/Arguments: rowspace

	Returns a basis for the rowspace of A, or nil if there is none. The
 rowspace is the space generated by the rows of A.

;; Arguments: A, a matrix.

;; Returns: A basis for the rowspace of A, or nil if there is none.

;; Notes: 

;; See Also: 

;; Examples:
 (rowspace #2a((1 0)(0 0)))
 (rowspace #2a((0 0)(0 0)))
 (rowspace (id 2))
 (rowspace #2a((1 1 1 0)(0 1 1 1)))
 (rowspace (transpose #2a((1 1 1 0)(0 1 1 1))))
"
  (if (= (sum (square (combine A))) 0)
      nil
    (span-vectors (transpose A)))
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				nullspace
;;
(defun nullspace(A)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			nullspace Doc:
;;
;; Author: Andy Long
;; Created: Sat Apr 18 09:03:02 1998
;;
;; Description/Arguments: nullspace

	Returns a basis for the nullspace of A. The nullspace is the space
 generated by all vectors having the property that Ax=0.

;; Arguments: A, a matrix.

;; Returns: A basis for the nullspace of A, or nil if there is none.

;; Notes: 

;; See Also: 

;; Examples:
 (nullspace #2a((1 0)(0 0)))
 (nullspace #2a((0 0)(0 0)(0 0)))
 (nullspace (id 2))
 (nullspace #2a((1 1 1 0)(0 1 1 1)))
 (nullspace #2a((1 1 1 0)))
 (nullspace #2a((1)(1)(1)(0)))

 (defun test-nullspace(A)
   (mapcar (lambda(x) (matmult A x)) (print (nullspace A))))
 ;; the result should be a zero vector:
 (test-nullspace (matrix '(3 4) (rand.u 12)))
"
  (let* (
	 (M (span-vectors (transpose A)))
	 (M (if M (apply #'bind-columns M)))
	 )
    (if M
	(span-vectors
	 (apply
	  #'bind-columns
	  (mapcar (lambda(x) (orthogonal-complement x M))
		  (column-list (id (first (size M)))))
	  )
	 )
      ;; otherwise the matrix is a zero matrix:
      (column-lists (id (second (size A))))
      )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
