#|

				Matrix.lsp

Functions and macros included here:

		size			 ;; matrix dimensions
		row			 ;; select a row of a matrix
		column			 ;; select a column of a matrix
                select-columns		 ;; select a sub-matrix of a list of
					 ;; columns or a matrix
                select-rows		 ;; select a sub-matrix of a matrix
					 ;; or list of rows 
                select-columns-by	 ;; select a sub-matrix of a list of
					 ;; rows or a matrix, by ....
                select-rows-by		 ;; select a sub-matrix of a list of
					 ;; rows or a matrix, by ....
                average-columns		 ;; average columns in a matrix
					 ;; (matrix returned has one fewer
					 ;; columns)
                average-rows		 ;; average rows in a matrix
					 ;; (matrix returned has one fewer
					 ;; rows)
                submatrix		 ;; a submatrix of a matrix
		change-matrix-value	 ;; changing elements of matrices,
					 ;; lists or arrays.
		setm			 ;; setting elements of matrices
		setl			 ;; setting elements of lists
                comp			 ;; components of a vector
                set-row			 ;; set a row to a new value
                set-column		 ;; set a column to a new value

		id			 ;; the identity matrix
		zeros			 ;; the zero matrix
		ones			 ;; the ones matrix

		svd			 ;; the svd
		polar-decomp		 ;; the polar-decomposition of a matrix
		pseudo-inverse		 ;; the pseudo-inverse of a matrix
					 ;; (obtained using the svd)
		pca			 ;; principal components analysis
		pca-factors		 ;; prints factors of principal
					 ;; components
		low-rank		 ;; low rank approx using the svd

		correlation		 ;; the correlation matrix
		matrix-row-stats	 ;; row statistics
		matrix-column-stats	 ;; column statistics

                sort-matrix		 ;; sort a matrix by a column
                mat			 ;; macro to define a matrix
                table			 ;; defines a matrix from a table
                mat-solve		 ;; macro to solve Ax=b (like solve)

                dot			 ;; dot-product of two vectors
                norm			 ;; norm of a vector
                cross			 ;; cross-product of two vectors

                combine-matrices	 ;; combines matrices with the same
                                            number of columns
		mprint			 ;; used to be an alias for
					 ;; print-matrix, but now more general

                flesh			 ;; densifies a matrix
                flesh2			 ;; densifies a matrix
                matroid			 ;; plots x,y,z matrices for a matroid
                matroid-to-matrix	 ;; given a matroid, creates the
					 ;; matrices needed by function matroid
                matrix-to-matroid	 ;; given a matrix, creates a matroid
		plot-matrix-columns	 ;; plots the columns of a matrix

                matrix-multiply		 ;; multiplies two matrices

                remove-missing		 ;; removes missing values from a
					 ;; matrix, by eliminating rows.

		mat-fill		 ;; fill in a matrix with repeated
					 ;; measures

		upper-to-full		 ;; creates a full matrix from an
					 ;; upper-triangular one.
		lower-to-full		 ;; creates a full matrix from an
					 ;; lower-triangular one.
		diagonal-to-matrix	 ;; Creates a diagonal matrix
		replace-diagonal	 ;; replaces a diagonal
		extract-upper-triangular ;; extracts the lower triangular
					 ;; matrix from a full one
		extract-lower-triangular ;; extracts the lower triangular
					 ;; matrix from a full one

		make-grid		 ;; makes a rectangular grid

		inside-nth		 ;; nth elements of the lists inside a
					 ;; matrix (or list, or vector).

;; The following routines were developed to see if I could get to the SVD on my
;; own, using only matrix multiplication. Yep!
		robust-svd		 ;; My own SVD routine, created more
					 ;; for my enjoyment than for use.

		Householder-triangular-form
		Householder-qr-decomp

		matrix-equalp		 ;; check equivalence of matrices:

		permutation-matrix	 ;; creates a permutation matrix

		backsolve		 ;; updated the function defined in
					 ;; stats.lsp 
		rotate-clock-wise-matrix ;; rotates a matrix about its center
					 ;; by 90 degrees.

		symmetric-matrix-p	 ;; checks for symmetry
		square-matrix-p		 ;; checks for square

		but-rows		 ;; selects all but rows
		but-cols		 ;; selects all but columns

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	The idea is to introduce some functions which are more natural than
those provided. In many cases, this reflects my prior experience: for example,
I want a "size" routine (which comes from MATLAB).

	Also, I define operators "row" and "column" which give me info about
those parts of a matrix (returned as a vector or as a list).

	Another important and useful operation is setting an element to a
certain thing: this I called "setm".

	Standard initialization matrices are included (with simpler names, e.g.
"id" (for identity-matrix), "zeros", "ones", etc.).

	Some statistical and other analysis tools creep in: starting with the
svd (svd) and the correlation matrix (correlation). And some summary
statistical operations, "matrix-row-stats" and "matrix-column-stats".

|#
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                size
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun size (&rest x)
  "
;; Description: function size (&rest x)

;; Args: object(s) such as vectors, lists or arrays. Calculates the array
;; dimensions. Vectorized.

;; Examples:
 (size #2a((1 2 3)(4 5 6)))
 (size #(1 2 3))
 (size (iseq 4))

 ;; size is vectorized:
 (size #(1 2 3) (id 3) (list 1 2))
"
  (if (= (length x) 1)
      (let ((obj (first x)))
	(if (or (listp obj) (vectorp obj))
	    (length obj)
	  (array-dimensions obj)
	  )
	)
    (mapcar #'size x)
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                column
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun column (columnnum amatrix &key vector)
  "
Description: (column nth matrix &key vector)

	Returns the nth column of a matrix as a list. Use the :vector option if
you want it returned as a vector.

;; Notes: Rows and columns are indexed as in fortran, from 1!

;; Examples:
 (column 2 (id 4))
 (column 2 (matrix '(3 2) (normal-rand 6)) :vector 't)
"
  (let (
	(rows (first (size amatrix)))
	(col (1- columnnum))
	)
    (if vector
	(coerce (mapcar #'(lambda(x) (aref amatrix x col)) (iseq rows)) 'vector)
      (mapcar #'(lambda(x) (aref amatrix x col)) (iseq rows))
      )
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                row
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun row (rownum amatrix &key vector)
  "
Description: (row nth matrix)

	Returns the nth row of a matrix as a list. Use the :vector option if
you want it returned as a vector.

;; Note: Rows and columns are indexed as in fortran, from 1!

;; Examples:
 (row 2 (id 4))
 (row 2 (matrix '(3 2) (normal-rand 6)) :vector 't)
"
  (let (
	(cols (second (size amatrix)))
	(row (1- rownum))
	)
    (if vector
	(coerce (mapcar #'(lambda(x) (aref amatrix row x)) (iseq cols)) 'vector)
      (mapcar #'(lambda(x) (aref amatrix row x)) (iseq cols))
      )
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                col
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun col (colnum amatrix &key vector)
  "
Description: (col nth matrix &key vector)

	Returns the nth col of a matrix as a list. Use the :vector option if
you want it returned as a vector.

;; Note: Cols and columns are indexed as in fortran, from 1!

;; Examples:
 (col 2 (id 4))
 (col 2 (matrix '(3 2) (normal-rand 6)) :vector 't)
"
  (row colnum (transpose amatrix) :vector vector)
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                setm
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmacro setm (amatrix elist value)
  "
Description: (setm amatrix elist value)

      Setm resets elements of vectors or matrices.

For example,
			(setm a(1 2) 666)
sets the (1,2) element of the matrix a to value 666.

Note: setm uses fortran indexing of matrices (i.e., starts with (1,1) (first row (1) and first column (1)).

Returns the value.

;; Examples:

 (print-matrix (setq a (matrix '(5 2) (normal-rand 10))))
 (setm a (1 2) 666)
 (print-matrix a)

 (setq ve (make-array 2 :initial-contents '(5 2)))
 (setm ve (1) 666)
 ve
"
  (if (eval `(vectorp ,amatrix))
      (let* (
	     (ele (coerce elist 'list))
	     (ele1 (1- (first ele)))
	     )
	`(setf (aref ,amatrix ,ele1) ,value)
	)
    (let* (
	   (ele (coerce elist 'list))
	   (ele1 (1- (first ele)))
	   (ele2 (1- (second ele)))
	   )
      `(setf (aref ,amatrix ,ele1 ,ele2) ,value)
      )
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                alter
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun alter(x new old)
  "
;; Description: alter(x new old)

Replaces a nsubst-if which misbehaves...

;; Examples:
 (alter '(1 2 4) 3 4)
 (alter '(\"hi\" 2 4) \"ho\" 4)
 (alter '(\"hi\" 2 4) 4 \"hi\")
"
  (mapcar (lambda(x) (if (equalp x old) new x)) x)
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                change-matrix-value
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun change-matrix-value (amatrix oldvalue value &key careful)
  "
Description: (change-matrix-value amatrix oldvalue value &key careful)

      Change-Matrix-Value resets elements of lists, vectors or matrices.

For example,

			(change-matrix-value a 1 666)

sets all elements equal to 1 of the matrix a to value 666. Returns the matrix.

	The careful option allows for comparisons of elements of a matrix that
are lists, matrices, etc. Otherwise we simply use combine to get the elements
of the list.

;; Examples:
 (change-matrix-value (id 3) 1 2)
 (change-matrix-value #2a((1 2) (2 3) (3 4) (5 1)) 1 2)
 (change-matrix-value (vector 1 2 33) 1 2)
 (change-matrix-value (list 1 2 33) 1 2)

 ;; If we don't set careful to true, we get into trouble:
 (change-matrix-value (matrix '(2 2) (list 3 '(1 2) 2 33)) '(1 2) 2 :careful t)
"
  (if (matrixp amatrix)
      (matrix (size amatrix)
	      (alter
	       (if careful (matrix-to-list amatrix) (combine amatrix)) 
	       value oldvalue))
    (if (vectorp amatrix)
	(coerce
	 (alter (coerce amatrix 'list) value oldvalue)
	 'vector)
      (alter amatrix value oldvalue)
      )
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                setl
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun setl (alist index value)
  "
Description: (setl alist indices values)

	Resets portions of a list to new values.

Notes:

1. Uses standard lisp ordering from 0

2. Based on:
	(setf (select mylist indices) values)

3. Returns the new list.

;; Examples:

 (setq a '(5 2 3))
 (setl a 2 666)
 (setl a '(0 1) '(2 1))
"
  (setf (select alist index) value)
  alist
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Description for ZEROS:
;;
(defun zeros (nrow &optional ncol)
  "
Documentation:

     ZEROS (nrow &optional ncol)

Zero matrix of dimensions nrow by ncol.

;; Examples:
 (zeros 3 4)
 (zeros 3)
" 
  (if ncol
      (make-array (list nrow ncol) :initial-element 0)
    (coerce (repeat 0 nrow) 'vector)
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Description for ONES:
;;
(defun ones (nrow &optional ncol)
  "
;; Documentation: ONES (nrow &optional ncol)

	\"Ones\" matrix (all 1 entries) of dimensions nrow by ncol. If only one
argument is provided, then a vector is returned.

;; Examples:
 (ones 3 4)
" 
  (if ncol
      (make-array (list nrow ncol) :initial-element 1)
    (coerce (repeat 1 nrow) 'vector)
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Description for ID:
;;
(defun id (n)
  "
;; Documentation: ID (n)

Identity matrix of dimensions n by n.

 (id 3)
" 
  (identity-matrix n)
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Description for SVD:
;;
(defun svd (a)
  "
;; Description: function svd

     performs the Singular Value Decomposition of a matrix. The native svd code
 (sv-decomp) requires that the number of rows be >= the number of columns. This
 being unnecessary, svd handles either case.

			A = U L V'

 where U and V are orthogonal matrices, and L is a 'diagonal' matrix containing
 the singular values on the diagonal (largest to smallest). For an mxn matrix,
 the U returned is m by (min m,n), L  is (min m,n) by (min m,n), and V is
 m by (min m,n). 

     It also assures that the singular values are returned as real (if a
 complex matrix is given, they are returned as complex with zero imaginary part
 by sv-decomp).

     Returns a list of the matrix U, a list of the singular values, the matrix
 V, and a flag indicating either that the algorithm converged (T), or NIL
 otherwise.

;; See Also: sv-decomp, pseudo-inverse

;; Examples:
 (def a (matrix '(3 2) (list 1 -1 -1 2 1 2)))
 (def svd-a (svd a))

 ;; as promised works on matrices where the number of rows is less than the
 ;; number of columns:
 (def a-transpose (transpose a))
 (def svd-a-transpose (svd a-transpose))

 ;; Here's how to put the matrix a back together again:
 (def u (first svd-a))
 (def lambda (second svd-a))
 (def v (third svd-a))
 (mprint (matmult u (diagonal-to-full lambda) (transpose v))) ;; = a

 ;; works for complex matrices, too:
 (def a #2a((2 #c(0 1))(#c(0 1) 2)))
 (def svd-a (svd a))
 (def u (first svd-a))
 (def lambda (second svd-a))
 (def v (third svd-a))

 ;; Notice that to put a complex matrix back together you need to conjugate the
 ;; (possibly complex) matrix v :
 (mprint (matmult u (diagonal-to-full lambda) (transpose (conjugate v))))
" 
  (if (matrixp a)
      (let* (    ; true, so get the matrix dimensions:
	     (size (size a))
	     (swap (< (first size) (second size)))
	     ;; if swap is true, then transpose:
	     (svd (if swap (sv-decomp (transpose a))
		    (sv-decomp a)))
	     (u (if swap (third svd) (first svd)))
	     ;; make sure that the lambda are real numbers, and return them as
	     ;; a list rather than as a vector:
	     (lambda (mapcar #'realpart (coerce (second svd) 'list)))
	     (v (if swap (first svd) (third svd)))
	     (converged (fourth svd))
	     )
	(list u lambda v converged)
	)
    )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                pca
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun pca (matrix)
  "
;; Description: (pca matrix)

	Carries out pca on a matrix: in reality, this is just the svd of a
matrix after it has been properly scaled.

;; Examples:
 (def test-matrix (matrix '(9 3) (rand.n 27)))
 (def pca-results (pca test-matrix))

 (second pca-results)
 (sum (^ (second pca-results) 2))

 (title \"the singular values\")
 (plot (second pca-results))
 (title \"the singular vectors (samples)\")
 (plot (first pca-results)) ;; the singular vectors ('factors')
 (title \"the singular vectors (factors)\")
 (plot (third pca-results)) ;; the singular vectors ('factors')
"
  (svd
   (apply
    #'bind-columns
    (mapcar
     #'(lambda(x)
	 (/ (standardize (coerce x 'list))
	    (sqrt (1- (first (size matrix))))))
     (column-list matrix)
     )
    )
   )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                pca-factors
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun pca-factors (pca)
  "
;; Description: prints variance and factors of principal components analysis.

   Columns represent the factors. It is common to 'normalize' the factors so
that the first entry in each vector is positive. I consider this a waste of
time, and dangerous, having heard people remark on the strange (and doubtless
important) fact that variable one has positive weights on all components. Ael.

;; Examples:
 (pca-factors (pca (matrix '(9 3) (rand.u 27))))
"
  (let* (
	 (lamsquare (square (second pca)))
	 (lamsquare (matrix (list 1 (length lamsquare)) lamsquare))
	)		   
    (princ "Squares of singular values (\"eigenvalues\"):")(terpri)
    (print-matrix lamsquare)
    (princ "Relative weights:")(terpri)
    (print-matrix (/ lamsquare (sum lamsquare)))
    (princ "Factors (columns are weights on variables):")(terpri)
    (print-matrix (third pca))
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; pca-analysis Doc:    Creation Date:5/27/96        Author: Andy Long
;;
;;
(defun pca-analysis (matrix &optional (prefix "pca"))
  "
;; Description/Arguments:

       (pca-analysis matrix &optional (prefix \"pca\"))
  
;; Notes: creates variables pca.sv, pca.factors, and pca.scores (by default,
;; although another prefix can be used). 

;; See Also:

;; Examples:
 (def data (matrix '(9 3) (rand.u 27)))
 (pca-analysis data \"data\")
 data.sv
 (pca-analysis data)
 pca.sv
"
  (let ((pca (pca matrix)))
    (genvars
     (strcat prefix ".sv") (second pca)
     (strcat prefix ".factors") (third pca)
     (strcat prefix ".scores") (first pca)
     (strcat prefix ".inverted") (fourth pca)
     )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; standardize Doc:    Creation Date:5/22/96        Author: Andy Long
;;
;;
(defun standardize(a)
  "
;; Description/Arguments:

       (standardize a)
  
Centers list a and makes standard deviation 1

;; Examples:
 (def a (standardize (rand.u 10)))
 (mean a)
 (sd a)
"
  (/ (- a (mean a)) (sd a))
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 replace-diagonal
;;
(defun replace-diagonal(a b)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          replace-diagonal Doc:
;;
;; Author: Andy Long
;; Created: Sat Nov 22 08:40:52 1997
;;
;; Description/Arguments:

       (replace-diagonal matrix new-diagonal)
  
;; Notes:

;; See Also:

;; Examples:
 (mprint (replace-diagonal (id 3) (list 1 2 3)))
"
  (let* ((xlen (length b)))
    (dotimes (i xlen) (setf (aref a i i) (elt b i)))
    a
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                diagonal-to-matrix
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun diagonal-to-matrix (x)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; diagonal-to-matrix Doc:    Creation Date:5/22/96        Author: Andy Long
;;
;; Description/Arguments:

       (diagonal-to-matrix x)
  
;; Examples:
 (diagonal-to-matrix (diagonal (id 3)))
"
  (let* (
	 (xlen (length x))
	 (tmp (zeromat xlen xlen))
	 )
    (dotimes (i xlen) (setf (aref tmp i i) (elt x i)))
    tmp
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 extract-upper-triangular
;;
(defun extract-upper-triangular(a)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          extract-upper-triangular Doc:
;;
;; Author: Andy Long
;; Created: Sat Nov 22 08:10:48 1997
;;
;; Description/Arguments:

       (extract-upper-triangular matrix)
  
;; Notes:

;; See Also:

;; Examples:
 (mprint (extract-upper-triangular #2a((1 2) (3 4))))
 (mprint (extract-upper-triangular 
          #2a((2 1 0 0) (1 2 1 0) (0 1 2 1) (0 0 1 2))))
 (mprint (extract-upper-triangular 
          #2a((0 0 0 0)(0 1 0 0)(0 0 1 0)(0 0 0 1))))
"
  (let* (
	 (xlen (first (size a)))
	 (1-xlen (1- xlen))
	 (tmp (zeromat xlen xlen))
	 )
    (dotimes (i xlen)
	     (setf (select tmp i (iseq i 1-xlen)) (select A i (iseq i 1-xlen))))
    tmp
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 extract-lower-triangular
;;
(defun extract-lower-triangular(a)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          extract-lower-triangular Doc:
;;
;; Author: Andy Long
;; Created: Sat Nov 22 08:10:48 1997
;;
;; Description/Arguments:

       (extract-lower-triangular matrix)
  
;; Notes:

;; See Also:

;; Examples:
 (mprint (extract-lower-triangular #2a((1 2) (3 4))))
 (mprint (extract-lower-triangular 
          #2a((2 1 0 0) (1 2 1 0) (0 1 2 1) (0 0 1 2))))
"
  (let* (
	 (xlen (first (size a)))
	 (tmp (zeromat xlen xlen))
	 )
    (dotimes (i xlen)
	     (setf (select tmp i (iseq 0 i)) (select A i (iseq 0 i))))
    tmp
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Description for LOW-RANK:
;;
(defun low-rank(a nkeep &optional use-svd)
  "
;; Documentation:

;;       LOW-RANK(a-matrix nkeep &optional use-svd)

;; LOW-RANK returns the best low-rank approximation of a given
;; rank nkeep. If svd is true, then the svd will be used
;; instead of principal components.

;; Examples:

;; flesh-mat is 6 by 3:
 (def flesh
     (matrix
      '(6 3)
      (list
       1 -1 -1 2 1 2
       1 -2 1 5 3 0
       -2 -3 -2 -1 -1 -3
       )
      )
     )

;; plot singular values:
 (def s (second (svd flesh-mat))) 
 (title \"singular values\")
 (plot s)

 (title \"Absolute Error of approximation\")
 (plot
  (mapcar 
   (lambda(i) (max (abs (- (low-rank flesh-mat i t) flesh-mat))))
   (iseq 1 3)))

 (format nil \"The difference (~e) is pretty small!\" 
	 (max (abs (- (low-rank flesh-mat 3 t) flesh-mat))))
" 
  (let* (
	 (dims (size a))
	 (recomp (zeromat (first dims) (second dims)))
	 (decomp (if use-svd (svd a) (pca a)))
	 (ucols (column-list (first decomp)))
	 (s (coerce (second decomp) 'list))
	 (vcols (column-list (third decomp)))
	 )
    (dotimes
     (i (min (length s) nkeep))
     (setq recomp
	   (+ recomp
	      (outer-product (* (nth i s) (nth i ucols)) (nth i vcols)))
	   )
     )
    (if use-svd
	recomp
      (apply
       #'bind-columns
       (+
	(mapcar #'mean (column-list a))
	(mapcar
	 #'(lambda(x) (coerce x 'list))
	 (column-list
	  (*
	   (sqrt (1- (first dims)))
	   (matmult
	    recomp
	    (diagonal-to-matrix (mapcar #'sd (column-list a)))
	    )
	   )
	  )
	 )
	)
       )
      )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                correlation
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun correlation (&rest args)
  "Args: 
 (args &keys
    latex caption names (float-digits 4) (stream *standard-output*) (print t))

Returns the sample correlation matrix of the data columns in ARGS.
ARGS may consist of lists, vectors or matrices.

;; Examples:

 (correlation (matrix '(4 4) (uniform-rand 16)))
 (correlation (id 4))
 (correlation (id 4) :latex t)
 (correlation (id 4) :latex t :caption \"New Year's Day\")
"
  (let* (
	 (latex (grep '(:latex) args))
	 (caption (first (catch-keys '(:caption) args)))
	 (names (first (catch-keys '(:names) args)))
	 (stream (if (grep '(:stream) args)
		     (first (catch-keys '(:stream) args))
		   *standard-output*))
	 (float-digits (if (grep '(:float-digits) args)
			   (first (catch-keys '(:float-digits) args)) 4))
	 (print (if (grep '(:print) args)
		    (first (catch-keys '(:print) args))
		  t))
	 (args (remove-keys
		'(:latex :caption :names :float-digits :stream :print)
		args))
	 (len-vec (if (matrixp (first args)) (first (size (first args)))
		    (length (first args))))
	 (mat 
	  (xtx
	   (apply #'bind-columns
		  (mapcar #'standardize
			  (column-list
			   (apply #'bind-columns args)
			   )
			  )
		  )
	   )
	  )
	 (mat (/ mat (1- len-vec)))
	 (names (if names names
		  (mapcar
		   (lambda(x) (format nil "var~d" x))
		   (iseq (second (size mat))))))
	 )
    (if latex (latex-table
	       (bind-rows
		(combine "" names)
		(bind-columns
		 names
		 mat
		 ))
	       :caption caption)
      (if print
	  (let ((size 0))
	    (map-elements
	     #'(lambda (x)
		 (setf size
		       (max size
			    (cond
			     ((floatp x) (+ 4 float-digits))
			     ((integerp x) (+ (flatsize x) 4))
			     (t (flatsize x))))))
	     mat)
	    (format stream "~%")
	    (dolist
	     (x (row-list
		 (bind-columns (combine "" names) (bind-rows names mat))))
	     (let ((n (length x)))
	       (dotimes (i n)
			(let ((y (aref x i)))
			  (cond
			   ((floatp y)
			    (format stream "~v,vf " size float-digits y))
			   (t (format stream "~va " size y))))
			))
	     (format stream "~%")
	     )
	    (format stream "~%")
	    )
	)
      )
    mat
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                matrix-row-stats
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun matrix-row-stats (a-matrix &key names latex caption)
  "
;; Description: matrix-row-stats (a-matrix &key names latex caption)

;; Examples:
 (matrix-row-stats (matrix '(4 6) (normal-rand 24)))
 (matrix-row-stats (matrix '(4 6) (normal-rand 24))
    :names '(\"hi\" \"ho\" \"hee\" \"hawwwwww\"))
 (matrix-row-stats (matrix '(4 6) (normal-rand 24))
    :latex \"caption\" :names '(\"hi\" \"ho\" \"hee\" \"hawwwwww\"))
"
  (let* (
	 (a-size (size a-matrix))
	 (rowlist (row-list a-matrix))
	 )
    (if latex
	(let ((result '(variable minimum mean maximum standard-deviation)))
	  (dotimes (i (first a-size))
		   (setq newlist-element (coerce (select rowlist i) 'list))
		   (setq
		    result 
		    (combine
		     result
		     (list
		      (if names
			  (elt names i)
			(num-to-string (1+ i))
			)
		      (min newlist-element)
		      (mean newlist-element)
		      (max newlist-element)
		      (standard-deviation newlist-element)
		      )
		     )
		    )
		   )
	  (latex-table
	   (matrix (list (1+ (first a-size)) 5) result)
	   :caption (if (stringp latex) latex "")
	   )
	  )
      (let ()
	  (format t " ~6,,2,'.@a ~12,,2,'.@a ~16,,2,'.@a ~16,,2,'.@a ~16,,2,'.@a ~%"
		  "var" "min" "mean" "max" "std")
	  (dotimes (i (first a-size))
		   (setq newlist-element (coerce (select rowlist i) 'list))
		   (format t " ~6,,0,'.@a ~16,6,g ~16,6,g ~16,6,g ~16,6,g ~%"
			   (if names
			       (select (elt names i)
				       (iseq (min (length (elt names i)) 6)))
			     (num-to-string (1+ i))
			     )
			   (min newlist-element)
			   (mean newlist-element)
			   (max newlist-element)
			   (standard-deviation newlist-element)
			   )
		   )
	  )
      )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                matrix-column-stats
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun matrix-column-stats (a-matrix &key names latex caption)
  "
;; Description: matrix-column-stats (a-matrix &key names latex caption)

;; Examples:
 (matrix-column-stats (matrix '(4 6) (normal-rand 24)))
 (matrix-column-stats (matrix '(4 6) (normal-rand 24))
    :names (list \"Fee\" \"Fi\" \"Fo\" \"Fum,\" \"I\" \"Smell.\")
    )
 (matrix-column-stats (matrix '(4 6) (normal-rand 24))
    :latex \"Silly\"
    :names (list \"Fee\" \"Fi\" \"Fo\" \"Fum,\" \"I\" \"Smell.\")
    )
"
  (if names
      (matrix-row-stats (transpose a-matrix) :names names
			:latex (if caption caption latex))
    (matrix-row-stats (transpose a-matrix)
		      :latex (if caption caption latex))
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; table Doc:    Creation Date:1/17/95        Author: Andy Long
;;
;;
(defmacro table (dataname i1 i2 &rest data)
  "
;; Description/Arguments:

        (table dataname i1 i2 &rest data)
  
Table creates a table (really a matrix) out of your data: it should have i1 rows and i2 columns, and the data values are given following.

;; Note: plot associates the x-axis with the rows, and the y-axis with the
;; columns.

;; Examples:

 (table beef 5 4 
       2.65 2.59 2.51 2.43
       4.14 4.05 3.94 3.88
       5.11 5.00 4.97 4.84
       5.35 5.29 5.19 5.07
       5.79 5.77 5.60 5.53
       )
 (plot beef)

 (table mothers 5 5
       2  4 4  2 1
       0 12 8  2 1
       1  7 12 4 3
       0  8 12 6 2
       0  1 3  4 1
       )
 (plot mothers)
"
  `(def ,dataname (matrix (list ,i1 ,i2) (list ,@data)))
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; dot Doc:    Creation Date:1/30/96        Author: Andy Long
;;
;;
(defun dot(v1 v2)
  "
;; Description/Arguments:

        (dot v1 v2)

Computes the dot product of two vectors v1 and v2
;; Notes:

;; See Also: inner-product

;; Examples:
 (def v1 (vector 1 2 3))
 (def v2 (vector 3 2 1))
 (dot v1 v2)

"
  (inner-product v1 v2)
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; norm Doc:    Creation Date:2/4/96        Author: Andy Long
;;
;;
(defun norm(v &key (p 2))
  "
;; Description/Arguments:

        (norm v &key (p 2))
  
where
	v	- is a number, vector/list, or matrix.
	&key
	(p 2)	- means the 2 norm, i.e. the sqrt of the square of the vector
		  This should be a positive real, or 'infinity for vectors; for
		  matrices, only 1, 2 and infinity are permitted..
		  
;; Returns - the non-negative norm.

;; See Also: abs, inner-product

;; Examples:
 (norm #(1 2 3))

 (norm #(.5 .5) :p 'infinity)
 (norm #(.5 .5) :p 10)
 (norm #(.5 .5)) ;;  :p 2
 (norm #(.5 .5) :p 1)

 (norm '(1 #c(0 1)))					;; 1.4142135623730951
 (norm #2a((3 0 0)(0 2 0)(0 0 1)))			;; 3

 (norm #2a((1 2)(1.0001 2)) :p 1)
 (norm #2a((1 2 -1)(0 3 -1)(5 -1 1)) :p 'infinity)
 ;; but the following will cause an error (as only 1,2, and infinity norms are
 ;; defined for matrices):
 ;; (norm #2a((1 2)(1.0001 2)) :p 10)
"
  (setq p (if (numberp p) (abs p) -666))
  (cond
   ((numberp v) (abs v))
   ((matrixp v) 
    (cond
     ((= p 2) (first (second (svd v))))
     ((= p 1) (max (mapcar #'sum (column-lists (abs v)))))
     ((= p -666) (max (mapcar #'sum (row-lists (abs v)))))
     (t
      (error
       "in function norm.
Sorry: norms other than the 1, 2, or infinity-norm are only available
for vectors.")
      )
     )
    )
   (t
    (cond
     ((= p 2) (realpart (sqrt (inner-product v v))))
     ((= p 1) (sum (abs v)))
     ((= p -666) (max (abs v)))
     (t (^ (sum (^ (abs v) p)) (/ p)))
     )
    )
   )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                normalize
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun normalize (v)
  "
;; Description: (normalize v)

;; Returns: the unit vector formed by dividing v by its norm (or signals an
   error, if v is the zero vector).

;; Examples:
 (normalize '(5 4))
 ;; causes an error:
 ;; (normalize '(0 0 0))
"
  (let ((l (norm v)))
    (if (= 0 l) 
	(error "in normalize: attempting to normalize the zero vector.")
      (/ v l))))
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; cross Doc:    Creation Date:1/30/96        Author: Andy Long
;;
;;
(defun cross(v1 v2)
  "
;; Description/Arguments:

        (cross v1 v2)

Computes the cross-product of two vectors v1 and v2
;; Notes:

;; See Also:

;; Examples:
 (def v1 (vector 1 2 3))
 (def v2 (vector 3 2 1))
 (cross v1 v2)

 (cross '(2 1 -2) '(3 0 1)) ; #(1 -8 -3)
 (cross '(2 1 -3) '(1 3 2)) ; #(11 -7 5)
"
  (let* (
	 (u (coerce v1 'list))
	 (v (coerce v2 'list))
	 (ux nil)
	 (uy nil)
	 (uz nil)
	 (vx nil)
	 (vy nil)
	 (vz nil)
	 )
    (cond
     ((and (/= 3 (length v1)) (/= 3 (length v2)))
      (print "Cross only works for 3-d vectors: sorry!"))
     (t
      (setq ux (first u))
      (setq uy (second u))
      (setq uz (third u))
      (setq vx (first v))
      (setq vy (second v))
      (setq vz (third v))
      (vector
       (- (* uy vz) (* uz vy))
       (- (* uz vx) (* ux vz))
       (- (* ux vy) (* uy vx))
       )
      )
     )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; comp Doc:    Creation Date:2/4/96        Author: Andy Long
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun comp(v n)
  "
;; Description/Arguments:

        (comp v n)

\"Comp\" stands for \"component\". Returns the nth component of a vector (indexed from 1)

;; Notes: uses select

;; See Also: select

;; Examples:
 (comp (vector 1 2.2 3) 2)
 (comp (vector 3 2 1) (iseq 1 2))
"
  (select v (1- n))
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 turn-it-into-a-matrix
;;
(defun turn-it-into-a-matrix(x &key (from 'turn-it-into-a-matrix))
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          turn-it-into-a-matrix Doc:
;;
;; Author: Andy Long
;; Created: Fri May 16 09:59:41 CDT 1997
;;
;; Description/Arguments:

       (turn-it-into-a-matrix x &key (from 'turn-it-into-a-matrix))
  
	Turns a list, a vector, a list of vectors, a vector of lists, a list of
lists, a vector of vectors, or a matrix into a matrix.

	from is a symbol of the function from which turn-it-into-a-matrix was
called, in case of a problem.

;; Notes:

;; See Also:

;; Examples:
 (turn-it-into-a-matrix (list (list 1 2) (list 2 3)))
 (turn-it-into-a-matrix (list #(1 2) #(2 3)))
 (turn-it-into-a-matrix #(#(1 2) #(2 3)))
 (turn-it-into-a-matrix #((1 2) (2 3)))
 (turn-it-into-a-matrix #2A((1 2) (2 3)))
 (def a #2A((1 2) (2 3)))
 (turn-it-into-a-matrix a)

 ;; you'll get an error with any of these:
 ;; (turn-it-into-a-matrix (plot a))
 ;; (turn-it-into-a-matrix \"hi\")
 ;; (turn-it-into-a-matrix 'ps)
"
  (cond
   ((matrixp x)
    x
    )
   ((listp x)
    (if (or (listp (first x)) (vectorp (first x)))
	(apply #'bind-columns x)
      (bind-columns x))
    )
   ;; This has to come before vector, or we'll get an element of type vector
   ;; from symbols and strings:
   ((or (stringp x) (symbolp x) (not (vectorp x)))
    (helpless from)
    (error (format nil "In function turn-it-into-a-matrix:
I can't turn an element of type ~s into a matrix in routine ~s.~%"
		   (type-of x) from)
	   )
    )
   ((vector x)
    (if (or (listp (aref x 0)) (vectorp (aref x 0)))
	(apply #'bind-columns (coerce x 'list))
      (bind-columns x))
    )
   )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; turn-it-into-a-list Doc:    Creation Date:3/24/96       Author: Andy Long
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmacro turn-it-into-a-list(x)
  "
 ;; Examples:
 (turn-it-into-a-list (2 2))
 (turn-it-into-a-list '(2 2))
 (setq a '(2 2))
 (turn-it-into-a-list a)
 (turn-it-into-a-list (rand.u 10))
 ;; but not
 ;; (mvsetq (a b) (1 2)) (turn-it-into-a-list (a b))
 ;; just type (list a b)!
  "
  (let ((bing `,x))
    (cond
     ((symbolp bing)
      (eval `',bing))
     ((consp bing)
      (cond
       ((numberp (first bing))
	`',bing
	)
       (t
	`,bing
	)
       )
      )
     (t `,bing)
     )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmacro mat(xin yin)
  "
;; description: (mat dimensions contents) 
	
        Mat is a macro designed to generalize the vanilla matrix function which
comes with xlisp-stat. It is tedious writing those quotes!

;; Examples:
 (mat '(2 2) '(3 2 1 2))
 (mat (2 2) (3 2 1 2))
 (setq a '(2 2))
 (setq b '(3 2 1 2))
 (mat a b)
 (mat (4 4) (rand.u 16))
 (mat (repeat 4 2) (rand.u 16))
 ;; but not
 ;; (def c 4) (def d 4) (mat (c d) (rand.u 16))
"
  (let (
	(x (eval `(turn-it-into-a-list ,xin)))
	(y (eval `(turn-it-into-a-list ,yin)))
	)
    `(matrix ',x ',y)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; mat-solve Doc:    Creation Date:5/1/96        Author: Andy Long
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmacro mat-solve(A b)
  "
;; Description/Arguments for MACRO mat-solve:

      (mat-solve A b)

;; Notes: Puts lu-solve together with lu-decomp, so we can skip a step!

;; See Also: lu-solve, lu-decomp, solve

;; Examples:
 (mat-solve (id 4) (1 2 3 4))
 (mat-solve (* 2 (id 4)) (1 2 3 4))
 (def a (id 4))
 (def b (vector 1 2 3 4))
 (mat-solve (* 2 a) b)

;; Compare with function solve, requiring quotes:
 (solve (id 4) '(1 2 3 4))
 (solve (* 2 (id 4)) '(1 2 3 4))
"
  (let (
	(aa (eval `(turn-it-into-a-list ,a)))
	(bb (eval `(turn-it-into-a-list ,b)))
	)
    `(solve ,aa ',bb)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                        select-rows, select-columns
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun select-columns (a col-list)
  "
;; Description: (select-columns matrix col-list)

Choose a sub-matrix of a given matrix by giving a list of columns (lisp
ordering, from 0).

;; Examples:
 (print-matrix (select-columns (id 6) (iseq 4)))
 (print-matrix (select-columns (id 6) '(1 2 3)))
"
  (apply #'bind-columns (select (column-list a) col-list)))

(defun select-rows (a row-list)
  "
;; Description: (select-rows matrix col-list)

Choose a sub-matrix of a given matrix by giving a list of rows (lisp
ordering, from 0).

;; Examples:
 (print-matrix (select-rows (id 6) (iseq 4)))
 (print-matrix (select-rows (id 6) '(1 2 3)))
 (print-matrix (select-rows (id 6) '(0)))
"
  (apply #'bind-rows (select (row-list a) row-list)))
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                  select-rows-by, select-columns-by
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun select-rows-by (matrix test-function)
  "
;; Description: (select-rows-by matrix test-function)

Choose a sub-matrix of a given matrix by giving a test for each row (treated as
a list).

;; Examples:
 (print-matrix (select-rows-by (id 6) #'(lambda(x) (= 1 (first x)))))
 (print-matrix (select-rows-by (id 6) #'(lambda(x) (= 1 (sum x)))))
 (select-rows-by (id 6) #'(lambda(x) (= 0 (sum x))))
 ;; (nil)
"
  (let (
	(stuff
	 (select-if
	  (mapcar #'(lambda(x) (coerce x 'list)) (row-list matrix))
	  test-function)
	 )
	)
    (if stuff (apply #'bind-rows stuff) nil)
    )
  )

(defun select-columns-by (matrix test-function)
  "
;; Description: (select-columns-by matrix test-function)

Choose a sub-matrix of a given matrix by giving a test for each column (treated
as a list).

;; Examples:
 (print-matrix (select-columns-by (id 6) #'(lambda(x) (= 1 (first x)))))
 (print-matrix (select-columns-by (id 6) #'(lambda(x) (= 1 (sum x)))))
 (select-columns-by (id 6) #'(lambda(x) (= 0 (sum x))))
 ;; (nil)
"
  (let (
	(stuff
	 (select-if
	  (mapcar #'(lambda(x) (coerce x 'list)) (column-list matrix))
	  test-function)
	 )
	)
    (if stuff (apply #'bind-columns stuff) nil)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                         set-row, set-column
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun set-row (row matrix value)
  "
;; Examples:
  (def a (matrix '(2 4) (rand.u 8)))
  (mprint a)
  (set-row 1 a (rand.u 4))
  (mprint a)
"
  (let((nrow (1- row)))
    (dotimes
     (i (length value))
     (setf (aref matrix nrow i) (elt value i))
     )
    )
  )

(defun set-column (column matrix value)
  "
;; Examples:
  (def a (matrix '(2 4) (rand.u 8)))
  (mprint a)
  (set-column 1 a (rand.u 2))
  (mprint a)
"
  (let((ncol (1- column)))
    (dotimes
     (i (length value))
     (setf (aref matrix i ncol) (elt value i))
     )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                flesh
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun flesh (mat &key (increase 2) (type 'cubic) ncols nrows (center t))
  "
;; Description: function
	(flesh mat &key (increase 2) (type 'cubic) ncols nrows (center t))

	Flesh is a function which \"densifies\" a matrix. It will interpolate
the matrix, to whatever degree you desire (although it presently adds exactly
as many new rows as columns). It creates a \"skin\" for a matrix, returning
this new and denser matrix.

;; Examples:
 (def x1 (rand.u 5))
 (def x2 (+ 2 (rand.u 5)))
 (def x3 (+ 4 (rand.u 5)))
 (def x4 (+ 6 (rand.u 5)))
 (def xmat (bind-columns x1 x2 x3 x4))
 (plot xmat)
 (plot (flesh xmat))
 (plot (flesh xmat :increase 4 :type 'linear))

 (def farouki (matrix '(6 3)
  (split-list '(
-3.0 -1.2  0.0
-3.0  0.0  0.1	
-2.7  1.3 -0.1	
-2.0 -1.0 -0.2
-1.7  0.2  0.3
-1.5  1.6  0.0
-1.1 -1.5  1.0
-0.6 -0.1  1.1
-0.8  1.2  0.3
-0.4 -0.8 -0.3
 0.0  0.1  0.1
 0.2  1.0  0.5
 0.8 -1.0  0.3
 1.5 -0.3  0.4
 2.2  1.5  0.6
 2.3 -1.7  0.2
 3.0 -0.1  0.3
 3.3  1.2  0.5) 3))
 )

 (def matrices (matroid-to-matrices farouki))
 (matroid
  (flesh (first matrices) :increase 4)
  (flesh (second matrices) :increase 4)
  (flesh (third matrices) :increase 4)
  )
"
  (let* (
	 (svd (if center (svd (- mat (mean mat))) (svd mat)))
	 (lambda (second svd))
	 (u (column-list (first svd)))
	 (v (row-list (transpose (third svd))))
	 (len (length lambda))
	 (rows (first (size mat)))
	 (cols (second (size mat)))
	 (numrvals (if nrows nrows (1- (* increase rows))))
	 (numcvals (if ncols ncols (1- (* increase cols))))
	 (result 0)
	 )
    (dotimes
     (i len)
     (let (
	   (newu 
	    (if (eq type 'cubic)
		(second (spline (iseq rows) (elt u i) :xvals numrvals))
	      (if (eq type 'hermite)
		  (second (hermite-spline (iseq rows) (elt u i) numrvals))
		(second (lin-spline (iseq rows) (elt u i) numrvals))
		)
	      )
	    )
	   (newv 
	    (if (eq type 'cubic)
		(second (spline (iseq cols) (elt v i) :xvals numcvals))
	      (if (eq type 'hermite)
		  (second (hermite-spline (iseq cols) (elt v i) numcvals))
		(second (lin-spline (iseq cols) (elt v i) numcvals))
		)
	      )
	    )
	   )
       (setq result (+ result (* (elt lambda i) (outer-product newu newv))))
       )
     )
    (if center
	(let ()
	  (andy-print "centering...")
	  (+ result (mean mat))
	  )
      (let ()
	(andy-print "not centering...")
	result
	)
      )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                flesh-function
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun flesh-function (mat &key x y center (type 'cubic))
  "
;; Description: function (flesh-function mat &key x y center (type 'cubic))

	Flesh-Function is a function which returns an interpolator to a matrix.

;; Examples:
 (def flesh-out (flesh-function (id 10) :x (rseq 1 10 10) :y (rseq 1 10 10)))
 (apply flesh-out '(1 1)) ;; should get 1 for both of these:
 (apply flesh-out '(10 10))
 (apply flesh-out '(10 9)) ;; should get 0
   
 (new-plot)
 (def flesh-out (flesh-function (id 10) :x (rseq -1 1 10) :y (rseq -1 1 10)))
 (spin-points 10)
 (apply flesh-out '(1 2))
 (plot (lambda(x y) (apply flesh-out (list x y))))

 (defun stew (x y) (apply flesh-out (list x y)))
 (plot stew)
"
  (let* (
	 (svd (if center (svd (- mat (mean mat))) (svd mat)))
	 (lambda (second svd))
	 (u (column-list (first svd)))
	 (v (row-list (transpose (third svd))))
	 (len (length lambda))
	 (rows (if x x (iseq (first (size mat)))))
	 (cols (if y y (iseq (second (size mat)))))
	 )
    (lambda (x y)
      (let ((result 0) newu newv)
	(dotimes
	 (i len)
	 (setq newu 
	       (if (eq type 'cubic)
		   (second (spline rows (elt u i) :xvals (list x)))
		 (if (eq type 'hermite)
		     (second (hermite-spline rows (elt u i) (list x)))
		   (second (lin-spline rows (elt u i) (list x)))
		   )
		 )
	       )
	 (setq newv 
	       (if (eq type 'cubic)
		   (second (spline cols (elt v i) :xvals (list y)))
		 (if (eq type 'hermite)
		     (second (hermite-spline cols (elt v i) (list y)))
		   (second (lin-spline cols (elt v i) (list y)))
		   )
		 )
	       )
	 (setq result (+ result (* (elt lambda i) newu newv)))
	 )
	(first result)
	)
      )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                matroid
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun matroid(xmat ymat zmat &key plot)
  "
;; Description: (matroid xmat ymat zmat &key plot)

	Matroid takes three matrices, which are essentially x, y, and z
components of a 'matroid' - a twisted grid - and recreates the grid.

;; See also: matroid-to-matrices

;; Examples:
 (def xmat (matrix '(4 3) '(1 1 1 2 2 2 3 3 3 4 4 4)))
 (def ymat (matrix '(4 3) '(1 2 3 1 2 3 1 2 3 1 2 3)))
 (def zmat (matrix '(4 3) '(1 1 1 2 2 2 1 1 1 -2 -2 -2)))
 (matroid xmat ymat zmat)
 (matroid (flesh xmat) (flesh ymat) (flesh zmat))

 (def eps .1)
 (def xmat (+ xmat (* eps (matrix '(4 3) (rand.n (* 4 3))))))
 (def ymat (+ ymat (* eps (matrix '(4 3) (rand.n (* 4 3))))))
 (def zmat (+ zmat (* eps (matrix '(4 3) (rand.n (* 4 3))))))
 (matroid xmat ymat zmat)
 (matroid (flesh xmat) (flesh ymat) (flesh zmat))

 (def eps .4)
 (def xmat (+ xmat (* eps (matrix '(4 3) (rand.n (* 4 3))))))
 (def ymat (+ ymat (* eps (matrix '(4 3) (rand.n (* 4 3))))))
 (def zmat (+ zmat (* eps (matrix '(4 3) (rand.n (* 4 3))))))
 (matroid xmat ymat zmat)
 (matroid (flesh xmat) (flesh ymat) (flesh zmat))
 (matroid (flesh xmat :increase 4) (flesh ymat :increase 4) (flesh zmat :increase 4))

 ;; this is the case of degenerate vertices:
 (def xmat (matrix '(3 2) '(1 1 1 2 1 3)))
 (def ymat (matrix '(3 2) '(1 2 1 2 1 2)))
 (def zmat (matrix '(3 2) '(1 1 1 2 2 1)))
 (matroid xmat ymat zmat)
 (matroid
   (flesh xmat :increase 4) (flesh ymat :increase 4) (flesh zmat :increase 4))
 (matroid
   (flesh xmat :increase 4 :type 'cubic)
   (flesh ymat :increase 4 :type 'cubic)
   (flesh zmat :increase 4 :type 'cubic))
"
  (let* (
	 (plot (if plot plot (send andyspin-proto :new 3)))
	 (nrows (first (size xmat)))
	 (ncols (second (size xmat)))
	 )
    (dotimes
     (j ncols)
     (let (
	   (xp (aref xmat 0 j))
	   (yp (aref ymat 0 j))
	   (zp (aref zmat 0 j))
	   )
       (dotimes
	(i (1- nrows))
	(let (
	      (xnext (aref xmat (1+ i) j))
	      (ynext (aref ymat (1+ i) j))
	      (znext (aref zmat (1+ i) j))
	      )
	  (send plot :add-lines (list
				    (list xp xnext)
				    (list yp ynext)
				    (list zp znext)
				    )
		)
	  (setq xp xnext)
	  (setq yp ynext)
	  (setq zp znext)
	  )
	)
       )
     )
    (dotimes
     (i nrows)
     (let (
	   (xp (aref xmat i 0))
	   (yp (aref ymat i 0))
	   (zp (aref zmat i 0))
	   )
       (dotimes
	(j (1- ncols))
	(let (
	      (xnext (aref xmat i (1+ j)))
	      (ynext (aref ymat i (1+ j)))
	      (znext (aref zmat i (1+ j)))
	      )
	  (send plot :add-lines (list
				    (list xp xnext)
				    (list yp ynext)
				    (list zp znext)
				    )
		)
	  (setq xp xnext)
	  (setq yp ynext)
	  (setq zp znext)
	  )
	)
       )
     )

    (if *title* (send plot :title *title*))
    (send plot :mouse-mode 'hand-rotate)
    (send plot :depth-cuing nil)
    (send plot :adjust-to-data)
    plot
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                        matroid-to-matrices
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun matroid-to-matrices(matroid)
  "
;; Description: function (matroid-to-matrices matroid)

	matroid-to-matrices takes a matroid and returns the three matrices
needed by function matroid to flesh out (\"densify\") the matroid.

	A matroid is a matrix of lists of three coordinates.

;; See also: matroid

;; Examples:
 (def matroid-a
   (matrix '(3 2) '((1 1 1) (1 2 1) (1 1 1) (2 2 2) (1 1 2) (3 2 1))))
 (def matrices (matroid-to-matrices matroid-a))
 (matroid
   (flesh (first matrices) :increase 4)
   (flesh (second matrices) :increase 4)
   (flesh (third matrices) :increase 4)
   )

 (def farouki (matrix '(6 3)
  (split-list '(-3.0 -1.2  0.0
-3.0  0.0  0.1
-2.7  1.3 -0.1
-2.0 -1.0 -0.2
-1.7  0.2  0.3
-1.5  1.6  0.0
-1.1 -1.5  1.0
-0.6 -0.1  1.1
-0.8  1.2  0.3
-0.4 -0.8 -0.3
 0.0  0.1  0.1
 0.2  1.0  0.5
 0.8 -1.0  0.3
 1.5 -0.3  0.4
 2.2  1.5  0.6
 2.3 -1.7  0.2
 3.0 -0.1  0.3
 3.3  1.2  0.5) 3)))

 (def matrices (matroid-to-matrices farouki))
 (matroid
   (flesh (first matrices) :increase 4)
   (flesh (second matrices) :increase 4)
   (flesh (third matrices) :increase 4)
   )
"
  (let* (
	 (nrows (first (size matroid)))
	 (ncols (second (size matroid)))
	 (xmat (zeros nrows ncols))
	 (ymat (zeros nrows ncols))
	 (zmat (zeros nrows ncols))
	 )
    (dotimes
     (i nrows)
     (dotimes
      (j ncols)
      (let ((vec (aref matroid i j)))
	(setf (aref xmat i j) (first vec))
	(setf (aref ymat i j) (second vec))
	(setf (aref zmat i j) (third vec))
	)
      )
     )
    (list xmat ymat zmat)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                            matrix-to-matroid
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun matrix-to-matroid(matrix &key x y)
  "
;; Documentation: (matrix-to-matroid matrix &key x y)

	:x refers to the rows, and should be a list of x values;
	:y refers to the columns.

;; Examples:
 (matrix-to-matroid (id 4) :x (iseq 4) :y (iseq 4))
"
  (let* (
	 (size (size matrix))
	 (num1 (first size))
	 (num2 (second size))
	 (x (if x x (iseq num1)))
	 (y (if y y (iseq num2)))
	 )
    (matrix size
	    (weave
	     (combine (mapcar #'(lambda(z) (repeat z num2)) x))
	     (combine (repeat y num1))
	     (matrix-to-list matrix)
	     )
	    )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                matrix-to-list
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun matrix-to-list (matrix)
  "
;; Description: 

	function (matrix-to-list matrix)

turns a matrix into a list, row by row: I once thought that combine would work,
but if the matrix elements are lists, then you get into trouble.

;; Examples:
 (matrix-to-list (id 3))
 (matrix-to-list (matrix '(3 4) '(1 2 3 4 5 6 7 8 9 10 11 12)))

 ;; Combine works, if you have numbers for matrix elements:
 (combine (matrix '(3 4) '(1 2 3 4 5 6 7 8 9 10 11 12)))
 ;; But it's disasterous for the following:
 (combine (matrix '(3 2) '((1 2) (3 4) (5 6) (7 8) (9 10) (11 12))))
 (matrix-to-list (matrix '(3 2) '((1 2) (3 4) (5 6) (7 8) (9 10) (11 12))))
"
  (let (
	(size (size matrix))
	newlist
	)
    (dolist
     (i (row-lists matrix))
     (setq newlist (append newlist i))
     )
    newlist
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                flesh2
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun flesh2 (mat &key (increase 2) (type 'hermite) columns rows (eps 1e-6))
  "
;; Description: function flesh2

         mat &key (increase 2) (type 'hermite) columns rows (eps 1e-6)

	Flesh2 is a function which \"densifies\" a matrix. It will interpolate
the matrix, to whatever degree you desire (although it presently adds exactly
as many new rows as columns). It creates a \"skin\" for a matrix, returning
this new and denser matrix.

	If columns are provided, these should be a list of functions which the
interpolator will be compelled to fit. There should be one for each column of
the matrix (assuming the the column dimension is the smaller of the two),
preceded by two values: the beginning and ending values of \"x\".

	Choices for type are hermite (default), cubic, or linear. This reflects
the type of interpolation which will be done on the singular vectors.

;; Examples:
 (def x1 (rand.u 5))
 (def x2 (+ 2 (rand.u 5)))
 (def x3 (+ 4 (rand.u 5)))
 (def x4 (+ 6 (rand.u 5)))
 (def xmat (bind-columns x1 x2 x3 x4))
 (plot xmat)
 (plot (flesh2 xmat :increase 2))
 (plot (flesh2 xmat :increase 4 :type 'linear))

 ;; the following are supposed to resemble each other:
 (def a (id 2))
 (def svd_a (svd a))
 (plot (flesh2 a :increase 10 :columns (list 0 pi/2 #'cos #'sin)
                       :rows (list 0 pi/2 #'cos #'(lambda (x) 0))))

 (func f(x y) (1-y)*(cos x) + y*(sin x))
 (spin-domain 0 pi/2 0 1)
 (spin-points 20)
 (plot f)

 ;; using rows and columns:
 (def a (matrix '(2 2) '(1 0 0 0)))
 (def svd_a (svd a))
 (plot (flesh2 a :increase 10 :columns (list 0 pi/2 #'cos #'(lambda (x) 0))
                       :rows (list 0 pi/2 #'cos #'(lambda (x) 0))))

 (func f(x y) (cos x) * (cos y))
 (spin-domain 0 pi/2 0 pi/2)
 (spin-points 20)
 (plot f)

 (def x (rseq 0 6 20))
 (def y (rseq 0 1 20))
 (def wild
	(bind-columns
	(sin x)
	(cos x)
	(ln (+ 1 x))
	(exp (- x))
	(^ (/ x 6) 2)
	(/ (+ 1 x))
	))
 (plot (flesh2 wild :increase 4
	:type 'hermite
	:columns (list 0 6
			#'sin
			#'cos
			#'(lambda (x) (ln (+ 1 x)))
			#'(lambda (x) (exp (- x)))
			#'(lambda (x) (^ (/ x 6) 2))
			#'(lambda (x) (/ (+ 1 x)))
	)))
 (plot wild)
"
  (let* (
	 (svd (svd mat))
	 (lambda (second svd))
	 (u (column-list (first svd)))
	 (v (row-list (transpose (third svd))))
	 (len (length lambda))
	 (nrows (first (size mat)))
	 (ncols (second (size mat)))
	 (numrvals (1- (* increase nrows)))
	 (numcvals (1- (* increase ncols)))
	 (result 0)
	 newmat
	 newrowmat
	 )

    (if columns
	(let (
	      (xvals (rseq (first columns) (second columns) numrvals))
	      (pseudo-lambda
	       (mapcar #'(lambda (x) (if (< x eps) 0 (/ x)))
		       (coerce lambda 'list)))
	      )
	  (dolist
	   (f (rest (rest columns)))
	   (setq newmat (if newmat
			    (bind-columns newmat (mapcar `,f xvals))
			  (mapcar `,f xvals))
		 )
	   )
	  (setq newmat
		(matmult
		 newmat
		 (third svd)
		 (diagonal-to-matrix pseudo-lambda)
		 )
		)
	  )
      )

    (if (and rows (= nrows ncols))
	(let (
	      (yvals (rseq (first rows) (second rows) numcvals))
	      (pseudo-lambda
	       (mapcar #'(lambda (x) (if (< x eps) 0 (/ x)))
		       (coerce lambda 'list)))
	      )
	  (dolist
	   (f (rest (rest rows)))
	   (setq newrowmat (if newrowmat
			       (bind-rows newrowmat (mapcar `,f yvals))
			     (mapcar `,f yvals))
		 )
	   )
	  (setq newrowmat
		(matmult
		 (transpose (first svd))
		 (diagonal-to-matrix pseudo-lambda)
		 newrowmat
		 )
		)
	  )
      (if rows
	  (print
	   "I only allow row choice if the matrix is square! Sorry about that...")
	)
      )
    
    (dotimes
     (i len)
     (let (
	   (newu 
	    (if columns
		(eval `(column (1+ ,i) ,newmat))
	      (if (eq type 'cubic)
		  (second (spline (iseq nrows) (elt u i) :xvals numrvals))
		(if (eq type 'hermite)
		    (second (hermite-spline (iseq nrows) (elt u i) numrvals))
		  (second (lin-spline (iseq nrows) (elt u i) numrvals))
		  )
		)
	      )
	    )
	   (newv 
	    (if rows
		(eval `(row (1+ ,i) ,newrowmat))
	      (if (eq type 'cubic)
		  (second (spline (iseq ncols) (elt v i) :xvals numcvals))
		(if (eq type 'hermite)
		    (second (hermite-spline (iseq ncols) (elt v i) numcvals))
		  (second (lin-spline (iseq ncols) (elt v i) numcvals))
		  )
		)
	      )
	    )
	   )
       (setq result (+ result (* (elt lambda i) (outer-product newu newv))))
       )
     )
    result
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				Combine-matrices
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun combine-matrices (&rest matrices)
  "
;; Description: (combine-matrices &rest matrices)

combines matrices along their columns (by default): if you want them
bound along rows, give the matrices as lists:

 (combine-matrices (list (id 3)) (list (id 3)))

;; Notes: this is the same as bind-columns, if only matrices are given.

;; See also: bind-columns, bind-rows.

;; Examples:
 (mprint (combine-matrices (id 3) (id 3)))
 (mprint (combine-matrices (list (id 2) (id 2)) (list (id 4))))

 (mprint
   (combine-matrices
     (list (repeat 0 2) (id 2))
     (list (repeat 2 2) (- (id 2)))
     )
   )
"
  (if (listp (first matrices))
      (apply #'bind-rows
	     (mapcar #'(lambda(x) (apply #'bind-columns x)) matrices)
	     )
    (apply #'bind-columns matrices)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				submatrix
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun submatrix (mat row1 row2 column1 column2)
  "
 (submatrix (id 5)  0 3 1 4)
"
  (let* ()
    (select-columns (select-rows mat (iseq row1 row2))
		    (iseq column1 column2))
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				average-rows
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun average-rows (mat)
  "
	Averages rows of a matrix. Returns a matrix one row short.

;; Examples:
 (average-rows (id 4))
 (pprint *)
;; #2A((0.5 0.5 0 0)
;;     (0 0.5 0.5 0)
;;     (0 0 0.5 0.5))
"
  (let (
	(matsize (size mat))
	)
    (matrix (list (1- (first matsize)) (second matsize))
	    (/
	     (combine
	      (mapcar
	       #'(lambda(x)
		   (+ (first x) (second x))
		   )
	       (weave (butlast (row-list mat)) (rest (row-list mat))))
	      )
	     2
	     )
	    )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				average-columns
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun average-columns (mat)
  "
	Averages columns of a matrix. Returns a matrix one column
short.

;; Examples:
 (average-columns (id 4))
 (pprint *)
;; #2A((0.5 0 0)
;;     (0.5 0.5 0)
;;     (0 0.5 0.5)
;;     (0 0 0.5))
"
  (let (
	(matsize (size mat))
	)
    (matrix (list (first matsize) (1- (second matsize)))
	    (/
	     (combine
	      (transpose
	       (mapcar
		#'(lambda(x)
		    (coerce (+ (first x) (second x)) 'list)
		    )
		(weave (butlast (column-list mat)) (rest (column-list mat))))
	       )
	      )
	      2
	     )
	    )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                matrix-and-data
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun matrix-and-data(mat xvals yvals data)
  "
;; Description: matrix-and-data(mat xvals yvals data)

	mat - a matrix
	xvals - the list of row values
	yvals - the list of column values
	data - a list of three lists of coordinates 

;; Examples:
  (matrix-and-data (id 4) '(0 1 2 3) '(0 1 2 3) '((0 1 1) (1 0 1) (.25 .5 .75)))
  (matrix-and-data (matrix '(6 4) (rand.u 24))
     '(0 1 2 3 4 5) '(0 1 2 3) '((0 1 1) (1 0 1) (.25 .5 .75)))
"
  (let (
	(myplot (matrix-plot mat :grid-vectors (list xvals yvals)))
	)
    (send myplot :add-points data)
    (send myplot :mouse-mode 'hand-rotate)
    myplot
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                       plot-matrix-columns
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun plot-matrix-columns (matrix &optional against)
  "
;; Description: plot-matrix-columns (matrix &optional against)

	plot-matrix-columns plots the columns of a matrix, oddly
enough.

;; Examples:
 (plot-matrix-columns (matrix '(6 4) (rand.u 24)) (iseq 1 6))
"
  (let* (
	 (against (if against against (iseq (first (size matrix)))))
	 (first-one (column 1 matrix))
	 (plot (plot `,against `,first-one))
	 )
    (mapcar #'(lambda(x)
		(send plot :add-plot-item (list against x))
		)
	    (rest (column-list matrix))
	    )
    (send plot :plot-item)
    (send plot :adjust-to-data)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				sort-matrix
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun sort-matrix(mat col &key high-low)
  "
;; Description: sort-matrix(mat col &key high-low)

	Sorts a matrix according to numerical values in a column,
from low to high (by default).

;; Examples:
 (sort-matrix (id 4) 1)
 ;; #2A((0 1 0 0) (0 0 1 0) (0 0 0 1) (1 0 0 0))
 (sort-matrix (id 4) 4 :high-low t)
 ;; #2A((0 0 0 1) (0 0 1 0) (0 1 0 0) (1 0 0 0))
"
  (select-rows
   mat
   (if high-low
       (reverse
	(second (simul-sort (column col mat) (iseq (first (size mat))))))
     (second (simul-sort (column col mat) (iseq (first (size mat)))))
     )
   )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				circulate
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun circulate(x &key backward)
  "
;; Description: circulate(x &key backward)

	Circulate returns a permuted x, with the first element moving to the
end of the line (unless backward is true, in which case the last element comes
to the front of the line).

;; Examples:
 (circulate (iseq 10))
 (circulate [1 2 3])
 (circulate (list [1 2 3] [1 2 4]))
 (circulate \"circulate\")

 (circulate (iseq 10) :backward t)
 (circulate [1 2 3] :backward t)
 (circulate \"circulate\" :backward t)
 ;; this one will generate an error:
 ;; (circulate (id 2))
"
  (cond
   (backward
    (reverse (circulate (reverse x))))
   ((listp x)
    (if (= (length x) 1) x
      (append (rest x) (list (first x)))
      ))
   ((stringp x)
    (strcat (substring x 1) (substring x 0 1)))
   ((vectorp x)
    (coerce (circulate (coerce x 'list)) 'vector))
   (t
    (error
     (format nil "in circulate:
Sorry: I don't know how to circulate an element of type ~s." (type-of x)))
    )
   )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			print-lower-diagonal
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun print-lower-diagonal(matrix &key (print t))
  "
;; Documentation: (print-lower-diagonal matrix &key (print t))

	Prints out the lower diagonal of a matrix. If print is nil, then the
lower-diagonal is merely returned as a list.

;; Examples:
 (print-lower-diagonal (id 4))
"
  (let (
	(k 1)
	result tmp
	)
    (dolist 
     (i (row-list matrix))
     (setq tmp (select (coerce i 'list) (iseq k)))
     (if print (format t "~g~%" tmp))
     (setq result (append result (list tmp)))
     (setq k (1+ k))
     )
    result
    )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			matrix-multiply
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun matrix-multiply(a b)
  "
;; Description: (matrix-multiply a b)

	Multiplies two matrices: works even if the components of the matrices
are not real numbers (note that matprod and matmult will fail: that's what
motivated this!).

;; Examples:
 (matrix-multiply (x-y-rotation (/ 2pi 3))
		 (matrix '(3 1) (list (id 3) (id 3) (id 3))))
"
  (let* (
	 (size-a (size a))
	 (size-b (size b))
	 (nrows (first size-a))
	 (ncols (second size-b))
	 (nover (second size-a))
	 (c (make-array (list nrows ncols)))
	 )
    (dotimes
     (i nrows)
     (dotimes
      (j ncols)
      (let ((sum 0))
	(dotimes
	 (k nover)
	 (setq sum (+ sum (matprod (aref a i k) (aref b k j))))
	 )
	(setf (aref c i j) sum)
	)))
    c
    )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			write-matrix-to-file
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun write-matrix-to-file
  (data wfile
	&key
	(format "~g")
	(separator " ")
	fmt
	header
	)
  "
;; Description (write-matrix-to-file matrix filename
                 &key (format \"~g\") (separator \" \") fmt header)

	Write your matrix to a file, with the same number of rows and columns
as the given matrix. Key options:

	:format		- format is ~g by default (for real decimal numbers)
	:separator	- space delimited by default
	:fmt		- if fmt is specified, overrides format and separator
			  to give explicit formatting instructions
	:header		- a format descriptor to write to the top of the file

;; Notes:
	If you use the :fmt option, the argument to format are fmt and
	the individual elements of each row.

;; Examples:
; (write-matrix-to-file (ones 2 2) \"wmtfdem.out\")
; (write-matrix-to-file (ones 2 2) \"wmtfdem.out\" :header \"hi~%\")
; (write-matrix-to-file (ones 2 2) \"wmtfdem.out\" :header \"hi~%\" :format \"~d \")
; (write-matrix-to-file (ones 2 2) \"wmtfdem.out\" :fmt \"~d ~f~%\")
"
  (let (
	(format
	 (if fmt
	     fmt
	   (strcat format "~{" separator format "~}" "~%")))
	)
    (with-open-file 
     (file wfile :direction :output)
     (if header (format file header))
     (dolist
      (i (row-list data))
      (let ((ii (coerce i 'list)))
	(if fmt 
	    (eval `(format ,file ,fmt ,@ii))
	  (format file format (first ii) (rest ii))
	  )
	)
      )
     )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				mat-fill
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun mat-fill(x &key (p1 1) (p2 2))
  "
;; Description: mat-fill(matrix &key (p1 1) (p2 2))

	The point of mat-fill is to take a dataset which has repeated measures
at certain positions and average them. This assumes positions in
two-dimensions.

	It returns a single matrix, which contains the two coordinates (in
positions one and two, unless the keys p1 and p2 are used), and the sums of the
rest of the values (preceded by the number of pairs of that sort). Thus,
averages for the remaining variables can be obtained by dividing by the number
at a site, as shown below.

;; Examples:
 (def mat (matrix '(6 4) '(
	 1 5 3 4
	 2 1 4 4
	 2 2 5 4
	 2 1 6 4
	 1 5 7 4
	 1 5 8 4
	 )))

 (def result (mat-fill mat))
 (mprint result)
 ;; averages:
 (mapcar #'(lambda(y) (/ (rest y) (first y))) (column 3 result))

 (mprint (mat-fill mat :p1 2 :p2 1))
"
  (let* (
	 (age-levels (sort-data (remove-duplicates (column p1 x))))
	 (lincpov-levels (sort-data (remove-duplicates (column p2 x))))
	 (p1 (1- p1))
	 (p2 (1- p2))
	 (ages (length age-levels))
	 (lincs (length lincpov-levels))
	 (nmat (make-array (list ages lincs) :initial-element 0))
	 (pmat (make-array (list ages lincs) :initial-element 0))
	 (lmat (make-array (list (* ages lincs) 3) :initial-element nil))
	 row
	 (keepers
	  (remove-if
	   #'(lambda(x) (or (= x p1) (= x p2))) (iseq (second (size x)))))
	 (k 0)
	 )
    (dotimes
     (z (first (size x)))
     (setq k (1+ k))
     (print-every 1000 k)
     (setq row (row k x))
     (let* (
	    (vals (coerce row 'list))
	    (age (elt vals p1))
	    (lincpov (elt vals p2))
	    (dead (select vals keepers))
	    (i (search (list age) age-levels))
	    (j (search (list lincpov) lincpov-levels))
	    )
       (setf (aref pmat i j) (+ dead (aref pmat i j)))
       (setf (aref nmat i j) (+ 1 (aref nmat i j)))
       )
     )
    (setq k 0)
    (dotimes
     (i ages)
     (let ((age (elt age-levels i)))
       (dotimes
	(j lincs)
	(let ((n (aref nmat i j)))
	  (if (/= n 0)
	      (let ()
		(setf (aref lmat k 0) age)
		(setf (aref lmat k 1) (elt lincpov-levels j))
		(setf (aref lmat k 2) (combine n (aref pmat i j)))
		(setq k (1+ k))
		)
	    )
	  )
	)
       )
     )
    (select lmat (iseq k) (iseq 3))
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				upper-to-full
;;
(defun upper-to-full(upper-matrix)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; upper-to-full Doc:    Creation Date:2/4/97        Author: Andy Long
;;
;; Description/Arguments:

       (upper-to-full upper-matrix)
  
	Takes a upper-triangular matrix and convert it to a full matrix.

;; Notes:

;; See Also: lower-to-full, diagonal-to-full

;; Examples:
 ;; creates all 1 matrix:
 (mprint (upper-to-full #2A((1 1) (0 1))))
 (mprint (upper-to-full #2A((0 1 1) (0 0 1) (0 0 0))))
 (mprint (upper-to-full (matrix '(4 4) (rand.u 16))))
"
  (let* (
	 (m upper-matrix)
	 (n (first (size m)))
	 )
    (dotimes
     (i (1- n))
     (dolist
      (j (iseq i (1- n)))
      (setf (aref m j i) (aref m i j))
      )
     )
    m
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 lower-to-full
;;
(defun lower-to-full(lower-matrix)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; lower-to-full Doc:    Creation Date:2/4/97        Author: Andy Long
;;
;; Description/Arguments:

       (lower-to-full lower-matrix)
  
	Takes a lower-triangular matrix and convert it to a full (necessarily
  symmetric) matrix.

;; Notes:

;; See Also: upper-to-full, diagonal-to-full

;; Examples:
 ;; creates all 1 matrix:
 (mprint (lower-to-full #2A((1 0) (1 1))))
 ;; all zero matrix:
 (mprint (lower-to-full #2A((0 1 1) (0 0 1) (0 0 0))))
 (mprint (lower-to-full (matrix '(4 4) (rand.u 16))))
"
  (let* (
	 (m lower-matrix)
	 (n (first (size m)))
	 )
    (dotimes
     (i (1- n))
     (dolist
      (j (iseq i (1- n)))
      (setf (aref m i j) (aref m j i))
      )
     )
    m
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 make-grid
;;
(defun make-grid(xstart xend xsteps ystart yend ysteps)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; make-grid Doc:    Creation Date:2/4/97        Author: Andy Long
;;
;; Description/Arguments:

       (make-grid xstart xend xsteps ystart yend ysteps)
  
	Makes a grid running from xend to xsteps taking xsteps, and from ystart
to yend taking ysteps. Returns it as a list of pairs.

	If you want to make a matrix out of the result, note that the x values
correspond to the *rows*, not the columns (see the first example below).

;; Notes:

;; See Also:

;; Examples:
 (make-grid 0 1 4 0 1 2)
 (mprint (matrix '(4 2) (make-grid 0 1 4 0 1 2)))
 (make-grid 0 1 4 0 0 1)
"
  (let (
	(x (if (= xstart xend) (list xend) (rseq xstart xend xsteps)))
	(y (if (= ystart yend) (list yend) (rseq ystart yend ysteps)))
	sites
	)
    (dolist
     (i x)
     (setq sites (append sites (mapcar #'(lambda(y) (list i y)) y)))
     )
    sites
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 orthogonal-2-by-2
;;
(defun orthogonal-2-by-2(alpha)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; orthogonal-2-by-2 Doc:    Creation Date:2/6/97        Author: Andy Long
;;
;; Description/Arguments:

       (orthogonal-2-by-2 alpha)
  
	Rotates the xy-plane by an angle of alpha in the counter-clockwise
sense.

;; Examples:
 ;; Watch what the rotation matrix does to the unit vectors:
 (matprod (orthogonal-2-by-2 (/ pi 4)) '(1 0))
 (matprod (orthogonal-2-by-2 (/ pi 4)) '(0 1))
"
  (let (
	(c (cos alpha))
	(s (sin alpha))
	)
    (matrix '(2 2) (list c (- s) s c))
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 average-duplicates
;;
(defun average-duplicates(matrix &optional column)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; average-duplicates Doc:    Creation Date://97        Author: Andy Long
;;
;; Description/Arguments:

       (average-duplicates matrix &optional column)
  
	The idea here is that there may be data with the last column
repeated (repeat measures for lab data, etc.), in which case we may want to
average them.

	average-duplicates returns the new matrix, as well as a list of those
entries which were kept (in a values statement, so you need to use
multiple-value-setq to catch the second).

;; Notes:

;; See Also:

;; Examples:
 (def stuff '(#(1 2) #(2 3) #(1 2) #(1 2)))
 (equalp (first stuff) (third stuff))
 (find-duplicates stuff)
 ;; ((0 2 3))

 (def stuff '(#(1 2) #(2 3) #(1 0)))
 (average-duplicates (apply #'bind-rows stuff) 2)
 (def stuff '(#(2 3) #(1 2) #(1 0)))
 (average-duplicates (apply #'bind-rows stuff) 2)
 (def stuff '(#(1 2) #(1 0) #(2 3)))
 (average-duplicates (apply #'bind-rows stuff) 2)
 ;; all give:
 ;; #2A((2 3) (1 1))

 (def stuff '(#(1 2) #(2 3) #(1 0) #(1 1)))
 (average-duplicates (apply #'bind-rows stuff) 2)
 ;; #2A((2 3) (1 1))

 (def stuff '(#(1 2) #(2 3) #(3 0) #(4 1)))
 ;; get it right back! No dups...
 (average-duplicates (apply #'bind-rows stuff) 2)

 ;; Here's how you 'catch' the list of rows chosen:
 (multiple-value-setq (hip hop)
   (average-duplicates (apply #'bind-rows stuff) 2)
   )
 hip ;; the new matrix
 hop ;; the list of indices
"
  (let* (
	 ;; by default the column is the last:
	 (column (if column column (second (size matrix))))
         (dups
          (find-duplicates
           (row-list
            (select-columns
             matrix
             (remove (1- column) (iseq (second (size matrix))))))))
	 )
    (if dups
	(let* (
	       (lep-dups
		(mapcar
		 #'mean
		 (mapcar
		  (lambda(x) (column column (select-rows matrix x)))
		  dups)))
	       (first-dups (mapcar #'first dups))
	       (rest-dups
		(mapcar
		 (lambda(x) (coerce (row x matrix) 'vector)) (1+ first-dups)))
	       (non-dups
		(set-difference (iseq (first (size matrix))) (combine dups)))
	       )
	  (mapcar
	   (lambda(i)
	     (setf (aref (elt rest-dups i) (1- column)) (elt lep-dups i)))
	   (iseq (length dups)))
	  (values 
	   (bind-rows
	    (select-rows matrix non-dups)
	    (apply #'bind-rows rest-dups)
	    )
	   (combine non-dups first-dups)
	   )
	  )
      (values matrix (iseq (first (size matrix))))
      )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun positive-inverse (mat) 
  "
from:
Thomas Lumley
-----------------------------------------------------+------
Biostatistics		: \"Never attribute to malice what  :
Uni of Washington	:  can be adequately explained by  :
Box 357232		:  incompetence\" - Hanlon's Razor  :
Seattle WA 98195-7232	:				   :
------------------------------------------------------------

	I think that this is one guy's way of coming up with an inverse that is
positive definite for a positive definite matrix. It might be useful in
variogram methods some day. ael.

;; Examples:
 ;; Here's a non-invertible matrix: I pith the identity matrix, and get a wild
 ;; matrix instead of an error.
 (positive-inverse (let ((x (id 4))) (setf (aref x 1 1) 0) x))
"
  (let ((Linv (inverse (first (chol-decomp mat)))))
    (matmult (transpose Linv) Linv)
    )
  )


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 pad-to-square-with-zeros
;;
(defun pad-to-square-with-zeros(matrix)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; pad-to-square-with-zeros Doc:    Creation Date:2/28/97    Author: Andy Long
;;
;; Description/Arguments:

       (pad-to-square-with-zeros matrix)
  
;; Notes:

;; See Also:

;; Examples:
 (mprint (pad-to-square-with-zeros #2a((1 2 3))))
 (mprint (pad-to-square-with-zeros #2a((1)(2)(3))))
 (mprint (pad-to-square-with-zeros (id 3)))
"
  (let* (
	 (size (size matrix))
	 (m (first size))
	 (n (second size))
	 )	
    (if (= n m)
	matrix
      (let* (
	     (diff (abs (- m n)))
	     (z (zeros (max m n) diff))
	     )
	(if (> m n)
	    (bind-columns matrix z)
	  (bind-rows matrix (transpose z))
	  )
	)
      )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 column-lists
;;
(defun column-lists(matrix)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; column-lists Doc:    Creation Date:3/31/97        Author: Andy Long
;;
;; Description/Arguments:

       (column-lists matrix)
  
	Returns the columns of a matrix as lists: this is to improve on
function column-list, which returns them as arrays.

;; Notes:

;; See Also: column-list

;; Examples:
 (column-lists (id 4))
 (column-lists (matrix (list 4 2) (rand.u 8)))
"
  (mapcar
   #'coerce
   (column-list matrix)
   (repeat 'list (second (size matrix)))
   )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 row-lists
;;
(defun row-lists(matrix)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; row-lists Doc:    Creation Date:3/31/97        Author: Andy Long
;;
;; Description/Arguments:

       (row-lists matrix)
  
	Returns the rows of a matrix as lists: this is to improve on
function row-list, which returns them as arrays.

;; Notes:

;; See Also: row-list

;; Examples:
 (row-lists (id 4))
 (row-lists (matrix (list 4 2) (rand.u 8)))
"
  (mapcar #'coerce (row-list matrix) (repeat 'list (first (size matrix))))
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 pseudo-inverse
;;
(defun pseudo-inverse(A &key (eps 1e-12) (verbose t) infinity)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          pseudo-inverse Doc:
;;
;; Author: Andy Long
;; Created: Thu Apr 24 16:05:57 CDT 1997
;;
;; Description/Arguments:

       (pseudo-inverse A &key (eps 1e-12) (verbose t) infinity)
  
	Calculates the pseudo-inverse of matrix A, using the singular value
decomposition. Once the sum of the first (largest to smallest) singular values
equals 1-eps or greater, the remainder of the singular values are considered to
be zero. Then, whereas the inverse of A would be given by

	V (Lambda)^{-1} U^T,

the pseudo-inverse of Lambda is given by entries of (1/lambda_i) for non-zero
lambda_i, and 0 otherwise:

	(pseudo-inverse A) = V (pseudo-inverse Lambda) U^T,

	If the singular value is less than eps, it is set to zero; if verbose
is true, a warning will be issued about the singularity of the matrix.

;; Notes: returns the pseudo-inverse, and by the values function a flag
;; indicating whether the matrix was numerically singular.

;; See Also: svd

;; Examples:
 (pseudo-inverse #2a((1 0) (0 0)))
 (pseudo-inverse #2a((1 0 0) (0 1 0) (1 1 0)))

 (def pseudo-inv (pseudo-inverse #2a((1 0 0) (0 1 0) (1 1 0))))
 (matmult pseudo-inv #2a((1 0 0) (0 1 0) (1 1 0)))

;; Here's an interesting example of the use of the pseudo-inverse:
 (title \"Skinning a Grid with the Pseudo-Inverse\")
 (func f(x y) sin(x)*sin(y))
 (def xs (rseq 0 1 3))
 (def ys xs) 
 (defun G(x) (mapcar (lambda(y) (f x y)) ys))
 (defun H(y) (mapcar (lambda(x) (f x y)) xs))
 (def pseudo-inv (pseudo-inverse (outer-product xs ys #'f)))
 (defun skin(x y) (matmult (g x) pseudo-inv (h y)))
 (defun diffy(x y) (- (skin x y) (f x y)))
 (spin-domain -pi pi -pi pi)
 (spin-points 21)
 (plot skin)
 (plot diffy)

 (def stuff 
  #2a(
     (  13.0000       97.0000       626.000       153.000       390.000    )
     (  97.0000       1139.00       4922.00       769.000       2620.00    )
     (  626.000       4922.00       33050.0       7201.00       15739.0    )
     (  153.000       769.000       7201.00       2293.00       4628.00    )
     (  390.000       2620.00       15739.0       4628.00       15062.0    )
    )
   )
 (mprint stuff)
 (mprint (pseudo-inverse stuff))
 (mprint (inverse stuff))
 (mprint (matmult stuff (pseudo-inverse stuff :eps 1e-12)))
 (mprint (matmult stuff (inverse stuff)))

 (def stuff #2a((0 #c(0 1))(#c(0 1) 0)))
 (svd stuff)
 (mprint (matmult stuff (pseudo-inverse stuff :eps 1e-12)))
"
  (let* (
	 (svd (svd A))
	 (u (first svd))
	 (lambda (second svd))
	 (v (third svd))
	 (len (length lambda))
	 (inv (repeat 0 len))
	 (sum 0)
	 ;; make sure it's real! The singular values may be returned as
	 ;; complex, for a complex matrix, even though their imaginary parts
	 ;; are all zero:
	 (total (sum lambda))
	 (k 0)
	 singular
	 lam
	 )
    (if (/= total 0)
	(do
	 ((i 1 (1+ i)))
	 ((> sum (- 1 eps)))
	 (setf
	  lam (elt lambda (1- i))
	  sum (+ sum (/ lam total))
	  (elt inv (1- i)) (/ lam))
	 (incf k)
	 )
      )
    (setq singular (/= k len))

    (if (and verbose singular)
	(format t
		"~%\tTo precision ~,4g the matrix is numerically singular,
with rank ~d out of ~d possible.~%" eps k len))
    
    (if (and singular infinity)
	(let ((n (where inv 0)))
	  (values
	   (list 
	    (select (column-lists u) n) lambda (select (column-lists v) n))
	   n))
      (values
       (matprod v (diagonal-to-matrix inv) (transpose (conjugate u)))
       singular
       )
      )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 pseudo-solve
;;
(defun pseudo-solve(matrix b &key (eps 1e-12) verbose)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          pseudo-solve Doc:
;;
;; Author: Andy Long
;; Created: Tue Dec 2 19:29:35 1997
;;
;; Description/Arguments:

       (pseudo-solve A b &key (eps 1e-12) verbose)
  
		A		- a square matrix
		b		- a vector, list, or matrix to be solved for
		eps		- a small number, such that if a singular value
  				  is less than eps, it is set to zero.
		verbose		- let's be talkative

;; Returns:
	The result of multiplying the right-hand side (b) by the
  pseudo-inverse of the matrix A.

;; Notes:
	The pseudo-inverse is defined by the use of the singular-value
  decomposition (svd). This is a 'safe' way to invert a matrix: it will give
  you an answer, even if the matrix is singular; that answer may, however, not
  be what you expect! So this should be used with some caution.

;; See Also: solve, pseudo-inverse, svd

;; Examples:
 (def A #2a((1 -1 0)(0 1 -1)(1 0 -1)))
 (def b (list 2 3 5))
 (pseudo-solve A b)

 (def b (matrix '(3 2) '(2 2 3 3 5 4)))
 (pseudo-solve A b)
"
  (matmult (pseudo-inverse matrix :eps eps :verbose verbose) b)
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Robust-SVD
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun robust-svd(amatrix &key (eps 1e-12) (its 100))
  "
;; Description: (robust-svd amatrix &key (eps 1e-12) (its 100))

	Robust-svd: based on a paper by R. R. Burnside and P. B. Guest, \"A
Simple Proof of the Transposed QR Algorithm\", in SIAM Review, Vol. 38, No. 2,
pp. 306--308, June 1996.

	Uses the transposed qr algorithm to diagonalize the sequence of
upper-triangular R matrices.	

;; Examples:
 (def amatrix #2a((1 2 3) (2 3 4) (1 -1 1)))
 (mprint amatrix)
 (def robust-svd (robust-svd amatrix))
 (mprint
   (matmult
     (first robust-svd)
     (diagonal-to-matrix (second robust-svd))
     (transpose (third robust-svd))
     )
   )

 ;; Rectangular:
 (def amatrix #2a((1 2) (2 3) (1 -1)))
 (def svd (svd amatrix))
 (def robust-svd (robust-svd amatrix))
 (mprint (first svd))
 (mprint (first robust-svd))
 (mprint (third svd))
 (mprint (third robust-svd))
 (print (- (second svd) (second robust-svd)))

 (def amatrix (transpose #2a((1 2) (2 3) (1 -1))))
 (def svd (svd amatrix))
 (def robust-svd (robust-svd amatrix))
 (mprint (first svd))
 (mprint (first robust-svd))
 (mprint (third svd))
 (mprint (third robust-svd))
 (print (- (second svd) (second robust-svd)))
"
  (let* (
	 (size (size amatrix))
	 (transpose (if (> (second size) (first size)) t nil))
	 (qr (qr-decomp (if transpose (transpose amatrix) amatrix)))
	 (q1 (first qr))
	 (a (second qr))
	 (i 1)
	 u lambda v
	 )
    (do
     (
      (off-diag
       (- (sum (abs a)) (sum (abs (diagonal a))))
       (- (sum (abs a)) (sum (abs (diagonal a))))
       )
      )
     (
      (or (> i its) (< off-diag eps))
      )
     (setq qr (qr-decomp (transpose a)))
     (if (evenp i)
	 (if u (setq u (matmult u (first qr))) (setq u (first qr)))
       (if v (setq v (matmult v (first qr))) (setq v (first qr)))
       )
     (incf i)
     (setq a (second qr))
     )
    (setq lambda (diagonal a))
    (if transpose
	(list v
	      (abs lambda)
	      (matmult q1 u (diagonal-to-matrix (signum lambda)))
	      i)
      (list (matmult q1 u (diagonal-to-matrix (signum lambda)))
	    (abs lambda)
	    v
	    i)
      )
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Householder-qr-decomp
;;
;; Creation Date:4/27/97        Author: Andy Long
;;
(defun Householder-qr-decomp(amatrix)
  "
;; Description/Arguments:

       (Householder-qr-decomp amatrix)
  
;; Notes:

;; See Also:

;; Examples:
 (def amatrix #2a((1 0 1) (0 1 2) (1 2 0)))
 (mapcar #'mprint (Householder-qr-decomp amatrix))
 (mapcar #'mprint (qr-decomp amatrix))

 (def amatrix #2a((1 -1 -1 -1) (-1 1 -1 -1) (-1 -1 1 -1) (-1 -1 -1 1)))
 (mapcar #'mprint (Householder-qr-decomp amatrix))

 (def amatrix #2a((3 4) (4 0)))
 (mapcar #'mprint (Householder-qr-decomp amatrix))

 (def amatrix #2a((1 2) (2 3) (1 -1)))
 (mapcar #'mprint (Householder-qr-decomp amatrix))
 (mapcar #'mprint (qr-decomp amatrix))

 ;; Also handles the case that the qr-decomp won't:
 (def amatrix (transpose #2a((1 2) (2 3) (1 -1))))
 (mapcar #'mprint (Householder-qr-decomp amatrix))
"
  (let* (
	 (size (size amatrix))
	 (square (if (= (second size) (first size)) t nil))
	 (transpose (if (> (second size) (first size)) t nil))
	 (amatrix (if transpose (transpose amatrix) amatrix))
	 (size (size amatrix))
	 (ntimes (first size))
	 (m (second size))
	 (id (id ntimes))
	 (result amatrix)
	 (1-ntimes (1- ntimes))
	 q
	 )
    (dotimes
     (i (if (= m ntimes) (1- m) m))
     (let* (
	    (avec (select (column (1+ i) result) (iseq i 1-ntimes)))
	    (rvec (repeat 0 (length avec)))
	    (tmp (setf (elt rvec 0) (norm avec)))
	    (avec (- avec rvec))
	    (dvec (if (= i 0) avec (combine (repeat 0 i) avec)))
	    (norm (norm avec))
	    (house 
	     (if (= 0 norm) id
	       (- id (* (/ 2 (dot avec avec)) (outer-product dvec dvec))))
	     )
	    )
       (setq result (matmult house result))
       (setq q (if q (matmult q house) house))
       )
     )
    (if square
	(list q result)
      (let (
	    (result (select-rows result (iseq m)))
	    (q (select-columns q (iseq m)))
	    )
	(if transpose
	    (list (transpose result) (transpose q))
	  (list q result)
	  )
	)
      )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Householder-triangular-form Doc:    
;; Creation Date:4/27/97        Author: Andy Long
;;
;;
(defun Householder-triangular-form(amatrix)
  "
;; Description/Arguments:

       (Householder-triangular-form amatrix)
  
;; Notes:

;; See Also:

;; Examples:
 (def amatrix #2a((1 0 1) (0 1 2) (1 2 0)))
 (mprint (Householder-triangular-form amatrix))

 (def amatrix #2a((1 -1 -1 -1) (-1 1 -1 -1) (-1 -1 1 -1) (-1 -1 -1 1)))
 (mprint (Householder-triangular-form amatrix))
"
  (let* (
	 (m (first (size amatrix)))
	 (id (id m))
	 (result amatrix)
	 (ntimes (if (evenp m) (/ m 2) (/ (1- m) 2)))
	 (1-m (1- m))
	 q
	 )
    (dotimes
     (i ntimes)
     (let* (
	    (avec (select (column (1+ i) result) (iseq i 1-m)))
	    (a1 (elt avec 0))

	    (rvec (repeat 0 (length avec)))
	    (tmp (setf (elt rvec 0) a1))
	    (tmp (setf (elt rvec 1) (sqrt (- (dot avec avec) (* a1 a1)))))

	    (avec (- avec rvec))
	    (dvec (if (= i 0) avec (combine (repeat 0 i) avec)))

	    (norm (norm avec))
	    (house 
	     (if (= 0 norm) id
	       (- id (* (/ 2 (dot avec avec)) (outer-product dvec dvec))))
	     )
	    )
       (setq q (if q (matmult house q) house))
       (setq result (matmult house result house))
       )
     )
    (values result q)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
(defun inside-nth(&rest args)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; inside-nth Doc:    Creation Date:5/3/97        Author: Andy Long
;;
;; Description/Arguments:

       (inside-nth item &optional (nth 0) &key apply)
  
	The idea is to get at the nth element of the elements of a list. I
especially wanted to be able to do this for matrices.

	Apply is a function to apply. The default is (elt x nth), but you may
supply another: for example, #'mean.

;; Notes: 

;; See Also:

;; Examples:
 ;; the following three calls give the same result:
 (inside-nth #2a(((1 2) (2 3))))
 (inside-nth #2a(((1 2) (2 3))) 0)
 (inside-nth #2a(((1 2) (2 3))) :apply #'first)
 
 (inside-nth #2a(((1 2) (2 3))) 1)
 (inside-nth #((1 2) (2 3)) 1)
 (inside-nth (list '(1 2) '(2 3)) 1)
 (inside-nth 'andy-is-a-hippo-lover)

 ;; This shows that it isn't only for nth anymore!
 (inside-nth #2a(((1 2) (2 3))) :apply #'mean)
"
  (let* (
	 (apply (first (catch-keys '(:apply) args)))
	 (args (remove-keys '(:apply) args))
	 (item (first args))
	 (nth (if (second args) (second args) 0))
	 (apply (if apply apply #'(lambda(x) (elt x nth))))
	 )
    (cond
     ((listp item)
      (mapcar apply (coerce item 'list))
      )
     ((vectorp item)
      (coerce
       (mapcar apply (coerce item 'list))
       'vector
       )
      )
     ((matrixp item)
      (matrix (size item) (mapcar apply (matrix-to-list item)))
      )
     (t
      (format t "~%Sorry - I don't know what to do with this:~%~20t~s~%" item))
     )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 matrix-equalp
;;
(defun matrix-equalp(a b &key (test #'equalp))
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          matrix-equalp Doc:
;;
;; Author: Andy Long
;; Created: Thu Nov 20 10:35:38 1997
;;
;; Description/Arguments:

	Function matrix-equalp(a b &key (test #'equalp))

uses equalp on the elements of matrices a and b to determine if the two
matrices are equivalent.

;; Notes:

;; See Also:

;; Examples:
 (setq A (matrix '(3 3) '(a b c d e f g h i)))
 (setq B (id 3))
 (matrix-equalp a b)
 (matrix-equalp a a)
"
  (eval `(and ,@(mapcar test (matrix-to-list a) (matrix-to-list b))))
  )


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 permutation-matrix
;;
(defun permutation-matrix(list)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          permutation-matrix Doc:
;;
;; Author: Andy Long
;; Created: Sat Nov 22 09:40:03 1997
;;
;; Description/Arguments:

       (permutation-matrix list)
  
Creates a permutation matrix given by the elements of list. This list should be
a permutation of the sequence with length (length list).

;; Notes:

;; See Also:

;; Examples:
 (permutation-matrix '(2 1 0))
 (permutation-matrix '(1 2 0))
 (permutation-matrix '(1 0 2))
"
  (apply #'bind-rows (select (row-list (id (length list))) list))
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun rotate-clock-wise-matrix(a n)
  "
;; Description: function rotate-clock-wise-matrix(A n)

	Rotates a matrix A about its center 90 degrees in a clock-wise sense.

	A - a matrix
	n - the number of trips around you want to make (n=4 gives back A)

;; Notes: created to help me evaluate when two tic-tac-toe boards are
;; equivalent! Now used in backsolve, to turn lower-triangular matrices into
;; upper-triangular ones. ael, 11/97.

;; Examples:
 (mprint (setq A (matrix '(3 3) '(a b c d e f g h i))))
 (mprint (rotate-clock-wise-matrix a 1))
 (mprint (rotate-clock-wise-matrix a 2))
 (mprint (rotate-clock-wise-matrix a 3))

 (mprint (setq A (matrix '(3 2) '(a b c d e f))))
 (mprint (rotate-clock-wise-matrix a 1))
 (mprint (rotate-clock-wise-matrix a 2))

"
  (do
   ((i n (decf i)))
   ((= i 0) a)
   (setq a (apply #'bind-columns (reverse (row-lists a))))
   )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
(defun backsolve (a b &key lower)
  "
Args: (a b &key lower)

Solves A x = B by backsolving, assuming A is triangular (assumed upper by
default). B should be a matrix, vector or sequence.

;; Notes: ael, 11/22/97: I absconded with this to make it a little friendlier
;; (used to demand that the matrix be upper, and only took a vector or list).

;; See Also: rotate-clock-wise-matrix

;; Examples:
 (def a #2a((1 0 0) (1/2 1 0) (1/3 2/3 1)))
 (backsolve a '(1 1/2 1/3) :lower t)		;; (1 0 0)
 (backsolve (transpose a) '(1 0 0))		;; (1 0 0)
 (backsolve a a :lower t)			;; ((1 0 0) (0 1 0) (0 0 1))
 (backsolve (transpose a) (transpose a))	;; ((1 0 0) (0 1 0) (0 0 1))
"
  (if (matrixp b)
      (mapcar (lambda(x) (backsolve a x :lower lower)) (column-lists b))
    (let* (
	   (n (length b))
	   (sol (make-array n))
	   )
      (if lower
	  ;; stand everything on its head!
	  (reverse
	   (backsolve
	    (rotate-clock-wise-matrix a 2) (reverse b)))
	(let ()
	  (dotimes
	   (i n)
	   (let* (
		  (k (- n i 1))
		  (val (elt b k))
		  )
	     (dotimes
	      (j i)
	      (let ((l (- n j 1)))
		(setq val (- val (* (aref sol l) (aref a k l))))))
	     (setf (aref sol k) (/ val (aref a k k)))))
	  (if (listp b) (coerce sol 'list) sol))
	)
      )
    )
  )


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                square-matrix-p
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun square-matrix-p (x)
  "
;; Description: function square-matrix-p (x)
Checks that x is really a square matrix.

;; Examples:
 (square-matrix-p (id 3))
 ;; true: 3 by 3 identity matrix
 (square-matrix-p (matrix '(2 3) (iseq 6)))
 ;; nil
"
  (and (matrixp x) (= (array-dimension x 0) (array-dimension x 1)))) 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 symmetric-matrix-p
;;
(defun symmetric-matrix-p(a)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          symmetric-matrix-p Doc:
;;
;; Author: Andy Long
;; Created: Wed Nov 26 22:55:42 1997
;;
;; Description/Arguments:

       (symmetric-matrix-p a)
  
where a is a matrix.

;; Returns:

;; Notes:

;; See Also:

;; Examples:
 (symmetric-matrix-p (id 3))				;; t
 (symmetric-matrix-p (matrix '(2 3) (iseq 6)))		;; nil
 (symmetric-matrix-p (matrix '(2 2) '(1 2 3 4)))	;; nil
 (symmetric-matrix-p (matrix '(2 2) '(1 2 2 1)))	;; t
"
  (if (square-matrix-p a)
      (matrix-equalp
       (extract-upper-triangular a)
       (extract-upper-triangular (transpose a))
       )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 matrix-trace
;;
(defun matrix-trace(a)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          matrix-trace Doc:
;;
;; Author: Andy Long
;; Created: Sat Nov 29 15:56:41 1997
;;
;; Description/Arguments:

       (matrix-trace matrix)
  
;; Returns: the trace, or sum of the diagonal elements of a matrix.

;; See Also: diagonal

;; Examples:
 (matrix-trace (id 3))		;; 3
 (matrix-trace (ones 3 3))	;; 3
"
  (if (matrixp a) (sum (diagonal a)))
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 hermitian
;;
(defun hermitian(a)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          hermitian Doc:
;;
;; Author: Andy Long
;; Created: Sat Nov 29 16:58:15 1997
;;
;; Description/Arguments:

       (hermitian matrix)
  
;; Returns: the conjugate of the transpose of the matrix.

;; Examples:
 (hermitian #2a((0 #c(0 1))(#c(0 1) 0)))
"
  (if (matrixp a) (conjugate (transpose a)))
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 hermitian-p
;;
(defun hermitian-p(a)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          hermitian-p Doc:
;;
;; Author: Andy Long
;; Created: Sat Nov 29 17:03:50 1997
;;
;; Description/Arguments:

       (hermitian-p matrix)
  
;; Returns: t or nil, depending on whether matrix is hermitian or not (that is,
  if the complex conjugate of the transpose is equal to the original matrix).

;; See Also: hermitian

;; Examples:
 (hermitian-p #2a((0 #c(0 1))(#c(0 -1) 0)))	;; t
 (hermitian-p #2a((0 #c(0 1))(#c(1 1) 0)))	;; nil
"
  (if (matrixp a) (matrix-equalp a (conjugate (transpose a))))
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 mprint
;;
(defun mprint(A &rest args)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          mprint Doc:
;;
;; Author: Andy Long
;; Created: Sun Feb 1 14:46:11 1998
;;
;; Description/Arguments:

   (mprint matrix &optional stream &key (float-digits 6))
  
	pretty-prints a matrix, or a vector/list as a column matrix.

;; Returns: nil, after pretty-printing the object.

;; Notes: used to be aliased to print-matrix, then I generalized it to handle
;; vectors/lists too.

;; See Also: print-matrix

;; Examples:
 (mprint (id 3))
 (def v (rand.u 3))
 (mprint v)
 (mprint (coerce v 'vector))
 (mprint v t :float-digits 10)
 (mprint v t :float-digits 2)
"
  (apply
   #'print-matrix
   (if (matrixp A) A (bind-columns A))
   args)
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 polar-decomp
;;
(defun polar-decomp(A)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          polar-decomp Doc:
;;
;; Author: Andy Long
;; Created: Sun Feb 1 15:04:51 1998
;;
;; Description/Arguments:

       (polar-decomp A)
  
	Where A is a matrix, giving the decomposition

				 A = QB

where B is the positive square-root of A'A, and Q is deduced as 

		Q = A*(pseudo-inverse B)

;; Returns: a list containing the two matrices Q and B.

;; Notes: Got this from Strang's 'Introduction to Applied Math'.

;; See Also: lu-decomp, qr-decomp, sv-decomp, svd, etc.

;; Examples:
 (polar-decomp #2a((1 -1)(0 1)(1 0)))
 ;; things look a little nicer if we use mprint:
 (mapcar #'mprint (polar-decomp #2a((1 -1)(0 1)(1 0))))
"
  (let* (
	 (svd (svd (matmult (transpose A) A)))
	 (B (matmult (first svd) (diagonal-to-full (sqrt (second svd)))
		     (transpose (third svd))))
	 )
    (list (matmult A (pseudo-inverse B)) B)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			inverse-of-A-with-one-column-swap
;;
(defun inverse-of-A-with-one-column-swap(A-inv ith y)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			inverse-of-A-with-one-column-swap Doc:
;;
;; Author: Andy Long
;; Created: Wed Mar 11 22:14:07 1998
;;
;; Description/Arguments: inverse-of-A-with-one-column-swap 

	If the inverse of A is known, and one wants the inverse of a matrix
which is A with one column changed, then it is easy to calculate the new
inverse.

	If the last column is the one replaced by vector y, then

				[1 0 ... 0 v1  ]-1
				[0 1 ... 0 v2  ]
		A-inv-new = 	[. . ... . ..  ]   A-inv
				[0 0 ... 1 vn-1]
				[0 0 ... 0 vn  ]

where v = A-inv y. The inverse of the first matrix is simple:

			[1 0 ... 0 -v1/vn  ]
			[0 1 ... 0 -v2/vn  ]
			[. . ... .   ...   ]
			[0 0 ... 1 -vn-1/vn]
			[0 0 ... 0   1/vn  ]

;; Arguments: A-inv ith y

	where

		A-inv	- the inverse of A,
		ith	- the index of the column being replaced, and
		y	- the replacement column.

;; Returns: the inverse.

;; Notes: This is described in Gil Strang's 'Introduction to Applied
   Mathematics'.

;; See Also: inverse, pseudo-inverse

;; Examples:
 (def A (matrix '(2 2) (rand.n 4)))
 (def b (list 2 3))
 (def random-column (rand.n 2))

 (def new-A (bind-columns (column 1 A) random-column))
 (matmult (inverse-of-A-with-one-column-swap (inverse A) 1 random-column) b)
 (matmult (inverse new-A) b)

 (def new-A (bind-columns random-column (column 2 A)))
 (matmult (inverse-of-A-with-one-column-swap (inverse A) 0 random-column) b)
 (matmult (inverse new-A) b)

 ;; this is a real test: everything is random!
 (def n 3)
 (def ith (first (sample (iseq n) 1)))
 (def A (matrix (list n n) (rand.n (* n n))))
 (def b (rand.n n))
 (def random-column (rand.n n))

 ;; Create new-A as a list of columns of A:
 (def new-A (column-list A))
 ;; Replace the ith column of A with the random column:
 (setf (elt new-A ith) random-column)
 ;; Now turn new-A into a matrix:
 (def new-A (apply #'bind-columns new-A))
 ;; Here's the test!
 (matmult (inverse-of-A-with-one-column-swap (inverse A) ith random-column) b)
 (matmult (inverse new-A) b)
"
  (let* (
	 (n (length y))

	 (v (matmult A-inv y))
	 (special (elt v ith))
	 (error
	  (if (= 0 special)
	      (error "In inverse-of-A-with-one-column-swap:
Matrix is not invertible.~%")))

	 (v (- (/ v special)))

	 (ith-row (select A-inv ith (iseq n)))
	 (op (outer-product v (combine ith-row)))
	 (a-inv-new (+ a-inv op))
	 )
    (setf (select A-inv-new ith (iseq n)) (/ ith-row special))
    a-inv-new
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 but-rows
;;
(defun but-rows(a list)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          but-rows Doc:
;;
;; Author: Andy Long
;; Created: 5/10/1999
;;
;; Description/Arguments:

       (but-rows matrix num-or-list)
  
	Removes the offending row or rows from the matrix (given by an integer
  or a list of integers). Rows are numbered from 0.

;; Notes:

;; See Also:

;; Examples:
 (but-rows (id 3) 0)
 (but-rows (id 3) (list 1))
 (but-rows (id 3) (iseq 2))
 (but-rows (id 3) (iseq 3))
"
  (let* (
	 (size (size a))
	 (ns (if (numberp list) (list list) list))
	 (keepers (sort-data (set-difference (iseq (first size)) ns)))
	 )
    (select a keepers (iseq (second size)))
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 but-cols
;;
(defun but-cols(a list)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          but-cols Doc:
;;
;; Author: Andy Long
;; Created: 5/10/1999
;;
;; Description/Arguments:

       (but-cols matrix num-or-list)
  
	Removes the offending col or cols from the matrix (given by an integer
  or a list of integers). Cols are numbered from 0.

;; Notes:

;; See Also:

;; Examples:
 (mprint (but-cols (id 3) 1))
 (mprint (but-cols (id 3) (list 1)))
 (mprint (but-cols (id 3) (iseq 2)))
 (mprint (but-cols (id 3) (list 0 2)))
 (but-cols (id 3) (iseq 3))
"
  (let* (
	 (size (size a))
	 (ns (if (numberp list) (list list) list))
	 (keepers (sort-data (set-difference (iseq (second size)) ns)))
	 )
    (select a (iseq (first size)) keepers)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
