;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;              This is nonlin.lsp, with 1e-6 replacing .0001
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;; XLISP-STAT 2.1 Copyright (c) 1990, by Luke Tierney
;;;; Additions to Xlisp 2.1, Copyright (c) 1989 by David Michael Betz
;;;; You may give out copies of this software; for conditions see the file
;;;; COPYING included with this distribution.

(provide "nonlin")
(require "regress")

;;;;
;;;;
;;;; Nonlinear Regression Model Prototype
;;;;
;;;;

(defproto nreg-model-proto 
          '(mean-function theta-hat epsilon count-limit verbose)
          '()
          regression-model-proto)

(defun nreg-model (mean-function y theta 
                                 &key 
                                 (epsilon 1e-6) 
                                 (print t) 
                                 (count-limit 20)
                                 parameter-names
                                 response-name
                                 case-labels
                                 weights
                                 included 
                                 (verbose print))
"Args: (mean-function y theta &key (epsilon 1e-6) (count-limit 20) 
                      (print t) parameter-names response-name case-labels
                      weights included (verbose print))
Fits nonlinear regression model with MEAN-FUNCTION and response Y using initial
parameter guess THETA. Returns model object."
  (let ((m (send nreg-model-proto :new)))
    (send m :mean-function mean-function)
    (send m :y y)
    (send m :new-initial-guess theta)
    (send m :epsilon epsilon)
    (send m :count-limit count-limit)
    (send m :parameter-names parameter-names)
    (send m :response-name response-name)
    (send m :case-labels case-labels)
    (send m :weights weights)
    (send m :included included)
    (send m :verbose verbose)
    (if print (send m :display))
    m))

(defmeth nreg-model-proto :save ()
"Message args: ()
Returns an expression that will reconstruct the regression model."
  `(nreg-model ',(send self :mean-function)
               ',(send self :y)
               ',(send self :coef-estimates)
               :epsilon ',(send self :epsilon)
               :count-limit ',(send self :count-limit)
               :predictor-names ',(send self :predictor-names)
               :response-name ',(send self :response-name)
               :case-labels ',(send self :case-labels)
               :weights ',(send self :weights)
               :included ',(send self :included)
               :verbose ',(send self :verbose)))

;;;
;;; Computing Method
;;;
              
(defmeth nreg-model-proto :compute ()
"Message args: ()
Recomputes the estimates. For internal use by other messages"
  (let* ((y (send self :y))
         (weights (send self :weights))
         (inc (if-else (send self :included) 1 0))
         (w (if weights (* inc weights) inc)))
    (setf (slot-value 'theta-hat)
          (nlreg (send self :mean-function) 
                 y 
                 (slot-value 'theta-hat) 
                 (send self :epsilon)
                 (send self :count-limit)
                 w
                 (send self :verbose)))
    (setf (slot-value 'x) 
          (funcall (make-jacobian (slot-value 'mean-function) 
                                  (length (slot-value 'theta-hat))) 
                   (slot-value 'theta-hat)))
    (setf (slot-value 'intercept) nil)
    (call-next-method)
    (let ((r (send self :residuals)))
      (setf (slot-value 'residual-sum-of-squares)
	    (sum (* inc r r))))))

;;;
;;; Slot Accessors and Mutators
;;;

(defmeth nreg-model-proto :new-initial-guess (guess)
"Message args: (guess)
Sets a new initial guess for parmeters."
  (setf (slot-value 'theta-hat) guess)
  (send self :needs-computing t))

(defmeth nreg-model-proto :theta-hat ()
"Message args: ()
Returns current parameter estimate."
  (if (send self :needs-computing) (send self :compute))
  (coerce (slot-value 'theta-hat) 'list))

(defmeth nreg-model-proto :mean-function (&optional f)
"Message args: (&optional f)
With no argument returns the mean function as supplied to m. With an 
argument F sets the mean function of m to F and recomputes the
estimates."
  (when (and f (functionp f))
        (setf (slot-value 'mean-function) f)
        (send self :needs-computing t))
  (slot-value 'mean-function))

(defmeth nreg-model-proto :epsilon (&optional eps)
"Message args: (&optional eps)
With no argument returns the tolerance as supplied to m. With an argument
EPS sets the tolerance of m to EPS and recomputes the estimates."
  (when (and eps (numberp eps))
        (setf (slot-value 'epsilon) eps)
        (send self :needs-computing t))
   (slot-value 'epsilon))

(defmeth nreg-model-proto :count-limit (&optional count)
"Message args: (&optional new-count)
With no argument returns the iteration count limit as supplied to m. With
an argument COUNT sets the limit to COUNT and recomputes the
estimates."
  (when (and count (numberp count))
        (setf (slot-value 'count-limit) count)
        (send self :needs-computing t))
  (slot-value 'count-limit))

(defmeth nreg-model-proto :parameter-names (&optional (names nil set))
"Method args: (&optional names)
Sets or returns parameter names."
  (if set (setf (slot-value 'predictor-names) names))
  (let ((p-names (slot-value 'predictor-names))
        (p (length (slot-value 'theta-hat))))
    (if (not (and p-names (= p (length p-names))))
        (setf (slot-value 'predictor-names)
              (mapcar #'(lambda (a) (format nil "Parameter ~a" a)) 
                      (iseq 0 (- p 1))))))
  (slot-value 'predictor-names))

(defmeth nreg-model-proto :verbose (&optional (val nil set))
"Method args: (&optional val)
Sets or retrieves verbose setting. If T iteration info is printed during
optimization."
  (if set (setf (slot-value 'verbose) val))
  (slot-value 'verbose))

;;;
;;; Overrides for Linear Regression Methods
;;;

(defmeth nreg-model-proto :x ()
"Message args: ()
Returns the Jacobian matrix at theta-hat."
   (call-next-method))

(defmeth nreg-model-proto :intercept (&rest args)
"Message args: ()
Always returns nil. (For compatibility with linear regression.)"
  nil)

(defmeth nreg-model-proto :fit-values () 
"Message args: ()
Returns the fitted values for the model."
  (coerce (funcall (send self :mean-function) (send self :theta-hat)) 
          'list))

(defmeth nreg-model-proto :coef-estimates (&optional guess)
"Message args: (&optional guess)
With no argument returns the current parameter estimate. With an 
argument GUESS takes it as a new initial guess and recomputes
the estimates."
  (if guess (send self :new-initial-guess guess)) 
  (send self :theta-hat))

(defmeth nreg-model-proto :predictor-names () (send self :parameter-names))

;;;;
;;;;
;;;; Linear Regression Coefficients
;;;;
;;;;

(defun regression-coefficients (x y &key (intercept T) weights)
"Args: (x y &key (intercept T) weights)
Returns the coefficients of the regression of the sequence Y on the columns of
the matrix X."
  (let* ((m (if weights
                (make-sweep-matrix x y weights)
                (make-sweep-matrix x y)))
         (n (array-dimension x 1)))
    (coerce (compound-data-seq
	     (if intercept
		 (select (car (sweep-operator m (iseq 1 n)))
			 (1+ n)
			 (iseq 0 n))
 	         (select (car (sweep-operator m (iseq 0 n)))
			 (1+ n)
			 (iseq 1 n))))
            'vector)))

;;;;
;;;;
;;;; Nonlinear Regression Functions
;;;;
;;;;
(defun nlreg1 (f j y initial-beta epsilon count-limit weights verbose)
"Args: (mean-function jacobian y initial-beta 
        epsilon count-limit weights verbose)
MEAN-FUNCTION returns the mean response vector for a given parameter vector.
JACOBIAN returns the jacobian of MEAN-FUNCTION at a given parameter vector.
Y is the observed response vector. Returns the estimated parameter vector 
obtained by a Gauss-Newton algorithm with backtracking that continues until
the COUNT-LIMIT is reached or no component of the parameter vector changes
by more than EPSILON."
  (labels ((rss (beta)                   ; residual sum of squares
                (let ((res (- y (funcall f beta)))) 
                  (sum (if weights (* res res weights) (* res res)))))
           (next-beta (beta delta rss)   ; next beta by backtracking
                      (do* ((lambda 1 (/ lambda 2))
                            (new-rss (rss (+ beta delta)) 
                                     (rss (+ beta (* lambda delta)))))
                           ((or (< new-rss rss) (< lambda 1e-6))
                            (if (>= lambda 1e-6)
                                (+ beta (* lambda delta))
                                beta)))))
    (do* ((delbeta (regression-coefficients
                    (funcall j initial-beta)
                    (- y (funcall f initial-beta))
                    :intercept nil
                    :weights weights)
                   (regression-coefficients
                    (funcall j beta)
                    (- y (funcall f beta))
                    :intercept nil
                    :weights weights))
          (beta initial-beta (next-beta beta delbeta rss))
          (rss (rss beta) (rss beta))
          (count 0 (1+ count)))
         ((or (> count count-limit) (> epsilon (max (abs delbeta))))
          (if (and verbose (> count count-limit))
              (format t "Iteration limit exceeded.~%"))
          beta)
         (if verbose
	     (let ()
	       (format t "Residual sum of squares: ~,6g~%" rss)
	       (format t "Coefficients: ~,6g~%" beta)
	       )
	   )
	 )))

(defun make-jacobian (f n)
"Args: (f n)
F is a function of an N-vector. Returns a function that approximates the
jacobian function of F by a symmetric difference."
  (let* ((h 1e-6)
         (del (* h (column-list (identity-matrix n)))))
    #'(lambda (b) 
      (let ((b+ (mapcar #'(lambda (x) (+ b x)) del))
            (b- (mapcar #'(lambda (x) (- b x)) del)))
        (apply #'bind-columns (/ (- (mapcar f b+) (mapcar f b-)) (* 2 h)))))))

(defun nlreg (f y guess &optional 
                (epsilon 1e-6) (count-limit 20) weights verbose)
"Args: (mean-function y guess &optional 
        (epsilon 1e-6) (count-limit 20) weights verbose)
MEAN-FUNCTION returns the mean response vector for a given parameter vector.
Y is the observed response vector. Returns the estimated parameter vector 
obtained by a Gauss-Newton algorithm that continues until the ITERATION-LIMIT
is reached or no component of the parameter vector changes by more than
EPSILON. The jacobian of MEAN-FUNCTION is approximated by a symmetric difference."
  (nlreg1 f (make-jacobian f (length guess)) y guess 
          epsilon count-limit weights verbose))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(defun nreg-minimize (b nlfunc data y
			&optional (square t) (count-limit 20))
  "
	This is to be a general non-linear regressor for binary data in the
case that data have been put into classes (as I've been doing with the big data
sets, e.g. NHIS: I've been turning 120000 men into 27000 men.
"
  (nreg-model
   #'(lambda(b)
       (let* (
	      (estimates (nlfunc b data))
	      (crud
	       (if square
		   (+
		    (* events (- 1 estimates) (- 1 estimates))
		    (* non-events estimates estimates)
		    )
		 (+
		  (* events (abs (- 1 estimates)))
		  (* non-events (abs estimates))
		  )
		 )
	       )
	      )
	 (sqrt crud)
	 )
       (repeat 0 (length (first data)))
       b
       :count-limit count-limit)
   )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                       logistic-model
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun logistic-model
  (x y &key weights guess (epsilon 1e-6) (count-limit 20) (print t) naive)
  "
;; Uses the nreg-model to create a logistic model.
  (x y &key weights guess (epsilon 1e-6) (count-limit 20) (print t) naive)

	At the moment, the trouble is that the standard errors (and perhaps
other things) are incorrect. The naive method will get the coefficients wrong,
but gives a closer fix on the standard errors.

;; See also: Use logistic!

;; Examples:
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;
 ;; From page 3-, Hosmer and Lemeshow, Applied Logistic Regression
 ;;
 (def chd '(0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 1 0 0 0 0 0 1 0 0 1 0 0
   0 0 1 0 1 0 0 0 0 0 1 0 0 1 0 0 1 1 0 1 0 1 0 0 1 0 1 1 0 0 1 0 1 0 0 1 1 1
   1 0 1 1 1 1 1 0 0 1 1 1 1 0 1 1 1 1 0 1 1 1 1 1 0 1 1 1))
 (def age '(20 23 24 25 25 26 26 28 28 29 30 30 30 30 30 30 32 32 33 33 34 34
   34 34 34 35 35 36 36 36 37 37 37 38 38 39 39 40 40 41 41 42 42 42 42 43 43
   43 44 44 44 44 45 45 46 46 47 47 47 48 48 48 49 49 49 50 50 51 52 52 53 53
   54 55 55 55 56 56 56 57 57 57 57 57 57 58 58 58 59 59 60 60 61 62 62 63 64
   64 65 69)) 
 (logistic-model (list age) chd :guess '(-7 .2) :print nil)
 (logistic-model (list age) chd :guess '(-7 .2))
 ;; -5.310 0.111 coefficients
 ;;  1.134 0.024 standard errors
 ;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;
 ;; From page 322, Afifi and Clark, Computer-Aided Multivariate Analysis:
 ;; 183 women, 111 men; 1, depressed; 0, not depressed:
 ;;
 (def xvals (combine (repeat 1 183) (repeat 0 111)))
 (def yvals
   (combine (repeat 1 40) (repeat 0 143) (repeat 1 10) (repeat 0 101)))
 (logistic-model (list xvals) yvals :guess (list -2 1))
 ;; (-2.3125445 1.0385787): Note that 1.039 = ln((40/143)/(10/101)),
 ;; and that -2.313 = ln(10/101):
 ;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
"
  (let (
	(weights (if weights (- weights) -1)) ;; note negative: it's necessary!
	(data (if (matrixp x) (column-list x) x))
	)
    #|
Note: the naive function doesn't work; the way I got this to work, sort of, was
to go back to the function that is being maximized by function logistic, and
re-write it so that nreg would minimize the appropriate squared errors. That
gets the estimates right, but the standard errors are wrong.
|#
    (nreg-model
     (if naive
	 #'(lambda(b)
	     (/ (+ 1 (exp (- (+ (first b) (apply #'+ (* (rest b) data))))))))
       #'(lambda(x)
	   (let* (
		  (a (first x))
		  (b (rest x))
		  )
	     (sqrt (*
		    weights
		    (ln
		     (abs
		      (-
		       1
		       (/ (+ 1 (exp (- (+ a (apply #'+ (* b data)))))))
		       y
		       )))))
	     )
	   )
       )
     (if naive y (repeat 0 (length y)))
     (if guess guess (repeat 0 (1+ (length data))))
     :count-limit count-limit
     :epsilon epsilon
     :print print
     )
    )
  )

#|
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Test of nreg-model
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Does it give the same thing in linear regression as in nreg with a linear
;; function?
(def x1 (rseq 0 10 100))
(def y1 (+ (* 2 x1) (rand.u 100)))
(defun freg (b) (+ (* (select b 1) x1) (select b 0))) ;; bx+a
(def puromycin (nreg-model #'freg y1 (list 1 1)))
(def purolinear (regression-model x1 y1))
;; Okay, that checks out.
|#

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                        :joint-coef-probs-function
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth nreg-model-proto :joint-coef-probs-function (&key (b 0))
  "
;; Description: regression-model-proto :joint-coef-probs-function (&key (b 0))
		  
	joint-coef-probs-function: see Draper and Smith, Applied Regression
Analysis, 2nd Edition, p. 484:

             f(b) = f-cdf[(beta-b)'X'X(beta-b) / (p*s^2), n, p]

	The joint-coef-probs-function is between 0 and 1: if you plot a contour
line at 1-alpha, then it gives the region in which the parameters will fall
1-alpha of the time. Obviously, as alpha gets smaller, the region will get
bigger.

				f(b) = 1-alpha

;; Examples:
 (title \"Exercise K, p. 521\")
 (def xs (list 4 10 17 22 25))
 (def ys (list 5 20 45 66 85))
 (def lreg (regress (ln xs) (ln ys)))
 (def beta (send lreg :beta))
 (def nreg
     (nreg-model
      (lambda(theta)
 	(let ((alpha (first theta))
 	      (beta (second theta))
 	      )
 	  (* alpha (^ xs beta))
 	  ))
       ys
      (list (exp (first beta)) (second beta))
       )
      )
 (def pf (send nreg :joint-coef-probs-function))
 (spin-points 20)
 (contour (lambda(x y) (funcall pf (list x y)))
 	 :levels '(.75 .95 .99 .995))
"
  (let* (
	 (f (send self :mean-function))
	 (p (send self :num-coefs))
	 (n-p (send self :n-p))
	 (y (send self :y))
	 (resss (send self :resss))
	 )
    (lambda(x) (f-cdf 
		(* n-p (/ p) (1- (/ (sum (square (- y (funcall f x)))) resss)))
		p n-p))
    )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                        :s-theta
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth nreg-model-proto :s-theta(&key (alpha .05))
  "
;; Description: regression-model-proto :s-theta(&key (alpha .05))

             S(theta) = S-hat(theta)*(1 + p/(n-p)*(f-quant(1-alpha,p,n-p)))

Parameters giving this sum of squares are at the border of the alpha confidence
region for the parameters (S-hat is the minimum sum-of-squares).

;; See Draper and Smith, Applied Regression Analysis, 2nd Edition, p. 492.

;; Examples:
 (title \"Exercise K, p. 521\")
 (def xs (list 4 10 17 22 25))
 (def ys (list 5 20 45 66 85))
 (def lreg (regress (ln xs) (ln ys)))
 (def beta (send lreg :beta))
 (def nreg
     (nreg-model
      (lambda(theta)
 	(let ((alpha (first theta))
 	      (beta (second theta))
 	      )
 	  (* alpha (^ xs beta))
 	  ))
       ys
      (list (exp (first beta)) (second beta))
       )
      )
 (send nreg :s-theta)

 (def x (iseq 1 7 ))
 (def y (list 82 112 153 163 176 192 200))
 (def nreg
    (nreg-model
	(lambda(theta)
	  (let ((alpha (first theta)) (beta (second theta)))
	    (* alpha (- 1 (exp (- (* x beta)))))
	    )
	  )
	 y
	 (list (mean y) 0)
	 )
    )
 (send nreg :s-theta)
"
  (let* (
	 (p (send self :num-coefs))
	 (n-p (send self :n-p))
	 (y (send self :y))
	 (resss (send self :resss))
	 )
    (* resss (+ 1 (* p (/ n-p) (f-quant (- 1 alpha) p n-p))))
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                        :regression-function
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth nreg-model-proto :regression-function(vars)
  "
;; Description: regression-model-proto :regression-function(vars)

;; Examples:
 (new-plot)
 (title \"Exercise K, p. 521\")
 (def xs (list 4 10 17 22 25))
 (def ys (list 5 20 45 66 85))
 (def lreg (regress (ln xs) (ln ys)))
 (def beta (send lreg :beta))
 (def nreg
     (nreg-model
      (lambda(theta)
 	(let ((alpha (first theta))
 	      (beta (second theta))
 	      )
 	  (* alpha (^ xs beta))
 	  ))
       ys
      (list (exp (first beta)) (second beta))
       )
      )
 (plot (send nreg :regression-function '(xs)) (min xs) (max xs))
 (send my-plot :add-plot-item (list 'points xs ys 'blue))
 (send my-plot :update-plot)
"
  (let* (
	 (me self)
	 (mf (send self :mean-function))
	 (le (get-lambda-expression mf))
	 (var (second le))
	 (exp (third le))
	 (exp (recursive-grep-replace var (list 'hippo-lover) exp))
	 (hippo-lover (send self :beta))
	 )
    (eval `(lambda ,vars (let ((hippo-lover (list ,@hippo-lover))) ,exp)))
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                :plot
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth nreg-model-proto :plot (var &optional xmin xmax)
  "
;; Description: :plot (var &optional xmin xmax)

	This method allows one to plot the regression model in a 1x1
regression. 

	You need to give this function the name of the variables that you used
in the mean-function. The mean function is defined as a function of the
parameters: we want the regression function, which is a function of the
variable x (or whatever you called it when you defined it).

	In the examples below the x variable is called 'xs', so we pass a list
containing the name 'xs'.

;; Notes: Nreg-model doesn't even have a slot for the x variable values. It
;; works just with the mean-function (which will be based on the x-values, of
;; course).

;; Examples:
 (title \"Exercise K, p. 521\")
 (def xs (list 4 10 17 22 25))
 (def ys (list 5 20 45 66 85))
 (def lreg (regress (ln xs) (ln ys)))
 (def beta (send lreg :beta))
 (def nreg
     (nreg-model
      (lambda(theta)
 	(let ((alpha (first theta))
 	      (beta (second theta))
 	      )
 	  (* alpha (^ xs beta))
 	  ))
       ys
      (list (exp (first beta)) (second beta))
       )
      )
 (send nreg :plot '(xs))
"
  (andy-print "entering :plot for nreg-model-proto")
  (let* (
	 (x (eval (first var)))
	 (domain (get-nice-range (min x) (max x) 5))
	 (xmin (if xmin xmin (first domain)))
	 (xmax (if xmax xmax (second domain)))
	 (y (send self :y))
	 (range (get-nice-range (min y) (max y) 5))
	 (myplot (plotter (send self :regression-function var) xmin xmax))
	 (n (length x))
	 (xmean (mean x))
	 (ssx (sum (square (- x xmean))))
	 (s (sqrt (send self :s-squared)))
	 )
    (andy-print "inside :plot for nreg-model-proto")
    (send myplot :add-plot-item (list 'points (list (mean x)) (list (mean y))
				      'blue 'disk))
    (send myplot :add-plot-item (list 'points x y 'black 'dot4))
    (send myplot :range 1 (first range) (second range))
    (send myplot :plot-item)
    (andy-print "leaving :plot for nreg-model-proto")
    myplot
    )
  )


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                nreg-model-proto :display ()
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
#|
	I just stole the usual :display method for nreg, since I modified the
method for standard regression-model-proto.
|#
(defmeth nreg-model-proto :display ()
"Message args: ()
Prints the least squares regression summary. Variables not used in the fit
are marked as aliased."
  (let ((coefs (coerce (send self :coef-estimates) 'list))
        (se-s (send self :coef-standard-errors))
        (x (send self :x))
        (p-names (send self :predictor-names)))
    (if (send self :weights) 
        (format t "~%Weighted Least Squares Estimates:~2%")
        (format t "~%Least Squares Estimates:~2%"))
    (when (send self :intercept)
          (format t "Constant~25t~13,6g~40t(~,6g)~%" (car coefs) (car se-s))
          (setf coefs (cdr coefs))
          (setf se-s (cdr se-s)))
    (dotimes (i (array-dimension x 1)) 
             (cond 
               ((member i (send self :basis))
                (format t "~a~25t~13,6g~40t(~,6g)~%"
                        (select p-names i) (car coefs) (car se-s))
                (setf coefs (cdr coefs) se-s (cdr se-s)))
               (t (format t "~a~25taliased~%" (select p-names i)))))
    (format t "~%")
;;    (format t "R Squared:~25t~13,6g~%" (send self :r-squared))
    (format t "Sigma hat:~25t~13,6g~%" (send self :sigma-hat))
    (format t "Number of cases:~25t~9d~%" (send self :num-cases))
    (if (/= (send self :num-cases) (send self :num-included))
        (format t "Number of cases used:~25t~9d~%" (send self :num-included)))
    (format t "Degrees of freedom:~25t~9d~%" (send self :df))
    (format t "~%")))

