;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                pde.lsp
;;
#|
	It is my intention to fill this up with (at least) the usual
finite-difference methods, maybe some finite-element stuff.

	I'm having a look at the stuff that Burden and Faires did in their book
'Numerical Analysis' (fifth edition): I have their fortran code, which I've
managed to convert to xlispstat. Only problem is that their stuff isn't very
good! I should probably have a look at numrec....

	At any rate, here are three of B&F's algorithms, used to solve the
examples in B&F. 

	Actually I deviated from their method on the poisson-pde, in what I
think a great improvement on their code. Rather than create a large, sparse,
linear system, I simply work within a full matrix. You can do all the linear
algebra entailed in their algorithm easily there. Check the documentation in
the code.
|#
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				poisson-pde
;;
(defun poisson-pde
  (f a b c d n m g
     &key
     (eps 1e-12)
     (maxits 200)
     jacobi
     (verbose nil)
     )
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			poisson-pde Doc:
;;
;; Author: Andy Long
;; Created: Wed Feb 11 21:46:11 1998
;;
;; Description/Arguments: poisson-pde
  
;; Arguments: f a b c d n m g
     &key
     (eps 1e-12)
     (maxits 200)
     jacobi
     (verbose nil)

Approximates the solution of

	u_{xx} + u_{yy} = f(x,y)

subject to rectangular boundary conditions u(x,y) = g(x,y) on R /

	R = {(x,y) | a <= x <= b and c <= y <= d}

by discretizing the domain, dividing the x-interval [a,b] into n sub-intervals,
and the y-interval [c,d] into m sub-intervals.

;; Returns: matrix of size (m+1)x(n+1), the solution (including the boundary),
   obtained using Gauss-Seidel iteration (Jacobi is also available).

;; Notes: Burden and Faires, 'Numerical Analysis' (fifth edition), p. 625. This
   solves the problems on pages 629 and 633, for the moment.

;; See Also:

;; Examples:
 ;; page 629:
 (defun g(x y)
  (cond
   ((or (= y 0)(= x 0)) 0)
   ((= x .5) (* 200 y))
   ((= y .5) (* 200 x))))

 (plot (poisson-pde (func f(x y) 0) 0 .5 0 .5 6 4 #'g))
 (plot (poisson-pde (func f(x y) 0) 0 .5 0 .5 4 6 #'g))

 (title \"p. 633, Burden and Faires\")
 (defun g(x y)
  (cond
   ((= x 0) 0)
   ((= y 0) x)
   ((= x 2) (* 2 (exp y)))
   ((= y 1) (* (exp 1) x))))

 (def result (poisson-pde (func f(x y) x*(exp y)) 0 2 0 1 6 5 #'g
               :verbose t))
 (plot result)
 (plot (poisson-pde (func f(x y) x*(exp y)) 0 2 0 1 6 5 #'g
   :jacobi t :verbose t))

 ;; right on!
 (mprint result)
"
  (let* (
	 ;; Step-sizes:
	 (h (/ (- b a) n))
	 (k (/ (- d c) m))
	 (r-sq (square (/ h k)))

	 ;; the diagonal element:
	 (di (/ (* 2 (1+ r-sq))))

	 ;; and a useful constant:
	 (d-r-sq (* di r-sq))

	 ;; Easier to work with m and n augmented by one:
	 (1+m (1+ m))
	 (1+n (1+ n))
	 (iseq1+m (iseq 1+m))
	 (iseq1+n (iseq 1+n))
	 
	 ;; matrix will contain our starting values for u (equivalent to
	 ;; assuming that the initial solution is 0 in the iterative process):
	 (matrix 
	  (* h (- h) di
	     (transpose (outer-product (rseq a b 1+n) (rseq d c 1+m) f))))
	 ;; A remark: had to transpose to make it run in the right sense:

	 ;; The f matrix will be the same thing: each time it is added to the
	 ;; matrix of u values:
	 (fmatrix
	  (select
	   (matrix (size matrix) (combine matrix))
	   (iseq 1 (- m 1)) (iseq 1 (- n 1))))
	 )
    
    ;; introduce the boundary values on the outer edges of our solution matrix:
    (setf 
     (select matrix iseq1+m '(0))
     (matrix (list 1+m 1) (mapcar g (repeat a 1+m) (rseq c d 1+m)))
     (select matrix iseq1+m (list (1- 1+n)))
     (matrix (list 1+m 1) (mapcar g (repeat b 1+m) (rseq d c 1+m)))
     (select matrix '(0) iseq1+n)
     (matrix (list 1 1+n) (mapcar g (rseq a b 1+n) (repeat d 1+n)))
     (select matrix (list m) iseq1+n)
     (matrix (list 1 1+n) (mapcar g (rseq a b 1+n) (repeat c 1+n)))
     )

    (do*
     (
      (i 0 (1+ i))
      ;; matrix now contains our initial guess on its interior, which we
      ;; will call inconnues:
      (inconnues (select matrix (iseq 1 (- m 1)) (iseq 1 (- n 1)))
		 (select matrix (iseq 1 (- m 1)) (iseq 1 (- n 1))))
      (old-norm 1e31 norm)
      (norm (sqrt (sum (square inconnues))) (sqrt (sum (square inconnues))))
      )
     ((or 
       (< (abs (- norm old-norm)) eps)
       (> i maxits)
       ) (if verbose (format t "~%Number of Iterations: ~d~%" i)))
     
     ;; we replace our old approximation with the new approximation,
     ;; effectively using Jacobi iteration only without recourse to the sparse
     ;; matrix of size (m-1)*(n-1). Instead we simply update the interior of
     ;; the matrix as a weighted sum of shifted versions of the matrix
     ;; (including the edge, or the boundary).
     (if jacobi
	 (setf ;; set the interior of the matrix
	  (select matrix (iseq 1 (- m 1)) (iseq 1 (- n 1)))
	  ;; to the sum
	  (+ 
	   ;; of the weighted left and right matrices
	   (* di
	      (+
	       (select matrix (iseq 1 (- m 1)) (iseq 0 (- n 2)))
	       (select matrix (iseq 1 (- m 1)) (iseq 2       n))
	       ))
	   ;; and of the weighted upp and down matrices:
	   (* d-r-sq
	      (+
	       (select matrix (iseq 0 (- m 2)) (iseq 1 (- n 1)))
	       (select matrix (iseq 2       m) (iseq 1 (- n 1)))
	       ))
	   ;; subtracting off the constant matrix with the rhs (I say
	   ;; subtracting because I already tacked a minus sign into the
	   ;; definition of fmatrix).
	   fmatrix)
	  )

       ;; In the event that we do Gauss-Seidel instead, we just do the same
       ;; thing element by element: this is just the little cross that is used
       ;; to approximate the derivatives:
       ;;
;;; u_{ij} =
;;;  di(u_{(i+1)j}+u_{(i-1)j}+r-sq(u_{i(j+1)}+u_{i(j-1)})-h^2f_{(i-1)(j-1)})
       ;;
       ;; where f is indexed a little screwily as it's not the same size as the
       ;; others.
       (dolist
	(i (iseq 1 (- m 1)))
	(dolist
	 (j (iseq 1 (- n 1)))
	 (setf (aref matrix i j)
	       (+ 
		(* di
		   (+
		    (aref matrix i (1- j))
		    (aref matrix i (1+ j))
		    ))
		(* d-r-sq
		   (+
		    (aref matrix (1- i) j)
		    (aref matrix (1+ i) j)
		    ))
		(aref fmatrix (1- i) (1- j))
		)
	       )
	 )
	)
       )
     )
    matrix
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				crank-nicolson
;;
(defun crank-nicolson(f alpha m l n TT)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			crank-nicolson Doc:
;;
;; Author: Andy Long
;; Created: Fri Feb 13 09:07:11 1998
;;
;; Description/Arguments: crank-nicolson
  
;; Arguments: f alpha m l n TT

;; Returns: An approximation to the solution of the parabolic PDE

		u_{t} - \alpha^2 u_{xx} = 0

   subject to boundary conditions u(0,t)=u(l,t)=0, for t up to TT, and to the
   initial condition

		u(x,0)=f(x).

;; Notes: Burden and Faires, 'Numerical Analysis' (fifth edition), p. 644.

;; See Also: poisson-pde

;; Examples:
 ;; solves the problem on pages 645:
 (pprint (crank-nicolson (func f(x) (sin (pi*x))) 1 10 1 50 .5))
"
  (let* (
	 (xs (rseq 0 l (1+ m)))
	 (ts (rseq 0 TT (1+ n)))
	 
	 (h (/ l m))
	 (k (/ TT n))
	 (lambda (* alpha alpha (/ k (* h h))))

	 ;; initial values:
	 (w (mapcar f xs))

	 ;; get the boundary condition right:
	 (w (combine 0 (rest (butlast w)) 0))
	 (u (copy-list w))
	 (l (copy-list w))
	 (z (copy-list w))
	 )

    (dolist
     (i (iseq 1 (- m 1)))
     (setf
      (elt l i) (+ 1 (* lambda (+ 1 (* .5 (elt u (1- i))))))
      (elt u i) (/ lambda (* -2 (elt l i)))
      )
     )
    (do
     ((j 1 (1+ j)))
     ((> j N))
     (dolist
      (i (iseq 1 (1- m)))
      (setf (elt z i) (/ (+ 
			  (* (- 1 lambda) (elt w i))
			  (* lambda .5 (+ (elt w (1- i))
					  (elt w (1+ i))
					  (elt z (1- i))
					  )))
			 (elt l i)))
      )
     (dolist
      (i (iseq (1- m) 1))
      (setf (elt w i) (- (elt z i) (* (elt u i) (elt w (1+ i)))))
      )
     )
    w
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				wave-pde
;;
(defun wave-pde(f g alpha m l n TT &key verbose)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			wave-pde Doc:
;;
;; Author: Andy Long
;; Created: Fri Feb 13 09:07:11 1998
;;
;; Description/Arguments: wave-pde
  
;; Arguments: f g alpha m l n TT &key verbose

;; Returns: An approximation to the solution of the parabolic PDE

		u_{tt} - \alpha^2 u_{xx} = 0

   subject to boundary conditions u(0,t)=u(l,t)=0, for t up to TT, and to the
   initial conditions

		u(x,0)=f(x)
   and
		u'(x,0)=g(x).

;; Notes: Burden and Faires, 'Numerical Analysis' (fifth edition), p. 652.

;; See Also: poisson-pde, crank-nicolson

;; Examples:
 ;; solves the problem on page 653:
 (def wave (wave-pde (func f(x) (sin (pi*x))) (func g(x) 0) 2 10 1 20 1))
 (pprint (last wave))
"
  (let* (
	 (xs (rseq 0 l (1+ m)))
	 (ts (rseq 0 TT (1+ n)))
	 
	 (h (/ l m))
	 (k (/ TT n))
	 (lambda (/ (* alpha k) h))
	 (lambda (if (<= lambda 1) lambda
		   (error "Method is not convergent with lambda > 1.
In function wave-pde.")))

	 (lambda-sq (* lambda lambda))

	 ;; initial values:
	 (w0 (mapcar f xs))
	 ;; first step:
	 (w1 (+
	      (* (- 1 lambda-sq) (rest (butlast w0)))
	      (* .5 lambda-sq (+ (rest (rest w0)) (butlast (butlast w0))))
	      (mapcar g (rest (butlast xs)))
	      ))
	 ;; get the boundary condition right:
	 (w1 (combine 0 w1 0))
	 (w2 (copy-list w1))
	 (result (list w0 w1))
	 )

    (do*
     (
      (j 1 (1+ j))
      (u0 w0 u1)
      (u1 w1 u2)
      (u2 (copy-list w2) (copy-list w2))
      )
     ((>= j N))
     (setf
      (select u2 (iseq 1 (- m 1)))
      (+
       (* 2 (- 1 lambda-sq) (select u1 (iseq 1 (- m 1))))
       (* lambda-sq (+ (rest (rest u1)) (butlast (butlast u1))))
       (- (select u0 (iseq 1 (- m 1))))
       )
      )
     (setq result (append result (list (copy-list u2))))
     (if verbose (if verbose (print u2)))
     )
    result
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
