;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;					Plotter
;;				anything! (just about)
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun plotter (&rest x)
  "
	Plotter plots just about anything.

;; See also:
	plot
	plotit
	plot-variables
	new-plot (resets the parameters above to nil)
 
;; Examples:
 (new-plot)
 (plotter (uniform-rand 10) (uniform-rand 10))
 (plotter (uniform-rand 10))

 (setq *title* \"Here's my plot!\")
 (setq *lines* 't)
 (range -2 2)
 (domain -2 2)
 (plotter (uniform-rand 10) (uniform-rand 10))
 (domain -1 11)
 (plotter (uniform-rand 10))

 (new-plot)
 (plotter (uniform-rand 10) (uniform-rand 10)  (uniform-rand 10))
 (plotter (id 4))

 (new-plot)
 (defun f (x y) (+ (cos x) (sin y)))
 (plotter #'f)

 (defun f (x) (cos x))
 (derivfunc 'fprime #'f 'x)
 (plotter #'f #'fprime)
 (plotter #'f #'fprime -pi pi)

;; plotter vectors of real or complex components
 (new-plot)
 (def my-array (coerce (iseq 0 15) 'array))		; make the array
 (plotter my-array)
 (def my-array (fft my-array))
 (plotter my-array)
"
  (let* (
	 (args (first (catch-keys '(:special-plot-args) x)))
	 (x (remove-keys '(:special-plot-args) x))
	 (len (length x))
	 )
    (cond
     ((= len 0)
      ;; If no arguments are given, then just set up my standard scatterplot:
      (def my-plot (send andyplot-proto :new 2 :title *title* :show nil))
      (send my-plot :mouse-mode *mouse-mode*)
      (send my-plot :range '(0 1) '(-1 -1) '(1 1))
      (mapcar (lambda(arg) (apply #'send my-plot arg)) args)
      my-plot
      )
     ((= len 1) (plotit (first x) :special-plot-args args))
     (t (plotit x :special-plot-args args))
     )
    )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				Plot:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmacro plot (&rest args)
  "
;; Description:

	Plot is designed to be a universal program: it should do something
interesting for almost any reasonable thing you give it. It will plot matrices,
vectors, functions, what-have-you (but not comic books).

	Optional arguments:

		spin-domain

;; See also: plotter, plot-variables, new-plot (resets plot parameters)

;; Examples:
 (new-plot) ;; initializes the plot environment
 (lines t)(range -2 2)(domain -2 2)
 (title \"Plotting User-defined Functions\")
 (func f(x) cos x)
 (plot f)

 (title \"Plot points:\")
 (lines off)
 (plot (rand.u 100) (rand.n 100))

 (new-plot) ;; re-initialize
 (title \"Easy to specify an interval\")
 (diff f x fprime)
 (plot f fprime -pi pi)

 (new-plot)
 (title \"Spin-plots for 3-D data:\")
 (func f(x y) (cos x + sin y))
 (plot f)
 (lines off)
 (plot (rand.u 100) (rand.n 100) (rand.n 100))

 (new-plot)
 (title \"Matrix plots:\")
 (plot (id 4))

 (title \"plot vectors of real/complex components\")
 (def my-array (coerce (iseq 0 15) 'array))		; make the array
 (plot my-array)
 (def my-array (fft my-array))
 (plot my-array)

 (title \"Plot standard functions, e.g. sine:\")
 (plot sin)

 (title \"Plot lambda expressions, or funcs:\")
 (plot (func f(x) (if (x = 0) nil (x - ln x))))
"
  (let* (
	 (plot-arg-names '(:location :size :show-window :spin-domain))
	 (plot-arg-vals (catch-keys plot-arg-names args))
	 (worth-keeping (which plot-arg-vals))
	 (special-plot-args
	  (if worth-keeping 
	      (transpose
	       (list (select plot-arg-names worth-keeping)
		     (select plot-arg-vals worth-keeping)))))
	 (special-plot-args (mapcar #'combine special-plot-args))
	 (args (remove-keys plot-arg-names args))

	 (y args)
	 (i 0)
	 bing
	 sd
	 )
    (dolist
     (xel args)
     (setq bing xel)
     (if (and (symbolp bing) (not (boundp bing))) ;; it's a function!
	 (cond
	  ((eq (eval `(type-of #',bing)) 'SUBR)
	   (setq bing  (eval `(function (lambda (x) (,bing x)))))
	   (setl y i bing)
	   )
	  ((eq (eval `(type-of #',bing)) 'BYTE-CODE-CLOSURE)
	   (setq bing  (eval `(function (lambda (x) (,bing x)))))
	   (setl y i bing)
	   )
	  (t
	   (list 
	    (setq bing `#',bing)
	    (setl y i bing)
	    ))))
     (setq i (1+ i))
     )
    (if special-plot-args
	`(plotter ,@y :special-plot-args ',special-plot-args)
      `(plotter ,@y))
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			Plot-points/lines variables:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;----------------------------------------------------------------------
(defun show-plots (&optional title)
  "
Sets plotting on or off for the plot command.
"
  (if title (setq *show-plots* title) (setq *show-plots* nil))
  )
;;----------------------------------------------------------------------
(defun title (&optional title)
  "
Sets or unsets the title for plot.
"
  (if title (setq *title* title) (setq *title* nil))
  )
;;----------------------------------------------------------------------
(defun x-axis (&optional x-axis)
  "
Sets or unsets the x-axis for plot. Toggles.
"
  (if x-axis (setq *x-axis* t) (setq *x-axis* nil))
  )
;;----------------------------------------------------------------------
(defun y-axis (&optional y-axis)
  "
Sets or unsets the y-axis for plot. Toggles.
"
  (if y-axis (setq *y-axis* t) (setq *y-axis* nil))
  )
;;----------------------------------------------------------------------
(defun range (&optional ymin ymax)
  "
Sets or unsets the range for plot.
"
  (if ymin (setq *range* (list ymin ymax)) (setq *range* nil))
  )
;;----------------------------------------------------------------------
(defun domain (&optional xmin xmax)
  "
Sets or unsets the domain for plot.
"
  (if xmin (setq *domain* (list xmin xmax)) (setq *domain* nil))
  )
;;----------------------------------------------------------------------
(defun polar-range (&optional ymin ymax)
  "
Sets or unsets the range for a polar plot.
"
  (if ymin (setq *polar-range* (list ymin ymax)) (setq *polar-range* nil))
  )
;;----------------------------------------------------------------------
(defun polar-domain (&optional xmin xmax)
  "
Sets or unsets the domain for a polar plot.
"
  (if xmin (setq *polar-domain* (list xmin xmax)) (setq *polar-domain* nil))
  )
;;----------------------------------------------------------------------
(defun points (&optional npnts)
  "
Sets or unsets the number of points for plot.
"
  (if npnts (setq *points* npnts) (setq *points* nil))
  )
;;----------------------------------------------------------------------
(defun x-tics (&optional npnts)
  "
Sets or reports the number of points for plot.
"
  (if npnts (setq *x-tics* npnts) *x-tics*)
  )
;;----------------------------------------------------------------------
(defun y-tics (&optional npnts)
  "
Sets or reports the number of points for plot.
"
  (if npnts (setq *y-tics* npnts) *y-tics*)
  )
;;----------------------------------------------------------------------
(defun polar-points (&optional npnts)
  "
Sets or unsets the number of polar-points for plot.
"
  (if npnts (setq *polar-points* npnts) (setq *polar-points* nil))
  )
;;----------------------------------------------------------------------
(defun lines (&optional lines)
  "
Toggles lines on or off in plot.
"
  (if lines (setq *lines* t) (setq *lines* nil)))
;;----------------------------------------------------------------------
(defun line-width (&optional linewidth)
  "
Sets line-width for plots.
"
  (if linewidth (setq *line-width* linewidth) *line-width*)
  )
;;----------------------------------------------------------------------
(defun line-type (&optional linetype)
  "
Sets line-type for plots.
"
  (if linetype (setq *line-type* linetype) *line-type*)
  )
;;----------------------------------------------------------------------
(defun zoom (&optional inf outf)
  "
Sets zoom factors to user-defined or defaults

;; Examples: to zoom in by factor of .5, out by twice,
 (zoom .5 2)

"
  (if inf
      (list
       (setq *zoom-in-factor* inf)
       (setq *zoom-out-factor* outf)
       )
    (list
     (setq *zoom-in-factor* .9)
     (setq *zoom-out-factor* 1.1)
     )
    )
  )
;;----------------------------------------------------------------------
(defun shift (&optional inf)
  "
Sets shift factors to user-defined or default

;; Examples: to shift by factor of .1
 (shift .1)

"
  (if inf
      (setq *shift-factor* inf)
    (setq *shift-factor* .1)
    )
  )

;;----------------------------------------------------------------------
(defun line-colors (&optional colors)
  "
Sets or uses default for the line-colors. If you have a preference, then you
would set it using this function, as follows:

;; Examples:
 (line-colors '(black blue yellow red))
;; In this case, at most four colors would be used, in the order given.
 (line-colors) ;; resets the line-colors to the default: effectively
;; (line-colors '(black blue red yellow cyan magenta green))
"
  (if colors (setq *line-colors* colors)
    (setq *line-colors* '(black blue red yellow cyan magenta green)))
  )

;;----------------------------------------------------------------------
(defun draw-color (&optional color)
  "
Sets or uses default for the draw-color. For example,
;; Examples:
 (draw-color 'black)
;; 
 (draw-color) ;; resets the draw color to the default (blue)
"
  (if color (setq *draw-color* color) (setq *draw-color* 'blue)))

;;----------------------------------------------------------------------
(defun mouse-mode (&optional mode)
  "
Sets or uses default for the usual mouse-mode for command plot
;; Examples:
 (mouse-mode 'zoom-in) ;; the default
"
  (if mode (setq *mouse-mode* mode) *mouse-mode*)
  )

;;----------------------------------------------------------------------
(defun use-color (&optional use-color)
  "
Toggles use-color on or off in plots.
"
  (if use-color (setq *use-color* t) (setq *use-color* nil)))


;;----------------------------------------------------------------------
(defun plot-symbol (&optional symbol)
  "
Sets or uses default for the plot-symbol. Choices are given in *plot-symbols*.
;; Examples:
 *plot-symbols*
"
  (if symbol (setq *plot-symbol* symbol) (setq *plot-symbol* 'disk)))

;;----------------------------------------------------------------------
(defun point-labels (&optional labels)
  "
Sets or uses default for the point-labels.
"
  (if labels (setq *point-labels* labels) (setq *point-labels* nil)))

;;----------------------------------------------------------------------
(defun variable-labels (&optional labels labels2)
  "
Sets or uses default for the point-labels.

;; Examples:
 (variable-labels '(\"eye\" \"nose\"))
 (variable-labels \"x\" \"y\")
"
  (if labels2 (setq *variable-labels* (list labels labels2))
    (if labels (setq *variable-labels* labels)
      (if *variable-labels* (setq *variable-labels* nil)
	(setq *variable-labels* '("x" "y"))
	)))
  )
(defun true-axes-labels (&rest labels)
  "
Sets or uses default for the true-axes-labels.

;; Examples:
 (true-axes-labels '(\"ear\" \"nose\" \"throat\"))
 (plot (func f(x y) (x*y)))
"
  (if labels
      (cond
       ((= 4 (length labels))
	(setq *true-axes-labels* labels)
	)
       ((= 3 (length labels))
	(setq *true-axes-labels* (combine "O" labels))
	)
       ((= 1 (length labels))
	(setq *true-axes-labels* (combine "O" labels))
	)
       (t
	*true-axes-labels*
	)
       )
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			Spin-function variables:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;----------------------------------------------------------------------
(defun spin-axes (&optional arg)
  "
Sets or unsets the spin-axes for plot.
"
  (if arg (setq *spin-axes* t) (setq *spin-axes* nil)))

;;----------------------------------------------------------------------
(defun real-axes (&optional arg)
  "
Sets or unsets the real-axes for plot.
"
  (if arg (setq *real-axes* t) (setq *real-axes* nil)))

;;----------------------------------------------------------------------
(defun spin-depth (&optional (arg nil))
  "
Sets or unsets the spin-depth for plot.
"
  (if arg (setq *spin-depth* t) (setq *spin-depth* nil)))

;;----------------------------------------------------------------------
(defun spin-domain (&optional xmin xmax ymin ymax)
  "
Sets or reports the spin-domain for plot.
"
  (if xmin (setq *spin-domain* (list xmin xmax ymin ymax)) *spin-domain*)
  )

;;----------------------------------------------------------------------
(defun spin-points (&optional npnts)
  "
Sets or uses default for the number of spin-points for plot.
"
  (if npnts (setq *spin-points* npnts) (setq *spin-points* 6)))

;;----------------------------------------------------------------------
(defun spin-spline (&optional npnts)
  "
Sets or uses default for the number of spin-spline for plot.
"
  (if npnts (setq *spin-spline* npnts) (setq *spin-spline* nil)))
;;----------------------------------------------------------------------
(defun spin-symbol (&optional symbol)
  "
Sets or uses default for the spin-symbol for 3-d plot.
"
  (if symbol (setq *spin-symbol* symbol) (setq *spin-symbol* 'x)))

;;----------------------------------------------------------------------
(defun contour-points (&optional npnts)
  "
Sets or uses default for the number of contour-points for plot.
"
  (if npnts (setq *contour-points* npnts) (setq *contour-points* 30)))

;;----------------------------------------------------------------------
(defun contour-levels (&optional npnts)
  "
Sets or uses default for the number of contour-levels for plot.
"
  (if npnts (setq *contour-levels* npnts) (setq *contour-levels* 10)))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;		A few useful matrix plotting routines
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Description for MATRIX-PLOT:
;;
(defun matrix-plot (a-matrix &rest args &key grid-vectors spline idle)
  "
;; Documentation:

     matrix-plot (a-matrix &rest args &key grid-vectors spline idle)

Matrix-plot creates a set of three-d data from a matrix, using the indices,
and the matrix value. This is useful for plotting Abe, for example. Returns
the plot.

;; Examples:
 (def a (matrix '(5 3) (+ (rseq 0 3 15) (normal-rand 15))))
 (matrix-plot a)
 (matrix-plot a :title \"howdy\" :depth-cuing 't)
 (matrix-plot a :title \"howdy\")
 (matrix-plot a :depth-cuing 't)
 (matrix-plot a :depth-cuing 't :title \"howdy\" :idle t)
" 
  (let* (
	 (args (if grid-vectors (remove-keys '(:grid-vectors) args) args))
	 (args (if spline (remove-keys '(:spline) args) args))
	 (args (if idle (remove-keys '(:idle) args) args))
	 (plot (apply #'send andyspin-proto :new 3 :show nil args))
	 (grid-vectors
	  (if grid-vectors grid-vectors
	    (list
	     (iseq 1 (length (first (column-list a-matrix))))
	     (iseq 1 (length (column-list a-matrix)))
	     )
	    )
	  )
	 )
    (send plot :andy-options (list ':axes t))
    (send plot :plot-item (list a-matrix))
    (apply #'send plot :add-surface 
	   (first grid-vectors)
	   (second grid-vectors)
	   a-matrix
	   :spline spline :draw nil args)
    (send plot :adjust-to-data :draw nil)
    (send plot :new-menu)
    (send plot :depth-cuing nil)
    (send plot :rotate-2 1 2 (/ pi -2))
    (send plot :show-window)
    (if idle
	(let ()
	  (send plot :do-idle)
	  (send plot :idle-on t)
	  ))
    ;;ael    (if args (apply #'send plot args))
    ;; getting an Error: too many arguments
    ;; Happened in: #<Subr: #810b9a8>
    (if args
	(mapcar #'(lambda(x) 
		    (if (send plot :has-method (first x))
			(send plot (first x) (second x))))
		(split-list args 2)))
    ;;ael
    plot
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Description for MATRIX-POINT-PLOT:
;;
(defun matrix-point-plot (a-matrix &rest args &key idle)
  "
Documentation:

      MATRIX-POINT-PLOT (a-matrix &rest args &key idle)

matrix-point-plot creates a set of three-d data from a matrix, using the
indices, and the matrix value. This is useful for plotting Abe, for
example. Returns the plot.

;; Examples:
 (def a (matrix '(5 3) (+ (rseq 0 3 15) (normal-rand 15))))
 (matrix-point-plot a)
 (matrix-point-plot a :depth-cuing t)
 (matrix-point-plot a :title \"howdy!\")
 (matrix-point-plot a :depth-cuing t :title \"howdy!\" :idle t)
" 
  (let* (
	 (args (if idle (remove-keys '(:idle) args) args))
	 (xlen (length (column-list a-matrix)))               ; 
	 (ylen (length (first (column-list a-matrix))))
	 (z (combine (column-list a-matrix)))
	 ;; points will be indexed from 1, by 1:
	 (y (repeat (iseq ylen) xlen))
	 (x (combine (mapcar (lambda(x) (repeat x ylen)) (iseq xlen))))
	 (plot (spin-plot (list x y z)))
	 )
    (dolist (i (iseq 2 xlen))
	    (setq x (combine x (repeat i ylen)))
	    )
    (send plot :andy-options (list ':axes t))
    (send plot :plot-item (list a-matrix))
    (send plot :depth-cuing nil)
    (if idle
	(let ()
	  (send plot :do-idle)
	  (send plot :idle-on t)
	  ))
    ;;ael    (if args (apply #'send plot args))
    ;; getting an Error: too many arguments
    ;; Happened in: #<Subr: #810b9a8>
    (if args
	(mapcar #'(lambda(x) 
		    (if (send plot :has-method (first x))
			(send plot (first x) (second x))))
		(split-list args 2)))
    ;;ael
    plot
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;		A couple of Handy derivative plotters: dplot is king
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun fplot (f xmin xmax)
  "
fplot plots the function and derivative on (xmin, xmax).

;; Examples:
 (lines on)
 (title \"2*sin(x) and derivative\")
 (func f(x) 2 * sin x)
 (fplot #'f -pi pi)
"
  (let (
	(var (second (get-lambda-expression f)))
	)
    (derivfunc 'fprime f (first var))
    (plot (list f #'fprime xmin xmax))
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun fpplot (f xmin xmax)
  "
fpplot plots the function, derivative, and second derivative on (xmin, xmax).

;; Examples:
 (lines on)
 (title \"2*sin(x) and derivatives\")
 (func f(x) 2 * sin x)
 (fpplot #'f -pi pi)

 (title \"ln(x) and derivatives\")
 (func f(x) ln x)
 (fpplot #'f 1 pi)
"
  (let (
	(var (second (get-lambda-expression f)))
	)
    (derivfunc 'fprime f (first var))
    (derivfunc 'fpprime #'fprime (first var))
    (plot (list f #'fprime #'fpprime xmin xmax))
    )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun fppplot (f xmin xmax)
  "
fppplot plots the function, first, second, and third derivatives
on (xmin, xmax).

;; Examples:
 (lines on)
 (title \"2*sin(x) and derivatives\")
 (func f(x) 2*sin x)
 (fppplot #'f -pi pi)

 (title \"ln(x) and derivatives\")
 (func f(x) ln x)
 (fppplot #'f 1 pi)
"
  (let (
	(var (first (second (get-lambda-expression f))))
	)
    (derivfunc 'fprime f var)
    (derivfunc 'fpprime #'fprime var)
    (derivfunc 'fppprime #'fpprime var)
    (plot (list f #'fprime #'fpprime #'fppprime xmin xmax))
    )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
(defmacro dplot (f &rest options)
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  "

		(dplot f &optional xmin xmax (nderivs 1))

dplot plots the function and derivative(s) (up to 3) on (xmin, xmax).

;; See also: fplot, fpplot, fppplot; plot; alias

;; Examples:

 (new-plot)
 (lines t)

 (defun f (x) (+ (+ (^ x 3) (- (* 2 x))) 3))
 (dplot f -2 3)
 (dplot f 3)
 (dplot f)
 (dplot f -1 2 2)

 (title \"2*sin(x) and derivative\")
 (func f (x) 2 * sin x)
 (dplot f -pi pi)
 (dplot f -pi pi 2)
 (dplot f -pi pi 3)

 (func f (x) x ^ 3)
 (dplot f -pi pi)
 (dplot f -pi pi 2)
 (dplot f -pi pi 3)

 (dplot sin)
 (dplot log .5 2 3)
"
  (let (nderivs xmin xmax)
    (if options
	(cond
	 ((= (length options) 1)
	  (setq nderivs (eval (first options)))
	  (if *domain*
	      (list (setq xmin (first *domain*)) (setq xmax (second *domain*)))
	    (list (setq xmin -1) (setq xmax 1)))
	  )
	 ((= (length options) 2)
	  (setq nderivs 1)
	  (setq xmin (first options))
	  (setq xmax (second options))
	  )
	 ((= (length options) 3)
	  (setq xmin (first options))
	  (setq xmax (second options))
	  (setq nderivs (eval (third options)))
	  )
	 (t
	  (error "Sorry, wrong number of optional arguments to dplot.~2%Type~2%
     (help dplot)~2%to see some examples of the proper use of dplot.~%")
	  )
	 )
      (if *domain*
	  (list
	   (setq nderivs 1)
	   (setq xmin (first *domain*))
	   (setq xmax (second *domain*)))
	(list (setq xmin -1) (setq xmax 1) (setq nderivs 1)))
      )
    (cond
     ((= nderivs 3) (eval `(alias dplot.func fppplot)))
     ((= nderivs 2) (eval `(alias dplot.func fpplot)))
     (t (eval `(alias dplot.func fplot)))
     )
    (if (symbolp f)
	(cond
	 ((eq (eval `(type-of #',f)) 'SUBR)
	  `(dplot.func #'(lambda (x) (,f x)) ,xmin ,xmax))
	 ((eq (eval `(type-of #',f)) 'BYTE-CODE-CLOSURE)
	  `(dplot.func #'(lambda (x) (,f x)) ,xmin ,xmax))
	 (t `(dplot.func #',f ,xmin ,xmax))
	 )
      `(dplot.func ,f ,xmin ,xmax))
    )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;	2-D derivatives (and group of function) plotters (splot is king):
;;
;;		splotter
;;		splot
;;
;;		spin-andy (1001 uses! replaces spin-function)
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				Splotter
;;
(defun splotter (f &optional xmin xmax ymin ymax xcenter ycenter)
  "
;; Examples:

 (spin-points 10)
 (spin-domain -pi pi -pi pi)
 (spin-depth nil)

 (title \"f(x y) = 2*(sin x)*y\")
 (defun f (x y) (* (* 2 (sin x)) y))
 (splotter #'f)

 (title \"f(x y) = x^2+y^2\")
 (defun f (x y) (+ (^ x 2) (^ y 2)))
 (splotter #'f)
"
  (let* (
	 (vars (second (eval `(get-lambda-expression ,f))))
	 (xvar (first vars))
	 (yvar (second vars))
	 fpxval fpyval fpointval save-spin-domain
	 )

    (eval `(derivfunc 'fpx ,f ',xvar))
    (eval `(derivfunc 'fpy ,f ',yvar))

    (if xcenter ;; plot the tangent plane, instead:
	(list
	 (setq fpxval (eval `(fpx ,xcenter ,ycenter)))
	 (setq fpyval (eval `(fpy ,xcenter ,ycenter)))
	 (setq fpointval (eval `(,f ,xcenter ,ycenter)))
	     
	 (setq save-spin-domain *spin-domain*)
	 (spin-domain  xmin xmax ymin ymax)
	 (def my-plot
	      (eval
	       `(plot ,f
		      #'(lambda ,vars
			  (+ ,fpointval (* ,fpxval (- ,xvar ,xcenter))
			     (* ,fpyval (- ,yvar ,ycenter)))
			  )
		      )
	       )
	      )
	 (setq *spin-domain* save-spin-domain)
	 )
      (if (not xmin)
	  (def my-plot (eval `(plot (list ,f #'fpx #'fpy))))
	(list
	 (setq save-spin-domain *spin-domain*)
	 (spin-domain  xmin xmax ymin ymax)
	 (def my-plot (eval `(plot ,f #'fpx #'fpy)))
	 (setq *spin-domain* save-spin-domain)
	 )
	)
      )
    )
  my-plot
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;					Splot
;;
(defmacro splot (f &optional xmin xmax ymin ymax xcenter ycenter)
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  "
;; macro splot

;; Description: 
;; (defmacro splot (f &optional xmin xmax ymin ymax xcenter ycenter)

;; Examples:
 (spin-points 10)
 (spin-domain -pi pi -pi pi)
 (spin-depth nil)

 (title \"f(x y) = 2*(sin x)*y\")
 (defun f (x y) (* (* 2 (sin x)) y))
 (splot f)

 (title \"f(x y) = x^2+y^2\")
 (defun f (x y) (+ (^ x 2) (^ y 2)))
 (splot f)
 (splot f -pi 2pi -pi 2pi)
;; the following will plot the tangent plane at (1,1):
 (splot f -pi pi -pi pi 1 1)
"
  (cond
   ((and xmin xcenter)
    `(splotter #',f ,xmin ,xmax ,ymin ,ymax ,xcenter ,ycenter))
   (xmin
    `(splotter #',f ,xmin ,xmax ,ymin ,ymax))
   (t
    `(splotter #',f))
   )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
#|
;; formerly:
(defmacro splot (f &optional xmin xmax ymin ymax)
  (if xmin `(splotter #',f ,xmin ,xmax ,ymin ,ymax)
    `(splotter #',f))
  )(help 'splot)
|#

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				spin-andy
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun spin-andy (flist xmin xmax ymin ymax &key (num-points 6) (spline 3))
  "
;; Args: (flist xmin xmax ymin ymax &key (num-points 6) (spline 3))

;; Description: function

	(spin-andy
		flist xmin xmax ymin ymax &key (num-points 6) (spline 3))

	This is spin-function for a list of functions: I had trouble when I
tried to use spin-function and then :add-function, so I went to the source and
rewrote spin-function.

	Rotatable plot of functions in Flist of two real variables over the
range between [xmin, xmax] x [ymin, ymax]. The function is evaluated at
NUM-POINTS points. If SPLINE is not NIL a spline is fit at (* SPLINE NUMPOINTS)
points.
"
  (let ((plot (apply #'send andyspin-proto :new 3 :show nil nil)))
    (dolist
     (f flist)
     (apply #'send plot :add-function f xmin xmax ymin ymax :draw nil
	    :num-points num-points :spline spline nil))
    (send plot :adjust-to-data :draw nil)
    (send plot :new-menu)
    (send plot :rotate-2 0 1 (/ pi 3) :draw nil)
    (send plot :rotate-2 1 2 (- (/ pi 3)) :draw nil)
    (send plot :show-window)
    plot
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				diffplot
;;
(defmacro diffplot(f &rest e)
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
"
;; Description: macro

	(diffplot f &rest e)

diffplot is meant to tak

;; See also: dplot, splot

;; Examples:
;; one-d:
 (new-plot)
 (func f(x) x ^ 3 - 2 * x + 3)
 (title \"f(x) = x^3 - 2*x + 3\")
 (diffplot f)
 (diffplot f -3 2)
 (diffplot f -3 2 2)
 (diffplot f -3 2 3)

;; two-d
 (func ff(x y) (cos x + sin y))
 (title \"f(x y) = (cos x + sin y)\")
 (spin-points 20)
 (spin-axes nil)
 (spin-domain -pi pi -pi pi)

 (diffplot ff -pi pi -pi pi 1.57 1.57)
"
  (let* (
	 (vars (second (eval `(get-lambda-expression #',f))))
	 )
    (if e
	(if (= 1 (length vars)) (eval `(dplot ,f ,@e)) (eval `(splot ,f ,@e)))
      (if (= 1 (length vars)) (eval `(dplot ,f)) (eval `(splot ,f)))
      )
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; contour Doc:    Creation Date:1/23/96        Author: Andy Long
;;
;; modified to take advantage of improvements in contour-matrix, 11/6/96. AEL.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun contour2 (f
		 &key
		 xmin xmax ymin ymax
		 x
		 y
		 (levels 10)
		 (labels t)
		 (color *use-color*)
		 )
  "
;; Description/Arguments: function

          (contour2 f &key (levels 10) xmin xmax ymin ymax)
		 x
		 y
		 (labels t) (color *use-color*)
  
workhorse for contour.

;; Notes: also plots the function (as a matrix)

;; See Also: contour-matrix, contour

;; Examples:
 (func f(x y) x + y)
 (spin-domain -pi pi -pi pi)
 (setq *contour-points* 30)
 (setq *contour-levels* 10)
 (contour2 #'f)
 (func g(x y) sin x + cos y)
 (setq *contour-levels* 30)
 (contour2 #'g)
"
  (let* (
	 (levels (if levels levels *contour-levels*))
	 (x (if x x
	      (rseq 
	       (if xmin xmin (first *spin-domain*))
	       (if xmax xmax (second *spin-domain*))
	       *contour-points*)))
	 (y (if y y 
	      (rseq
	       (if ymin ymin (third *spin-domain*))
	       (if ymax ymax (fourth *spin-domain*))
	       *contour-points*)))
	 my-contour-plot
	 result
	 )
    (let ((z (outer-product x y `,f)))
      (setq my-contour-plot
	    (contour-matrix z
			    :x x
			    :y y
			    :levels levels
			    :labels labels
			    :color color
			    ))
      (send my-contour-plot :title (if *title* *title* "contour"))
      (send my-contour-plot :location 400 20)
;;ael      (list my-contour-plot (plot `,z))
;; otherwise the coordinates don't correspond!
      (list my-contour-plot (plot `,f))
;;ael
      )
    )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; contour Doc:    Creation Date:1/23/96        Author: Andy Long
;;
;; modified to take advantage of improvements in contour-matrix, 11/6/96. AEL.
;; Also modified to use a function (contour2) rather than operate solo.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmacro contour (f &rest args)
  "
;; Description/Arguments:

          (contour f
		 &key
		 xmin xmax ymin ymax
		 x
		 y
		 (levels 10)
		 (labels t)
		 (color *use-color*)
  
;; Notes: also plots the function (as a matrix)

;; See Also: contour-matrix, contour2 (workhorse for contour)

;; Examples:
 (func f(x y) x + y)
 (spin-domain -pi pi -pi pi)
 (setq *contour-points* 30)
 (setq *contour-levels* 10)
 (contour f)
 (func g(x y) sin x + cos y)
 (setq *contour-levels* 30)
 (contour g)
 (contour g :xmin -pi :xmax pi)
"
  `(contour2 #',f ,@args)
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; scatterplot methods that facilitate labels in contours (contour-matrix):
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth andyplot-proto :add-surface-contour
  (x y z v &key (draw t) labels (color 'black) fast)
  (let ((c (surface-contour x y z v)))
    (setq c  (split-list c 2))
    (if labels
	(let ()
	  (setq c (make-lines-from-scraps c))
	  (dolist
	   (i c)
	   (let ((j (transpose i)))
	     (send self :add-plot-item
		    (list labels (first (first j)) (first (second j)))
		    (list labels (final (first j)) (final (second j)))
		    (list
		     'lines
		     (first j)
		     (second j)
		     color
		     )
		    ))))
      (if fast
	  (dolist (i c) (send self :add-lines (transpose i) :draw nil))
	(dolist (i c)
		(send self :add-plot-item (list
					   'lines
					   (first (transpose i))
					   (second (transpose i))
					   color
					   )
		      )
		)
	)
      )
    )
  (if draw (send self :redraw-content))
  )

(defmeth andyplot-proto :add-surface-contours
  (x y z &optional v &key (draw t) labels fast (color t) (decimals 2))
  (let ((v (if v 
               (if (numberp v) (list v v) (coerce v 'list))
               (let ((min (min z))
                     (max (max z)))
                 (+ min (* (- max min) '(.2 .4 .6 .8))))))
	(colors (if color *line-colors*))
	)
    (dolist (v v)
	    (let ()
	      (if color (setq colors (circulate colors)))
	      (if labels
		  (send self
			:add-surface-contour x y z v
			:draw nil
			:labels (format nil (strcat "~," (num-to-string decimals) "g") v)
			:color (if color (first colors) 'black)
			:fast fast
			)
		(send self
		      :add-surface-contour x y z v
		      :draw nil
		      :color  (if color (first colors) 'black)
		      :fast fast
		      )
		)
	      )
	    )
    (if draw
	(let()
	  (send self :plot-item)
	  (send self :adjust-to-data)
	  )
      )
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			make-lines-from-scraps
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun make-lines-from-scraps (pairs)
  "  
	This function was created to make pieces of contour plots into
whole lines. It takes snippets of a line and looks for pieces that
join to them.
  
;; Examples:
 (make-lines-from-scraps '(((0 1)(1 2)) ((0 1)(1 2)) ((4 1)(4 2)) ((0 1)(2 1))))
"
  (let (
	(results nil)
	(newresults nil)
	(others pairs)
	)
    (tagbody
     (dotimes
      (i (length pairs))
      (if others
	  (let ()
	    (setq newresults (make-lines-from-scrap others))
	    (setq results (append results (first newresults)))
	    (setq others (second newresults))
	    )
	(go done)
	)
      )
     done
     )
    results
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			make-lines-from-scrap
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun make-lines-from-scrap (pairs)
  "
	This function was created to make pieces of contour plots into
whole lines. It takes a snippet of a line and looks for pieces that
join to it.
  
;; Examples:
 (make-lines-from-scrap '(((0 1)(1 2)) ((0 1)(1 2)) ((4 1)(4 2)) ((0 1)(2 1))))
"
  (let* (
	 (pair (first pairs))
	 (p1 (first pair))
	 (p2 (second pair))
	 (result (list p1 p2))
	 (others nil)
	 )
    (dolist
     (i (rest pairs))
     (let* (
	    (p3 (first i))
	    (p4 (second i))
	    )
       (if (equal p1 p3)
	   (let ()
	     (setq result (append (list p4) result))
	     (setq p1 p4)
	     )
	 (if (equal p1 p4)
	     (let ()
	       (setq result (append (list p3) result))
	       (setq p1 p3)
	       )
	   (if (equal p2 p3)
	       (let ()
		 (setq result (append result (list p4)))
		 (setq p2 p4)
		 )
	     (if (equal p2 p4)
		 (let ()
		   (setq result (append result (list p3)))
		   (setq p2 p3)
		   )
	       (setq others (append others (list i)))
	       )
	     )
	   )
	 )
       )
     )
    (list (list result) others)
    )
  )


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; contour-matrix Doc:    Creation Date:1/23/96        Author: Andy Long
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun contour-matrix
  (z
   &key
   x
   y
   labels
   (levels 10)
   title
   (color *use-color*)
   fast
   (decimals 2)
   (variable-labels *variable-labels*)
   )
  
  "
;; Description/Arguments:

 (contour-matrix 
  (matrix
   &key
   x
   y
   labels
   (levels 10)
   title
   (color *use-color*)
   fast
   (decimals 2)
   (variable-labels *variable-labels*)
   )
  
	Produces a contour plot of a matrix. Options:
x - specify the row values (corresponding to rows)
y - specify the column values (corresponding to values)
levels - either a number of a list of levels - defaults to 10 levels
labels - whether you want labels on your lines or not
fast - won't use color or labels (won't work if labels are turned on)
color - use colors (otherwise black). This will be t if *use-colors*
	is true, and nil otherwise.
decimals - use two decimals for labels

;; Notes:

;; See Also: contour

;; Examples:
 (contour-matrix (id 4) :labels t)

 (load (strcat *andy-default-path* \"examples\" separator \"abe.lsp\"))
 (contour-matrix abemat :levels 10)
 (contour-matrix abemat :levels 20 :fast t)
 (contour-matrix abemat :x (iseq 1 49) :y (iseq 1 36) :levels 30 :fast t)
"
  (let* (
	 (v (if (listp levels) levels (rseq (min z) (max z) levels)))
	 (x (if x x (iseq (first (size z)))))
	 (y (if y y (iseq (second (size z)))))
	 )
    (def my-plot (apply #'send andyplot-proto :new 2 :show nil nil))
    (send my-plot :andy-options (list ':axes t))
    (send my-plot :plot-item nil)
    (send my-plot :add-surface-contours x y z v
	  :draw t :labels labels :fast fast :color color :decimals decimals)
    (send my-plot :title (if title title *title*))
    (if variable-labels 
	(send my-plot :variable-label '(0 1) variable-labels))
    (send my-plot :adjust-to-data :draw nil)
    (send my-plot :new-menu)
    (send my-plot :show-window)
    my-plot)
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; section Doc:    Creation Date:1/26/96        Author: Andy Long
;;
;;
(defun section2(f which-variable value &optional (section-name 'my-func))
  "
;; Description/Arguments:

       (section2 f which-variable value &optional section-name)

      The idea of section is to turn two-variable functions into univariate
functions: slices, or cross-sections for a particular value of one of the
variables.
  
;; Notes:

;; See Also: slice

;; Examples:
 (func ff (x y) x + y)
 (section2 #'ff 1 0)
 (section2 #'ff 1 0 'gg)
"
  (let* (
	 (var (second (eval `(get-lambda-expression ,f))))
	 inde
	 )
    (cond
     ((= which-variable 1)
      (setq inde (second var))
      (eval `(defun ,section-name (,inde) (,f ,value ,inde)))
      )
     (t
      (setq inde (first var))
      (eval `(defun ,section-name (,inde) (,f ,inde ,value)))
      )
     )
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; section Doc:    Creation Date:1/26/96        Author: Andy Long
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmacro section (f &rest stuff)
  "
;; Description/Arguments:

       (section g which-variable value &optional section-name)

      The idea of section is to turn two-variable functions into univariate
functions: slices, or cross-sections for a particular value of one of the
variables.
  
;; Notes:
      This macro is built on function section2.

;; See Also: slice, section2

;; Examples:
 (func ff (x y) x ^ 2 - y ^ 2)
 (section ff 1 0)
;; the default name of the function returned from section is my-func:
 (plot my-func)
;; but you may give it another name if you wish, e.g. gg:
 (section ff 1 0 gg)
 (plot gg)
"
  (setq last-stuff (last `,stuff))
  (setq but-last (butlast `,stuff))
  (if (= 2 (length stuff))
      `(section2 #',f ,@stuff)
    `(section2 #',f ,@but-last ',@last-stuff)
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				window
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun window (xmin xmax ymin ymax)
  "
Description: window
       simultaneously sets the domain and range

Usage:
        (window xmin xmax ymin ymax)
"
  (domain xmin xmax)
  (range ymin ymax)
  (list xmin xmax ymin ymax)
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; polar2 Doc:    Creation Date:2/25/96        Author: Andy Long
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun polar2 (f &key
		 rmin rmax
		 thmin thmax
		 my-plot
		 cartesian
		 (r-curves t)
		 (theta-curves t)
		 (r-points *polar-points*)
		 (theta-points *polar-points*)
		 (view '(210 -60))
		 )
  "
;; Description/Arguments:

	(polar2 f &key
		 rmin rmax
		 thmin thmax
		 my-plot
		 (r-curves t)
		 (theta-curves t)
		 (r-points *polar-points*)
		 (theta-points *polar-points*)
		 (view '(210 -60))
		 )

;; Notes: workhorse for polar

;; See Also: polar

;; Examples:
 (polar-points 20)
 (title \"A pair of cones\")
 (func f (r th) r)
 (polar2 #'f :rmin -1)

 (title \"Skirts?\")
 (func f (r th) r * sin (2 * th))
 (polar2 #'f :thmax pi)
 (func f (r th) r * sin th)
 (polar2 #'f :thmax 2pi)

 (title \"Paraboloid!\")
 (func f (r th) r ^ 2)
 (polar2 #'f :thmax pi)
  "
  (let (polyx polyy polyz polyr polyt)
    (if (not my-plot)
	(setq my-plot (apply #'send andyspin-proto :new 3 :show nil '()))
      (setq my-plot (first (active-graph-windows))))

    (if (not rmin) (setq rmin (first *polar-domain*)))
    (if (not rmax) (setq rmax (second *polar-domain*)))
    (if (not thmin) (setq thmin (first *polar-range*)))
    (if (not thmax) (setq thmax (second *polar-range*)))

    (setq polyr (rseq rmin rmax r-points))
    (setq polyt (rseq thmin thmax theta-points))
    (setq polyx (outer-product polyr (cos polyt)))
    (setq polyy (outer-product polyr (sin polyt)))
    (setq polyz (outer-product polyr polyt f))

    (if cartesian
	(send my-plot :add-function `,f
	      (min polyx) (max polyx)
	      (min polyy) (max polyy))
      (list 
       (if r-curves
	   (dotimes (i r-points)
		    (setq j (1+ i))
		    (setq orbit (list
				 (row j polyx) (row j polyy) (row j polyz)))
		    (send my-plot :add-lines orbit)
		    )
	 )
       (if theta-curves
	   (dotimes (i theta-points)
		    (setq j (1+ i))
		    (setq orbit
			  (list
			   (column j polyx)
			   (column j polyy)
			   (column j polyz)))
		    (send my-plot :add-lines orbit)
		    )
	 )
       )
      )
    (send my-plot :adjust-to-data :draw nil)
    (send my-plot :new-menu)
    (send my-plot :showing-axes t)
    (send my-plot :rotate-2 0 1 (/ (* ( first view) pi) 180) :draw nil)
    (send my-plot :rotate-2 1 2 (/ (* (second view) pi) 180) :draw nil)
    (send my-plot :show-window)
    (send my-plot :mouse-mode 'hand-rotate)
    (if *title* (send my-plot :title *title*))
    (if *spin-axes* (send my-plot :showing-axes t)
      (send my-plot :showing-axes nil))
    (if *spin-depth* (send my-plot :depth-cuing t)
      (send my-plot :depth-cuing nil))
    (send my-plot :real-axes)
    my-plot
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; polar Doc:    Creation Date:2/25/96        Author: Andy Long
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmacro polar (f &rest e)
  "
;; Description/Arguments:

	(polar f &key
		 rmin rmax
		 thmin thmax
		 (r-curves t)
		 (theta-curves t)
		 (r-points *polar-points*)
		 (theta-points *polar-points*)
		 (view '(210 -60))
		 )

  
;; Notes: workhorse for polar

;; See Also: polar

;; Examples:
 (polar-points 20)
 (title \"A pair of cones\")
 (func f (r th) r)
 (polar f :rmin -1)

 (title \"Skirts?\")
 (func f (r th) r * sin (2 * th))
 (polar f :thmax pi)
 (func f (r th) r * sin th)
 (polar f :thmax 2pi)

 (title \"Paraboloid!\")
 (func f (r th) r ^ 2)
 (polar f :thmax pi)
  "
  `(polar2 #',f ,@e)
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; view Doc:    Creation Date:2/27/96        Author: Andy Long
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun view (th phi)
  "
;; Description/Arguments:

 (view th phi)
  
;; Notes:

;; See Also:

;; Examples:
 (func f(r th) r)
 (polar f)
 (dotimes (i 360) (view 0 1))
 (dotimes (i 360) (view 1 0))
"
  (let ((my-plot (first (active-graph-windows))))
    (send my-plot :rotate-2 0 1 (/ (* th pi) 180) :draw nil)
    (send my-plot :rotate-2 1 2 (/ (* phi pi) 180) :draw nil)
    (send my-plot :redraw)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;	2-D quadratic approximations, for my multivariate class
;;
;;		s-taylor-too
;;		s-taylor
;;
;; A quick butcher of splot and splotter.
;;
;; Modified: 2/3/98 - ael
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				S-Taylor-Too
;;
(defun s-taylor-too (f &optional xmin xmax ymin ymax xcenter ycenter)
  "
;; Description:

	(s-taylor-too f &optional xmin xmax ymin ymax xcenter ycenter)

 Plots tangent or 2-D quadratic approximations to a function of two dimensions.

 It will plot the f and the quadratic fit if all optional arguments are
 given. If the centers are not given, then the function f and its partial
 derivatives will be plotted.

;; Examples:
 (spin-points 10)
 (spin-domain -pi pi -pi pi)
 (spin-depth nil)

 (func f(x y) 2*(sin x)*y)
 (title \"f(x y) = 2*(sin x)*y and partials\")
 (s-taylor-too #'f)
 (title \"f(x y) = 2*(sin x)*y and quadratic at (1,1)\")
 (s-taylor-too #'f -pi pi -pi pi 1 1)

 (title \"f(x y) = x^2+y^2\")
 (func f(x y) x^2+y^2)
 (s-taylor-too #'f)
"
  (let* (
	 (vars (second (get-lambda-expression f)))
	 (xvar (first vars))
	 (yvar (second vars))
	 (fpx (lambda-deriv f xvar))
	 (fpy (lambda-deriv f yvar))
	 (fpxx (lambda-deriv fpx xvar))
	 (fpxy (lambda-deriv fpx yvar))
	 (fpyy (lambda-deriv fpy yvar))
	 )
    
    (if xcenter ;; plot the quadratic
	(let (
	      (fpxval (funcall fpx xcenter ycenter))
	      (fpyval (funcall fpy xcenter ycenter))
	      (fpxxval (funcall fpxx xcenter ycenter))
	      (fpxyval (funcall fpxy xcenter ycenter))
	      (fpyyval (funcall fpyy xcenter ycenter))
	      (fpointval (funcall f xcenter ycenter))
	      (save-spin-domain *spin-domain*)
	      )
	  (spin-domain xmin xmax ymin ymax)
	  (def my-plot
	       (eval
		`(plot ,f
		       #'(lambda ,vars
			   (let ((xdiff (- ,xvar ,xcenter))
				 (ydiff (- ,yvar ,ycenter)))
			     (+ ,fpointval
				(* xdiff
				   (+ ,fpxval
				      (* .5 ,fpxxval xdiff)
				      (* ,fpxyval ydiff)))
				(* ydiff
				   (+ ,fpyval
				      (* .5 ,fpyyval ydiff)))
				)
			     )
			   )
		       )
		)
	       )
	  (setq *spin-domain* save-spin-domain)
	  )
      (if (not xmin)
	  (def my-plot (eval `(plot ,f ,fpx ,fpy)))
	(let ((save-spin-domain *spin-domain*))
	  (spin-domain  xmin xmax ymin ymax)
	  (def my-plot (eval `(plot ,f ,fpx ,fpy)))
	  (setq *spin-domain* save-spin-domain)
	  )
	)
      )
    )
  my-plot
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;					S-Taylor
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmacro s-taylor (f &optional xmin xmax ymin ymax xcenter ycenter)
  "
;; macro s-taylor

;; Description: macro 
	
	(s-taylor f &optional xmin xmax ymin ymax xcenter ycenter)

 Plots tangent or 2-D quadratic approximations to a function of two dimensions.

 It will plot the f and the quadratic fit if all optional arguments are
 given. If the centers are not given, then the function f and its partial
 derivatives will be plotted.

;; Examples:
 (spin-points 10)
 (spin-domain -pi pi -pi pi)
 (spin-depth nil)

 (title \"f(x y) = 2*(sin x)*y\")
 (func f(x y) 2*(sin x)*y)
 (title \"f(x y) = 2*(sin x)*y and partials\")
 (s-taylor f)
 (title \"f(x y) = 2*(sin x)*y and quadratic at (0,0)\")
 (s-taylor f -pi pi -pi pi 0 0)

 (title \"f(x y) = x^2+y^2\")
 (func f(x y) x^2+y^2)
 (s-taylor f)
"
  (cond
   ((and xmin xcenter)
    `(s-taylor-too #',f ,xmin ,xmax ,ymin ,ymax ,xcenter ,ycenter))
   (xmin
    `(s-taylor-too #',f ,xmin ,xmax ,ymin ,ymax))
   (t
    `(s-taylor-too #',f))
   )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                       parametric plots:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmacro parametric (f1 f2 &rest e)
  "
;; Description/Usage: macro parametric: either of 

		(parametric f1 f2 tstart tend tsteps)   ; 2-d
or
		(parametric f1 f2 f3 tstart tend tsteps); 3-d

	Generates and plots the path of a parametric curve.

;; See Also: parametric2

;; Examples:
 (func f(x) cos(3*x)) ;; x will serve as 't': remember, you can't use 't'!
 (func g(x) sin(5*x))
 (title \"Lissajous Curve\")
 (parametric f g 0 2pi 200) ;; plots (f(x),g(x)) as x goes from 0 to 2pi in
			    ;; 200 steps.
 (func h(x) x)
 (parametric f g h 0 2pi 200)

 (title \"A closed space curve\")
 (func f(x) (4+sin(3*x))*cos(2*x))
 (func g(x) (4+sin(3*x))*sin(2*x))
 (func h(x) cos(3*x))
 (parametric f g h 0 2pi 200)
"
  (let* (
	 (args `,e)
	 (ff1 (eval `(turn-it-into-a-function ,f1)))
	 (ff2 (eval `(turn-it-into-a-function ,f2)))
	 )
    (if (= (length args) 3) ;; we're in two dimensions
	(let (
	      (tstart (first args))
	      (tend (second args))
	      (tsteps (third args))
	      )
	  (plot (eval `(parametric2 ,ff1 ,ff2 ,tstart ,tend
				    ,tsteps)))
	  )
      (let* (
	     (f3 (first args))
	     (tstart (second args))
	     (tend (third args))
	     (tsteps (fourth args))
	     (ff3 (eval `(turn-it-into-a-function ,f3)))
	     )
	(let (
	      (my-plot
	       (plot (eval `(parametric2  ,ff1 ,ff2 ,ff3
					  ,tstart ,tend ,tsteps))))
	      )
	 (send my-plot :points-showing nil)
	 (send my-plot :add-lines
	       (eval `(parametric2  ,ff1 ,ff2 ,ff3 ,tstart ,tend ,tsteps)))
	 )))
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                        parametric2
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun parametric2 (&rest e)
  "
;; Description: function parametric2

;; Arguments: two or three functions, a starting and ending time, and number of
;; points.

;; Examples:
 (func f(x) cos x)
 (func g(x) sin x)
 (plot (parametric2 #'f #'g 0 (/ 2pi 1.2) 100))

 (func h(x) x)
 (plot (parametric2 #'f #'g #'h 0 (/ 2pi 1.2) 100))
"
  (let ((args `,e))
    (cond
     ((= (length args) 5)
      (let ((tvals (rseq (third args) (fourth args) (nth 4 args))))
	(list (mapcar (first args) tvals) (mapcar (second args) tvals))
	))
     ((= (length args) 6)
      (let ((tvals (rseq (fourth args) (fifth args) (nth 5 args))))
	(list (mapcar (first args) tvals)
	      (mapcar (second args) tvals) (mapcar (third args) tvals))
	))
     (t (error "Wrong number (~d) of arguments to function parametric2."
	       (length args)))
     )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 tile
;;
(defun tile(x
	    &key
	    ncol
	    nrow
	    height
	    width
	    (xshift 10)
	    (yshift 30)
	    (scale .9)
	    (title-bar 25)
	    linked
	    )
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; tile Doc:    Creation Date:1/2/97        Author: Andy Long
;;
;; Description/Arguments:

 (tile x &key
	    ncol		; sets the number of columns in the tile
	    nrow		; sets the number of rows in the tile
	    height		; sets the height of the windows
	    width		; sets the width of the windows
	    (xshift 10)		; a shift in where the windows start in x
	    (yshift 30)		; a shift in where the windows start in y
	    (scale .9)		; scale scales the plots
	    (title-bar 25)	; allows a certain number of pixels for title
	    linked		; links plots for comparison
            )

where x is a list of plots. This plots them all in such a way that they are all
visible, and about as big as they can be.

;; Notes:

;; See Also:

;; Examples:
 (tile (list (plot {sin(x)}) (plot {x}) (plot {x^2}) (plot {x^3})))
 ;; will make a 2x2 tiling.
"
  (let* (
	 (screen-width #+unix 1000 #+macintosh 620 #+msdos 620)
	 (screen-height #+unix 800 #+macintosh 420 #+msdos 420)
         (plots (if (matrixp x) (matrix-to-list x) x))
         (n (length plots))
         (ncol (if ncol ncol
		 (if (matrixp x) (second (size x)) (greatest-divisor n))))
         (nrow (if nrow nrow
		 (if (matrixp x) (first (size x)) (/ n ncol))))
         (width (if width width (/ screen-width ncol)))
         (width (floor width))
         (height (if height height (/ screen-height nrow)))
         (height (floor height))
	 (indices
	  (matrix-to-list
	   (outer-product
	    (iseq ncol) (iseq nrow) #'(lambda(x y) (list x y)))
	   )
	  )
	 )
    (mapcar #'(lambda(x)
		(send x :size
		      (floor (* scale width))
		      (floor (* scale height)))
		)
	    plots)
    (setq height (+ title-bar height))
    (if linked (mapcar #'(lambda(x) (send x :linked t)) plots))

    (setq
     indices
     (select
      (if (< (length indices) n)
	  (smoosh (mapcar (lambda(x) indices) (iseq n)))
	indices
	)
      (iseq n)
      )
     )

    (dolist
     (x (weave plots indices))
     (let (
	   (plot (first x))
	   (location (second x))
	   )
       (send plot :location
	     (+ xshift (* (first location) width))
	     (+ yshift (* (second location) height)))
       )
     )
    (mapcar (lambda(x) (send x :show-window)) plots)
    t
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 make-title
;;
(defun make-title(s &key (yper 1.05) xper plot sub-title)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; make-title Doc:    Creation Date:1/6/97        Author: Andy Long
;;
;; Description/Arguments:

       (make-title string &key (yper 1.05) xper plot sub-title)
  
   I've rigged up a title maker that aspires to put a label on the plot
in about the top center. This has not been tested much!

   yper adjusts how high the title is placed above the top of the box (that's
why it's up above 1).

   The default plot is the most recent. Use the :plot key if you want to set
the title of another.

;; Notes:

;; See Also:

;; Examples:
 (plot sin)
 (make-title \"Hi There, and Hello World!\")
 (make-title \"Hi There, and Hello World!\" :sub-title \"King of Spain\")
 (send my-plot :update-plot)
"
  (let* (
	 (plot (if plot plot (first (agw)))) ;; most recent
         (xpos (send plot :range 0))
         (ypos (send plot :range 1))
         (slen (max 1 (length s)))
         (size (first (send plot :size)))
         (xper (if xper xper (+ (* slen (/ -5 size)) .5)))
         (xper2 (if sub-title
		    (+ (* (max 1 (length sub-title)) (/ -5 size)) .5)))
         )
    (send plot :add-plot-item
          (list
           s
           (+ (* xper (- (apply #'- xpos))) (first xpos))
           (+ (* yper (- (apply #'- ypos))) (first ypos))
           )
          )
    (if sub-title
	(send plot :add-plot-item
	      (list
	       sub-title
	       (+ (* xper2 (- (apply #'- xpos))) (first xpos))
	       (+ (* .95 yper (- (apply #'- ypos))) (first ypos))
	       )
	      )
      )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 prob-plot
;;
(defun prob-plot(list-o-data)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; prob-plot Doc:    Creation Date:1/30/97        Author: Andy Long
;;
;; Description/Arguments:

       (prob-plot list-o-data)
  
	Just a souped up probability-plot to use my plot-item stuff (so I can
make postscript images, etc.)

;; Notes:

;; See Also: probability-plot

;; Examples:
 (prob-plot (rand.n 100))
"
  (def my-plot (probability-plot list-o-data))
  (let ((n (send my-plot :num-points)))
    (send my-plot :andy-options (list ':axes t))
    (send my-plot :plot-item)
    (send my-plot :add-plot-item
	  (list 'points
		(send my-plot :point-coordinate 0 (iseq n))
		(send my-plot :point-coordinate 1 (iseq n))
		)
	  )
    (send my-plot :update-plot)
    )
  my-plot
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 quant-plot
;;
(defun quant-plot(list-o-data)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; quant-plot Doc:    Creation Date:1/30/97        Author: Andy Long
;;
;; Description/Arguments:

       (quant-plot list-o-data)
  
	Just a souped up quantile-plot to use my plot-item stuff (so I can
make postscript images, etc.)

;; Notes:

;; See Also: quantile-plot

;; Examples:
 (quant-plot (rand.n 100))
"
  (def my-plot (quantile-plot list-o-data))
  (let ((n (send my-plot :num-points)))
    (send my-plot :andy-options (list ':axes t))
    (send my-plot :plot-item)
    (send my-plot :add-plot-item
	  (list 'points
		(send my-plot :point-coordinate 0 (iseq n))
		(send my-plot :point-coordinate 1 (iseq n))
		)
	  )
    (send my-plot :update-plot)
    )
  my-plot
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 hist-plot
;;
(defun hist-plot(list-o-data)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; hist-plot Doc:    Creation Date:1/30/97        Author: Andy Long
;;
;; Description/Arguments:

       (hist-plot list-o-data)
  
	Just a souped up version of histo to use my plot-item stuff (so I can
make postscript images, etc.)

;; Notes:

;; See Also: histo

;; Examples:
 (hist-plot (rand.n 100))
"
  (def my-plot (histo list-o-data))
  (let* (
	 (counts (send my-plot :slot-value 'counts))
	 (midpoints (send my-plot :slot-value 'mid-points))
	 (width (- (second midpoints) (first midpoints)))
	 (halfwidth (half width))
	 (lefts (- midpoints halfwidth))
	 (rights (+ midpoints halfwidth))
	 )
    (send my-plot :andy-options (list ':axes t))
    (send my-plot :plot-item
	  (mapcar
	   #'(lambda(x)
	       (let (
		     (x1 (first x))
		     (x2 (second x))
		     (c (third x))
		     )
	     (list 'lines (list x1 x1 x2 x2) (list 0 c c 0) 'black)
	     ))
	   (weave lefts rights counts)
	   )
	  )
    (send my-plot :update-plot)
    )
  my-plot
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
