;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			 newtons-method-demo-zero
;;
(defun newtons-method-demo-zero
  (f a.val tolerance
     &key (first 't) plot (iterations 20) (newton-pause 1))
     
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  "
;; (newtons-method-demo-zero f a tolerance &key first plot (iterations 20))

Uses Newton's method to find a zero given an initial approximation.

;; Notes:

	Newton's method is guaranteed to work (see Burden and Faires) if

 1) f has a root r, and
 2) f is twice differentiable on an open interval containing r, and
 3) the initial guess r_0 is sufficiently close to r.

;; See Also: newtons-method-demo

;; Examples:

 (func f(x) sin x)
 (domain -1 2)
 (newtons-method-demo-zero #'f 1 .001 :first 't)

 (defun f(x) (- (+ (+ (^ x 4) (* 3 (^ x 3))) (* 3 (^ x 2))) 2))
 (domain -2 1)
 (newtons-method-demo-zero #'f 1 .001 :first 't)
 (newtons-method-demo-zero #'f -1 .001 :first 't)

 (func f(x) x)
 (domain -1 1)
 (newtons-method-demo-zero #'f 1 .001 :first 't)

 (func f(x) x ^ 3)
 (newtons-method-demo-zero #'f 1 .00001 :first 't)

 (domain -1 2)
 (func f(x) 1 - x ^ 3)
 (newtons-method-demo-zero #'f 2 .00001 :first 't)
"
  (let* (
	 (a.val (if (listp a.val) a.val (list a.val)))
	 (var (second (eval `(get-lambda-expression ,f))))
	 (newton.points (if *points* *points* 100))
	 (xmin  (first *domain*))
	 (xmax  (second *domain*))
	 (xvals (rseq xmin xmax newton.points))
	 (fvals (mapcar f xvals))
	 (lines *lines*)
	 (ctmp (lines on))
	 (my-plot (plotter f))
	 (fder-hippo (lambda-deriv f (first var)))
	 result
	 a fa fpa
	 )
    (send my-plot :range 0 xmin xmax)
    (send my-plot :add-plot-item
	  (list 'lines (list xmin xmax) (list 0 0) 'black))
    
    (dotimes
     (i (length a.val))
     (let* (
	    (b (nth i a.val))
	    (k 0)
	    )
       (do
	((fb (funcall f b) (funcall f b)))
	((or (> k iterations) (< (abs fb) tolerance)))
	(setq
	 a b
	 fa (funcall f a)
	 fpa (funcall fder-hippo a)
	 b (- a (/ fa fpa))
	 )
	(send my-plot :start-buffering)
	(send my-plot :clear-lines :draw t)
	(send my-plot :add-lines (list xvals fvals) :color 'blue)
	(send my-plot :add-lines (list (list xmin xmax) (list 0 0))
	      :color 'black)
	(send my-plot :add-lines (list (list xmin xmax) 
				       (mapcar
					(lambda(x)
					  (+ (* fpa x)
					     (- fa (* fpa a))))
					(list xmin xmax)))
	      :type 'dashed :color 'black)
	(send my-plot :buffer-to-screen)
	(paws newton-pause)
	(format t "~%root: ~f, with error ~f" b fb)
	(setq result b)
	(incf k)
	)
      )
     (send my-plot :add (list 'points (list result) (list 0)))
     (send my-plot :add
	   (list
	    'function (lambda(x) (+ (- fa (* fpa a)) (* x fpa))) :color 'blue)
	   )
     (lines lines)
     my-plot
     )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				 newtons-method-demo
;;
(defmacro newtons-method-demo (f a &optional (tolerance 1e-6))
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  "
;; Description: macro (newtons-method-demo f a &optional (tolerance 1e-6))

	Demonstrates Newton's method graphically.

;; Notes:

	Newton's method is guaranteed to work (see Burden and Faires) if

 1) f has a root r, and
 2) f is twice differentiable on an open interval containing r, and
 3) the initial guess r_0 is sufficiently close to r.

;; See also: roots, newtons-method-demo-zero, newton-zero-value, newton-zero-values

;; Examples:

 (func ff(x) sin x)
 (newtons-method-demo ff 1 .001)
 ;; or, more directly,
 (newtons-method-demo (lambda (x) (sin x)) 1 .001)
 (newtons-method-demo (func f(x) sin x) 1 .001)
"
  (let ((bing (eval `(turn-it-into-a-function ,f))))
    `(newtons-method-demo-zero ,bing ,a ,tolerance))
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			linear-approximation
;;
(defun linear-approximation (f a &optional b)
  "
;; Description

	Linear-approximation generates a plot of the tangent line, and the
appromation using the tangent line at the point with abscissa b.

;; Examples:
 (title \"f(x) = (sin x)\")
 (func f(x) sin x)
 (linear-approximation #'f -.5 1)

 (title \"f(x) = (ln x)\")
 (func f(x) ln x)
 (linear-approximation #'f 1 1.5)
"
  (let* (
	 (var (second (eval `(get-lambda-expression ,f))))
	 (fa (funcall f a))
         (xvar (first var))
         (fder (eval `(derfunc fder ,f ,xvar)))
         (fdera (fder a))
	 h xmin xmax fb
	 )
    (if b (let()
	    (setq 
	     h (- a b)
	     xmin  (- a (abs h))
	     xmax  (+ a (abs h))
	     fb (funcall f b)
	     )
	    (format t "Approximation: ~e~%" (+ fa (* fdera (- b a))))
	    (format t "True value   : ~e~%" fb)
	    )
      (let()
	(setq xmin (first *domain*))
	(setq xmax (second *domain*))
	)
      )
    (format t "Tangent line: ~f x + (~f)~%" fdera (- fa (* fdera a)))
    (eval `(plot ,f #'(lambda (x) (+ ,fa (* ,fdera (- x ,a)))) ,xmin ,xmax))
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; tangent Doc:    Creation Date:12/20/95        Author: Andy Long
;;
;;
(defmacro tangent (f x0 &optional xm)
  "
;; Description: macro

	(tangent f x0 &optional xm)
  
;; Arguments
	f: a user-defined function
	x0: the position at which one wants the tangent line
	xm: the extreme position at which one wants to see the tangent

;; Examples:
 (title \"f(x) = (sin x)\")
 (func f(x) sin x)
 (tangent f -.5 1)

 (title \"f(x) = (ln x)\")
 (func f(x) ln x)
 (tangent f 1 1.5)
 (tangent (func f(x) ln x) 1 1.5)

;; See Also: linear-approximation

"
  (let ((bing (eval `(turn-it-into-a-function ,f))))
    (if xm `(linear-approximation ,bing ,x0 ,xm)
      `(linear-approximation ,bing ,x0))))
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			 newton-zero-values
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun newton-zero-values (f a.val &optional (tolerance 1e-12) (maxits 1000))
  "
;; Description:
     (newton-zero-values f a &optional (tolerance 1e-12) (maxits 1000))
Where
	f		- a function
	a		- a guess, or a list of guesses
	tolerance	- an error tolerance for the size of the
			  function value
	maxits		- maximum number of iterations

;; Returns: a list of the calculated roots

;; Notes:

	Newton's method is guaranteed to work (see Burden and Faires) if

 1) f has a root r, and
 2) f is twice differentiable on an open interval containing r, and
 3) the initial guess r_0 is sufficiently close to r.

;; Examples:

 (func f(x) ln x)
 (newton-zero-values (function f) .9 1e-6)
 (newton-zero-values #'f .9)

 (func f(x) sin x)
 (newton-zero-values #'f .9)

 ;; Here's a 'nice' function for which Newton's method won't work. The problem
 ;; is that the first derivative is infinite (f is not differentiable at 0).
 ;; This is f(x)=x^(1/3), written to force the choice of the real root:
 (func f(x) (signum x)*(abs x)^(1/3))
 (new-plot)
 (title \"Newton's Method fails\")
 (plot f)
 (newton-zero-values #'f .9)
"
  (let* (
	 (var (second (get-lambda-expression f)))
	 (a.vals (if (listp a.val) a.val (list a.val)))
	 (fder (lambda-deriv f (first var)))
	 )
    (mapcar
     #'(lambda(a) (newton-zero-value f fder a tolerance maxits))
     a.vals)
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			 newton-zero-value
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun newton-zero-value (f fp guess &optional (tolerance 1e-12) (maxits 1000))
  "
;; Description:

	(newton-zero-value f fp guess &optional (tolerance 1e-12) (maxits 1000))
where
	f		- a function of a single variable
	fp		- the derivative function
	guess		- a guess for the root
	tolerance	- the method quits when |f(x)|<tolerance
	maxits		- the method quits when iterations > maxits

	Implements Newton's method for the solution of a f(x)=0.

;; See also: newton-zero-values

;; Notes:

	Newton's method is guaranteed to work (see Burden and Faires) if

 1) f has a root r, and
 2) f is twice differentiable on an open interval containing r, and
 3) the initial guess r_0 is sufficiently close to r.

;; Examples:
 (func f(x) ln x)
 (newton-zero-value #'f (lambda-deriv #'f 'x) .9)

 (func f(x) x^2+2)
 ;; Newton will never find a complex root with a real initial guess:
 (newton-zero-value #'f (lambda-deriv #'f 'x) 0.9)
 ;; but a root can be found with a complex guess:
 (newton-zero-value #'f (lambda-deriv #'f 'x) #c(0 .9))
 ;; even a 'real guess' expressed as a complex number:
 (newton-zero-value #'f (lambda-deriv #'f 'x) #c(.9 0))
 ;; would cause a handled error, as f' is zero at 0:
 ;; (newton-zero-value #'f (lambda-deriv #'f 'x) 0)

 (func f(x) x^2)
 (newton-zero-value #'f (lambda-deriv #'f 'x) .9)
 ;; no error, as the function value is 0 (we tested that first!)
 (newton-zero-value #'f (lambda-deriv #'f 'x) 0)
"
  (do* (
	(i 0 (1+ i))
	(a guess (- a (/ fa fpa)))
	(fa (funcall f a) (funcall f a))
	(fpa (funcall fp a) (funcall fp a))
	)
       ((or
	 (> i maxits)
	 (= fpa 0)		;; derivative is zero
	 (< (abs fa) tolerance)	;; a root
	 )
	(cond
	 ((and (= fpa 0) (/= fa 0))
	  (error "Zero derivative value in newton-zero-value.
 Newton's method fails."))
	 (t
	  (if (> i maxits)
	      (format
	       t
	       "~%Newton failed to converge before reaching maximum number (~d)
of iterations.~%" maxits))
	  (values a fa fpa i))
	 )
	)
       )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; roots Doc:    Creation Date:12/20/95        Author: Andy Long
;;
;;
(defmacro roots (f xvals &optional (tolerance 1e-12) (maxits 100))
  "
;; Description/Arguments: macro roots

	(roots f xvals &optional (tolerance 1e-12) (maxits 100))

Calculates roots given a list (or single) initial value. Includes options for a
tolerance.
  
;; Notes:

;; See Also: newton, newton-zero-values

;; Examples:

 (func f(x) sin x)
 (roots f (.3 1.8) .0001)
 (roots f 2 .01)
 (roots f 2)
 (roots f 2 1e-3) ;; find a pretty good (not a great) root.
 
 (func f(x) ln x)
 (roots f (.3 1.8) .0001) ;; root is 1, for both
"
  (let ((bing (eval `(turn-it-into-a-function ,f))))
    (if (numberp xvals) 
	`(newton-zero-values ,bing ,xvals ,tolerance ,maxits)
      `(newton-zero-values ,bing ',xvals ,tolerance ,maxits))
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                newton2
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun newton2 (f x00 &optional (maxits 100) (tolerance 1e-6) (multiplicity 1))
  "
 (newton2 f x00 &optional (maxits 100) (tolerance 1e-6) (multiplicity 1))

;; Examples:
 (newton2 (func f(x) sin x) .1 10) 
 (newton2 (lambda(x) (cos x)) 1 10)
"
  (let* (
	 (x0 x00)
	 (var (second (get-lambda-expression f)))
	 (xvar (first var))
	 (fx0 (funcall f x0))
	 (newton-its (list x0))
	 (newton-vals (list fx0))
	 (fprime (lambda-deriv f xvar))
	 )
   
    (format t "~%(Iterate, Function value)~%")
    (print (list x0 fx0))
    (dotimes
     (i maxits)
     (if (or (= 0 (funcall fprime x0)) (< (abs fx0) tolerance))
	 (return nil)
       (let()
	 (setq
	  x0 (- x0 (* multiplicity (/ fx0 (funcall fprime x0))))
	  fx0 (funcall f x0)
	  newton-its (combine newton-its (list x0))
	  newton-vals (combine newton-vals (list fx0))
	  )
	 (print (list x0 fx0))
	 )
       )
     )
    (format t "~2%function value at final iterate ~,6g: ~,6g~2%" x0 fx0)
    (format t "Following is the list of iterates and their function values:~2%")
    (mapcar
     (lambda(x y) (format t "~10t ~,6g ~30t ~,6g~%" x y))
     newton-its newton-vals
     )
    (values x0 fx0)
    )    
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                newton-root
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmacro newton-root (f &rest stuff)
  "
;; Description:
       (newton-root f guess
           &optional (maxits 100) (tolerance 1e-6) (multiplicity 1))

;; Examples:
 (func f(x) sin x)
 (newton-root f .2 10 1e-12)
 (newton-root f 3.1 10 1e-12)

;; This one requires relaxation:
 (func f(x) sqrt (abs x))
 (newton-root f .2 10 1e-12 .25)
 (newton-root (func f(x) sqrt (abs x)) .2 10 1e-12 .25)
"
  (let ((bing (eval `(turn-it-into-a-function ,f))))
    `(newton2 ,bing ,@stuff)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                secant2
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun secant2 (f x0 x1 &key (maxits 100) (tolerance 1e-6) (verbose nil))
  "
;; Description:

 (secant2 f x0 x1 &key (maxits 100) (tolerance 1e-6) (verbose nil))

;; Examples:
 (func f(x) sin x)
 (secant2 #'f .1 .2 :maxits 10)
 (secant2 #'f .1 .2 :maxits 10 :verbose t)

 ;; Would cause a handled error, as function values are equal:
 ;; (secant2 (func f(x) 3) .1 .2 :maxits 10)
"
  (let* (
	 (fx0 (funcall f x0))
	 (fx1 (funcall f x1))
	 (best-f (abs fx1))
	 (best-x x1)
	 (secant-its (list x0 x1))
	 (secant-vals (list fx0 fx1))	 
	 )
    (if verbose
	(format
	 t "~2%(Iterate, Function value):~%(~,4g, ~,4g)~%(~,4g, ~,4g)~%"
	 x0 fx0 x1 fx1))
    (dotimes
     (i maxits)
     (if (or
	  (= 0 (- fx1 fx0))
	  (< (abs fx1) tolerance))
	 (if (= 0 (- fx1 fx0))
	     (error (format
		     nil
		     "Secant fails, as f(~g)=f(~g)." x0 x1))
	   (return nil)
	   )
       (let* (
	      (x2 (- x1 (/ (* fx1 (- x1 x0)) (- fx1 fx0))))
	      (fx2 (funcall f x2))
	      )
	 (setq
	  x0 x1
	  x1 x2
	  secant-its (combine secant-its (list x2))
	  secant-vals (combine secant-vals (list fx2))
	  fx0 fx1
	  fx1 fx2
	  )

	 ;; keep track of the best guess, in case we're not getting anywhere!
	 (if (< (abs fx2) best-f)
	     (setq
	      best-x x2
	      best-f (abs fx2)
	      ))
	 )
       )
     )
    (if verbose
	(format t "~2%Function value at final x=~,4g: ~,4g~%
Following is the list of iterates and their function values:~%~{~g~%~}~2%"
		x1 fx1 (weave secant-its secant-vals))
      )
    (values best-x best-f)
    )    
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                secant
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmacro secant (f &rest stuff)
  "
 (secant f x00 x01 &key (maxits 100) (tolerance 1e-6) (verbose t))

;; Examples:
 (func f(x) sin x)
 (secant f .1 .2 :maxits 10 :tolerance 1e-12)
 (secant f 3 3.1 :maxits 10 :tolerance 1e-12)
 (secant (func f(x) sin x) 3 3.1 :maxits 10 :tolerance 1e-12)
"
  (let ((bing (eval `(turn-it-into-a-function ,f))))
    `(secant2 ,bing ,@stuff)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                regula-falsi2
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun regula-falsi2 (f x0 x1 &key (maxits 100) (tolerance 1e-6) (verbose nil))
  "
;; Description:

 (regula-falsi2 f x0 x1 &key (maxits 100) (tolerance 1e-6) (verbose nil))

	Implements the regula-falsi algorithm, which is essentially secant with
 root bracketing. The two guesses x0 and x1 must bracket a root!

;; Examples:
 (func f(x) sin x)
 (regula-falsi2 #'f .1 -.11 :maxits 10)

 ;; If a zero is given, it's found:
 (regula-falsi2 #'f 0 -.11)

 ;; regula-falsi may work even if the root isn't bracketed, but don't count on
 ;; it!
 (regula-falsi2 #'f .1 .2 :maxits 10 :verbose t)

 ;; arcsin!
 (plot
  (lambda(p)
   (regula-falsi2
    (lambda(x) (- (funcall #'sin x) p))
    -pi/2 pi/2)))
"
  (let* (
	 (fx0 (funcall f x0))
	 (fx1 (funcall f x1))
	 (regula-falsi-its (list x0 x1))
	 (regula-falsi-vals (list fx0 fx1))	 
	 )
    (if verbose
	(format
	 t "~2%(Iterate, Function value):~%(~,4g, ~,4g)~%(~,4g, ~,4g)~%"
	 x0 fx0 x1 fx1))
    (dotimes
     (i maxits)
     (if (or
	  (= 0 (- fx1 fx0))
	  (< (abs fx0) tolerance)
	  (< (abs fx1) tolerance)
	  )
	 (if (= 0 (- fx1 fx0))
	     (error (format
		     nil
		     "Regula-Falsi fails, as f(~g)=f(~g)." x0 x1))
	   (let()
	     (setq
	      regula-falsi-vals (list (min (abs fx0) (abs fx1)))
	      regula-falsi-its (select (list x0 x1)
					(where (list (abs fx0) (abs fx1))
					       (first regula-falsi-vals)))
	      )
	     (return nil)
	     )
	   )
       (let* (
	      (x2 (- x1 (/ (* fx1 (- x1 x0)) (- fx1 fx0))))
	      (fx2 (funcall f x2))
	      (sign (if (< (* fx2 fx1) 0) t nil))
	      )
	 (setq
	  x0 (if sign x1 x0)
	  x1 x2
	  regula-falsi-its (combine regula-falsi-its (list x2))
	  regula-falsi-vals (combine regula-falsi-vals (list fx2))
	  fx0 (if sign fx1 fx0)
	  fx1 fx2
	  )
	 )
       )
     )
    (if verbose
	(format t "~2%Function value at final x=~,4g: ~,4g~%
Following is the list of iterates and their function values:~%~{~g~%~}~2%"
		x1 fx1 (weave regula-falsi-its regula-falsi-vals))
      )
    (values (final regula-falsi-its) (final regula-falsi-vals))
    )    
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                regula-falsi
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmacro regula-falsi (f &rest stuff)
  "
 (regula-falsi f x00 x01 &key (maxits 100) (tolerance 1e-6) (verbose t))

;; Examples:
 (func f(x) cos x - x)
 ;; Checks with Burden and Faires:
 (regula-falsi f .5 pi/4 :verbose t :tolerance 1e-12)

 (func f(x) sin x)
 (regula-falsi f .1 -.11)
 ;; regula-falsi may work even if the root isn't bracketed, but don't count on
 ;; it!
 (regula-falsi f .1 .2 :maxits 10 :tolerance 1e-12)
 (regula-falsi f 3 3.1 :maxits 10 :tolerance 1e-12)
 (regula-falsi (func f(x) sin x) 3 3.1 :maxits 10 :tolerance 1e-12)
"
  (let ((bing (eval `(turn-it-into-a-function ,f))))
    `(regula-falsi2 ,bing ,@stuff)))
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; muller2 Doc:    Creation Date:1/4/96        Author: Andy Long
;;
;;
(defun muller2
  (f xx0 xx1 xx2
     &key (maxits 100) (tolerance 1e-6) (num-roots 1) real (verbose t))
  "
;; Description/Arguments:

  (muller2
    (f xx0 xx1 xx2
       &key (maxits 100) (tolerance 1e-6) (num-roots 1) real (verbose t))
  
  Muller2 will iteratively find your roots. It defaults to finding a single
root. The idea of Muller2 is that it uses a parabola to fit your function,
and so generalizes Newton's method (or really the secant method, which uses
a line). 

;; Notes: to find successive roots, muller uses deflation.

;; See Also: newton, muller

;; Examples:
 (func f(x) x ^ 2  + 3)
 (muller2 #'f -4 0 4 :num-roots 2)

 (func f(x) x - 3)
 (muller2 #'f -4 0 4 :num-roots 1)
 ;; causes handled errors:
 ;; (muller2 #'f -4 0 4 :num-roots 2)
 ;; (func f(x) 3)
 ;; (muller2 #'f -4 0 4)

 (func f(x) x ^ 3 - 2 * x + 3)
 (muller2 #'f -4 0 4 :num-roots 3 :tolerance 1e-12)

 (func f(x) sin x)
 (muller2 #'f -4.2 0.1 4)

 ;; find 10 roots of sine:
 (func f(x) sin x)
 (muller2 #'f -4.2 0.1 4 :maxits 1000 :tolerance 1e-12 :num-roots 10)
"
  (let* (
	 (x0 xx0)
	 (x1 xx1)
	 (x2 xx2)
	 (y0 (funcall f x0))
	 (y1 (funcall f x1))
	 (y2 (funcall f x2))
	 (h1 (- x1 x0))
	 (h2 (- x2 x1))
	 (tester t)
	 h c f10 f210 f21 x3 y3 result
	 )
    (dotimes (i maxits) ; while abs(y2) > tolerance do
	     (if tester
		 (let()
		   (setq
		    f21 (/ (- y2 y1) h2)
		    f10 (/ (- y1 y0) h1)
		    f210 (/ (- f21 f10) (+ h2 h1))
		    c (+ f21 (* h2 f210))
		    h (if (= c 0) 
			  (error "Failure in Muller: iterates equal.")
			(/ (* -2 y2)
			   (+ c (* (signum (realpart c))
				   (sqrt (- (^ c 2) (* 4 y2 f210)))))))
		    x3 (+ x2 h)
		    )
		   (if real (setq x3 (realpart x3)))
		   (setq
		    y3 (funcall f x3)
		    y0 y1
		    y1 y2
		    y2 y3
		    x0 x1
		    x1 x2
		    x2 x3
		    h1 h2
		    h2 h)
		   (if (< (abs y2) tolerance) (setq tester nil))
		   )
	       )
	     )
    (if verbose (format t "~%Root: ~,5g, with error ~,5g." x2 (abs y2)))
    (setq result x2)
    (if num-roots
	(cond
	 ((= 1 num-roots)
	  (if verbose (format t "~%All done!~%"))
	  )
	 ((and (not tester) (> num-roots 0))
	  (setq
	   result
	   (combine
	    result
	    (eval `(muller2 #'(lambda (x)
				(/ (,f x) (- x ,x2))) ,xx0 ,xx1 ,xx2
				:maxits ,maxits
				:tolerance ,tolerance
				:num-roots (1- ,num-roots))))
	   ))
	 (t
	  (format
	   t
	   "~%Muller failed to find additional roots to tolerance ~g.~%"
	   tolerance)
	  )
	 )
      (if (not tester) 
	  (eval `(muller2 #'(lambda (x)
			      (/ (,f x) (- x ,x2))) ,xx0 ,xx1 ,xx2
			      :maxits ,maxits
			      :tolerance ,tolerance))
	)
      )
    result
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; muller Doc:    Creation Date:1/25/96        Author: Andy Long
;;
(defmacro muller (f &rest stuff)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  "
;; Description/Arguments: macro

        (muller f x0 x1 x2 &key (maxits 100) (tolerance 1e-6) num-roots)
  
Muller will iteratively find your roots. It defaults to finding a single root.
The idea of Muller is that it uses a parabola to fit your function, and so
generalizes Newton's method (which uses a line).

;; Notes: Muller's method was introduced in 1956 by D. E. Muller, 'A method for
;; solving algebraic equations using an automatic computer', in Mathematical
;; Tables and other aids to computation, 10, 208-215.

;; See Also: newton, secant, muller2 (the function which this macro calls).

;; Examples:
 (func f(x) x^2+3)
 (muller f -4 0 4 :num-roots 2)

 (func f(x) x-3)
 (muller f -4 0 4 :num-roots 1)

 (muller (func f(x) x^3-2*x+3) -4 0 4 :num-roots 3 :tolerance 1e-12)

 (muller (func f(x) sin x) -4.2 0.1 4)
 (muller f -4.2 0.1 4 :maxits 1000 :tolerance 1e-12 :num-roots 10)
"
  (let ((bing (eval `(turn-it-into-a-function ,f))))
    `(muller2 ,bing ,@stuff)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; muller-2 Doc:    Creation Date:4/2/96        Author: Andy Long
;;
;;
(defun muller-2
  (f &key verbose guesses guess (scale 1) (maxits 100) (tolerance 1e-6))
  "
;; Description/Arguments: function muller-2

  (f &key guesses guess (scale 1) (maxits 100) (tolerance 1e-6) verbose)
  
	maxits: the number of times it will iterate in seeking a solution.
	tolerance: the tolerance for changes: small change means we're done.
	verbose: if off, will not print the iterates; otherwise it will.
	guess: a single guess for the true solution (default: (0 0))
	scale: a real number guess for the distance about the guess of the
               solution

Muller-2 will iteratively find a root of functions of two variables. The idea
of Muller-2 is that it uses a paraboloid to fit your function, and so
generalizes the univariate technique known as the secant method, which uses a
line.

;; See Also: muller. This is beautiful stuff! No derivatives.

;; Examples:
;; You can give some guesses (you'll need six)
 (data first-guesses (0 1) (1 0) (0 0) (1 1) (1 4) (3 2))
 (muller-2 (func f(x y) (x - 3.33) ^ 2 + (sin (y - .123)) - 1)
           :guesses first-guesses :maxits 10)

;; or the code will automatically choose some values around (0 0)
 (muller-2 (func g(x y) (x + 1) ^ 4 + (y - 1) ^ 4 + 1 / (x ^ 2 * y ^ 2 + 1)))

;; compare this one to the ineffective demonstration of steep-path:
 (def xvals '(1 2 3))
 (def yvals '(1 1 3))
 (def ones '(1 1 1))
 (func f(m b) (sum ((yvals - (b * ones + m * xvals)) ^ 2)))
 (muller-2 #'f)
"
  (andy-print "muller-2")
  (let* (
	 (guess (if guess guess '(0 0)))
	 (xvals
	  (if guesses (xs guesses) (+ (first guess) (* scale (rand.n 6)))))
	 (yvals
	  (if guesses (ys guesses) (+ (second guess) (* scale (rand.n 6)))))
	 (fvals (rand.n (length xvals)))
	 )
    (dotimes
     (i maxits)
     (let* (
	    (xnot (first xvals))
	    (ynot (first yvals))
	    (testvec (vector xnot ynot))
	    (index -1)
	    (list-of-matrix-a nil)
	    (a-matrix nil)
	    (params nil)
	    )
       (dolist
	(x xvals)
	(setq index (1+ index))
	(let* (
	       (y (nth index yvals))
	       (fval (funcall f x y))
	       (xdiff (- x xnot))
	       (ydiff (- y ynot))
	       )
	  (setl fvals index fval)
	  (setq list-of-matrix-a
		(combine list-of-matrix-a
			 (list (^ xdiff 2) (* xdiff ydiff) (^ ydiff 2)
			       xdiff ydiff 1)
			 )
		)
	  )
	)
       (setq a-matrix (matrix '(6 6) (rest list-of-matrix-a)))
       (setq params (coerce (lu-solve (lu-decomp a-matrix) fvals) 'list ))
       (setq alpha (first params))
       (setq beta (second params))
       (setq gamma (third params))
       (setq delta (fourth params))
       (setq epsilon (fifth params))
       (setq helper (- (* 4 alpha gamma) (^ beta 2)))
       (setq newxval
	     (+ xnot (/ (- (* beta epsilon) (* 2 gamma delta)) helper)))
       (setq newyval
	     (- ynot (/ (- (* 2 alpha epsilon) (* beta delta)) helper)))
       (setq oldtestvec testvec)
       (setq testvec (vector newxval newyval))
       (if (> tolerance
	      (/ (norm (- testvec oldtestvec)) (+ tolerance (norm oldtestvec))))
	   (let()
	     (princ
	      (format nil "~%root: (~e, ~e), after ~d iterates.~%"
		      newxval newyval i)
	      )
	     (return (list newxval newyval))
	     )
	 )
       (setq xvals
	     (combine
	      newxval
	      (butlast xvals) 
	      )
	     )
       (setq yvals
	     (combine
	      newyval
	      (butlast yvals) 
	      )
	     )
       (if verbose
	   (princ
	    (format nil "~%(~e, ~e), after ~d iterates, with f=~e."
		    newxval newyval i (funcall f newxval newyval))
	    )
	 )
       )
     )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; muller-1 Doc:    Creation Date:4/2/96        Author: Andy Long
;;
;;
(defun muller-1 (f &key guesses (maxits 100) (tolerance 1e-6) (print nil))
  "
;; Description/Arguments:

        (muller-1 f &key guesses (maxits 100) (tolerance 1e-6) (print off))
  
	Muller-1 will iteratively find extrema of a function of a single
variable. The idea of Muller-1 is that it uses a parabola to fit your
function.

	It's not really a Muller method, although it's similar. I've got it
figured out for the most part: we're looking for extrema of a function f.

	Newton		requires f\", f', and one guess
	Secant		requires f', and two guesses
	Muller		requires three guesses

	Newton uses the taylor series at our one guess, x_0: it computes the
quadratic approximation, and takes the extremum of the quadratic as its next
guess:
	q(x) = .5 f\"(x_0)(x-x_0)^2 + f'(x_0)(x-x_0) + f(x_0)

Secant uses a related quadratic: it replaces f\"(x_0) by m, where

	m=(f'(x_1)-f'(x_0))/(x_1-x_0), to get

	q(x) = .5    m   (x-x_0)^2 + f'(x_0)(x-x_0) + f(x_0)

The Mulleresque method approximates all derivatives using the three points, so

	q(x) = f(x_0)*L(x-x_0) + f(x_1)*L(x-x_1) + f(x_2)*L(x-x_2)

;; Notes:
	maxits: the number of times it will iterate in seeking a solution.
	tolerance: the tolerance for changes: small change means we're done.
	print: if off, will not print the iterates; otherwise it will.

;; See Also: muller

;; Examples:
;; You can give some guesses (you'll need three: the first is the best, etc.)
 (data first-guesses 0 -1 1)
 (muller-1 (func f(x) (cos (x - .123) - 1)) :guesses first-guesses :maxits 10)

;; or the code will automatically choose some values around 0:
 (muller-1 (func g(x) (x + 1.23) ^ 2))
 (muller-1 (func g(x) tan x - x))
"
  (let* (
	 (xvals (if guesses guesses (rand.n 3)))
	 (fvals (mapcar f xvals))
	 (x3 (first xvals))
	 (x2 (second xvals))
	 (x1 (third xvals))
	 )
    (dotimes
     (i maxits)
     (setq xnew x3)
     (let* (
	    (xold x3)
	    (diff12 (- x1 x2))
	    (diff23 (- x2 x3))
	    (diff31 (- x3 x1))
	    (diffs (list diff12 diff31 diff23))
	    (weights (* diffs fvals))
	    (other-weights (* weights (list (- diff23 diff31) (- diff31) diff23)))
	    (xnew (if (< tolerance (abs (sum (* 2 weights))))
		      (+ x3 (/ (sum other-weights) (sum (* 2 weights))))
		    xold
		    )
		  )
	    (fxnew (funcall f xnew))
	    )
       (if (> tolerance (/ (abs (- xold xnew)) (+ tolerance (abs xold))))
	   (let()
	     (format t "~2%extremum: ~e, after ~d iterates~2%" xnew i)
	     (return (list xnew fxnew))
	     )
	 )
       (setq x1 x2 x2 x3 x3 xnew fvals (combine fxnew (butlast fvals)))
       (if print (print (list xnew fxnew)))
       )
     )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; bisection Doc:    Creation Date:5/18/96        Author: Andy Long
;;
;;                   Modification :1/16/98
;;
(defun bisection(f x0 x1 &key (eps 1e-6) (tol 1e-6) (maxits 100))
  "
;; Description: bisection(f x0 x1 &key (eps 1e-6) (tol 1e-6) (maxits 100))

	Bisection method for the solution of f(x)=0;

;; Inputs: f, a function (assumed continuous); x0 and x1, two reals for which f
	takes opposite sign (i.e., f(x0)*f(x1) < 0).
	
;; Optional arguments: eps, tol, and maxits, which refer to three stopping
	criteria: |x1-x0|<eps|x0|, |f(xm)|<tol, and iterations > maxits.

;; See Also: newton, roots

;; Examples:
 (func f(x) x^3)
 (bisection (function f) -1 2)

 (def maxits 100)
 (def tol .000001)
 (def eps .000001)
 (def guess0 -2)
 (def guess1 1)
 (bisection #'f guess0 guess1 :maxits maxits :tol tol :eps eps);
 (bisection #'f guess0 guess1 :maxits 3 :tol tol :eps eps);
 (bisection #'f 0 guess1 :maxits 3 :tol tol :eps eps);
 (bisection #'f 1 -1 :maxits 3 :tol tol :eps eps);
"
  ;; Define local variables using let:
  (let (
	(f0 (funcall f x0))
	(f1 (funcall f x1))
	)

    (cond
     ;; We now begin testing. If we've stumbled upon a root at an endpoint,
     ;; then we're done (the first two tests of the cond statement):
     ((= f0 0) x0)
     ((= f1 0) x1)

     ;; Otherwise, we check to see if the function values are of opposite sign.
     ;; If so, then we signal an input error:
     ((> (* f0 f1) 0)
      (error "in bisection: function values are not opposite in sign."))

     ;; Okay, looks like it's time to get started: t means that this test is
     ;; always true, so the do* loop will be executed:
     (t
      (do*
       ;; local variables i, xm, and fm are initialized. Since the
       ;; initialization of fm uses xm (another local variable, just defined),
       ;; we need to use do* rather than do. Note that the local variable
       ;; definitions include the update statement as well as the
       ;; initialization statement.
       (
	(i 0 (incf i))
	(xm (/ (+ x0 x1) 2) (/ (+ x0 x1) 2))
	(fm (funcall f xm) (funcall f xm))
	(max-error (abs (/ (- x1 x0) 2)) (abs (/ (- x1 x0) 2)))
	)

       ;; We test two of our starting criteria. If either is true, we leave the
       ;; loop with either a zero or a list containing the best approximation,
       ;; the function value, and the maximum error we're making:
       ((or (< (abs fm) tol) (< max-error eps)) 
	(if (= fm 0) xm (list xm fm max-error))
	)

       ;; Now we test the third criterion: it is slightly different, as it
       ;; seems to imply that we haven't really found a root. For that reason,
       ;; we'll print out a warning:
       (if (> i maxits)
	   (let ()
	     (format t "
Bisection failed to achieve error ~g in ~d iterations.
Final estimate for root     : ~g
Corresponding function value: ~g~%" eps maxits xm fm)
	     (return (list xm fm max-error))
	     )
	 )

       ;; Otherwise, we carry out the update step, comparing fm with f1 to see
       ;; which endpoint gets replaced. Notice the setq statement, which (in
       ;; the first case) sets x1 to xm, then f1 to fm.
       (if (> (* f1 fm) 0)
	   (setq x1 xm f1 fm)
	 (setq x0 xm f0 fm)
	 )
       )
      )
     )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                traub
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun traub (f x0
		&key
		(maxits 100)
		(eps 1e-6)
		verbose
		)
  "
;; Description:
      (traub f x0
		&key
		(maxits 100)
		(eps 1e-6)
		verbose
		)

;; Notes: I read about the Methode de Traub in the book 'Methodes Numeriques
;; Appliquees', by A. Gourdin and M. Boumahrat, Lavoisier, 1983. The claim is
;; that this method is of order 3 (Newton is order 2, Secant is order golden
;; mean, that is (1 + sqrt(5))/2).

;; See also: newton2

;; Examples:
 (traub (func f(x) sin x) .1 :maxits 10) 
 (traub (lambda(x) (cos x)) 1 :maxits 10 :verbose t)
 (traub (func f(x) x^2) 0 :maxits 10) 
"
  (let* (
	 (var (second (get-lambda-expression f)))
	 (xvar (first var))
	 (fx0 (funcall f x0))
	 (newton-its (list x0))
	 (newton-vals (list fx0))
	 (fprime (lambda-deriv f xvar))
	 z fz
	 )
    (dotimes
     (i maxits)
     (cond
      ((= 0 (funcall fprime x0))
       (format t "Error in traub: derivative zero. Leaving....")
       (return nil))
      ((< (abs fx0) eps) (return nil))
      (t
       (setq

	;; z is given by the Newton step:
	z (- x0 (/ fx0 (funcall fprime x0)))
	fz (funcall f z)

	;; x0 is now determined by a secant step using the z values:
	x0 (- z (* fz (/ (- z x0) (- (* 2 fz) fx0))))

	fx0 (funcall f x0)
	newton-its (combine newton-its x0)
	newton-vals (combine newton-vals fx0)
	)
       )
      ))
    (if verbose
	(let ()
	  (format t "~%
Function value at final iterate = ~,6g: ~,6g.~%
Following is the list of iterates and their function values:~2%"
		  x0 fx0)
	  (mapcar
	   (lambda(x y) (format t "~10t ~,6g ~30t ~,6g~%" x y))
	   newton-its newton-vals
	   )))
    (values x0 fx0)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;	The following based on reading in Numerical Recipes:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				zbrac
;;
(defun zbrac(f x1 x2 &key (maxits 50) (factor 1.6))
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			zbrac Doc:
;;
;; Author: Andy Long
;; Created: Sun Mar 29 18:40:21 1998
;;
;; Description/Arguments: zbrac

	Finds an interval containing a root. Assumes a continuous function, and
geometrically increases the size of the region until success, or failure after
a number of tries given (maxits)

;; Arguments: f x1 x2 &key (maxits 50) (factor 1.6)

	f	- the function
	x1,x2	- the interval endpoints
	maxits	- the maximum number of interval expansions permitted
	factor	- the expansion factor

;; Returns: ordered interval (smallest first) bracketing a root, or nil if
;; unsuccessful.

;; Notes: NR, p. 245

;; See Also: 

;; Examples:
 ;; If an endpoint is a zero, the region is returned as is:
 (zbrac #'sin 0 .2)

 ;; otherwise, the region is expanded until a root is bracketed, if possible:
 (zbrac #'sin .1 .2)
 (zbrac #'sin .2 -.1)

 ;; if not possible, nil is returned.
 (zbrac #'square .1 .2)
 ;; causes an error: degenerate region.
 ;; (zbrac #'square .1 .1)
"
  (let* (
	 (x3 (min x1 x2))
	 (x2 (max x1 x2))
	 (x1 x3)
	 (error (if (= x1 x2) (error "In zbrac:
You have to give a REGION to start, not a point.")))
	 f1 f2
	 )
    (dotimes
     (i maxits)
     (setq
      f1 (funcall f x1)
      f2 (funcall f x2)
      )
     (if (<= (* f1 f2) 0) (return (list x1 x2)))
     (if (< (abs f1) (abs f2))
	 (setq x1 (+ x1 (* factor (- x1 x2))))
       (setq x2 (+ x2 (* factor (- x1 x2))))
       )
     )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				newton-safe
;;
(defun newton-safe(f fp x1 x2 &key (eps 1e-12) (maxits 100))
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			newton-safe Doc:
;;
;; Author: Andy Long
;; Created: Sun Mar 29 19:37:10 1998
;;
;; Description/Arguments: newton-safe

	'Using a combination of Newton-Raphson and bisection, find the root of a
function bracketed between x1 and x2. The root ... will be refined until its
accuracy is known within plus or minus eps.' from NR, on root-finding.

;; Arguments: f fp x1 x2 &key (eps 1e-6) (maxits 100)

;; Returns: the root

;; Notes: based on rtsafe of Numerical Recipes.

;; See Also: zbrac

;; Examples:
 (newton-safe #'sin #'cos .1 .2 )
 (newton-safe #'sin #'cos 0 .2 )
 (newton-safe (lambda(x) (- x pi)) (lambda(x) 1) .1 .2 )
 ;; if there's no root, the response is nil:
 (newton-safe (lambda(x) (+ (square x) pi)) (lambda(x) (* 2 x)) .1 .2 )
"
  (let ((interval (zbrac f x1 x2)))
    (if interval
	(let* (
	       (x1 (first interval))
	       (x2 (final interval))
	       (dxold (abs (- x2 x1)))
	       (dx dxold)
	       (rtsafe (mean (list x1 x2)))
	       (f1 (funcall f x1))
	       (f2 (funcall f x2))
	       (fval (funcall f rtsafe))
	       (df (funcall fp rtsafe))
	       temp
	       )
	  (do
	   ((i 0 (1+ i)))
	   ((or 
	     (> i maxits)
	     (< (abs f2) eps)
	     (< (abs f1) eps))
	    (if (> i maxits)
		(let ()
		  (format t
			  "Maximum number of iterations exceeded:
Possibilities: (x,y)=(~,g, ~,g) and  (~,g, ~,g)"
			  x1 f1 x2 f2)
		  nil
		  )
	      (first 
	       (select (list x1 x2)
		       (where (list (abs f1) (abs f2))
			      (min (abs f1) (abs f2)))))))
	   (if (or
		(>= (*
		     (- (* (- rtsafe x2) df) fval)
		     (- (* (- rtsafe x1) df) fval))
		    0
		    )
		(> (abs (* 2 fval)) (abs (* dxold df)))
		)
	       (setq
		dxold dx
		dx (* .5 (- x2 x1))
		rtsafe (+ x1 dx)
		temp x1)
	     (setq
	      dxold dx
	      dx (/ fval df)
	      temp rtsafe
	      rtsafe (- rtsafe dx))
	     )
	   (if (or (< (abs dx) eps) (= temp rtsafe)) (return rtsafe))
	   (setq
	    fval (funcall f rtsafe)
	    df (funcall fp rtsafe)
	    )
	   (if (< fval 0)
	       (setq
		x1 rtsafe
		f1 fval
		)
	     (setq
	      x2 rtsafe
	      f2 fval
	      )
	     )	     
	   )     
	  )
      )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
