
;; add to manual: moran-I
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				moran-I
;;
(defun moran-I(z x d)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			moran-I Doc:
;;
;; Author: Andy Long
;; Created: Thu Sep 10 19:50:36 1998
;;
;; Description/Arguments: moran-I

;; Arguments: z x d 

;; Returns: the Moran I statistic for the vector z at locations x of pairs
;; separated by a distance of d 

;; Notes: From An overview of spatial statistics, by Shuming Bao, China Data Center, University of Michigan

;; See Also: 

;; Key Functions:

;; Examples:
 (moran-I (iseq 5) (rseq 0 1 5) 1)
"
  (let* (
	 (n (length z))
	 (zbar (mean z))
	 (ssquare (sample-variance z))
	 (s 0)
	 (w 0)
	 (moran 0)
	 )
    (dotimes
     (i (1- n))
     (let* (
	    (loc (elt x i))
	    (val (elt z i))
	    (diff (- val zbar))
	    )
       (dolist
	(j (iseq (+ 1 i) (- n 1)))
	(if (< (norm (- loc (elt x j))) d)
	    (setq
	     s (+ s (* diff (- (elt z j) zbar)))
	     w (incf w))
	  )
	)
       )
     )
    (/ s (* ssquare w))
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; add to manual: whittemore
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				whittemore
;;
(defun whittemore(z cases x &key weights)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			whittemore Doc:
;;
;; Author: Andy Long
;; Created: Thu Sep 10 19:50:36 1998
;;
;; Description/Arguments: whittemore

;; Arguments: z cases x

;; Returns: the Mean pair case distance statistic for the vector z at locations
;; x 

;; Notes: From Whittemore, et al., Biometrika 74, 631-635, 1987. I
;; think. Actually I deduced it from other sources, notably Case Studies in
;; Biometry!

;; See Also: 

;; Key Functions:

;; Examples:
 (whittemore (iseq 5) (list 1 0 0 1 0) (rseq 0 1 5))
 (whittemore (iseq 5) (list 1 0 1 0 1) (rseq 0 1 5))
 (whittemore (iseq 5) (list 1 0 1 0 1) (rseq 0 1 5) :weights t)
 (whittemore (list .4 .6 1) (list 1 1 1) '((0 0) (2 0) (0 3)) :weights t)
"
  (let* (
	 (z (select z (where cases 1)))
	 (cases (select x (where cases 1)))
	 (n (length cases))
	 (dist 0)
	 )
    (do*
     (
      (i (first cases) (first cases))
      (val (first z) (first z))
      (cases (cdr cases) (cdr cases))
      (z (cdr z) (cdr z))
      )
     ((not cases) nil)
     (setq dist (+ dist 
		   (if weights
		       (* val (dot z 
				   (mapcar
				    #'norm
				    (mapcar (lambda(x) (- i x)) cases))))
		     (sum (mapcar #'norm (- i cases))))))
     )
    (* 2 (/ dist (* n (1- n))))
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; add to manual: stones-test
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				stones-test
;;
(defun stones-test(focus z ns x)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			stones-test Doc:
;;
;; Author: Andy Long
;; Created: Thu Sep 10 19:50:36 1998
;;
;; Description/Arguments: stones-test

;; Arguments: focus z ns x

;; Returns: Stone's statistic for the vector z with populations ns at locations x,
;; with respect to the focus.

;; Notes: From Case Studies in Biometry

;; See Also: 

;; Key Functions:

;; Examples:
 (load-calc waller)
 (stones-test (list (first ix) (first iy)) inc pop (transpose (list x y)))
 ;; Now to test the test against some actual results:
 (mapcar (lambda(xx yy)
	   (stones-test (list xx yy) inc pop (transpose (list x y))))
	 ix iy)
 ;; should get
 ;; 2.5 2.53 1.48 1.48 2.13 1.77 2.13 3.37 1.52 1.19 1.60 1.88
"
  (let* (
	 (order (order (mapcar #'norm (mapcar (lambda(x) (- focus x)) x))))
	 (z (select z order))
	 (x (select x order))
	 (ns (select ns order))
	 (expecteds (* ns (/ (sum z) (sum ns))))
	 )
    (max (/ (cumsum z) (cumsum expecteds)))
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
#|
(let ()
  (load-calc spatial)
  (try-out stones-test)
  )
|#

;; add to manual: besag-and-newell
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				besag-and-newell
;;
(defun besag-and-newell(focus z ns x k)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			besag-and-newell Doc:
;;
;; Author: Andy Long
;; Created: Thu Sep 10 19:50:36 1998
;;
;; Description/Arguments: besag-and-newell

;; Arguments: focus z ns x k

;; Returns: Focused Besag and Newell's test statistic for the vector z with
;; populations ns at locations x, with respect to the focus.

;; Notes: From Case Studies in Biometry

;; See Also: 

;; Key Functions:

;; Examples:
 (load-calc waller)
 (besag-and-newell (list (first ix) (first iy)) inc pop (transpose (list x y)) 6)
 ;; Now to test the test against some actual results:
 (mapcar (lambda(xx yy k)
	   (besag-and-newell (list xx yy) inc pop (transpose (list x y)) k))
	 ix iy (repeat 6 (length ix)))
 ;; should get
 ;; 0.0694 0.227 0.665 0.665 0.108 0.551 0.337 0.0131 0.769 0.843 0.847
 (mapcar (lambda(xx yy k)
	   (besag-and-newell (list xx yy) inc pop (transpose (list x y)) k))
	 ix iy (repeat 8 (length ix)))
 ;; should get
 ;; 0.0725 0.056 0.725 0.725 0.133 0.254 0.258 0.492 0.547 0.726 0.875
 (mapcar (lambda(xx yy k)
	   (besag-and-newell (list xx yy) inc pop (transpose (list x y)) k))
	 ix iy (repeat 10 (length ix)))
 ;; should get
 ;; 0.0133 0.072 0.47 0.47 0.143 0.168 0.12 0.237 0.284 0.879 0.802
"
  (let* (
	 (order (order (mapcar #'norm (mapcar (lambda(x) (- focus x)) x))))
	 (z (select z order))
	 (x (select x order))
	 (ns (select ns order))
	 (expecteds (* ns (/ (sum z) (sum ns))))
	 (l
	  (do*
	   (
	    (i 0 (incf i))
	    (z z (rest z))
	    (sum (first z) (+ sum (first z)))
	    )
	   ((>= sum k) i)
	   )
	  )
	 )
    (- 1
       (sum
	(poisson-pmf
	 (iseq k)
	 (* (sum (select ns (iseq 0 l))) (/ (sum z) (sum ns))))
	)
       )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
#|
(let ()
  (load-calc spatial)
  (try-out besag-and-newell)
  )
|#


;; add to manual: waller-and-lawson
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				waller-and-lawson
;;
(defun waller-and-lawson(focus z ns x k)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			waller-and-lawson Doc:
;;
;; Author: Andy Long
;; Created: Thu Sep 10 19:50:36 1998
;;
;; Description/Arguments: waller-and-lawson

;; Arguments: focus z ns x k

;; Returns: Focused Besag and Newell's test statistic for the vector z with
;; populations ns at locations x, with respect to the focus. k is the exposure
;; function.

;; Notes: From Case Studies in Biometry

;; See Also: 

;; Key Functions:

;; Examples:
 (load-calc waller)
 ;; Now to test the test against some actual results:
 (def
 act
 (mapcar
  (lambda(xx yy)
    (waller-and-lawson
     (list xx yy) inc pop (transpose (list x y)) (lambda(x) (/ x))))
  ix iy))
;; should get
 (setq res (list 4.12 3.39 2.47 2.46 2.06 .29 2.56 1.97 1.87 -.78 .81))
 act
 (plot act res)
"
  (let* (
	 (dists (mapcar (lambda(x) (norm (- focus x))) x))
	 (rate (/ (sum z) (sum ns)))
	 (expecteds (* ns rate))
	 (exposure (mapcar k dists))
	 )
    (/
     (sum (* exposure (- z expecteds)))
     (sqrt (sum (* exposure exposure expecteds))))
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
#|
(let ()
  (load-calc spatial)
  (try-out waller-and-lawson)
  )
|#

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                           score test
;;
#|
	Based on a pascal program by Lance Waller, score.p, provided for
the GeoMed project.
|#
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun score(xfocus yfocus x y o pop numsim &key (stream t))
  "
;; Examples:
 (load-calc \"californ.lsp\")
 (score -121 37 x y o pop 40)
 (score -121 37 x y o pop 40 :stream nil)
 (score -121 37 x y o pop 40 :stream nil)
 (score -121.85668 37.43443175 x y o pop 40 :stream nil)
"
  (let* (
	 (regions (length pop))
	 (tbn  0)
	 (toto (sum o))
	 (totp (sum pop))
	 (rate (/ toto totp))
	 (d (sqrt (+ (* (- x xfocus) (- x xfocus))
		     (* (- y yfocus) (- y yfocus)))))
	 (d (mapcar (lambda(x) (max x 1.0e-10)) d))
	 (g (/ d))
	 (e (* rate pop))
	 (u (sum (* g (- o e))))
	 (varu (sum (* g g e)))
	 (ustar (/ u (sqrt varu)))
	 (rate-pop (* rate pop))
	 (ustarsim
	  (mapcar
	   (lambda(sim)
	     (let (
		   (usim
		    (sum
		     (*
		      g 
		      (- (mapcar (lambda(x) (poisson-rand 1 x)) rate-pop)
			 e))))
		   (varusim (sum (* g g e)))
		   )
	       (/ usim (sqrt varusim))))
	   (iseq 1 numsim)
	   )
	  )
	 (numge (length (where ustarsim ustar #'>=)))
	 (psim (/ numge numsim))
	 (asim (- 1 (normal-cdf ustar)))
	 )
    (format stream "X coordinate of focus = ~g~%" xfocus)
    (format stream "Y coordinate of focus = ~g~%" yfocus)
    (format stream "test statistic = ~g~%" ustar)
    (format stream "Monte Carlo Simulation Method:~%")
    (format stream "Number of Monte Carlo simulations = ~d~%" numsim)
    (format stream "P-value from simulation = ~g~%" psim)
    ;;  calculate p-value by integrating Normal distribution
    (format stream "Normal Distribution Approximation Method:")
    (format stream "Approx. p-value = ~g~%" asim)
    (list ustar psim asim)
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; add to manual: matrix-moran-I
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				matrix-moran-I
;;
(defun matrix-moran-I(a &key (type 'rook))
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			matrix-moran-I Doc:
;;
;; Author: Andy Long
;; Created: Sun Dec 6 17:52:28 1998
;;
;; Description/Arguments: matrix-moran-I

;; Arguments: a    - a matrix
              type - 'rook, 'bishop, 'queen

;; Returns: Moran's I, oddly enough

;; Notes: 

;; See Also: 

;; Key Functions:

;; Examples:
 (matrix-moran-I (id 3))
 (matrix-moran-I (id 3) :type 'bishop)
 (matrix-moran-I (id 3) :type 'queen)
"
  (let* (
	 (size (size a))
	 (nrows (first size))
	 (ncols (second size))
	 (x (combine a))
	 (mean (mean x))
	 (z (- x mean))
	 (sc
	  (case type
		('rook
		 (* 2 (- (* 2 nrows ncols) nrows ncols)))
		('bishop
		 (* 4 (- (* nrows ncols) nrows ncols -1)))
		('queen
		 (* 2 (- (* 4 nrows ncols) (* 3 (+ nrows ncols)) -2)))
		))
	 (ss (sum (square z)))
	 (n (length z))
	 
	 ;; I pad a, so that it's easier to do the masking by the usual
	 ;; contiguity matrices....
	 (padded-a
	  (combine-matrices
	   (list (zeros 1 (+ 2 ncols)))
	   (list (zeros nrows 1) (- a mean) (zeros nrows 1))
	   (list (zeros 1 (+ 2 ncols)))
	   ))
	 
	 (type
	  (case type
		('rook
		 (lambda(i j)
		   (sum
		    (mapcar
		     (lambda(i j) (aref padded-a i j))
		     (list (1+ i) (1- i) i i)
		     (list j j (1+ j) (1- j))
		     )
		    )
		   )
		 )
		('bishop
		 (lambda(i j)
		   (sum
		    (mapcar
		     (lambda(i j) (aref padded-a i j))
		     (list (1+ i) (1- i) (1+ i) (1- i))
		     (list (1+ j) (1- j) (1- j) (1+ j))
		     )
		    )
		   )
		 )
		('queen
		 (lambda(i j)
		   (sum
		    (mapcar
		     (lambda(i j) (aref padded-a i j))
		     (list (1+ i) (1- i) (1+ i) (1- i) (1+ i) (1- i) i i)
		     (list (1+ j) (1- j) (1- j) (1+ j) j j (1+ j) (1- j))
		     )
		    )
		   )
		 )
		)
	  )
	 (result 0)
	 )
    ;; if the ss is 1, the field is perfectly autocorrelated:
    (if (= ss 0)
	1
      (dolist
       (i (iseq 1 nrows))
       (dolist
	(j (iseq 1 ncols))
	(setq 
	 result (+ result (* (aref padded-a i j) (funcall type i j)))
	 )
	)
       )
      )
    (* n result (/ (* sc ss))))    
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 another-matrix-moran-i
;;
(defun another-matrix-moran-i(list matrix)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          another-matrix-moran-i Doc:
;;
;; Author: Andy Long
;; Created: //1997
;;
;; Description/Arguments:

       (another-matrix-moran-i list matrix)
  
	Where
		list   - data list
		matrix - connection weight matrix

;; Notes:

;; See Also:

;; Examples:
 (another-matrix-moran-i (list 2 1 3) (matrix '(3 3) '(0 1 1 1 0 1 1 1 0)))
"
  (let* (
	 (z (- list (mean list)))
	 (n (length z))
	 (a (sum matrix))
	 (i (/ (* n (matmult z matrix z)) a (sum (* z z))))
	 )
    i
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
