;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Description for HEAVISIDE:
;;
(defun heaviside(x)
  "
;; Documentation: heaviside(x)
				(vectorized)

Heaviside function is zero for negatives, 1 for positives. It's currently going
to give 1 for zero: not sure if that's what Heaviside would do....

;; Examples:
 (heaviside '(4 8 -2))
 (heaviside (- '(4 8 2) '(5 5 5)))
 (plot heaviside -2 2)
" 
  (if (numberp x) (if (> x 0) 1 0) (mapcar #'heaviside (coerce x 'list)))
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Description for BUMP:
;;
(defun bump(x a b)
  "
;; Documentation: (bump x a b)

The bump function is 1 from a to b, and 0 otherwise. 

;; Examples:

 (bump 1 2 4)
 (bump 3 2 4)
 (bump 5 2 4)
 (plot (lambda(x) (bump x 0 1)) -2 2)
" 
  (- (heaviside (- x a)) (heaviside (- x b)))
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Description for COMPOSITE:
;;
(defun composite(f g x)
  "Args: (f g x)
Documentation:

	Composite creates the composite function of f and g. It is the user's
responsibility to see to its domain and range!

 (func f (x) (sin x))
 (func g (x) (2 * x))
 (func h (x) (composite #'f #'g x))
 (plot f g h -pi pi)

;; Note that if you now change function g, say,
;; you do not need to re-evaluate h!
 (func g (x) (x / 2))
 (plot f g h -pi pi)
" 
  (eval `(,f (,g ,x)))
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                factorial
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun factorial (n)
  "
Description: (factorial n)

	The factorial function. (factorial n) = n*(n-1)*(n-2)*...*3*2*1.

Notes: as given in Luke Tierney's book \"Lisp-stat\".

;; Examples:
 (factorial 3) ;; 6
 (factorial 10) ;; 3628800
"
  (let ((prod 1))
    (dotimes (count n)
	     (setf prod (* prod (+ count 1)))
	     )
    prod)
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                fibonacci
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun fibonacci (n)
  "
	The fibonacci sequence, based on the model of factorial as given in
Luke's book. The fibonacci sequence is very old and very famous. It has many
applications in nature. The first few numbers in the sequence are

                        0 1 1 2 3 5 8 13 etc.,

each term the sum of the two previous terms.

;; Examples:

 (fibonacci 10)

;; See also: fib (which also works for sequences).
"
  (let (
	(prod1 0)
	(prod2 1)
	(prod 1)
	)
    (if (= n 0) (setf prod 0)
      (if (> n 2)
	  (dotimes (count (1- n))
		   (setf prod (+ prod1 prod2))
		   (setf prod1 prod2)
		   (setf prod2 prod)
		   )
	)
      )
    prod
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun fib (n)
  "
;; Description: fibonacci numbers. See fibonacci.

;; Examples:
 (fib 3)
 (fib (iseq 1 15))
 (calc (fib (iseq 2 16)) / (fib (iseq 1 15))) ;; after the golden mean!
"
  (if (numberp n)
      (fibonacci n)
    (if (consp n)
	(mapcar #'fibonacci n)
      )
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Error function
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;; Error Function

;;; References
;;;   M. Abramowitz and I. Stegun, eds,
;;;     Handbook of Mathematical Functions,
;;;     National Bureau of Standards, 1964
;;;   Digital Equipment Corporation,
;;;     PDP-11 Paper Tape Software Programming Handbook,
;;;     DEC-11-GGPA-D, Section 7.7.
;;;   Cecil Hastings, Jr.,
;;;     Approximations for Digital Computers,
;;;     Princeton University Press, Princeton, NJ, 1955.

;;;  (c) Copyright Gerald Roylance 1982, 1984, 1985, 1988
;;;      All Rights Reserved.
;;;  This file may be distributed noncommercially provided
;;;  that this notice is not removed.

;;; Bugs and fixes
;;;   
;;; derived from NBS 7.1.26
;;;   eps < 1.5e-7
;;;
#|
(defun erfc (x)
  (if (< x 0.0)
      (+ 1.0 (erf (- x)))
    (let ((z (/ 1.0 (1+ (* 0.3275911 x)))))
      (declare (float z))
      (* (exp (- (* x x)))
	 (poly z 
	       +0.000000000
	       +0.254829592
	       -0.284496736
	       +1.421413741
	       -1.453152027
	       +1.061405429))))
  )
|#
(defun erfc (x)
  "
;; Description: Related to the error function

;; See also: erf

;; Examples:
 (erfc 5.01)
 (plot erfc -5 5)
"
  (if (< x 0.0)
      (+ 1.0 (erf (- x)))
    (let ((z (/ 1.0 (1+ (* 0.3275911 x)))))
      (* (exp (- (* x x)))
	 (eval
	  (poly-expand z
		       (list
			+0.000000000
			+0.254829592
			-0.284496736
			+1.421413741
			-1.453152027
			+1.061405429)))))))
;;
(defun erf (x)
  "
;; Description: the error function

	This is calculated with a polynomial approximation. By 5, the function
is assigned a value of 1. It is an odd function.

;; See also: erfc (a helper)

;; Examples:
 (erf 5)
 (plot erf -5 5)
"
  (cond ((<  x 0.0) (- (erf (- x))))
	((>= x 5.0) 1.0)
	(t (- 1.0 (erfc x)))))
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
(defun composition2 (g f gf)
  "
;; Description: (composition g f gf)

	This one's for Dad: he told me that he'd like to see it!
 Produces a function of one variable which produces a triplet, of
 (x, f(x), g(f(x))).

 ;; Examples:
 (func g(y) sin y)
 (func f(x) exp x)
 (composition2 #'g #'f 'gf)
 (plot (gf (rseq 0 3 200)))
 (send my-plot :points-showing nil)

 (composition2 #'sin #'exp 'gf)
 (plot (gf (rseq 0 3 200)))
 (send my-plot :points-showing nil)
"
  (eval
   `(let ()
      (defun ,gf (x)
	(let ((fx (funcall ,f x)))
	  (list x fx (funcall ,g fx))))
      (function ,gf)))
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; composition Doc:    Creation Date:3/23/96        Author: Andy Long
;;
;;
(defmacro composition(&rest x)
  "
;; Description: (composition g f gf) 

;; Dedicated to Cliff Long:

	This one's for Dad: he told me that he'd like to see it!
 It produces a function of one variable which produces a triplet, of
 (x, f(x), g(f(x))).

;; See Also: composition2

;; Examples:
 ;; Three different ways to obtain the same effect:
 (composition (func g(y) sin y) (func f(x) exp x) gf)
 (plot (gf (rseq 0 3 200)))
 (send my-plot :points-showing nil)

 (func g(y) sin y)
 (func f(x) exp x)
 (composition g f gf)
 (plot (gf (rseq 0 3 200)))
 (send my-plot :points-showing nil)

 (composition sin exp gh)
 (plot (gh (rseq 0 3 200)))
 (send my-plot :points-showing nil)
"
  (let (
	(bing (first x))
	(bong (second x))
	(bang (third x))
	)
    (setq
     bing (eval `(turn-it-into-a-function ,bing))
     bong (eval `(turn-it-into-a-function ,bong)))
    `(composition2 ,bing ,bong ',bang)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                      Functions stolen from sapaclisp
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defconstant +log-pi+ (log pi))
(defconstant +euler-constant+ 0.577215664901532860606512d0)

(defun digamma-general (x &key (x-recursion 8.5) (x-small 1.0e-5))
  "given
   [1] x (required)
       ==> a positive number
   [2] x-recursion (keyword; 8.5)
       ==> for noninteger x,
           if x-small < x < x-recursion,
           recursive formula is used
   [3] x-small (keyword; 1.0e-5)
       ==> if x <= x-small,
           small x approximation used (default )
returns
   [1] value of digamma (psi, derivative of log(gamma)) function at x
---
Note: see Abramowitz and Stegun's equation 6.3.2; expansion 6.3.18 plus
recurrence 6.3.5; the small x formula is Equation (5) from Algorithm AS 103 --
Psi (Digamma) Function -- by J. M. Bernardo in Applied Statistics, Vol. 25,
No. 3, 1976, pp. 315--317; note that better accuracy can be obtained by
increasing x-recursion --- 10.0 to 100.0 are probably useful values

;; Examples:
 (digamma-general .2)
 (plot (lambda(x) (digamma-general x)) .1 1)
"
  (assert (plusp x))
  (cond
   ((integerp x)
    (let ((sum (- +euler-constant+)))
      (dotimes (i (1- x) sum)
        (incf sum (/ (1+ i))))))
   ((<= x x-small)
    (- (+ +euler-constant+ (/ x))))
   ((< x x-recursion)
    (- (digamma-general (1+ x) :x-recursion x-recursion) (/ x)))
   (t
    (let* ((x2 (* x x))
           (x4 (* x2 x2)))
      (- (log x)
         (/ (* 2.0 x))
         (/ (* 12.0 x2))
         (/ (* -120.0 x4))
         (/ (* 252.0 x2 x4)))))))

(defun trigamma-general (x &key (x-recursion 2.0))
  "given
   [1] x (required)
       ==> a positive number
   [2] x-recursion (keyword; 2.0)
       ==> if  x < x-recursion,
           recursive formula is used
returns
   [1] value of trigamma function at x
---
Note: expansion 6.4.12 plus recurrence 6.4.6
of Abramowitz and Stegun;
better accuracy can be obtained by increasing
x-recursion ---10.0 to 30.0 is probably a useful upper limit

;; Examples:
 (trigamma-general .2)
 (plot (lambda(x) (trigamma-general x)) .1 1)
"
  (assert (plusp x))
  (if (< x x-recursion)
    (+ (trigamma-general (1+ x) :x-recursion x-recursion) (/ (* x x)))
    (let* ((x2 (* x x))
           (x3 (* x x2))
           (x5 (* x2 x3))
           (x7 (* x2 x5)))
      (+ (/ x)
         (/ (* 2.0 x2))
         (/ (* 6.0 x3))
         (/ (* -30.0 x5))
         (/ (* 42.0 x7))
         (/ (* -30.0 x2 x7))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; gamma Doc:    Creation Date:5/25/96        Author: Andy Long
;;
;;
(defun gamma(x)
  "
;; Description/Arguments:

       (gamma x)
  
;; Notes: The gamma function. It is defined in terms of the log-gamma function,
;; and is a generalization of the factorial function.

;; See Also: log-gamma, digamma, trigamma

;; Examples:
 (gamma 5) ;==> 24
 (gamma (iseq 1 5))
 (gamma pi)
"
  (exp (log-gamma x))
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun digamma (x)
  " Type

         (help digamma-general)

for more info. I've defined it to be imaginary for x < 0 (in fact, it's not
defined there at all - a bit of a klugy fix so that it will plot without
problems, even if we get below zero).

;; Examples:
 (integral g.int digamma 1 x .01) ;; log-gamma is zero at 1, perfect!
 (plot log-gamma digamma g.int .1 1) ;; g.int should fall on the plot of the
				     ;; log-gamma function
"
  (if (> x 0) (digamma-general x)
    #c(0 1))
  )

(defun trigamma (x)
  " Type
         (help trigamma-general)

for more info. I've defined it to be imaginary for x < 0 (in fact, it's not
defined there - a bit of a klugy fix so that it will plot without problems,
even if we get below zero).

;; Examples:
 (integral g.int trigamma 1 x .001)
 (plot digamma trigamma g.int 1 2) ;; g.int should look like the digamma
				    ;; function, shifted up or down
"
  (if (> x 0) (trigamma-general x)
    #c(0 1))
  )

;; add to manual: tent
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				tent
;;
(defun tent(a b &key (peak (mean (list a b))) (height 1))
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			tent Doc:
;;
;; Author: Andy Long
;; Created: Mon Aug 10 12:32:45 1998
;;
;; Description/Arguments: tent

	A tent function looks like a pup tent. It is compact, and formed of two
 linear segments rising to a peak.

;; Arguments: a b &key (peak (mean (list a b))) (height 1)

	a	-	left endpoint
	b	-	right endpoint
	peak	-	peak position (mean of a and b by default)
	height	-	value at the top of the peak (1 by default)

;; Returns: the tent function which is 0 everywhere but in [a,b], with a peak
   at peak of height 1 by default.

;; Notes: 

;; See Also: bump, heaviside

;; Key Functions:

;; Examples:
 (plot (tent 0 2) -1 3)
 (plot (tent 0 2 :peak 1.5 :height 2) -1 3)
"
  (lambda(x)
    (cond
     ((or (<= x a) (>= x b)) 0)
     ((< x peak) (* height (/ (- x a) (- peak a))))
     (t (* height (/ (- b x) (- b peak)))))))
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; add to manual: bent
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				bent
;;
(defun dome(a b &key (peak (mean (list a b))) (height 1))
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			dome Doc:
;;
;; Author: Andy Long
;; Created: Mon Aug 10 12:32:45 1998
;;
;; Description/Arguments: dome

	A dome function looks like a dome tent, only it has derivative
 continuity. It is compact, and formed of two sinusoidal segments rising to a
 peak.

;; Arguments: a b &key (peak (mean (list a b))) (height 1)

	a	-	left endpoint
	b	-	right endpoint
	peak	-	peak position (mean of a and b by default)
	height	-	value at the top of the peak (1 by default)

;; Returns: the dome function which is 0 everywhere but in [a,b], with a peak
   at peak of height 1 by default.

;; Notes: 

;; See Also: bump, heaviside

;; Key Functions:

;; Examples:
 (plot (dome 0 2) -1 3)
 (plot (dome 0 2 :peak 1.5 :height 2) -1 3)
"
  (lambda(x)
    (cond
     ((or (<= x a) (>= x b)) 0)
     ((< x peak) (* height (half (- 1 (cos (* pi (/ (- x a) (- peak a))))))))
     (t (* height (half (- 1 (cos (* pi (/ (- b x) (- b peak)))))))))))
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
