;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                lin-spline
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun lin-spline (xx yy xvals &optional (eps 1e-6) (max-min #'mean))
  "
;; Description: (lin-spline xx yy xvals &optional (eps 1e-6) (max-min #'mean))

Computes a linear spline to data.

;; Examples:
 (def a (rand.n 10))
 (def b (rand.n 10))
 (def c (lin-spline a b 200))
 (plot (first c) (second c))
 (send my-plot :add-points a b)
"
  (andy-print "entering lin-spline")
  (let* (
	 (x (simul-sort (coerce xx 'list) (coerce yy 'list)))
	 (y (second x))
	 (x (first x))
	 (spvals
	  (if (numberp xvals)
	      (rseq (min x) (max x) xvals)
	    (sort-data xvals)
	    )
	  )
	 (index 0)
	 vals xlast x1 x2 y1 y2 i
	 )
    (dotimes
     (i (- (length x) 2))
     (if (< (abs (- (elt x (1+ i)) (elt x i))) eps)
	 (let()
	   (setf (elt x i) nil)
	   (setf (elt y (1+ i)) 
		 (funcall max-min (list (elt y (1+ i)) (elt y i))))
	   (setf (elt y i) nil)
	   )
       )
     )
    (setq x (remove-if 'null x))
    (setq y (remove-if 'null y))
    (setq xlast (first (last x)))
    (dolist
     (z spvals)
     (setq x1 (elt x index))
     (cond
      ((> x1 z)
       (setq i 0)
       )
      ((< xlast z)
       (setq i (- (length x) 2))
       )
      (t
       (setq i
	     (do
	      ((ii index (1+ ii)))
	      ((>= (elt x ii) z) (1- ii))
	      )
	     )
       (setq i (if (< i 0) 0 i))
       )
      )
     (setq index i)
     (setq x1 (elt x i))
     (setq x2 (elt x (1+ i)))
     (setq y1 (elt y i))
     (setq y2 (elt y (1+ i)))
     (setq vals
	   (append vals
		   (list (+ y1 (* (- z x1) (/ (- y2 y1) (- x2 x1)))))
		   )
	   )
     )
    (andy-print "leaving lin-spline")
    (list spvals vals)
    )
  )



#|
%
%         function [yy]=clamp_cubic_spline(x,y,m1,m2,xx)
%
%	Gives the y-values yy corresponding to the x-values in
% vector xx, with end-point slopes given in m1 and m2.
%
%			x: set of node abscissa values
%			y: set of node y-values
%			m1: left-endpoint slope
%			m2: right-endpoint slope
%
%                                        Andy Long, 7/24/95
%
%
% Strategy: calculate the slopes that the cubic spline is going
% to hit and put them into an array, then call hermite!
%
% This is starting out as the natural spline, from an algorithm found on
% page 319 on the book "Numerical Analysis", by Cheney and Kincaid.

             function [yy]=clamp_cubic_spline(xin,yin,m1,m2,xxin)
			 
% x is the x-values of the nodes (sorted)
[x,index]=sort(xin);
y=yin(index);
slope=slopes(index);

[xx,index]=sort(xxin);

% nn is the length of the vector for which we want y values
nn=length(xx);
% n is the length of the node vector
n=length(x);

b=zeros(n,1);
for i=1:n-1
	h(i)=x(i+1)-x(i);
	b(i)=6*(y(i+1)-y(i));
end
u(2)=2(h(2)+h(1));
v(2)=b(2)-b(1);
for i=3:n-1
	u(i)=2(h(i)+h(i-1))-h(i-1)^2/u(i-1);
	v(i) = b(i)-b(i-1)-h(i-1)*v(i-1)/u(i-1);
end
z(n) = 0;
slopes(n) = m2;
for i=n-1:-1:2
  z(i)=(v(i)-h(i)*z(i+1))/u(i);
  slopes(i) = -h(i)*(z(i)+z(i+1)/2)/3 + (y(i+1)-y(i))/h(i);
end
slopes(1) = m1;
z(1) = 0;

yy=hermite(x,y,slopes,xx);
|#
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                clamped-spline
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun clamped-spline (xx yy xvals &key m1 m2)
  "
;; Description: function clamped-spline

                (xx yy xvals &key m1 m2)

Computes a clamped spline to data (xx,yy), then computes yvals corresponding to
the xvals. Clamped-spline requires (ordinarily) derivative values at the
endpoints. If you don't have a clue, clamped-spline will use the values of the
first order divided differences. This should give a good fit provided points
are dense at the ends, and fairly linear.

         xx: a list of x values
         yy: a corresponding list of y values
         xvals: either
                 1) a list of x values at which y values are to be computed, or
                 2) an integer, in which case points will be taken evenly
                               spaced, from left endpoint to right.
Optional:
         m1: left endpoint slope
         m2: right endpoint slope

;; Note: following Conte and de Boor, p. 290. This code derives the slopes at
;; the nodes, then calls hermite-spline.

;; See Also: hermite-spline, lin-spline, spline.

;; Examples:
 (print \"This should be the cubic x^3:\")
 (plot (clamped-spline '(0 1 2 3) '(0 1 8 27) 100 :m1 0 :m2 36 ))
 (plot (clamped-spline '(0 1 2 3) '(0 1 8 27) 100))

 (def a (rseq 0 pi 30))
 (def b (sin a))
 (title \"clamped\")
 (plot (clamped-spline a b 200 :m1 0 :m2 0))

 (def a '(1 2 5 6 7 8 10 13 17))
 (def b '(3 3.7 3.9 4.2 5.7 6.6 7.1 6.7 4.5))
 (def e (clamped-spline a b 100 :m1 1 :m2 -.67))
 (def c (first e))
 (def d (second e))
 (def a '(17 20 23 24 25 27 27.7))
 (def b '(4.5 7 6.1 5.6 5.8 5.2 4.1))
 (def e (clamped-spline a b 100 :m1 3 :m2 -4))
 (def c (combine c (first e)))
 (def d (combine d (second e)))
 (def a '(27.7 28 29 30))
 (def b '(4.1 4.3 4.1 3))
 (def e (clamped-spline a b 100 :m1 .33 :m2 -1.5))
 (def c (combine c (first e)))
 (def d (combine d (second e)))

 (title \"Snoopy\")
 (plot c d)
 (send my-plot :size 909 306)

 (title \"A Duck?\")
 (def a '(.9 1.3 1.9 2.1 2.6 3 3.9 4.4 4.7 5 6 7 8 9.2 10.5
         11.3 11.6 12 12.6 13 13.3))
 (def b '(1.3 1.5 1.85 2.1 2.6 2.7 2.4 2.15 2.05 2.1 2.25 2.3
         2.25 1.95 1.4 .9 .7 .6 .5 .4 .25))
 (plot (clamped-spline a b 100 :m1 .54 :m2 -0.552))
 (send my-plot :size 682 260)
 ;; should give the same as the natural spline below:
 (plot (spline a b :xvals 100))
 (send my-plot :size 682 260)
"
  (andy-print "entering clamped-spline")
  (let* (
	 (x (simul-sort xx yy))
	 (y (second x))
	 (x (first x))
	 (m1 (if m1 m1 (/ (- (second y) (first y)) (- (second x) (first x)))))
	 (m2 (if m2 m2 (/
			(- (second (reverse y)) (first (reverse y)))
			(- (second (reverse x)) (first (reverse x))))))

	 (n (1- (length x)))
	 (n-1 (1- n))

	 (b (iseq 0 n))
	 (d (iseq 0 n))
	 (diag (iseq 0 n))
	 g
	 )

    (setf (elt diag 0) 1)
    (setf (elt d 0) 0)
    (dolist
     (m (iseq 1 n))
     (setf (elt d m) (- (elt x m) (elt x (1- m))))
     (setf (elt diag m) (/ (- (elt y m) (elt y (1- m))) (elt d m)))
     )
    
    (setf (elt b 0) m1)
    (setf (elt b n) m2)
    (dolist
     (m (iseq 1 n-1))
     (setf (elt b m) (* 3 (+ (* (elt d m) (elt diag (1+ m)))
			     (* (elt diag m) (elt d (1+ m)))
			     )
			)
	   )
     (setf (elt diag m) (* 2 (+ (elt d m) (elt d (1+ m)))))
     )

    (dolist
     (m (iseq 1 n-1))
     (setq g (- (/ (elt d (1+ m)) (elt diag (1- m)))))
     (setf (elt diag m) (+ (* g (elt d (1- m))) (elt diag m)))
     (setf (elt b m) (+ (elt b m)
			 (* g (elt b (1- m)))
			 )
	   )
     )

    (dolist
     (m (iseq n-1 1))
     (setf (elt b m) (/
		       (-
			(elt b m)
			(* (elt d m) (elt b (1+ m)))
			)
		       (elt diag m)
		       )
	   )
     )

    (andy-print "leaving clamped-spline")
    (hermite-spline x y xvals :slopes b)
    )  
  )  
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Lagrange Stuff:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
(defmacro Lagrange (&rest points)
  "
;;
;; Lagrange Doc:    Creation Date:2/20/96        Author: Andy Long
;;
;; Description: macro

	(lagrange points)
  
where points is a bunch of (x,y) pairs.

;; See Also: lagrange-polynomial

;; Examples:
 (data crumbs (1 2) (2 2) (3 4) (4 2))
 (def stuff (lagrange crumbs))
 (defun lpoly(x)
  (if (listp x)
      (mapcar stuff x)
    (first (mapcar stuff (list x)))
    )
  )
 (lpoly 1)
 (plot (rseq 1 4 100) (lpoly (rseq 1 4 100)))
"
  `(lagrange-polynomial ,@points)
  )


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                lagrange-polynomial
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun lagrange-polynomial (points)
  "
;; Description:

	lagrange-polynomial (points)

where points is a list of pairs of points.

;; Examples:
 (data crumbs (1 2) (2 2) (3 4) (4 2))
 (def stuff (lagrange-polynomial crumbs))
 (defun lpoly(x)
  (if (listp x)
      (mapcar stuff x)
    (first (mapcar stuff (list x)))
    )
  )
 (lpoly 1)
 (plot (rseq 1 4 100) (lpoly (rseq 1 4 100)))
"
  (let (
	(pl (length points))
	(xvals (mapcar #'first points))
	(yvals (mapcar #'second points))
	(len (length points))
	)
    #'(lambda (x)
	(let ((lpoly-val 0))
	  (dotimes
	   (i len)
	   (let ((xs (select xvals (remove i (iseq 0 (1- pl))))))
	     (setq lpoly-val
		   (+
		    lpoly-val
		    (* (elt yvals i)
		       (/
			(apply #'* (- x xs))
			(apply #'* (- (elt xvals i) xs))
			)
		       )
		    )
		   )
	     )
	   )
	  lpoly-val
	  )
	)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                lagrange-spline
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun lagrange-spline (xx yy xvals &optional (eps 1e-6) (max-min #'mean))
  "
;; Description:

   (lagrange-spline xx yy xvals &optional (eps 1e-6) (max-min #'mean))

Computes a Lagrange polynomial spline to data. If there are x values less than
eps apart, then they will be replaced by a single point, given by max-min of
those points.

;; Examples:
 (def a '(1 2 3 4))
 (def b '(2 2 4 3))

 (def c (lagrange-spline a b 200))
 (plot (first c) (second c))
 (send my-plot :add-plot-item (list 'points a b))

 (def c (lagrange-spline a b (rseq 1 2 100)))
 (plot (first c) (second c))
 (send my-plot :add-plot-item (list 'points a b))

 ;; It doesn't matter if they're ordered incorrectly or we have duplicates:
 (def a (reverse '(1 2 1 2 5 6 7 8 10 13 15 17 17)))
 (def b (reverse '(2 3 3 3.7 3.9 4.2 5.7 6.6 7.1 6.7 4.5 5.3 5.5)))
 (def e (lagrange-spline a b 100))
 (plot e)
 (send my-plot :add-plot-item (list 'points a b))
"
  (andy-print "entering lagrange-spline")
  (let* (
	 (xx (coerce xx 'list))
	 (yy (coerce yy 'list))
	 (order (order xx))
	 (x (select xx order))
	 (y (select yy order))
	 (spvals
	  (if (numberp xvals) (rseq (min x) (max x) xvals) (sort-data xvals)))
	 vals xlast stuff
	 )
    (dotimes
     (i (- (length x) 1))
     (if (< (abs (- (elt x (1+ i)) (elt x i))) eps)
	 (let ()
	  (setf (elt x i) nil)
	  (setf (elt y (1+ i))
		(funcall max-min (list (elt y (1+ i)) (elt y i))))
	  (setf (elt y i) nil)
	  )
       )
     )
    (setq x (remove-if 'null x))
    (setq y (remove-if 'null y))
    (setf stuff (lagrange-polynomial (transpose (list x y))))

    (andy-print "leaving lagrange-spline")
    (list spvals (mapcar stuff spvals))
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                             B-spline stuff:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                b-spline
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun b-spline(i q x)
"
	b-spline(i q x)

	This routine requires that you define a, b, m, d, and csi, as well:

        a - lower limit of the interval;
        b - upper limit of the interval;
        m - number of inner knots;
	d - the degree of the spline (actually degree is d-1); and 
	csi - the list of knots (inner and boundary).

	I wrote this one because I read an article entitle 'Regression Splines
in Models of Time Trends and Interactions', Biometrics 53, 161-177, March 1997,
by Carsten Heuer, and it was inspirational.

;;
;; Non-periodic B-spline, according to
;; http://www.astro.virginia.edu/~eww6n/math/B-Spline.html
;;

;; Examples:
 (def a 0)
 (def b 1)
 (def m 6)
 (def d 4) ;; cubic b-splines
 (def q 3)
 (def csi (combine (repeat a q) (/ (iseq 1 m) (1+ m)) (repeat b q)))
 ;; The b-splines should be defined from i=0...m+q-1.

;;; This doesn't work, as the i change is reflected in the lambda expression.
;; (plot (lambda(x) (b-spline 0 4 x)) a b)
;; (dolist
;;  (i (iseq 1 (+ m q -1)))
;;  (send my-plot :add-plot-item
;;       (list
;;	'function
;;	(lambda(x)(b-spline i 4 x))
;;	)
;;       )
;;  )

 ;; This works! Somehow the dolist is more pernicious than the mapcar:
 (plot (lambda(x) (b-spline 0 4 x)) a b)
 (mapcar
  (lambda(i)
    (send my-plot :add-plot-item
 	 (list
 	  'function
 	  (lambda(x)(b-spline i 4 x))
          'black
          t
 	  )
         :draw nil
 	 ))
  (iseq 1 (+ m q -2))
  )
"
  (let* (
	 (1+i (+ i 1))
	 (csi_i (elt csi i))
	 (csi_i+1 (elt csi 1+i))
	 )
    (cond
     ((= q 1)
      (if (<= i (+ m (* 2 d) -3))
	  (if (and (>= x csi_i) (< x csi_i+1)) 1 0)
	(if (= i (+ m (* 2 d) -2))
	    (if (and (>= x csi_i) (<= x csi_i+1)) 1 0)
	  ;; Otherwise (and this shouldn't happen):
	  (error
	   (format
	    t
	    "Index i=~d exceeds acceptible value ~d in (b-spline ~d ~d ~g).
Looks like there's a problem in the definition of b-spline."
	    i (+ m (* 2 d) -2) i q x
	    )
	   "Sorry!"
	   )
	  )
	)
      )
     (t
      (let* (
	     (i+q (+ i q))
	     (i+q-1 (- i+q 1))
	     (csi_i+q (elt csi i+q))
	     (csi_i+q-1 (elt csi i+q-1))
	     (diff1 (- csi_i+q-1 csi_i))
	     (diff2 (- csi_i+q csi_i+1))
	     (r1
	      (if (/= 0 diff1)
		  (* (- x csi_i) (b-spline i (1- q) x) (/ diff1))
		0
		)
	      )
	     (r2
	      (if (/= 0 diff2)
		  (* (- csi_i+q x) (b-spline (1+ i) (1- q) x) (/ diff2))
		0
		)
	      )
	     )
	(+ r1 r2)
	)
      )      
     )
    )
  )


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                b-spline-double-prime
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun b-spline-double-prime(i q x)
  "
	b-spline-double-prime(i q x)

	This is the second derivative function of the b-splines, defined as in
'Regression Splines in Models of Time Trends and Interactions', Biometrics 53,
161-177, March 1997, by Carsten Heuer.

;; See also: b-spline

;; Examples:
 (def a 0)
 (def b 1)
 (def m 6)
 (def d 4)
 (def q 3)
 (def csi (combine (repeat a q) (/ (iseq 1 m) (1+ m)) (repeat b q)))

 (plot (lambda(x) (b-spline 0 4 x)) a b)
 (mapcar
  (lambda(i)
    (send my-plot :add-plot-item
 	 (list
 	  'function
 	  (lambda(x)(b-spline i 4 x))
          'black
          t
 	  )
         :draw nil
 	 ))
  (iseq 1 (+ m q -2))
  )

 (plot (lambda(x) (b-spline-double-prime 0 4 x)) a b)
 (mapcar
  (lambda(i)
    (send my-plot :add-plot-item
 	 (list
 	  'function
 	  (lambda(x)(b-spline-double-prime i 4 x))
          'black
          t
 	  )
         :draw nil
 	 ))
  (iseq 1 (+ m q -2))
  )
"
  (let* (
	 (i+1 (+ i 1))
	 (i+2 (+ i 2))
	 (q-1 (- q 1))
	 (q-2 (- q 2))
	 (i+q (+ i q))
	 (i+q-1 (- i+q 1))
	 (i+q-2 (- i+q 2))

	 (csi_i (elt csi i))
	 (csi_i+1 (elt csi i+1))
	 (csi_i+2 (elt csi i+2))
	 (csi_i+q (elt csi i+q))
	 (csi_i+q-1 (elt csi i+q-1))
	 (csi_i+q-2 (elt csi i+q-2))

	 (diff1 (- csi_i+q-1 csi_i))
	 (diff2 (- csi_i+q csi_i+1))
	 (diff3 (- csi_i+q-2 csi_i+1))
	 (diff4 (- csi_i+q-1 csi_i+1))
	 (diff5 (- csi_i+q csi_i+2))

	 (r1
	  (if (/= 0 diff1)
	      (/
	       (- 
		(if (= diff3 0) 0 (/ (b-spline i q-2 x) diff3))
		(if (= diff4 0) 0 (/ (b-spline i+1 q-2 x) diff4))
		)
	       diff1
	       )
	    0
	    )
	  )
	 (r2
	  (if (/= 0 diff2)
	      (/
	       (-
		(if (= diff4 0) 0 (/ (b-spline i+1 q-2 x) diff4))
		(if (= diff5 0) 0 (/ (b-spline i+2 q-2 x) diff5))
		)
	       diff2
	       )
	    0
	    )
	  )
	 )
    (* q-1 q-2 (- r1 r2))
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Hermite Stuff:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
#|
Here's the matlab code that I'm using to make hermite-spline
%
%         function [yy]=hermite(x,y,slopes,xx)
%
%	Gives the y-values yy corresponding to the x-values in
% vector xx, with node slopes given in the vector slopes.
%
%			x: set of node abscissa values
%			y: set of node y-values
%			slopes: set of node slope values
%
%                                        Andy Long, 7/21/95

             function [yy]=hermite(xin,yin,slopes,xxin)

% x is the x-values of the nodes (sorted)
[x,index]=sort(xin);
y=yin(index);
slope=slopes(index);

xx,index]=sort(xxin);

% nn is the length of the vector for which we want y values
nn=length(xx);
% n is the length of the node vector
n=length(x);

minx = x(1);
maxx = x(n);
ynot=xx;

jmin=1;
for i=1:nn,	% for all x-values of interest
	xnot=xx(i);
	if xnot <= minx
		j1=1;
		j2=2;
	elseif xnot >= maxx
		j1=n-1;
		j2=n;
	else
		for j=jmin:n-1,
			if ((xnot >= x(j)) & (xnot <= x(j+1)))
				break;
			end
		end
		j1=j;
		j2=j+1;
	end
	x1=x(j1);
	x2=x(j2);
	y1=y(j1);
	y2=y(j2);
 	m1=slope(j1);
	m2=slope(j2);

	del=x2-x1;
	ydel=y2-y1;
	ratio=(xnot-x1)/del;
	
	gamma=m1*del;
	delta=y1;
	alpha=del*(m1+m2)-2*ydel;
	beta=3*ydel-del*(2*m1+m2);
	
	ynot(i)=delta+ratio*(gamma+ratio*(beta+alpha*ratio));
	
	jmin=j1;
end
yy=ynot(index);	
|#
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                hermite-spline
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun hermite-spline (xx yy xvals &key slopes (eps 1e-6) (max-min #'mean))
  "
;; Description: function hermite-spline

                (xx yy xvals &optional (eps 1e-6) (max-min #'mean))

Computes a hermite spline to data (xx,yy), then computes yvals corresponding to
the xvals.

	max-min is a choice that's only important if you have repeat x values
	in your sequence. If so, the max-min function will be used to get a new
	value at that location.

	eps represents the distance between two x values below which they are
	treated as identical.

;; Examples:
 (def a (rseq 0 pi 30))
 (def b (sin a))
 (title \"hermite\")
 (plot (hermite-spline a b 200))
 (send my-plot :add-plot-item (list 'points a b))

 (title \"spline\")
 (plot (spline a b :xvals 200))
 (send my-plot :add-plot-item (list 'points a b))

 (title \"linear spline\")
 (plot (lin-spline a b 200))
 (send my-plot :add-plot-item (list 'points a b))

 ;; Here we have duplicate points - there're averaged:
 (def a '(1 1 2 2 5 6 7 8 10 13 17 17))
 (def b '(2 3 3 3.7 3.9 4.2 5.7 6.6 7.1 6.7 4.5 5.5))
 (def e (hermite-spline a b 100))
 (plot e)
 (send my-plot :add-plot-item (list 'points a b))

 ;; It doesn't matter if they're ordered incorrectly:
 (def a (reverse '(1 2 1 2 5 6 7 8 10 13 17 17)))
 (def b (reverse '(2 3 3 3.7 3.9 4.2 5.7 6.6 7.1 6.7 4.5 5.5)))
 (def e (hermite-spline a b 100))
 (plot e)
 (send my-plot :add-plot-item (list 'points a b))

 (def a '(1 2 5 6 7 8 10 13 17))
 (def b '(3 3.7 3.9 4.2 5.7 6.6 7.1 6.7 4.5))
 (def e (hermite-spline a b 100))
 (def c (first e))
 (def d (second e))
 (def a '(17 20 23 24 25 27 27.7))
 (def b '(4.5 7 6.1 5.6 5.8 5.2 4.1))
 (def e (hermite-spline a b 100))
 (def c (combine c (first e)))
 (def d (combine d (second e)))
 (def a '(27.7 28 29 30))
 (def b '(4.1 4.3 4.1 3))
 (def e (hermite-spline a b 100))
 (def c (combine c (first e)))
 (def d (combine d (second e)))

 (title \"snoopy\")
 (plot c d)
 (send my-plot :size 834 286)
"
  (andy-print "entering hermite-spline")
  (let* (
         (xx (coerce xx 'list))
	 (yy (coerce yy 'list))
	 (order (order xx))
	 (x (select xx order))
	 (y (select yy order))
	 (slopes (if slopes (select slopes order) nil))
	 (spvals
	  (if (numberp xvals) (rseq (min x) (max x) xvals) (sort-data xvals))
	  )
	 (index 0)
	 (xlast (final x))
	 vals
	 )

    ;; Here we handle cases where two xvalues are too close (as defined by eps)
    ;; We clean them by putting nils in, and then remove them below.
    (dotimes
     (i (- (length x) 1))
     (if (< (abs (- (elt x (1+ i)) (elt x i))) eps)
	 (let()
	   (setf (elt x i) nil)
	   (setf (elt y (1+ i))
		 (funcall max-min (list (elt y (1+ i)) (elt y i))))
	   (setf (elt y i) nil)
	   (if slopes (setf (elt slopes i) nil))
	   )
       )
     )
    (setq x (remove-if 'null x))
    (setq y (remove-if 'null y))

    (setq slopes 
	  (if slopes (remove-if 'null slopes)
	    ;; otherwise, use divided differences within, forward and aft at
	    ;; the ends:
	    (if (> (length y) 2)
		(combine
		 (/ (- (second y) (first y)) (- (second x) (first x)))
		 (/ (- (rest (rest y)) (butlast (butlast y)))
		    (- (rest (rest x)) (butlast (butlast x))))
		 (/ (- (first (reverse y)) (second (reverse y)))
		    (- (first (reverse x)) (second (reverse x))))
		 )
	      (let (
		    (s (/ (- (second y) (first y)) (- (second x) (first x))))
		    )
		(list s s))
	      )
	    )
	  )
    
    ;; Now we head over the list of x values at which we want y values.
    (dolist
     (z spvals)
     (let* (
	    ;; we start with the nearest element of the x values below our 
	    ;; datapoint: 
	    (x1 (elt x index))
	    ;; we bound the value z of interest between two x values (if
	    ;; possible):
	    (index
	     (cond
	      ((> x1 z) 0 );; z is less than any x value;
	      ((< xlast z) (- (length x) 2)) ;; z is greater than any x value;
	      (t ;; we move the index until we find an x value greater than z:
	       (do
		((ii 1 (1+ ii)))
		((>= (elt x ii) z) (1- ii))
		)
	       )
	      )
	     )
	    (x1 (elt x index))
	    (x2 (elt x (1+ index)))
	    (y1 (elt y index))
	    (y2 (elt y (1+ index)))
	    (m1 (elt slopes index))
	    (m2 (elt slopes (1+ index)))
	    (del (- x2 x1))
	    (ydel (- y2 y1))
	    (ratio (/ (- z x1) del))
	    (gamma (* m1 del))
	    (delta y1)
	    (alpha (- (* del (+ m1 m2)) (* 2 ydel)))
	    (beta (- (* 3 ydel) (* del (+ (* 2 m1) m2))))
	    )
       (setq
	vals
	(append
	 vals
	 (list
	  (+ delta
	     (* ratio (+ gamma (* ratio (+ beta (* alpha ratio))))))
	  )
	 )
	)
       )
     )
    (andy-print "leaving hermite-spline")
    (list spvals vals)
    )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 hermite-spline-function
;;
(defun hermite-spline-function
  (xx yy
      &key slopes (eps 1e-6) (max-min #'mean))
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          hermite-spline-function Doc:
;;
;; Author: Andy Long
;; Created: Fri May 2 09:14:56 CDT 1997
;;
;; Description/Arguments:

 (hermite-spline-function
    xx yy
       &key slopes (eps 1e-6) (max-min #'mean))

	max-min is a choice that's only important if you have repeat x values
	in your sequence. If so, the max-min function will be used to get a new
	value at that location.

	eps represents the distance between two x values below which they are
	treated as identical.

;; Examples:
 (def a (rseq 0 pi 30))
 (def b (sin a))
 (title \"hermite\")
 (plot (hermite-spline-function a b) 0 pi)
 (send my-plot :add-plot-item (list 'points a b))

 (def a '(1 2 5 6 7 8 10 13 17))
 (def b '(3 3.7 3.9 4.2 5.7 6.6 7.1 6.7 4.5))
 (plot (hermite-spline-function a b) (min a) (max a))
 (send my-plot :add-plot-item (list 'points a b))

 (def a '(17 20 23 24 25 27 27.7))
 (def b '(4.5 7 6.1 5.6 5.8 5.2 4.1))
 (plot (hermite-spline-function a b) (min a) (max a))
 (send my-plot :add-plot-item (list 'points a b))

 (def a '(27.7 28 29 30))
 (def b '(4.1 4.3 4.1 3))
 (plot (hermite-spline-function a b) (min a) (max a))
 (send my-plot :add-plot-item (list 'points a b))
"
  (let* (
	 (x (coerce xx 'list))
	 (y (coerce yy 'list))
	 (slopes (coerce slopes 'list))
	 (order (order x))
	 (x (select x order))
	 (y (select y order))
	 (slopes (if slopes (select slopes order) nil))
	 (xlast (final x))
	 (x1 (elt x 0))
	 )
    
    ;; Here we handle cases where two xvalues are too close (as defined by eps)
    ;; We clean them by putting nils in, and then remove them below.
    (dotimes (i (- (length x) 1))
	     (if (< (abs (- (elt x (1+ i)) (elt x i))) eps)
		 (let()
		   (setf (elt x i) nil)
		   (setf (elt y (1+ i)) 
			 (funcall max-min (list (elt y (1+ i)) (elt y i))))
		   (setf (elt y i) nil)
		   (if slopes (setf (elt slopes i) nil))
		   )
	       )
	     )
    (setq x (remove-if 'null x))
    (setq y (remove-if 'null y))

    (setq slopes 
	  (if slopes (remove-if 'null slopes)
	    ;; otherwise, use divided differences within, forward and aft at
	    ;; the ends:
	    (combine
	     (/ (- (second y) (first y)) (- (second x) (first x)))
	     (/ (- (rest (rest y)) (butlast (butlast y)))
		(- (rest (rest x)) (butlast (butlast x))))
	     (/ (- (first (reverse y)) (second (reverse y)))
		(- (first (reverse x)) (second (reverse x))))
	     )
	    )
	  )

    (lambda(z)
      ;; we bound the value z of interest between two x values (if we can!)
      (let* (
	     (index
	      (cond
	       ((> x1 z) 0 );; z is less than any x value
	       ((< xlast z) (- (length x) 2)) ;; z is greater than any x value
	       (t  ;; we move the index until we find an x value greater than z:
		(do
		 ((ii 1 (1+ ii)))
		 ((>= (elt x ii) z) (1- ii))
		 )
		)
	       )
	      )
	     (x1 (elt x index))
	     (x2 (elt x (1+ index)))
	     (y1 (elt y index))
	     (y2 (elt y (1+ index)))
	     (m1 (elt slopes index))
	     (m2 (elt slopes (1+ index)))
	     (del (- x2 x1))
	     (ydel (- y2 y1))
	     (ratio (/ (- z x1) del))
	     (gamma (* m1 del))
	     (delta y1)
	     (alpha (- (* del (+ m1 m2)) (* 2 ydel)))
	     (beta (- (* 3 ydel) (* del (+ (* 2 m1) m2))))
	     )
	(+ delta (* ratio (+ gamma (* ratio (+ beta (* alpha ratio))))))
	)
      )
    )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 hermite-spline-derivative
;;
(defun hermite-spline-derivative
  (xx yy
      &key slopes (eps 1e-6) (max-min #'mean))
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          hermite-spline-derivative Doc:
;;
;; Author: Andy Long
;; Created: Fri May 2 09:14:56 CDT 1997
;;
;; Description/Arguments:

 (hermite-spline-derivative
    xx yy
       &key slopes (eps 1e-6) (max-min #'mean))

	max-min is a choice that's only important if you have repeat x values
	in your sequence. If so, the max-min function will be used to get a new
	value at that location.

	eps represents the distance between two x values below which they are
	treated as identical.

;; Examples:
 (def a (rseq 0 pi 30))
 (def b (sin a))
 (title \"hermite - cosine?\")
 (plot cos (hermite-spline-derivative a b) 0 pi)

 (def a (rseq 0 pi 30))
 (def b (square a))
 (title \"hermite - cosine?\")
 (plot {2*x} (hermite-spline-derivative a b) 0 pi)

 (def a '(17 20 23 24 25 27 27.7))
 (def b '(4.5 7 6.1 5.6 5.8 5.2 4.1))
 (plot (hermite-spline-derivative a b) (min a) (max a))

 ;; Repeat points are okay: they'll be averaged by default:
 (def a '(17 17 20 23 24 25 27 27.7))
 (def b '(4.5 5.5 7 6.1 5.6 5.8 5.2 4.1))
 (plot (hermite-spline-derivative a b) (min a) (max a))
"
  (let* (
	 (xx (coerce xx 'list))
	 (yy (coerce yy 'list))
	 (slopes (if slopes (coerce slopes 'list)))
	 (order (order xx))
	 (x (select xx order))
	 (y (select yy order))
	 (slopes (if slopes (select slopes order) nil))
	 (xlast (final x))
	 (x1 (elt x 0))
	 )
    
    ;; Here we handle cases where two xvalues are too close (as defined by eps)
    ;; We clean them by putting nils in, and then remove them below.
    (dotimes (i (- (length x) 1))
	     (if (< (abs (- (elt x (1+ i)) (elt x i))) eps)
		 (let()
		   (setf (elt x i) nil)
		   (setf (elt y (1+ i))
			 (funcall max-min (list (elt y (1+ i)) (elt y i))))
		   (setf (elt y i) nil)
		   (if slopes (setf (elt slopes i) nil))
		   )
	       )
	     )
    (setq x (remove-if 'null x))
    (setq y (remove-if 'null y))

    (setq slopes 
	  (if slopes (remove-if 'null slopes)
	    ;; otherwise, use divided differences within, forward and aft at
	    ;; the ends:
	    (combine
	     (/ (- (second y) (first y)) (- (second x) (first x)))
	     (/ (- (rest (rest y)) (butlast (butlast y)))
		(- (rest (rest x)) (butlast (butlast x))))
	     (/ (- (first (reverse y)) (second (reverse y)))
		(- (first (reverse x)) (second (reverse x))))
	     )
	    )
	  )


    (lambda(z)
      ;; we bound the value z of interest between two x values (if we can!)
      (let* (
	     (index
	      (cond
	       ((> x1 z) 0 );; z is less than any x value
	       ((< xlast z) (- (length x) 2)) ;; z is greater than any x value
	       (t  ;; we move the index until we find an x value greater than z:
		(do
		 ((ii 1 (1+ ii)))
		 ((>= (elt x ii) z) (1- ii))
		 )
		)
	       )
	      )
	     (x1 (elt x index))
	     (x2 (elt x (1+ index)))
	     (y1 (elt y index))
	     (y2 (elt y (1+ index)))
	     (m1 (elt slopes index))
	     (m2 (elt slopes (1+ index)))
	     (del (- x2 x1))
	     (ydel (- y2 y1))
	     (ratio (/ (- z x1) del))
	     (gamma (* m1 del))
	     (delta y1)
	     (alpha (- (* del (+ m1 m2)) (* 2 ydel)))
	     (beta (- (* 3 ydel) (* del (+ (* 2 m1) m2))))
	     )
	(/ (+ gamma (* ratio (+ (* 2 beta) (* ratio (* 3 alpha))))) del)
	)
      )
    )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 hermite-spline-second-derivative
;;
(defun hermite-spline-second-derivative
  (xx yy
      &key slopes (eps 1e-6) (max-min #'mean))
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          hermite-spline-second-derivative Doc:
;;
;; Author: Andy Long
;; Created: Fri May 2 09:14:56 CDT 1997
;;
;; Description/Arguments:

 (hermite-spline-second-derivative
    xx yy
       &key slopes (eps 1e-6) (max-min #'mean))

	max-min is a choice that's only important if you have repeat x values
	in your sequence. If so, the max-min function will be used to get a new
	value at that location.

	eps represents the distance between two x values below which they are
	treated as identical.

;; Examples:
 (def a (rseq 0 pi 30))
 (def b (sin a))
 (title \"hermite - cosine?\")
 (plot {-(sin x)} (hermite-spline-second-derivative a b) 0 pi)

 (def a (rseq 0 pi 30))
 (def b (square a))
 (title \"hermite - cosine?\")
 (plot {2} (hermite-spline-second-derivative a b) 0 pi)

 (def a '(17 20 23 24 25 27 27.7))
 (def b '(4.5 7 6.1 5.6 5.8 5.2 4.1))
 (plot (hermite-spline-second-derivative a b) (min a) (max a))

 ;; Repeated points are okay: they'll be averaged:
 (def a '(17 17 20 23 24 25 27 27.7))
 (def b '(4.5 5.5 7 6.1 5.6 5.8 5.2 4.1))
 (plot (hermite-spline-second-derivative a b) (min a) (max a))
"
  (let* (
	 (xx (coerce xx 'list))
	 (yy (coerce yy 'list))
	 (slopes (if slopes (coerce slopes 'list)))
	 (order (order xx))
	 (x (select xx order))
	 (y (select yy order))
	 (slopes (if slopes (select slopes order) nil))
	 (xlast (final x))
	 (x1 (elt x 0))
	 )
    
    ;; Here we handle cases where two xvalues are too close (as defined by eps)
    ;; We clean them by putting nils in, and then remove them below.
    (dotimes (i (- (length x) 1))
	     (if (< (abs (- (elt x (1+ i)) (elt x i))) eps)
		 (let()
		   (setf (elt x i) nil)
		   (setf (elt y (1+ i))
			 (funcall max-min (list (elt y (1+ i)) (elt y i))))
		   (setf (elt y i) nil)
		   (if slopes (setf (elt slopes i) nil))
		   )
	       )
	     )
    (setq x (remove-if 'null x))
    (setq y (remove-if 'null y))
    
    (setq slopes 
	  (if slopes (remove-if 'null slopes)
	    ;; otherwise, use divided differences within, forward and aft at
	    ;; the ends:
	    (combine
	     (/ (- (second y) (first y)) (- (second x) (first x)))
	     (/ (- (rest (rest y)) (butlast (butlast y)))
		(- (rest (rest x)) (butlast (butlast x))))
	     (/ (- (first (reverse y)) (second (reverse y)))
		(- (first (reverse x)) (second (reverse x))))
	     )
	    )
	  )
    
    
    (lambda(z)
      ;; we bound the value z of interest between two x values (if we can!)
      (let* (
	     (index
	      (cond
	       ((> x1 z) 0 );; z is less than any x value
	       ((< xlast z) (- (length x) 2)) ;; z is greater than any x value
	       (t  ;; we move the index until we find an x value greater than z:
		(do
		 ((ii 1 (1+ ii)))
		 ((>= (elt x ii) z) (1- ii))
		 )
		)
	       )
	      )
	     (x1 (elt x index))
	     (x2 (elt x (1+ index)))
	     (y1 (elt y index))
	     (y2 (elt y (1+ index)))
	     (m1 (elt slopes index))
	     (m2 (elt slopes (1+ index)))
	     (del (- x2 x1))
	     (ydel (- y2 y1))
	     (ratio (/ (- z x1) del))
	     (gamma (* m1 del))
	     (delta y1)
	     (alpha (- (* del (+ m1 m2)) (* 2 ydel)))
	     (beta (- (* 3 ydel) (* del (+ (* 2 m1) m2))))
	     )
	(/ (+ (* 2 beta) (* ratio 6 alpha)) (* del del))
	)
      )
    )
  )
