;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                grand-stats
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun grand-stats (&rest means-sds-ns)
  "
;; Description: grand-stats (&rest means-sds-ns)

	Grand-stats returns the grand-mean, grand-sd, and grand-n for a set of
sub-groups of a population, each with its own mean, standard deviation, and n.

;; Examples:
 (def gsx (rand.u 10))
 (def gsy (rand.u 20))
 (def gsboth (combine gsx gsy))  
 (list (mean gsboth) (sd gsboth) (length gsboth))
 (grand-stats
   (mean gsx) (sd gsx) (length gsx)
   (mean gsy) (sd gsy) (length gsy)
   )

 (grand-stats
   11.625 10.141 8
   20.5 32.72 8
   13 16.31 8
   )
 (grand-stats
   22.857 23.639 7
   23 23.875 5
   14.125 16.915 8
   )
"
  (let*(
	(x (transpose (split-list means-sds-ns 3)))
	(mean (first x))
	(sd (second x))
	(n (third x))
	(len (length n))
	(grand-n (sum n))
	(grand-mean (/ (sum (* n mean)) grand-n))
	(sum-xi-squared (+ (* (- n 1) (square sd)) (* n (square mean))))
	)
    (list
     grand-mean
     (sqrt (/ (- (sum sum-xi-squared)
		 (* grand-n (square grand-mean))
		 )
	      (1- grand-n)))
     grand-n
     )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                            power
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun power (mean sd n &key (prob .975) values function type)
  "
;; Description: power  (mean sd n &key (prob .975) values function type)
                                       ;; .975 for two-sided

	Power plots the two-sided power curve for a normal random variable with
given parameters mean and sd, and given sample size n. One may also specify the
probability desired.

	Options:
		:prob		1-alpha/2 of test
		:values		a list of values at which the power should be
				given. The powers are returned.
		:function	return the function
		:type		If you want the t distribution, rather than
				normal, give :type 't

If neither values nor function are specified, then a plot of the power curve is
made.

;; See also: start-power (computes power for many distributions, one- and
;; two-sided, and sample-size needed for given power)

;; Examples:
 (new-plot)
 (title \"Power Curves and Calculations\")
 (lines on)
 ;; this example comes from the book 'Biostatistics: a Foundation for analysis
 ;; in the health sciences', by Wayne W. Daniel, 6th edition; reproduces
 ;; figure, p. 257, and the table 7.9.1 on 256.

 (domain 15 20)
 (power 17.5 3.6 100)
 (power 17.5 3.6 100 :prob .85)
 (power 17.5 3.6 100 :values '(16 16.5 17 18 18.5 19))
 (power 17.5 3.6 100 :values '(16 16.5 17 18 18.5 19) :type 't)

 (plot (power 17.5 3.6 100 :function t) 15 20)
 (title \"As alpha varies from .05 to 1:\")
 (animate
  (lambda(y x)
    (funcall (power 17.5 3.6 100 :prob y :function t) x))
  (rseq .975 .51 30)
  15 20
  :loops 1)
 (title \"As n varies from 105 to 9:\")
 (animate
  (lambda(n x)
    (funcall (power 17.5 3.6 n :function t) x))
  (* 3 (iseq 35 3))
  15 20
  :loops 1)
"
  (let* (
	 (denom (/ sd (sqrt n)))
	 (spread (* denom (normal-quant prob)))
	 (spread-around (list (+ mean spread) (- mean spread)))
	 (1-n (1- n))
	 (f
	  (if (equalp type 't)
	      (lambda(x)
		(- 1 
		   (apply
		    #'-
		    (t-cdf (/ (- spread-around x) denom) 1-n))))
	    (lambda(x)
	      (- 1 
		 (apply
		  #'-
		  (normal-cdf (/ (- spread-around x) denom)))))
	    )
	  )
	 )
    (if function f (if values (mapcar f values) (plotter f)))
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                            power-n
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun power-n (half-width sd &key (alpha .05))
  "
;; Description: power-n (half-width sd &key (alpha .05))

	Power-n calculates the power needed to estimate the mean of a normal
population with a standard deviation of sd to a given half-width at a given
alpha level.

;; See also: power

;; Examples:
 ;; this example comes from the book 'Biostatistics: a Foundation for analysis
 ;; in the health sciences', by Wayne W. Daniel, 6th edition; ex. 6.7.1, p. 179
 (power-n 5 20)
"
  (square (/ (* (normal-quant (- 1 (half alpha))) sd) half-width))
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                            power-right
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun power-right (mean sd n &key (prob .95))
  "
;; Description: power-right (mean sd n &key (prob .95))

;; Examples:
 (new-plot)
 (lines on)
 ;; this example comes from the book 'Biostatistics: a Foundation for analysis
 ;; in the health sciences', by Wayne W. Daniel, 6th edition; reproduces
 ;; figure, p. 257. 
 (domain 60 80)
 (power-right 65 15 20)
 (new-plot)
"
  (let* (
	 (denom (/ sd (sqrt n)))
	 (spread (* denom (normal-quant prob)))
	 (xu (+ mean spread))
	 )
    (plotter (lambda(x) (- 1 (normal-cdf (/ (- xu x) denom)))))
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                            power-left
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun power-left (mean sd n &key (prob .95))
  "
;; Description: power-left (mean sd n &key (prob .95))

;; Examples:
 (new-plot)
 (lines on)
 ;; this example comes from the book 'Biostatistics: a Foundation for analysis
 ;; in the health sciences', by Wayne W. Daniel, 6th edition; reproduces
 ;; figure, p. 258.
 (domain 49 65)
 (power-left 65 15 20)
 (new-plot)
"
  (let* (
	 (denom (/ sd (sqrt n)))
	 (spread (* denom (normal-quant prob)))
	 (xl (- mean spread))
	 )
    (plotter (lambda(x) (normal-cdf (/ (- xl x) denom))))
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                            power-difference
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun power-difference (mean1 mean2 sd n1 n2 &key (prob .975))
  "
;; inverses:
;; (normal-cdf 1.96)
;; (normal-quant .025)
;; (plot (lambda(x) (normal-quant (normal-cdf x))))

;; Examples:
 (new-plot)
 (lines on)
 (domain -3 3)
 (title \"small numbers, wide spread\")
 (power-difference .3 1 2 20 20)
 (title \"large numbers, small spread\")
 (power-difference .3 1 2 200 200) ;; see increasing observations will do!
 (new-plot)
"
  (let* (
	 (mean (abs (- mean1 mean2)))
	 (denom (* sd (sqrt (+ (/ n1) (/ n2)))))
	 (spread (* denom (normal-quant prob)))
	 (xu (+ mean spread))
	 (xl (- mean spread))
	 )
    (plotter
     (lambda(x)
       (- 1 
	  (-
	   (normal-cdf (/ (- xu x) denom))
	   (normal-cdf (/ (- xl x) denom))
	   )
	  )
       )
     )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                            power-weighted-difference
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun power-weighted-difference (mean1 mean2 w1 w2 sd n1 n2 &key (prob .975))
  "

;; Examples:
 (new-plot)
 (lines on)
 (domain -3 3)
 (title \"small numbers, wide spread\")
 (power-weighted-difference 19.4 15.04 .44 .56 20.83 22 28)
 (new-plot)
"
  (let* (
	 (mean (abs (- mean1 mean2)))
	 (denom (* sd (sqrt (+ (/ n1) (/ n2)))))
	 (denom2 (* sd (sqrt (/ n2))))
	 (spread (* denom (normal-quant prob)))
	 (xx (/ (- spread (* w1 mean)) w2))
	 )
    (plotter (lambda(x) (- 1 (normal-cdf (/ (- xx x) denom)))))
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                univariate
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun univariate(&rest stuff)
  "
;; Documentation: (univariate arguments)

	Where arguments can be lists, matrices, or whatnot. If matrices, then
statistics will be performed on the columns as variables.

;; Examples:
 (univariate (rand.n 100))
 (univariate (id 4))
" 
  (let (result)
    (dolist
     (i stuff)
     (if (matrixp i) (setq result (mapcar #'univariate (column-list i)))
       (let (
	     (mini (min i))
	     (meani (mean i))
	     (mediani (median i))
	     (maxi (max i))
	     (sdi (sd i))
	     (ske (skewness i))
	     (kur (kurtosis i))
	     (lengthi (length i))
	     )
	 (format t "
minimum :~15t ~g
mean :~15t ~g
median :~15t ~g
maximum :~15t ~g
sd :~15t ~g
skewness :~15t ~g
kurtosis :~15t ~g
n :~15t ~g
" 
		 mini meani mediani maxi sdi ske kur lengthi)
	 (setq result
	       (append result (list mini meani mediani maxi sdi ske kur lengthi)))
	 )
       )
     )
    result
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                chi-square
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun chi-square(x &key (p .05))
  "
;; Description: calculates the chi-square of a matrix x:

		(chi-square x &key (p .05))

	x - contingency table
	p - significance level

	Returns values of the chi-square, degrees of freedom, and the
 probability of a chi-square value that large or larger for contingency tables
 (i.e., tables for which a sample of size n is calculated, then each
 observation falls into one cell of a matrix).

	If the chi-square calculated exceeds the critical value, then there is
 reason to reject the null hypothesis of no interaction.

;; Reference: Eason, et al. 1980. Mathematics and Statistics for the
;; Bio-Sciences. Wiley.

;; Examples:
 (chi-square (matrix '(2 2) '(3 4 8 9)))
 ;; should be the same as
 (* 24.0 25 (/ (* 7 11 13 17)))

 (chi-square (matrix '(2 3) '(3 4 5 8 9 10)))
 (chi-square (matrix '(3 3) '(3 4 5 6 8 10 9 12 15)))

 ;; Jives with Reference, p. 507:
 (chi-square (matrix '(2 3) '(49 64 84 33 25 95)) :p .01)
 ;; Jives with Reference, p. 509:
 (chi-square (matrix '(3 2) '(23 20 17 22 7 11)))
"
  (let* (
	 (rows (mapcar #'sum (row-list x)))
	 (cols (mapcar #'sum (column-list x)))
	 (n (sum cols))
	 (rank-one (/ (outer-product rows cols) n))
	 (chisq (sum (/ (square (- x rank-one)) rank-one)))
	 (df (* (1- (length rows)) (1- (length cols))))
	 (chisqcrit (chisq-quant (- 1 p) df))
	 )
    (format
     t
     "
Chi-Square:          ~30t~g
Critical Chi-Square: ~30t~g
Degrees of Freedom:  ~30t~d
Probability:         ~30t~,6f~%"
     chisq chisqcrit df (- 1 (chisq-cdf chisq df))
     )
    (values
     chisq df (- 1 (chisq-cdf chisq df))
     )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                            t-test
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun t-test(x y &key (alpha .05) (print t))
  "
;; Documentation: (t-test x y &key (alpha .05) (print t))

	Returns the difference in means and reports the confidence interval
corresponding to the two-sided t-test for the difference of means at alpha
level alpha.

;; Examples:
 (t-test (rand.u 100) (rand.u 100) :alpha .025)
 (t-test (rand.u 100) (1+ (rand.u 10)) :alpha .025)
"
  (let* (
	 (n1 (length x))
	 (n2 (length y))
	 (s1 (square (sd x)))
	 (s2 (square (sd y)))
	 (mean1 (mean x))
	 (mean2 (mean y))
	 (meandiff (- mean1 mean2))
	 (df (- (+ n1 n2) 2))
	 (pooled-s (/ (+ (* (1- n1) s1) (* (1- n2) s2)) df))
	 (tval (t-quant (- 1 (/ alpha 2)) df))
	 (spread (* tval (sqrt (* pooled-s (+ (/ n1) (/ n2))))))
	 (cochranes-s1 (/ s1 n1))
	 (cochranes-s2 (/ s2 n2))
	 (cochranes-tval
	  (/
	   (+
	    (* cochranes-s1 (t-quant (- 1 (/ alpha 2)) (1- n1)))
	    (* cochranes-s2 (t-quant (- 1 (/ alpha 2)) (1- n2)))
	    )
	   (+ cochranes-s1 cochranes-s2)
	   )
	  )
	 (cochranes-spread
	  (* cochranes-tval (sqrt (+ cochranes-s1 cochranes-s2))))
	 )
    (if print
	(let ()
	  (format t "Mean difference: ~g~2%" meandiff)
	  (format t "~%~30tAssuming Equal Variances:~2%")
	  (format t "Confidence interval (alpha=~,6f): ~%~20t~g~%"
		  alpha (list (- meandiff spread) (+ meandiff spread)))
	  
	  (format t "~%~30tAssuming Unequal Variances:~2%")
	  (format t "Confidence interval (alpha=~,6f): ~%~20t~g~%"
		  alpha
		  (list (- meandiff cochranes-spread)
			(+ meandiff cochranes-spread)))
	  )
      )
    (values meandiff
	    (list (- meandiff spread) (+ meandiff spread))
	    (list (- meandiff cochranes-spread) (+ meandiff cochranes-spread))
	    tval
	    )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                            t-test-summary
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun t-test-summary(mean1 mean2 s1 s2 n1 n2 &key (alpha .05) (print t))
  "
;; Documentation:

  (t-test-summary m1 m2 s1 s2 n1 n2 &key (alpha .05) (print t))

	Returns the confidence interval corrsponding to the two-sided t-test
for the difference of means at alpha level alpha.

;; Examples:
 (t-test-summary 106.42143 111.26154 (sqrt 24.780275) (sqrt 48.060897) 14 13)
 (t-test-summary 106.42143 111.26154 (sqrt 24.780275) (sqrt 48.060897) 14 13
    :alpha .1)
 (t-test-summary 4.5 3.7 .3 1 13 17)
"
  (let* (
	 (s1 (square s1))
	 (s2 (square s2))
	 (meandiff (- mean1 mean2))
	 (df (- (+ n1 n2) 2))
	 (pooled-s (/ (+ (* (1- n1) s1) (* (1- n2) s2)) df))
	 (tval (t-quant (- 1 (/ alpha 2)) df))
	 (spread (* tval (sqrt (* pooled-s (+ (/ n1) (/ n2))))))
	 (cochranes-s1 (/ s1 n1))
	 (cochranes-s2 (/ s2 n2))
	 (cochranes-tval
	  (/
	   (+
	    (* cochranes-s1 (t-quant (- 1 (/ alpha 2)) (1- n1)))
	    (* cochranes-s2 (t-quant (- 1 (/ alpha 2)) (1- n2)))
	    )
	   (+ cochranes-s1 cochranes-s2)
	   )
	  )
	 (cochranes-spread
	  (* cochranes-tval (sqrt (+ cochranes-s1 cochranes-s2))))
	 )
    (if print
	(let ()
	  (format t "Mean difference: ~g~2%" meandiff)
	  (format t "Pooled variance: ~g~2%" pooled-s)
	  (format t "~%~30tAssuming Equal Variances:~2%")
	  (format t "Confidence interval (alpha=~,6f): ~%~20t~g~%"
		  alpha (list (- meandiff spread) (+ meandiff spread)))
	  
	  (format t "~%~30tAssuming Unequal Variances:~2%")
	  (format t "Confidence interval (alpha=~,6f): ~%~20t~g~%"
		  alpha
		  (list (- meandiff cochranes-spread)
			(+ meandiff cochranes-spread)))
	  )
      )
    (values meandiff
	    (list (- meandiff spread) (+ meandiff spread))
	    (list (- meandiff cochranes-spread) (+ meandiff cochranes-spread))
	    tval
	    cochranes-tval
	    )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 ratio-of-variances-ci
;;
(defun ratio-of-variances-ci(v1 v2 n1 n2 &key (alpha .05))
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          ratio-of-variances-ci Doc:
;;
;; Author: Andy Long
;; Created: Thu May 29 11:24:49 CDT 1997
;;
;; Description/Arguments:

       (ratio-of-variances-ci v1 v2 n1 n2 &key (alpha .05))
  
	From Biostatistics: a Foundation for analysis in the health sciences',
  by Wayne W. Daniel, p. 187, 6th edition.

;; Notes:

;; See Also:

;; Examples:
 (ratio-of-variances-ci 33.64 11.56 11 4)
 (ratio-of-variances-ci 24.780275 48.060897 14 13)
"
  (let* (
	 (ratio (/ v1 v2))
	 (df1 (1- n1))
	 (df2 (1- n2))
	 (half-alpha (half alpha))
	 )
    (list
     (/ ratio (f-quant (- 1 half-alpha) df1 df2))
     (/ ratio (f-quant half-alpha df1 df2))
     )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 variance-ci
;;
(defun variance-ci(v n &key (alpha .05))
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          variance-ci Doc:
;;
;; Author: Andy Long
;; Created: Thu May 29 11:24:49 CDT 1997
;;
;; Description/Arguments:

       (variance-ci v n &key (alpha .05))
  
	From Biostatistics: a Foundation for analysis in the health sciences',
  by Wayne W. Daniel, p. 182, 6th edition.

;; Notes:

;; See Also:

;; Examples:
 ;; Example 6.9.1, p. 185 is wrong: sd = .391868, not variance; and the
 ;; right-hand side of the CI is wrong: it's not 1.3548....
 (def data (list 6 6.4 7 5.8 6 5.8 5.9 6.7 6.1 6.5 6.3 5.8))
 (variance-ci (variance data) (length data))
 (variance-ci .391868 12)
"
  (let* (
	 (df (1- n))
	 (numer (* df v))
	 (half-alpha (half alpha))
	 )
    (list
     (/ numer (chisq-quant (- 1 half-alpha) df))
     (/ numer (chisq-quant half-alpha df))
     )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          ANALYSIS-OF-VARIANCE
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun analysis-of-variance(sets &key latex)
  "
;; Documentation: (analysis-of-variance sets &key latex)

;; Examples:
 ;; Page 259, Snedecor and Cochrane, Sixth Edition, Statistical Methods
 (def fat1 (list 64 72 68 77 56 95))
 (def fat2 (list 78 91 97 82 85 77))
 (def fat3 (list 75 93 78 71 63 76))
 (def fat4 (list 55 66 49 64 70 68))
 (analysis-of-variance (list fat1 fat2 fat3 fat4))

 ;; Page 278, Snedecor and Cochrane, Sixth Edition, Statistical Methods
 (def mice-9d
   (combine
     (mapcar (lambda(x) (repeat (first x) (second x)))
       (weave (iseq 2 7) '(6 4 9 8 3 1)))))
 (def mice-11c
   (combine
     (mapcar (lambda(x) (repeat (first x) (second x)))
       (weave (iseq 2 13) '(1 3 3 6 6 14 11 4 6 2 3 1)))))
 (def mice-dsc1
   (combine
     (mapcar (lambda(x) (repeat (first x) (second x)))
       (weave (iseq 2 14) '(3 5 5 8 19 23 22 14 14 7 8 4 1)))))
 (analysis-of-variance (list mice-9d mice-11c mice-dsc1))

 ;; Page 290, Snedecor and Cochrane, Sixth Edition, Statistical Methods
 (def bull-1 (list 46 31 37 62 30))
 (def bull-2 (list 70 59))
 (def bull-3 (list 52 44 57 40 67 64 70))
 (def bull-4 (list 47 21 70 46 14))
 (def bull-5 (list 42 64 50 69 77 81 87))
 (def bull-6 (list 35 68 59 38 57 76 57 29 60))
 (analysis-of-variance (list bull-1 bull-2 bull-3 bull-4 bull-5 bull-6))

 ;; Page 295, Snedecor and Cochrane, Sixth Edition, Statistical Methods
 (analysis-of-variance
   (split-list ;; finger ridges on twelve sets of identical twin girls:
      (list 71 71 79 82 105 99 115 114 76 70 83 82 114 113 57 44 114 113 94 91
            75 83 76 72) 2))

 ;; Page 72, The 1-way classification of variance (Z. Li gave it to me):
 (analysis-of-variance (list '(3 3 12) '(11 13 17 7) '(4 2)))
"
  (let* (
	 (n-s (mapcar #'length sets))
	 (n (sum n-s))
	 (sum-n-squares (sum (square n-s)))
	 (groups (length sets))
	 (sum-x-s (mapcar #'sum sets))
	 (sum-xsquares-s (mapcar (lambda(x) (sum (square x))) sets))

	 (sum-x-squared-over-n (/ (square sum-x-s) n-s))
	 
	 (df-of-groups (1- groups))
	 (df-total (1- n))
	 (df-within-groups (- df-total df-of-groups))
	 
	 (correction-for-the-mean (/ (square (sum sum-x-s)) n))
	 (between-classes
	  (- (sum sum-x-squared-over-n) correction-for-the-mean))
	 (total (- (sum sum-xsquares-s) correction-for-the-mean))
	 (within-classes (- total between-classes))
	 
	 (between-mean-square (/ between-classes df-of-groups))
	 (within-mean-square (/ within-classes df-within-groups))
	 (total-mean-square (/ total df-total))

	 (f-stat (/ between-mean-square within-mean-square))
	 )
    (if latex
	(latex-table
	 (matrix
	  '(6 4)
	  (list
	   "Source of Variation" "d.f." "Sum of Squares" "Mean Square"
	   "Between samples" df-of-groups between-classes between-mean-square
	   "Within samples" df-within-groups within-classes within-mean-square
	   "Total" df-total total total-mean-square
	   "F-statistic:" f-stat "Probability of F:"
	   (- 1 (f-cdf f-stat df-of-groups df-within-groups))
	   "\\sigma_e^2" within-mean-square "\\sigma_\\alpha^2" 
	   (* (- between-mean-square within-mean-square)
	      (/ (* df-of-groups n) (- (* n n) sum-n-squares)))
	   )
	  )
	 )
      (let ()
	(format t "~%~5t~a ~25t~a ~40t~a ~55t~a~%"
	   "Source of Variation" "d.f." "Sum of Squares" "Mean Square")
	(format t "~7t~a ~27t ~d ~42t~,5g ~57t~,5g~%"
	   "Between samples" df-of-groups between-classes between-mean-square)
	(format t "~7t~a ~27t ~d ~42t~,5g ~57t~,5g~%"
  	   "Within samples" df-within-groups within-classes within-mean-square)
	(format t "~7t~a ~27t ~d ~42t~,5g ~57t~,5g~%~%"
	   "Total" df-total total total-mean-square)
	(format t "~7t~a ~25t ~,5f ~42t~a ~57t~,5f~%"
		"F-statistic:" f-stat "Probability:"
		(- 1 (f-cdf f-stat df-of-groups df-within-groups)))
	(format t "~7t~a ~25t ~,5f ~42t~a ~57t~,5f~%"
		"Error Variance" within-mean-square "Model Variance" 
		(* (- between-mean-square within-mean-square)
		   (/ (* df-of-groups n) (- (* n n) sum-n-squares)))
		)
	)
      )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                missing-mean
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun missing-mean(x 
		    &key
		    (test #'(lambda(x) (not (not-a-numberp x))))
		    (missing #.not-a-number)
		    )
  "
;; Description: function

 (missing-mean x 
	    &key
	    (test #'(lambda(x) (not (not-a-numberp x))))
	    (missing #.not-a-number)
	    )

 Calculates a mean even if there is missing data (simply ignores it). Only
 values satisfying the test are kept. The keyword missing is only useful if all
 values are missing (in which case missing is returned).

;; Examples:
 (missing-mean '(0 1 2 #.not-a-number))
 (missing-mean (list 0 1 2 nan))

 (missing-mean (list 0 1 2 '.) :test #'numberp :missing '.)
"
  (let (
        (n 0)
        (sum 0)
	)
    (mapcar
     (lambda(i)
       (if (funcall test i)
	   (let ()
	     (setq n (1+ n))
	     (setq sum (+ sum i))
	     )
	 )
       )
     x)
    (if (= 0 n) (values missing 0) (values (/ sum n) n))
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 skewness
;;
(defun skewness(list)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; skewness Doc:    Creation Date:2/21/97        Author: Andy Long
;;
;; Description/Arguments:

       (skewness list)
  
;; Notes:

;; See Also:

;; Examples:
 (skewness (list 6 8 10 12 14))
 ;; 0
 (skewness (list 6 8 10 11 15))
 ;; .387
 (skewness (rand.u 1000))
 ;; Should be around 0.
"
  (let (
	(m2 (/ (sum (square (- list (mean list)))) (length list)))
	(m3 (/ (sum (^ (- list (mean list)) 3)) (length list)))
	)
    (/ m3 (^ m2 1.5))
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 kurtosis
;;
(defun kurtosis(list)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; kurtosis Doc:    Creation Date:2/22/97        Author: Andy Long
;;
;; Description/Arguments:

       (kurtosis list)
  
;; Notes:

;; See Also:

;; Examples:
 (kurtosis (list 6 8 10 12 14))
 ;; -1.3
 (kurtosis (list 5.64 9 10 11 14.36))
 ;; -.74
 (kurtosis (rand.n 1000))
 ;; Should be around 0.
"
  (let (
	(m2 (/ (sum (square (- list (mean list)))) (length list)))
	(m4 (/ (sum (^ (- list (mean list)) 4)) (length list)))
	)
    (- (/ m4 (* m2 m2)) 3)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; fishers-z Doc:    Creation Date:2/27/97        Author: Andy Long
;;
;;
(defun fishers-z(r n &optional (alpha .05))
  "
;; Description/Arguments:
 
       (fishers-z r n &optional (alpha .05))
 
	Test to give confidence intervals for a correlation.

;; Notes: Got this out of Draper and Smith, Applied Regression Analysis, 2nd
;; edition, p. 46.
 
;; See Also:
 
;; Examples:
 \"Example, p. 47, Draper and Smith: (.339 .632):\"
 (fishers-z .5 103)
"
  (let (
        (z (atanh r))
        (val (/ (normal-quant (- 1 (half alpha))) (sqrt (- n 3))))
        )
    (list (tanh (- z val)) (tanh (+ z val)))
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 kernel-density
;;
(defun kernel-density(x &key (xvals *points*) width type)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; kernel-density Doc:    Creation Date:3/7/97        Author: Andy Long
;;
;; Description/Arguments:

       (kernel-density x
		&key
		(xvals *points*)
		width - Default: minimum of sd, interquartile-range/1.349
		(type 'e))
  
;; Notes:

	This is a replacement for the kernel-dens which comes with the
xlispstat distribution, and which I consider kind of murky. There are not
enough details. I have also included other models.

	Note that for the kernel-density and kernel-dens to give the same
values on the gaussian model requires different widths: the kernel-dens version
doesn't divide the exponent by 2.

;; Thanks:

	This is based on Stata's kdensity function and documentation, and I
thank them.

;; See Also: kernel-dens

;; Examples:
 (def sample-trial (rand.n 100))
 (def ker (kernel-density sample-trial))
 (title \"Epanechnikov\")
 (plot (first ker) (second ker))

 (def ker (kernel-density sample-trial :type 'b :width 1))
 (title \"Biweight\")
 (plot (first ker) (second ker))

 (def ker (kernel-density sample-trial :type 'c :width 1))
 (title \"Cosine\")
 (plot (first ker) (second ker))

 (def ker (kernel-density sample-trial :type 'g :width 1))
 (title \"Gaussian\")
 (plot (first ker) (second ker))
 (def ker2 (kernel-dens sample-trial :type 'g :width 4 :xval *points*))
 (title \"Gaussian from kernel-dens\")
 (plot (first ker2) (second ker2))

 (def ker (kernel-density sample-trial :type 'p :width 1))
 (title \"Parzen\")
 (plot (first ker) (second ker))

 (def ker (kernel-density sample-trial :type 'r :width 1))
 (title \"Rectangular\")
 (plot (first ker) (second ker))

 (def ker (kernel-density sample-trial :type 't :width 1))
 (title \"Triangular\")
 (plot (first ker) (second ker))
 (def ker2 (kernel-dens sample-trial :type 't :width 1 :xval *points*))
 (title \"Triangular from kernel-dens\")
 (plot (first ker2) (second ker2))

 (tile (select (agw) (iseq 9)) :nrow 3 :ncol 3)
"
  (let* (
	 (3sqrt5/4 (* .25 3 (sqrt 5)))
	 (15sixteenths (/ 15 16))
	 (1/sqrt2pi (/ (sqrt 2pi)))
	 (4thirds (/ 4 3))
	 (8thirds (/ 8 3))

	 (type
	  (cond
	   ((equalp type 'b) ;; 'b - biweight
	    (lambda(z) (let ((val (- 1 (* z z))))
			 (if (< val 0) 0 (* 15sixteenths val val)))))
	   ((equalp type 'c) ;; 'c - cosine
	    (lambda(z) (if (< (abs z) .5) (+ 1 (cos (* 2pi z))) 0)))
	   ((equalp type 'g) ;; 'g - Gaussian
	    (lambda(z) (* 1/sqrt2pi (exp (* -.5 (* z z))))))
	   ((equalp type 'p) ;; 'p - Parzen
	    (lambda(z) (let ((val (* z z)))
			 (cond
			  ((> val 1) 0)
			  ((> val .25) (* 8thirds (^ (- 1 (abs z)) 3)))
			  (t (- 4thirds (* 8 val (- 1 (abs z)))))
			  ))))
	   ((equalp type 'r) ;; 'r - Rectangular
	    (lambda(z) (if (< (abs z) 1) .5 0)))
	   ((equalp type 't) ;; 't - Triangular
	    (lambda(z) (let ((val (abs z))) (if (< val 1) (- 1 val) 0))))
	   (t ;; 'e - Epanechnikov
	    (lambda(z) (let ((val (* z z)))
			 (if (< val 5) (* 3sqrt5/4 (- 1 (* .2 val))) 0))))
	   ))

	 (width (if width width (min
				 (standard-deviation x)
				 (/ (interquartile-range x) 1.349)
				 )))
	 (xvals (if (numberp xvals) (rseq (min x) (max x) xvals) xvals))

	 (yvals
	  (mapcar
	   (lambda(xx) (sum (mapcar type (/ (- x xx) width))))
	   xvals
	   ))
	 )
    (list xvals (/ yvals (* (length x) width)))
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 binomial-ci
;;
(defun binomial-ci(s n &key (alpha .05) verbose)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; binomial-ci Doc:    Creation Date:3/10/97        Author: Andy Long
;;
;; Description/Arguments:

       (binomial-ci s n &key (alpha .05) verbose)
  
	Computes the exact confidence intervals for a binomial with s successes
in n trials. The default alpha level is .05, and the secant method is used to
find the roots p to the two equations

	\sum_{i=0}^{i=o}b(i,n,p) = \alpha/2
and
	\sum_{i=0}^{i=o}b(i,n,p) = 1-\alpha/2,

where b(i,n,p) = p^i*(1-p)^{n-i}.

	Returns the two roots, as well as the error in the root (should be
zeros). 

;; Notes: This is a two-sided confidence interval, so the alpha is split
;; between the low end and the high end.

;; See Also: binomial-cdf, binomial-quant, binomial-pmf

;; Examples:
 (binomial-ci 1 3)
 (binomial-ci 24 100)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;   The following plot demonstrates the significance of these intervals:
;;   the p-values associated with them have alpha area in their tails. The
;;   upper limit is associated with the yellow area, while the lower limit
;;   is associated with the red. Each area represents alpha/2 of the total
;;   area.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 (new-plot)
 (lines off)
 (def result (binomial-ci 3 10))
 (plot (iseq 11) (binomial-pmf (iseq 11) 10 .3))
 (send my-plot :range '(0 1) '(0 0) '(10 .3))

 (send my-plot :points-to-histogram (iseq 11) (binomial-pmf (iseq 11) 10 .3))
 (send my-plot :points-to-histogram
	(- (iseq 0 3) .25) (binomial-pmf (iseq 4) 10
		   (second result)) :fill-color 'yellow :squeeze 4)
 (send my-plot :points-to-histogram
	(+ (iseq 3 10) .25) (binomial-pmf (iseq 3 10) 10
		   (first result)) :fill-color 'red :squeeze 4)
"
  (let* (
	 (p (/ s n))

	 (switch (if (> p .5) t nil))
	 (p (if switch (- 1 p) p))
	 (s (if switch (- n s) s))

	 (f (lambda(x)
	      (cond
	       ((< x 0) (- 2 (binomial-cdf s n (- x))))
	       ((> x 1) 0)
	       (t (binomial-cdf s n x)))))

	 (g (lambda(x)
	      (cond
	       ((< x 0) (- 2 (binomial-cdf (1- s) n (- x))))
	       ((> x 1) 0)
	       (t (binomial-cdf (1- s) n x)))))

	 (alpha (/ alpha 2))
	 (eps (max 1e-6 (min 1e-6 (half p))))
	 low hi
	 )
    (def low
	 (if (= p 0) (list 0 0)
	   (secant2 (lambda(x) (- 1 alpha (funcall g x)))
		    p (+ p eps) :verbose verbose)))
    (def hi
	 (secant2 (lambda(x) (- alpha (funcall f x))) 
		  (- p eps) p :verbose verbose))

    (values 
     (if switch (reverse (- 1 (mapcar #'first (list low hi))))
       (mapcar #'first (list low hi)))
     (if switch (reverse (mapcar #'second (list low hi)))
       (mapcar #'second (list low hi)))
     )
    )
  )
;;
  #|
 binomial-quant is the sum of the binomial-pmf 
 (sum (mapcar {binomial-pmf x 100 .2 :dash t} (iseq 0 24))) ;; .8686
 (binomial-cdf 24 100 .2) ;; .8686
 (binomial-quant .8686 100 .2) ;; gives 24
 (plot {binomial-quant .05 100 x :dash t} 0 1)
 (plot {(binomial-cdf 24 100 x) :dash t} 0 1)
 (plot {binomial-pmf 24 100 x :dash t} 0 1)
 (muller {24 - (binomial-quant .05 100 x) :dash t} .1 .2 .3)

 (sum (mapcar {binomial-pmf x 3 .0084025 :dash t} (iseq 0 1)))
|#
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 poisson-ci
;;
(defun poisson-ci(p n &key (alpha .05) verbose (tol 1e-12))
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; poisson-ci Doc:    Creation Date:3/10/97        Author: Andy Long
;;
;; Description/Arguments:

       (poisson-ci p n &key (alpha .05) verbose (tol 1e-12))
  
	Computes the exact confidence intervals for a poisson with p outcomes
in n intervals. The default alpha level is .05, and the secant method is used
to find the roots mu to the two equations

	\sum_{i=0}^{i=(p-1)}poisson(i,\mu) = 1-\alpha/2
and
	\sum_{i=0}^{i=p}poisson(i,\mu) = \alpha/2

where poisson(i,mu) = e^\mu \mu^i/(i!)

	Returns the two roots, as well as the error in the root (should be
zeros). 

;; Notes: This is a two-sided confidence interval, so the alpha is split
;; between the low end and the high end.

;; See Also: poisson-cdf, poisson-quant, poisson-pmf

;; Examples:
 (poisson-ci 3 1)
 (poisson-ci 27 2)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;   The following plot demonstrates the significance of these intervals:
;;   the mu-values associated with them have alpha area in their tails. The
;;   larger value is associated with the yellow area, while the smaller value
;;   is associated with the red:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 (new-plot)
 (lines off)
 (def result (poisson-ci 5 1))
 (plot (iseq 10) (poisson-pmf (iseq 10) 5))
 (send my-plot :range '(0 1) '(0 0) '(10 .18))

 (send my-plot :points-to-histogram (iseq 10) (poisson-pmf (iseq 10) 5))
 (send my-plot :points-to-histogram (- (iseq 6) .25) (poisson-pmf (iseq 10)
   (second result)) :fill-color 'yellow :squeeze 4)
 (send my-plot :points-to-histogram (+ (iseq 5 10) .25) (poisson-pmf (iseq 5 10)
   (first result)) :fill-color 'red :squeeze 4)
"
  (let* (
	 (g (lambda(x)
	      (cond
	       ((< x 0) (- 2 (poisson-cdf (1- p) (- x))))
	       (t (poisson-cdf (1- p) x)))))

	 (f (lambda(x)
	      (cond
	       ((< x 0) (- 2 (poisson-cdf p (- x))))
	       (t (poisson-cdf p x)))))

	 (alpha (/ alpha 2))
	 (eps (max 1e-6 (min 1e-6 (half p))))
	 low hi
	 )
    (setq
     low
     (if (= p 0) (list 0 0)
       (multiple-value-list
	(secant2 (lambda(x) (- 1 alpha (funcall g x)))
		 (- p eps) p :verbose verbose :tolerance tol)))
     hi
     (multiple-value-list
      (secant2 (lambda(x) (- alpha (funcall f x))) 
	       p (+ p eps) :verbose verbose :tolerance tol)))
    
    (values 
     (mapcar #'first (/ (list low hi) n))
     (mapcar #'second (list low hi))
     )
    )
  )
#|
 (simpson {:dash t (poisson-pmf 9 x)})
 (plot {:dash t (poisson-pmf 27 x)} 0 100)
 (plot {:dash t (poisson-cdf 27 x)} 0 100)

 (title \"numerical differentiation - poisson cdf\")
 (plot (rseq 0 100 101) (num-diff (mapcar (lambda(x) (poisson-cdf 27 x))
	(rseq 0 100 101)) 1))
 (title \"should look like poisson density:\")
 (plot (rseq 0 100 101) (mapcar (lambda(x) (poisson-pmf 27 x))
	(rseq 0 100 101)))

 (sum (poisson-pmf (iseq 27) 17.79773))
|#
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 partial-correlation
;;
(defun partial-correlation(x y &rest var)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; partial-correlation Doc:    Creation Date:3/27/97        Author: Andy Long
;;
;; Description/Arguments:

       (partial-correlation x y &rest var)
  
	x - a matrix
	y - the dependent variable
	var - the number of the variable that is to be partialled out (start
		ordering variables from 1, rather than from 0)

Returns: list of partial correlations for other variables.

;; Notes:

;; See Also:

;; Examples:
 (def x3 (/ (list 61 64 78 49 84 74 42 87 75 76 65 62 79 60 70 74 85 67 45 95 64
		 75 70 67 70) 100))
 (def x4 (/ (list 74 80 74 75 55 89 41 41 41 45 103 69 66 73 81 84 61 49 46 37
		 47 53 62 68 75)10))
 (def x5 (list 31 29 31 30 31 30 31 31 30 31 30 31 31 28 31 30 31 30 31 31 30 31
	      30 31 31))
 (partial-correlation (bind-columns x3 x4) x5 2)
 (partial-correlation (bind-columns x3 x4) x5 1)

 (def x1 (list 7 1 11 11 7 11 3 1 2 21 1 11 10))
 (def x2 (list 26 29 56 31 52 55 71 31 54 47 40 66 68))
 (def x3 (list 6 15 8 8 6 9 17 22 18 4 23 9 8))
 (def x4 (list 60 52 20 47 33 22 6 44 22 26 34 12 12))
 (def x5 (/ (list 785 743 1043 876 959 1092 1027 725 931 1159 838 1133 1094)
	    10))
 (def appendixB (regression-model (list x1) x5))

 (square (partial-correlation (bind-columns x1 x2 x3 x4) x5 1))
 (square (partial-correlation (bind-columns x1 x2 x3 x4) x5 1 2))
 (partial-correlation (bind-columns x1 x2 x3 x4) x5 1 2)
"
  (let* (
	 (len (second (size x)))
	 (parted (select-columns x (1- var)))
	 (kept (select-columns
		x (sort-data (set-difference (iseq len) (1- var)))))
	 (reg (regression-model parted y :print nil))
	 (res (send reg :residuals))
	 (newvars
	  (mapcar
	   (lambda(x) (regression-model parted x :print nil))
	   (column-list kept)))
	 (newvars (mapcar (lambda(x) (send x :residuals)) newvars))
	 (newvars (apply #'bind-columns newvars))
	 (cor (row 1 (correlation (bind-columns res newvars))))
	 )
    (rest cor)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 value-with-se
;;
(defun value-with-se(value se
			   &key
			   (value-fmt "~,4g")
			   (se-fmt "~,4g")
			   (bracket (list " (" ")"))
			   )
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; value-with-se Doc:    Creation Date:3/31/97        Author: Andy Long
;;
;; Description/Arguments:

       (value-with-se value se
			   &key
			   (value-fmt \"~,4g\")
			   (se-fmt \"~,4g\")
			   (bracket (list \" (\" \")\"))
			   )
  
	This guy just pasts a list of values with a list of standard errors (or
really whatever you want - it's just usually used for that sort of thing).
	It creates a list of strings, with the value followed by the se in
parentheses.

;; Notes:

;; See Also:

;; Examples:
 (value-with-se (rand.n 10) (rand.u 10))

 ;; You can weave more complicated things into the brackets:
 (value-with-se (rand.n 10) (weave (rand.u 10) (rand.u 10)))
 (value-with-se (rand.n 10) (weave (rand.u 10) (rand.u 10))
   :brackets (list \" [\" \"]\")
   )
"
  (let
      (
       (fmt (strcat value-fmt (first bracket) se-fmt (second bracket)))
       (fmt2 (strcat value-fmt (first bracket) "~a" (second bracket)))
       )
    (mapcar
     (lambda(x y)
       (if (listp y)
	   (format
	    nil
	    fmt2
	    x
	    (space-trim
	     (apply
	      #'strcat
	      (mapcar (lambda(y) (strcat " " (format nil se-fmt y))) y)
	      )
	     )
	    )
	 (format nil fmt x y)
	 )
       )
     value se
     )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 binomial-coef
;;
(defun binomial-coef(n p)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          binomial-coef Doc:
;;
;; Author: Andy Long
;; Created: Wed Apr 9 11:47:52 CDT 1997
;;
;; Description/Arguments:

       (binomial-coef n p)
  
;; Notes:

;; See Also:

;; Examples:
 (binomial-coef 12 3) ;; 220
 (binomial-coef 0 0) ;; 1
 (binomial-coef 4 2) ;; 6
"
  (/ (factorial n) (* (factorial (- n p)) (factorial p)))
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 durbin-watson
;;
(defun durbin-watson(seq)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          durbin-watson Doc:
;;
;; Author: Andy Long
;; Created: Tue Apr 15 17:07:01 CDT 1997
;;
;; Description/Arguments:

       (durbin-watson sequence)
  
	Test for serial correlation in a sequence. See Draper and Smith,
Applied Regression Analysis, 2nd Edition, p. 162. Includes a table for
examination of the d value obtained.

;; Notes:

;; See Also:

;; Examples:
 (durbin-watson (iseq 20))
 (durbin-watson (rand.n 20))

"
  (/ (sum (square (- (rest seq) (butlast seq))))
     (sum (square seq)))
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 runs-test
;;
(defun runs-test(seq &key (verbose t))
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          runs-test Doc:
;;
;; Author: Andy Long
;; Created: Tue Apr 15 17:13:14 CDT 1997
;;
;; Description/Arguments:

       (runs-test sequence &key (verbose t))
  
	Test for serial correlation in a sequence. See Draper and Smith,
Applied Regression Analysis, 2nd Edition, p. 157. 

;; Notes:

;; See Also:

;; Examples:
 (runs-test (rand.n 10))
 (runs-test (rand.n 100))
"
  (let* (
	 (runs (heaviside seq))
	 (plus (sum runs))
	 (len (length seq))
	 (minus (- len plus))
	 (u 1)
	 (tmp (/ (* 2 plus minus) (+ plus minus)))
	 mu sigma z p
	 )
    (mapcar (lambda(x y) (if (/= x y) (setq u (1+ u))))
	    (rest runs) (butlast runs))
    (setq 
     mu (+ 1 tmp)
     sigma (sqrt (/ (* tmp (- tmp 1)) (- (+ plus minus) 1)))
     z (/ (+ u (- mu) .5) sigma)
     p (normal-cdf z)
     )
    (if verbose
	(format t "
n: ~20t~d
+: ~20t~d
-: ~20t~d
runs: ~20t~d
z: ~20t~,4f
probability: ~20t~,4f
" len plus minus u z p)
      )
    (values z u p)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 simulate-runs-test-probability
;;
(defun simulate-runs-test-probability(n1 n2 &key (ntimes 1000))
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          simulate-runs-test-probability Doc:
;;
;; Author: Andy Long
;; Created: Wed Apr 16 10:10:55 CDT 1997
;;
;; Description/Arguments:

       (simulate-runs-test-probability n1 n2 &key (ntimes 1000))
  
;; Notes: See Draper and Smith, Applied Regression Analysis, 2nd Edition,
;; p. 160. This is to simulate the table that is reproduced there, or exact
;; runs-test values.

;; See Also:

;; Examples:
 (simulate-runs-test-probability 2 3)
 (simulate-runs-test-probability 5 7)
 ;; takes a long time!
 ;; (simulate-runs-test-probability 10 10 :ntimes 100000)
"
  (let* (
	 (all 
	  (if (= 0 (* n1 n2))
	      (repeat 1 (+ n1 n2))
	    (combine (repeat -1 n1) (repeat 1 n2)))
	  )
	 (len (size all))
	 (result (repeat 0 len))
	 )
    (dotimes
     (i ntimes)
     (incf (elt result
		(let* (
		       (runs (heaviside (sample all len)))
		       (u 1)
		       )
		  (mapcar (lambda(x y) (if (/= x y) (setq u (1+ u))))
			  (rest runs) (butlast runs))
		  (1- u)
		  )
		)
	   )
     )
    (/
     (reverse (mapcon (lambda(x) (list (sum x))) (reverse result)))
     ntimes)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 andrews-pregibon
;;
(defun andrews-pregibon(x y)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          andrews-pregibon Doc:
;;
;; Author: Andy Long
;; Created: Wed Apr 16 11:06:26 CDT 1997
;;
;; Description/Arguments:

       (andrews-pregibon x y)
  
	This is another outlier test. Small values of AP are considered
deviant.

;; Notes: See Draper and Smith, Applied Regression Analysis, 2nd Edition,
;; p. 171. This procedure is untested, and implements the form of AP for k=1,
;; as described on p. 172!

;; See Also:

;; Examples:
 (andrews-pregibon (list (rand.u 10)) (rand.u 10))
 ;; small values are considered deviant, so check out the first value:
 (andrews-pregibon (list (combine 10 (rand.u 10))) (rand.u 11))
"
  (let* (
	 (matrix (if (matrixp x) x (apply #'bind-columns x)))
	 (x (if (matrixp x) (column-list x) x))
	 (reg (regression-model x y :print nil))
	 (square-residuals (square (send reg :residuals)))
	 (sum (sum square-residuals))
	 (xtxinv (send reg :xtxinv))
	 (rii (mapcar
	       (lambda(x)
		 (let ((y (combine 1 (coerce x 'list))))
		   (matprod y xtxinv y)))
	       (row-list matrix)))
	 )
    (size rii xtxinv x matrix)
    (- 1 rii (/ square-residuals sum))
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 correlated-variables
;;
(defun correlated-variables(n r &key (type #'rand.n))
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          correlated-variables Doc:
;;
;; Author: Andy Long
;; Created: Fri Apr 25 12:16:30 CDT 1997
;;
;; Description/Arguments:

       (correlated-variables n r &key (type #'rand.n))
  
	Generates a pair of variables of given type, sorted by the first
variable. The default type is normal.

;; Notes:

;; See Also:

;; Examples:
 (def vars (correlated-variables 1000 .5))
 (correlation (first vars) (second vars))

 (def vars (correlated-variables 1000 .5 :type #'rand.u))
 (correlation (first vars) (second vars))
"
  (let* (
	 (x1 (funcall type n))
	 (x1 (select x1 (order x1)))
	 (x2 (+
	      (* r x1)
	      (*
	       (funcall type n)
	       (sqrt (- 1 (^ r 2)))
	       )
	      )
	     )
	 )
    (list x1 x2)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 one-way-anova
;;
(defun one-way-anova(groups &key difference compare pooled)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          one-way-anova Doc:
;;
;; Author: Andy Long
;; Created: Mon May 5 14:09:46 CDT 1997
;;
;; Description/Arguments:

       (one-way-anova groups &key difference compare pooled)
  
	groups is a list of groups.

	I want to use my :anova regression-model-proto method to do one-way
anova. Here's my first attempt. This is based on some examples from Snedecor
and Cochrane, 6th edition, chapter 10.

	The first group is taken as the referent group: its mean is the
constant of the regression. Thus, the regression coefficients of the other
variables will tell you whether they are significantly different from the first
group's value.

	If difference is true, then the variables will be the indicators times
the means, rather than just indicators. Depends on how you like your anova
presented!

	Compare allows you to specify a certain linear combination of the
means. This should be a list of the same length as the list of groups, or a
list of such lists. If pooled is true, then the pooled variance will be used;
otherwise the variance of each group will be used.

;; Notes:

;; See Also:

;; Examples:
 (def g1 (list 64 72 68 77 56 95))
 (def g2 (list 78 91 97 82 85 77))
 (def g3 (list 75 93 78 71 63 76))
 (def g4 (list 55 66 49 64 70 68))
 (one-way-anova (list g1 g2 g3 g4))

 (one-way-anova (list g1 g2 g3 g4) :difference t)

 ;; Examples 10.12.1, 10.12.2, p. 278-279 (different sample sizes):
 (def days (iseq 2 14))
 (def vars
     (mapcar
      (lambda(x)
	(combine
	 (mapcar
	  (lambda(x y) (repeat y x))
	  x (select days (iseq (length x)))
	  )
	 )
	)
      (list
       (list 6 4 9 8 3 1)
       (list 1 3 3 6 6 14 11 4 6 2 3 1)
       (list 3 5 5 8 19 23 22 14 14 7 8 4 1)
       )
      ))

 (one-way-anova vars)
 (one-way-anova vars :compare '(0 1 -1))
 ;; Top of p. 279:
 (one-way-anova vars :compare '(-1 1 0))
 (one-way-anova vars :compare '(0 -1 1))
 (one-way-anova vars :compare '((-1 1 0) (0 -1 1)))
 (one-way-anova vars :compare '(1 -1 0) :pooled t)
"
  (let* (
	 (y (combine groups))
	 (length (length y))
	 (glen (1- (length groups)))
	 (vars
	  (mapcar
	   (lambda(i)
	     (let* (
		    (mean (if difference (mean (elt groups i)) 1))
		    (front
		     (repeat
		      0 (length (combine (select groups (iseq i))))))
		    (back
		     (if (/= glen i)
			 (repeat
			  0
			  (length
			   (combine
			    (select groups (iseq (1+ i) glen)))))
		       ))
		    )
	       (if back
		   (combine front (repeat mean (length (elt groups i))) back)
		 (combine front (repeat mean (length (elt groups i))))
		 )))
	   (iseq 1 glen)
	   )
	  )
	 (reg (regress vars y))
	 )
    (send reg :anova-no-dups)
    (if compare
	(mapcar
	 (lambda(compare)
	   (format t "~%Comparison: ~s; estimate: ~,6g; standard error: ~,6g.~%"
		   compare
		   (dot compare (mapcar #'mean groups))
		   (sqrt
		    (sum
		     (/
		      (*
		       (square compare)
		       (if pooled (send reg :ss) (mapcar #'variance groups))
		       ) 
		      (mapcar #'length groups))))
		   )
	   )
	 (if (listp (first compare)) compare (list compare))
	 ))
    reg
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 categorize
;;
(defun categorize(values types
			 &key (sort-values t) (sort-types t))
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          categorize Doc:
;;
;; Author: Andy Long
;; Created: Tue May 6 09:35:26 CDT 1997
;;
;; Description/Arguments:

	(categorize(values types
			 &key (sort-values t) (sort-types t))
  
	This function sorts the values into the types. Each value should have a
type associated with it (e.g. age and sex): then a table containing values and
numbers of each type will be provided.

	If you're using non-numeric values or types, you must set the sort-
variables to nil.

;; Notes: 

;; See Also:

;; Examples:
 (def age (sample (iseq 10) 100 t))
 (def sex (sample (iseq 2) 100 t)) ;; Say that 0 is male, 1 is female:
 (def table (categorize age sex))
 ;; The sum of the following should be 100:
 (sum (mapcar #'second table)) ;; men
 (sum (mapcar #'third table)) ;; women

 ;; On planet picto they have three sexes:
 (def age (sample (iseq 10) 100 t))
 (def sex (sample (iseq 3) 100 t))
 (def table (categorize age sex))
 ;; The sum of the following should be 100:
 (sum (mapcar #'second table)) ;; men
 (sum (mapcar #'third table)) ;; women
 (sum (mapcar #'fourth table)) ;; other

 ;; If you use latex, and want to make a table from the output, it's easy:
 (latex-table (apply #'bind-rows table))

 ;; Here we have an example with non-numeric values and types:
 (def age (sample (list 'old 'middle 'whippersnapper) 10 t))
 (def sex (sample (list 'm 'f) 10 t))
 (def table (categorize age sex :sort-values nil :sort-types nil))
 ;; The sum of the following should be 10:
 (sum (mapcar #'second table)) ;; men
 (sum (mapcar #'third table)) ;; women
"
  (let* (
	 (distinct-ages
	  (if sort-values (sort-data (remove-duplicates values))
	    (remove-duplicates values)))
	 (distinct-types
	  (if sort-types (sort-data (remove-duplicates types))
	    (remove-duplicates types)))
	 (n (length distinct-types))
	 (pairs (transpose (list age sex)))
	 )
    (format t "~%Types: ~s~%" distinct-types)
    (mapcar
     (lambda(distinct-age)
       (let ((type (repeat 0 n)))
	 (dolist
	  (pair pairs)
	  (if (equalp distinct-age (first pair))
	      (let* (
		     (index (search (list (second pair)) distinct-types))
		     )
		(setf (elt type index) (1+ (elt type index)))
		)
	    ) 
	  )
	 (combine distinct-age type)
	 )
       )
     distinct-ages
     )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 de-categorize
;;
(defun de-categorize(categories &key types)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          de-categorize Doc:
;;
;; Author: Andy Long
;; Created: Tue May 6 12:29:03 CDT 1997
;;
;; Description/Arguments:

       (de-categorize categories &key types)
  
;; Notes:

;; See Also: categorize

;; Examples:
 (def age (sample (iseq 10) 100 t))
 (def sex (sample (iseq 2) 100 t)) ;; Say that 0 is male, 1 is female:
 (def table (categorize age sex))
 (de-categorize table)

 ;; On planet picto they have three sexes:
 (def age (sample (iseq 10) 100 t))
 (def sex (sample (iseq 3) 100 t))
 (def table (categorize age sex))
 (de-categorize table)

 ;; Here we have an example with non-numeric values and types:
 (def age (sample (list 'old 'middle 'whippersnapper) 10 t))
 (def sex (sample (list 'm 'f) 10 t))
 (def table (categorize age sex :sort-values nil :sort-types nil))
 (de-categorize table :types (remove-duplicates sex))
 (size *)

 (mapcar #'first (de-categorize '((0 6) (1 4) (2 9) (3 8) (4 3) (5 1))))

"
  (let (
	result
	(types (if types types (iseq (1- (length (first categories))))))
	)
    (mapcar
     (lambda(x)
       (let ((val (first x)))
	 (mapcar
	  (lambda(y z)
	    (dotimes (i z) (setq result (append result (list (list val y))))))
	  types
	  (rest x)
	  )
	 )
       )
     categories
     )
    result
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 two-way-anova
;;
(defun two-way-anova(a)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          two-way-anova Doc:
;;
;; Author: Andy Long
;; Created: Tue May 6 12:59:07 CDT 1997
;;
;; Description/Arguments:

       (two-way-anova matrix)
  
;; Notes: This was developed following Snedecor and Cochran, Statistical
;; Methods, 6th edition; see page 300.

;; See Also: one-way-anova, anova, regression methods :anova, etc.

;; Examples:
 ;; this is Table 11.2.1, p. 300:
 (def a (matrix '(5 5) (list
		8 10 12 13 11
		2 6 7 11 5
		4 10 9 8 10
		3 5 9 10 6
		9 7 5 5 3)))
 (two-way-anova a)

 ;; Example 11.2.1, p. 301:
 (def a (matrix '(3 3) (list
		112 90 123
		86 73 89
		80 62 81
		)))
 (two-way-anova a)

 ;; Table 11.5.1
 (def a (matrix '(4 3) (list 30 29 46 35 34 33 31 30 32 16 11 21)))
 (mapcar #'sum (row-list a))
 (mapcar #'sum (column-list a))
 (two-way-anova a)

 ;; the following a is rank-two, and meets the same row-column standards as the
 ;; a matrix above:
 (def a (matrix '(4 3) (list 34 32 39 33 31 38 30 28 35 15 13 20)))
 (mapcar #'sum (row-list a))
 (mapcar #'sum (column-list a))
 (two-way-anova a)
"
  (let* (
	 (size (size a))
	 (row-df (first size))
	 (column-df (second size))
	 (df (apply #'* size))

	 (c (/ (square (sum a)) df))
	 (row-sums (mapcar #'sum (row-list a)))
	 (column-sums (mapcar #'sum (column-list a)))

	 (total-ss (- (sum (square a)) c))
	 (row-ss (- (/ (sum (square row-sums)) column-df) c))
	 (column-ss (- (/ (sum (square column-sums)) row-df) c))
	 (residual-ss (- total-ss row-ss column-ss))

	 (row-df (1- row-df))
	 (column-df (1- column-df))
	 (residual-df (* row-df column-df))

	 (mean-row-ss (/ row-ss row-df))
	 (mean-column-ss (/ column-ss column-df))
	 (mean-residual-ss (/ residual-ss residual-df))

	 (row-f-ratio 
	  (if (= mean-residual-ss 0) 1e31 (/ mean-row-ss mean-residual-ss)))
	 (column-f-ratio
	  (if (= mean-residual-ss 0) 1e31 (/ mean-column-ss mean-residual-ss)))

	 (fmt1 "~%  ~a ~14t~a ~26t~a ~43t~a ~55t~a ~67t~a~2%")
	 (fmt2 "~1t~a ~14t~d ~22t~8,8g ~39t~8,8g ~51t~8,8g ~63t~,4f~%")
	 (fmt3 "~1t~a ~14t~d ~22t~8,8g ~39t~8,8g ~51t")
	 )    
    (format t fmt1 "Source" "df" "SS" "MS" "F ratio" "Prob(f)")
    (format t fmt2 "Columns"  column-df column-ss mean-column-ss column-f-ratio
	    (- 1 (f-cdf column-f-ratio column-df residual-df)))
    (format t fmt2 "Rows"  row-df row-ss mean-row-ss row-f-ratio 
	    (- 1 (f-cdf row-f-ratio row-df residual-df)))
    (format t fmt3 "Residual"  residual-df residual-ss mean-residual-ss)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 two-way-regression
;;
(defun two-way-regression(a &key (display t) (test #'equalp) set-names missing)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          two-way-regression Doc:
;;
;; Author: Andy Long
;; Created: Tue May 6 13:53:20 CDT 1997
;;
;; Description/Arguments:

       (two-way-regression matrix-or-list
           &key (display t) (test #'equalp) set-names missing)
  
;; Args:
	The matrix-or-list is a matrix (if all positions have equal n) or a
 list of lists of triples (row column value) or three lists (row) (column)
 (value).

	The test option refers to the test that remove-duplicates will perform
to decide how many groups there are. If the rows and columns are given by
numbers, then this should be #'=, but #'equalp is conservative (it will work
for symbols or strings, for example).

;; Notes:

	I just wanted to try a different method for calculating the essential
f-statistics for two-way anova. This uses a method whereby I turn the matrix
problem into a vector problem, and send the :anova-sequential method to the
regressions, with the :sets option.

	The rows referred to correspond to the first dimension of the matrix,
or the first list given; the columns refer to the other.

	The two sets are the row indicators and the column indicators. A new
method is defined, :matrix-estimate, to give the estimated matrix from the
regression.

;; See Also: two-way-anova

;; Examples:
 (def a (matrix '(4 3)
	       (list
		30 29 46 
		35 34 33
		31 30 32
		16 11 21
		)))
 (two-way-anova a)
 (def reg (two-way-regression a))
 (mprint (send reg :matrix-estimate))
 (mprint (- a (send reg :matrix-estimate)))

 ;; The following is the same problem as given above, only with a given as
 ;; triples:
 (def a
   '((0 0 30) (0 1 29) (0 2 46) (1 0 35) (1 1 34) (1 2 33) (2 0 31) (2 1 30)
     (2 2 32) (3 0 16) (3 1 11) (3 2 21)))
 (def reg (two-way-regression a))
 (mprint (send reg :matrix-estimate))
 (mprint (- (third (matroid-to-matrices (matrix '(4 3) a)))
            (send reg :matrix-estimate)))

 ;; From Draper and Smith, Applied Regression Analysis:
 (title \"Example A, p. 454\")
 (def x (list 1 1 2 2 3 3))
 (def x (repeat x 3))
 (def y (combine (repeat 10 6) (repeat 20 6) (repeat 30 6)))
 (def z (/ (list 452 460 400 390 359 341 418 206 278 190 225 177 235 331 446 522
		427 486) 10))
 (def reg (two-way-regression (list x y z)))
"
  (let* (
	 (lists 
	  (if (matrixp a)
	      (let ((aa (matrix-to-list (matrix-to-matroid a))))
		(list
		 (mapcar #'first aa)
		 (mapcar #'second aa)
		 (mapcar #'third aa)))
	    (if (= (length a) 3) a
	      (list
	       (mapcar #'first a)
	       (mapcar #'second a)
	       (mapcar #'third a)))))
	 (rows (first lists))
	 (cols (second lists))
	 (vals (third lists))
	 (unique-row (rest (remove-duplicates rows :test test)))
	 (unique-col (rest (remove-duplicates cols :test test)))
	 
	 (col-indicators
	  (mapcar
	   (lambda(i) (indicator cols (where cols i #'equalp)))
	   unique-col
	   ))
	 
	 (row-indicators
	  (mapcar
	   (lambda(i) (indicator rows (where rows i #'equalp)))
	   unique-row
	   ))
	 
	 (reg (regress
	       (smoosh (list row-indicators col-indicators))
	       vals
	       :print display
	       :missing missing 
	       :predictor-names
	       (combine
		(mapcar (lambda(x) (format nil "row ~s" x)) unique-row)
		(mapcar (lambda(x) (format nil "col ~s" x)) unique-col)
		)
	       )
	      )
	 (beta (send reg :beta))
	 )
    (send reg :add-slot 'matrix-estimate 
	  (+ (first beta)
	     (outer-product
	      (combine 0 (select beta (iseq 1 (length unique-row))))
	      (combine 0 (select beta (iseq (1+ (length unique-row))
					    (1- (length beta)))))
	      #'+))
	  )
    (defmeth reg :matrix-estimate() (slot-value 'matrix-estimate))
    (send reg
	  :anova-sequential
	  :sets (list row-indicators col-indicators)
	  :pure-error t
	  :set-names set-names
	  )
    reg
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 paired-t
;;
(defun paired-t(a b &key (hyp 0) (alpha .05))
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          paired-t Doc:
;;
;; Author: Andy Long
;; Created: Mon May 12 11:52:50 CDT 1997
;;
;; Description/Arguments:

       (paired-t a b &key (hyp 0) (alpha .05))
  
;; Notes:

;; See Also:

;; Examples:
 (def a (list 117.3 111.4 98.6 104.3 105.4 100.4 81.7 89.5 78.2))
 (def b (/ (list 833 859 758 829 823 777 627 690 639) 10))
 (paired-t a b)
"
  (let* (
	 (diff (- a b))
	 (mean (mean diff))
	 (n (length a))
	 (var (/ (sum (square (- diff mean))) (1- n)))
	 (tval (abs (/ (- mean hyp) (sqrt (/ var n)))))
	 (tcrit (t-quant (- 1 (/ alpha 2)) (1- n)))
	 )
    (format t "~%Alpha: ~,4f~2%" alpha)
    (format t "T-statistic:    ~g~%" tval)
    (format t "Critical Value: ~g~%" tcrit)
    (format t "Confidence Int: ~g +- ~g~%" tval (* tcrit (sqrt (/ var n))))
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                   Lumley's Adjusted p-value functions
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
#|
I don't have Dunnett's test but I do have three p-value based 
corrections (Bonferroni, Holm, and Hochberg).  Holm's test is a more 
powerful version of Bonferroni and Hochberg's test is still more 
powerful. They work well when the tests are correlated and when several 
of the null hypotheses are false (the situation when Bonferroni's 
inequality is conservative). The theory is explained in "Adjusted 
p-values for simultaneous inference" by SP Wright, Biometrics 48: 1005-1013.
Both tests multiple the smallest of k p-values by k, the next smallest by 
k-1 and so on, but they differ in the way they exploit the original 
ordering of the p-values.

Code follows:

thomas lumley
UW biostatistics
|#
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                holm
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun holm (p-values)
  "Args: p-values
Adjusts a list of p-values by Holm's method

;; Examples:
 (holm (list .04 .02 .01 .005))
 ;; This is table 3 from the paper cited above:
 (holm (/ (list 294 668 6940 13903 13903 70222) 100000))
"
  (let* (
	 (n (length p-values))
	 (ranks (rank p-values))
	 (qi (* p-values (- n ranks)))
	 )
    (dotimes
     (i (- n 1) qi)
     (let (
	   (index (position (+ 1 i) ranks))
	   (lastindex (position i ranks))
	   )
       (setf (elt qi index) (max (elt qi index) (elt qi lastindex)))))
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                hochberg
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun hochberg (p-values) 
  "Args: p-values
Adjusts a list of p-values by Hochberg's method

;; Examples:
 (hochberg (list .04 .02 .01 .005))
 ;; This is table 3 from the paper cited above:
 (hochberg (/ (list 294 668 6940 13903 13903 70222) 100000))
"
  (let* (
	 (n (length p-values))
	 (ranks (rank p-values))
	 (qi (* p-values (- n ranks)))
	 )
    (dotimes
     (i (- n 1) qi)
     (let (
	   (index (position  (- n i 2) ranks))
	   (lastindex (position  (- n i 1) ranks))
	   )
       (setf (elt qi index) (min (elt qi index) (elt qi lastindex)))
       )
     )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                bonferroni
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun bonferroni (p-values)
  "Args: p-values
Adjusts a list of p-values by Bonferroni's method.
Use (holm) or (hochberg) instead.

;; Examples:
 (bonferroni (list .04 .02 .01 .005))
 ;; This is table 3 from the paper cited above:
 (bonferroni (/ (list 294 668 6940 13903 13903 70222) 100000))
"
  (* p-values (length p-values))
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				sample-variance
;;
(defun sample-variance(b)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			sample-variance Doc:
;;
;; Author: Andy Long
;; Created: Sun Apr 19 10:27:14 1998
;;
;; Description/Arguments: sample-variance

	The sample variance s^2 is defined by

			s^2 = (\sum_{i=1}^n (y_i-y_bar)^2)/n

;; Arguments: b - a list of values

;; Returns: the sample variance

;; Notes: this is different from the unbiased variance, given by the same
;; formula above only divided by n-1 rather than n.

;; See Also: variance

;; Examples:
 (def b (list 5 0 2 40 27 3 0 22 14 0 4 19 38 2 5 16 0 7 42 15 39 0 2 0 29 26 14
	     0 3 27 32 20 3 0 17 35 29 12 16 6))
 (variance b)
 (sample-variance b)
"
  (/ (sum (square (- b (mean b)))) (length b)))
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				repeated-sampling-subsets
;;
(defun repeated-sampling-subsets(n &optional (vals '(0 1)))
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			repeated-sampling-subsets Doc:
;;
;; Author: Andy Long
;; Created: Sun Apr 19 13:53:57 1998
;;
;; Description/Arguments: repeated-sampling-subsets

	This function is for creating the sample space of an experiment where
one chooses repeatedly a single element of a set. It was created for coin
tosses, so the default set is '(0 1), which can be considered as heads and
tails. Thus three tosses in a row would be generated by the command

		 (repeated-sampling-subsets 3)

;; Arguments: n &optional (vals '(0 1))

	n	- the number of tosses
	vals	- the set of choices

;; Returns: all possible samples

;; Examples:
 (repeated-sampling-subsets 1)
 (repeated-sampling-subsets 2)
 (repeated-sampling-subsets 3)
 (repeated-sampling-subsets 3 '(h t))
 (repeated-sampling-subsets 2 '(h t n))
"
  (if (= n 1) vals
    (smoosh 
     (let ((fewer-tosses (repeated-sampling-subsets (1- n) vals)))
       (mapcar (lambda(i) (mapcar (lambda(x) (combine i x)) fewer-tosses))
	       vals)))))
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				combinations
;;
(defun combinations(n r)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			combinations Doc:
;;
;; Author: Andy Long
;; Created: Sun Apr 19 14:05:30 1998
;;
;; Description/Arguments: combinations

	This is a standard probabilistic function.

;; Arguments: n r

;; Returns: The number of distinct ways of forming subsets of size r of a set
;; of size n.

;; See Also:  partitions, permutations.

;; Examples:
 (combinations 10 4)
 (combinations 4 10)
"
  (if (and (integerp n) (integerp r) (> n 0) (>= r 0))
      (if (< n r) 0
	(/ (factorial n) (* (factorial r) (factorial (- n r)))))
    (helpless 'combinations)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				permutations
;;
(defun permutations(n r)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			permutations Doc:
;;
;; Author: Andy Long
;; Created: Sun Apr 19 14:05:30 1998
;;
;; Description/Arguments: permutations

	This is a standard probabilistic function.

;; Arguments: n r

;; Returns: The number of ways of ordering n distinct objects taken r at a
;; time.

;; See Also: combinations, partitions.

;; Examples:
 (permutations 10 4)
 (permutations 10 0)
"
  (if (and (integerp n) (integerp r) (> n 0) (>= r 0))
      (/ (factorial n) (factorial (- n r)))
    (helpless 'permutations)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				partitions
;;
(defun partitions(n &rest r)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			partitions Doc:
;;
;; Author: Andy Long
;; Created: Sun Apr 19 14:05:30 1998
;;
;; Description/Arguments: partitions

	This is a standard probabilistic function.

;; Arguments: n n1 ... nk

;; Returns: The number of ways of partitioning n distinct objects into k
;; distinct groups containing n1,...nk objects, respectively.

;; See Also: combinations, permutations.

;; Examples:
 (partitions 10 4)
 (partitions 10 4 2)
 (partitions 10 4 2 0)
"
  (if (v-and (combine (mapcar #'integerp (combine n r))
		      (mapcar (lambda(x) (>= x 0)) (combine n r))))
      (/ (factorial n) (apply #'* (mapcar #'factorial r)))
    (helpless 'partitions)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun prob-of-getting-some-right-by-guessing (a n)
  "
;; Author: Andy Long
;; Fri May 1 16:05:50 1998

;; Description: (prob-of-getting-some-right-by-guessing a n)

	This is for matching problems. You've got n definitions and n words:
what's the chance that you get a of them right by guessing?

;; Examples:
 (prob-of-getting-some-right-by-guessing 0 4)
 (prob-of-getting-some-right-by-guessing 1 4)
 (prob-of-getting-some-right-by-guessing 2 4)
 (prob-of-getting-some-right-by-guessing 3 4)
 (prob-of-getting-some-right-by-guessing 4 4)
"
  (/ 
   (number-of-ways-of-getting-some-right-by-guessing a n)
   (factorial n)))

(defun number-of-ways-of-getting-some-right-by-guessing
  (a n)
  "
;; Author: Andy Long
;; Fri May 1 16:05:50 1998

;; Description: number-of-ways-of-getting-some-right-by-guessing

	This is for matching problems. You've got n definitions and n words:
what's the chance that you get a of them right by guessing?

;; Arguments: (a n)

;; Notes: A helper for prob-of-getting-some-right-by-guessing.

;; Examples:
 (number-of-ways-of-getting-some-right-by-guessing 0 4)
 (number-of-ways-of-getting-some-right-by-guessing 1 4)
 (number-of-ways-of-getting-some-right-by-guessing 2 4)
 (number-of-ways-of-getting-some-right-by-guessing 3 4)
 (number-of-ways-of-getting-some-right-by-guessing 4 4)
"
  (if (= a n);; Then we've just got 1 out of A^n_a chances!
      1	;; otherwise, 
    (*	;; we multiply the number of ways of ways of
     ;; picking a objects right out of the n
     (combinations n a)
     ;; and the difference of 
     (-
	 ;; the number of ways of choosing all others,
      (factorial (- n a))
      ;; and the sum of
      (sum 		   
       ;; the ways of getting some right by guessing:
       (mapcar
	(lambda(i)
	  (number-of-ways-of-getting-some-right-by-guessing i (- n a)))
	(iseq 1 (- n a))))))))
