;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defmacro 2steep (f &rest e)
  "
;; Description: (2steep f guess n (eps 1e-6))

	Performs 2steepest descents on the function f. The default for epsilon
is .000001.

;; Examples from multivariate calc, Harvard project:
;; #17, page 205
 (new-plot)
 (title \"#17, page 205\")
 (func g(x y) (x + 1) ^ 4 + (y - 1) ^ 4 + 1 / (x ^ 2 * y ^ 2 + 1))
 (spin-domain  -2 -1 1 2)
 (contour g)
 (2steep g (list 6 60) 100)
;; (-1.3910390862916089 1.3910410135806546)

;; #24, page 206
 (new-plot)
 (title \"#24, page 206\")
 (func g(x y) x ^ 2 + x * y)
 (contour g)
 (2steep g (list 6 6) 100)
;; (-4.724386193528013E-5 1.3690994923364145E-4) 
"
  (let* (
	 (newe `,e)
	 (guess (first newe))
	 (n (second newe))
	 (eps (if (= 3 (length newe)) (third newe) 1e-6))
	 )
    `(2steepest #',f ,guess ,n ,eps)
    )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defmacro steep (f &rest e)
  "
;; Description: (steep f guess n &optional (eps 1e-6))

	Performs steepest descents on the function f. The default for epsilon
is .000001.

;; Examples from multivariate calc, Harvard project:
 (new-plot) ;; initializes plotting environment

 (title \"#17, page 205\")
 (func g(x y) (x + 1) ^ 4 + (y - 1) ^ 4 + 1 / (x ^ 2 * y ^ 2 + 1))
 (spin-domain  -2 -1 1 2)
 (contour g)
 (steep g (list 6 60) 100) ;; (-1.3910390862916089 1.3910410135806546)

 (title \"#24, page 206\")
 (func g(x y) x ^ 2 + x * y)
 (contour g)
 (steep g (list 6 6) 100) ;; (-4.724386193528013E-5 1.3690994923364145E-4) 
"
  (let* (
	 (newe `,e)
	 (guess (first newe))
	 (n (second newe))
	 (eps (if (= 3 (length newe)) (third newe) 1e-6))
	 )
    `(steepest #',f ,guess ,n ,eps)
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Steepest Descent
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun steepest (f guess n &optional (eps 1e-6))
  "
;; Description: (steepest f guess n &optional (eps 1e-6))
	See steep.

;; Examples:
;; from multivariate calc, Harvard project: #17, page 205
 (func g(x y) (x + 1) ^ 4 + (y - 1) ^ 4 + 1 / (x ^ 2 * y ^ 2 + 1))
 (steepest #'g (list 6 6) 100)
;; (-1.391039007575002 1.3910410917778635)
"
  (let* (
	 ;; initial guess x and y values:
	 (oldguess `,guess)

	 ;; What are the independent variables?
	 (newvar (second (eval `(get-lambda-expression ,f))))
	 (newx (first newvar))
	 (newy (second newvar))
	 
	 ;; get the partials:
	 (crum (eval `(derivfunc 'fx.quill ,f ',newx)))
	 (crum (eval `(derivfunc 'fy.quill ,f ',newy)))
	 
	 ;; form the objective function:
	 (lef (third (get-lambda-expression `,f)))

	 (newguess oldguess)
	 xo yo px py crum to
	 )

    ;; let's do it again and again!
    (dotimes
     (i n)

     (setq
      oldguess newguess
      xo (first newguess)
      yo (second newguess)
      px (fx.quill xo yo)
      py (fy.quill xo yo)
      le.gx lef
      crum (list (list newx '- `,xo (list '* 'z `,px)))
      le.gx (sublis crum le.gx)
      crum (list (list newy '- `,yo (list '* 'z `,py)))
      le.gx (sublis crum le.gx)
     
      crum (eval `(defun f.quill (z) ,le.gx))
      ;; get the partials of the new objective function:
      crum (eval `(derivfunc 'f2.quill #'f.quill 'z))
      )
     
     ;; follow the gradient to a zero:
     (if (= 0 (first (mapcar #'f2.quill '(0)))) (return newguess)
       (setq
	to (first (eval `(roots f2.quill 0)))
	;; and so we have a new guess:
	newguess (- oldguess (* to (list px py)))
	)
       )
     
     ;; this is our test to see if we've converged:
     (if (< (/ (norm (- newguess oldguess))
	       (+ eps (norm oldguess))) eps) (return newguess))
     )
    newguess
    )
  )

(defun 2steepest (f guess n &optional (eps 1e-12))
  "
;; Description: (2steepest f guess n &optional (eps 1e-12))
	See 2steep.

;; Examples:
;; from multivariate calc, Harvard project: #17, page 205
 (func g(x y) (x + 1) ^ 4 + (y - 1) ^ 4 + 1 / (x ^ 2 * y ^ 2 + 1))
 (2steepest #'g (list 6 6) 100)
;; (-1.391039007575002 1.3910410917778635)
"
  (let* (
	 
	 (oldguess `,guess)

	 ;; What are the independent variables?
	 (newvar (second (eval `(get-lambda-expression ,f))))
	 (newx (first newvar))
	 (newy (second newvar))
	 
	 ;; get the partials:
	 (crum (eval `(derivfunc 'fx.quill ,f ',newx)))
	 (crum (eval `(derivfunc 'fy.quill ,f ',newy)))
	 
	 ;; form the objective function:
	 (le.gx (third (get-lambda-expression #'fx.quill)))
	 (le.gy (third (get-lambda-expression #'fy.quill)))
	 (crum (eval `(defun f.quill (,newx ,newy)
			(+ (^ ,le.gx 2) (^ ,le.gy 2)))))

	 ;; get the partials of the new objective function:
	 (crum (eval `(derivfunc 'fx.quill #'f.quill ',newx)))
	 (crum (eval `(derivfunc 'fy.quill #'f.quill ',newy)))

	 (newguess oldguess)
	 xo yo px py to
	 )

    ;; let's do it again and again!
    (dotimes (i n)

	     (setq
	      oldguess newguess
	      xo (first oldguess)
	      yo (second oldguess)
	      px (fx.quill xo yo)
	      py (fy.quill xo yo)
	      to (/ (f.quill xo yo) (+ (^ px 2) (^ py 2)))
	      newguess (- oldguess (* to (list px py)))
	      )
	     
	     ;; this is our test to see if we've converged:
	     (if (< (/ (norm (- newguess oldguess)) (+ eps (norm oldguess))) eps)
		 (return newguess))
	     )
    newguess
    )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                 Steep-path: illustrates steepest descent
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun steep-path (f guess n &key (eps 1e-12) (cont t))
  "
;; Description: (steep-path f guess max-iterations &key (eps 1e-12) (cont t))

	This is designed to illustrate steepest descent to students.

;; Examples:
;; from multivariate calc, Harvard project: #17, page 205
 (func g(x y) (x + 1) ^ 4 + (y - 1) ^ 4 + 1 / (x ^ 2 * y ^ 2 + 1))
 (spin-domain -2 0 0 2) ;; min at (-1.391039007575002 1.3910410917778635)
 (steep-path #'g (list 6 6) 100)

;; A least-squares problem, as vectors:
 (def xvals '(1 2 3))
 (def yvals '(1 1 3))
 (def ones '(1 1 1))
 (func f(m b) (sum ((yvals - (b * ones + m * xvals)) ^ 2)))
 (spin-domain 0 2 -1 0) ;; min at (1 -1/3)
 (steep-path #'f (list 0 0) 300)
 (steep-path #'f (list 1 -1) 300 :cont nil)
"
  (let* (
	 (oldguess `,guess)
	 (newguess oldguess)

	 ;; What are the independent variables?
	 (newvar (second (eval `(get-lambda-expression ,f))))
	 (newx (first newvar))
	 (newy (second newvar))
	 
	 ;; get the partials:
	 (crum (eval `(derivfunc 'fx.quill ,f ',newx)))
	 (crum (eval `(derivfunc 'fy.quill ,f ',newy)))
	 
	 ;; form the objective function:
	 (le.gx (third (get-lambda-expression #'fx.quill)))
	 (le.gy (third (get-lambda-expression #'fy.quill)))
	 (crum (eval `(defun f.quill (,newx ,newy)
			(+ (^ ,le.gx 2) (^ ,le.gy 2)))))

	 ;; get the partials of the new objective function:
	 (crum (eval `(derivfunc 'fx.quill #'f.quill ',newx)))
	 (crum (eval `(derivfunc 'fy.quill #'f.quill ',newy)))

	 (plots (if cont (eval `(contour f.quill))
		  (select (active-graph-windows) (iseq 1 0))))
	 (my-contour-plot (first plots))
	 (my-spin-plot (second plots))

	 xo yo px py to fval
	 )
    (printall plots my-contour-plot my-spin-plot)
    ;; let's do it again and again!
    (dotimes (i n)
	     (setq 
	      oldguess newguess
	      xo (first oldguess)
	      yo (second oldguess)
	      px (fx.quill xo yo)
	      py (fy.quill xo yo)
	      fval (f.quill xo yo)
	      to (/ fval (+ (^ px 2) (^ py 2)))
	      newguess (- oldguess (* to (list px py)))
	      )
	     
	     (send my-contour-plot :add-points (list (list xo) (list yo)))
	     (send my-spin-plot
		   :add-points
		   (list (list xo) (list yo) (list fval)))

	     (print oldguess)

	     ;; this is our test to see if we've converged:
	     (if (< (/ (norm (- newguess oldguess)) (+ eps (norm oldguess))) eps)
		 (return newguess))
	     )
    newguess
    )
  )
