;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun write-latex-table
  (x &key (title "table.tex") (how "r") (rule nil)
     (form "full") (size "normalsize") (caption "caption"))
  "
Function: write-latex-table

Args: (x &key (title \"table.tex\") (how \"r\") (rule nil)
        (form \"full\") (size \"normalsize\") (caption \"caption\"))

Takes a matrix of numbers or strings, and writes it to a file (default
is \"table.tex\") in a LaTeX table. Optional parameters
for the title of the file, for the caption, for the alignment (r, c, l),
for the format (full, upper, lower), for rules between the lines,
and for the LaTeX size (normalsize, tiny, LARGE, etc). Returns nil.

;; Examples:
 (write-latex-table (id 4)
		   :title \"crud\" :caption \"The 4x4 identity matrix\")
"
  (let* (
	 (n (first (array-dimensions x)))
	 (m (second (array-dimensions x)))
	 (s (with-output-to-string (form)
				   (format form "~a" "|")
				   (dotimes (i m)
					    (format form "~a~a" "|" how)
					    (if (= i (1- m)) (format form "~a"
								     "||")))))
	 )
    (with-open-file (texfile title :direction :output)
		    (format texfile "~a[h!]~%" "\\begin{table}")
		    (format texfile "~a~%" "\\centering")
		    (format texfile "\\~a~%" size)
		    (format texfile "~a{~a}\\hline~%" "\\begin{tabular}" s)
		    (dotimes (i n)
			     (dotimes (j m)
				      (if (or (string= form "full")
					      (and (string= form "upper") (>= j i))
					      (and (string= form "lower") (<= j i)))
					  (format texfile "~a" (aref x i j)))
				      (if (= j (1- m))
					  (if (= i (1- n)) (format texfile " \\\\\\hline~%")
					    (if rule (format texfile " \\\\\\hline~%")
					      (format texfile " \\\\~%")))
					(format texfile " & "))))
		    (format texfile "~a~%" "\\end{tabular}")
		    (format texfile "~a{~a}~%" "\\caption" caption)
		    (format texfile "~a~%" "\\end{table}"))
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

#|
(defun latex-table
  (x &key 
     (file *standard-output*)
     (how "c")
     (rule nil)
     (form "full")
     (size "normalsize")
     (caption "")
     (label "")
     row-names
     column-names
     select-rows
     select-columns
     extra-rule
     (fmt "~a")
     )
  (let*
      (
       (x 
	(if (and row-names column-names)
	    (bind-columns (combine "" row-names) (bind-rows column-names x))
	  (if column-names (bind-rows column-names x)
	    (if row-names (bind-columns row-names x)
	      x))))

       (select-rows (if (and select-rows column-names)
			(combine 0 (+ 1 select-rows))
		      select-rows))
       (select-columns (if (and select-columns row-names)
			   (combine 0 (+ 1 select-columns))
			 select-columns))
       (xsize (size x))
       (n (if select-rows select-rows (iseq (first xsize))))
       (m (if select-columns select-columns (iseq (second xsize))))
       
       (last-n (final n))
       (last-m (final m))
       (s (with-output-to-string
	   (form)
	   (format form "~a" "|")
	   (dolist
	    (i m)
	    (format form "~a~a" "|" how)
	    (if (= i last-m) (format form "~a" "||"))
	    )
	   )
	  )

       (k 0)
       (extra-rule
	(if extra-rule
	    (if (numberp extra-rule)
		(1- (* extra-rule (1+ (iseq (first xsize)))))
	      extra-rule)))
       (extra-rule
	(if extra-rule
	    (if column-names (combine 0 (1+ extra-rule))
	      extra-rule)
	  (if column-names '(0))
	  )
	)
       (fmt (if (stringp fmt) (repeat fmt (second xsize)) fmt))
       )
    (if (not (stringp file))
        (let ()
	  (format file "
\\\begin{table}[h!]
\\\caption{~a}
\\\smallskip
\\\centering
\\\~a
\\\label{~a}
\\\begin{tabular}{~a}\\hline
"
		  caption size label s)
	  (dolist
	   (i n)
	   (dolist
	    (j m)
	    (if (or (string= form "full")
		    (and (string= form "upper") (>= j i))
		    (and (string= form "lower") (<= j i)))
		(format file (elt fmt j) (aref x i j)))
	    (if (= j last-m)
		(if (= i last-n) (format file " \\\\\\hline~%")
		  (if rule (format file " \\\\\\hline~%")
		    (format file " \\\\~%")))
	      (format file " & "))
	    )
	   (if (and extra-rule (search (list k) extra-rule))
	       (format file " \\hline~%"))
	   (setq k (+ 1 k))
	   )
	  (format file "\\\end{tabular}~%\\\end{table}~%")
	  )
      (with-open-file
       (texfile file :direction :output)
       (format texfile "
\\\begin{table}[h!]
\\\caption{~a}
\\\smallskip
\\\centering
\\\~a
\\\label{~a}
\\\begin{tabular}{~a}\\hline
"
	       caption size label s)
       (dolist
	(i n)
	(dolist
	 (j m)
	 (if (or (string= form "full")
		 (and (string= form "upper") (>= j i))
		 (and (string= form "lower") (<= j i)))
	     (format texfile (elt fmt j) (aref x i j)))
	 (if (= j last-m)
	     (if (= i last-n) (format texfile " \\\\\\hline~%")
	       (if rule (format texfile " \\\\\\hline~%")
		 (format texfile " \\\\~%")))
	   (format texfile " & "))))
       (format texfile "\\\end{tabular}~%\\\end{table}~%")
       )
      )
    )
  )
|#

(defun latex-table
  (x &key 
     (file *standard-output*)
     (how "c")
     (rule nil)
     (form "full")
     (size "normalsize")
     (caption "")
     (label "")
     row-names
     column-names
     select-rows
     select-columns
     extra-rule
     (fmt "~a")
     )
  "
Function: latex-table

Args:   
  (x &key 
     (file *standard-output*) ;; the filename of the file
     (how \"c\") ;; centered, versus right (r) or left (l) justified.
     (rule nil) ;; if you want lines between each entry
     (form \"full\") ;; format: full, upper, lower.
     (size \"normalsize\") ;; tiny, LARGE, scriptsize, etc.
     (caption \"\") ;; the caption for the table
     (label \"\") ;; the label that you'll refer to in your tex files
     row-names ;; if you want, add a list of row names
     column-names ;; if you want, add a list of column names
     select-rows ;; use only the select-rows rows (numbered from 0)
     select-columns ;; use only the select-columns columns (numbered from 0)
     extra-rule ;; add extra lines every so often (an integer), or specify the
		;; rows.
     (fmt \"~a\") ;; either a string or a list of strings (one for each column).
     )

	Writes a matrix of numbers or strings to a file (default is the
standard output) in a LaTeX table. Returns nil.

;; Examples:
 (latex-table (id 4) :caption \"The 4x4 identity matrix\" :label \"table1\")

 (latex-table (id 4) :form \"upper\")

 (latex-table (id 4) :rule t)

 (latex-table (id 4) :extra-rule 2)
 (latex-table (id 4) :extra-rule '(0 1))

 (latex-table (id 4)
              :select-rows (iseq 0 1)
              :select-columns (iseq 0 1))
 (latex-table (id 4)
              :select-rows (iseq 0 1)
              :select-columns (iseq 2 3))
              

 ;; this one will create a file called \"crud\" in your current directory:
 ;; (latex-table (id 4) :file \"crud\" :caption \"The 4x4 identity matrix\")

 (latex-table (id 4) :row-names (list \"Hi\" \"Lo\" \"Up\" \"Down\"))

 (latex-table (id 4)
   :row-names (list \"Hi\" \"Lo\" \"Up\" \"Down\")
   :column-names (list \"Hi\" \"Lo\" \"Up\" \"Down\")
   :select-rows (list 1 2)
   :select-columns (list 1 2)
   )

 (latex-table (id 4) :fmt \"~,4g\")
 (latex-table (id 4) :fmt (list \"~,3f\" \"~,2f\" \"~,1f\" \"~d\"))
 (latex-table (id 4) :fmt (list \"~,3g\" \"~,2g\" \"~,1g\" \"~d\"))
"
  (let*
      (
       (x 
	(if (and row-names column-names)
	    (bind-columns (combine "" row-names) (bind-rows column-names x))
	  (if column-names (bind-rows column-names x)
	    (if row-names (bind-columns row-names x)
	      x))))

       (select-rows (if (and select-rows column-names)
			(combine 0 (+ 1 select-rows))
		      select-rows))
       (select-columns (if (and select-columns row-names)
			   (combine 0 (+ 1 select-columns))
			 select-columns))
       (xsize (size x))
       (n (if select-rows select-rows (iseq (first xsize))))
       (m (if select-columns select-columns (iseq (second xsize))))
       
       (last-n (final n))
       (last-m (final m))
       (s (with-output-to-string
	   (form)
	   (format form "~a" "|")
	   (dolist
	    (i m)
	    (format form "~a~a" "|" how)
	    (if (= i last-m) (format form "~a" "||"))
	    )
	   )
	  )

       (extra-rule
	(if extra-rule
	    (if (numberp extra-rule)
		(1- (* extra-rule (1+ (iseq (first xsize)))))
	      extra-rule)))
       (extra-rule
	(if extra-rule
	    (if column-names (combine 0 (1+ extra-rule))
	      extra-rule)
	  (if column-names '(0))
	  )
	)
       (extra-rule (if rule n extra-rule))
       (fmt (if (stringp fmt) (repeat fmt (second xsize)) fmt))
       (result
	(with-output-to-string
	 (texfile)
	 (format texfile "
\\\begin{table}[h!]
\\\caption{~a}
\\\smallskip
\\\centering
\\\~a
\\\label{~a}
\\\begin{tabular}{~a}\\hline
"
		 caption size label s)
	 (dolist
	  (i n)
	  (dolist
	   (j m)
	   (if (or (string= form "full")
		   (and (string= form "upper") (>= j i))
		   (and (string= form "lower") (<= j i)))
	       (format texfile (elt fmt j) (aref x i j)))
	   (if (= j last-m) (format texfile " \\\\") (format texfile " & "))
	   )
	  (if extra-rule
	      (if (search (list i) extra-rule) (format texfile "\\hline")))
	  (if (= i last-n) (format texfile "\\hline~%") (format texfile "~%"))
	  )
	 (format texfile "\\\end{tabular}~%\\\end{table}~%")
	 )
	)
       )
    (if (stringp file)
	(with-open-file
	 (texfile file :direction :output)
	 (format texfile "~a" result)
	 )
      (format file "~a" result)
      )
    )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun random-permutation (n)
  "
Function to randomly permute the indices of a list (iseq 0 n)

;; Examples:
 (random-permutation 10)
"
  (sample (iseq n) n)
  )

(defmacro swap (x i j)
  "
Macro to swap the ith and jth elements of an array x.

;; Examples:
 (def x (iseq 10))
 (swap x 0 3)
"
  `(let ()
     (setf (select ,x (list ,i ,j)) (select ,x (list ,j ,i)))
     ,x
     )
  )
(defmacro aset (x i y)
  "
Macro to set the ith element of an array x to value y.

;; Examples:
 (setq x (iseq 10))
 (aset x 3 666)
 x
"
  `(setf (elt ,x ,i) ,y)
  )
