(tutorial-dialog (combine 
(list "
;; As an example, we can examine a data set that records the number of months
;; prior to an interview when individuals remember a stressful event (originally
;; from Haberman, [2, p. 2,]): 

 (def months-before (iseq 1 18))
 (def event-counts '(15 11 14 17 5 11 10 4 8 10 7 9 11 3 6 1 1 4))
")
(list "
;; The data are multinomial, and we can fit a log-linear Poisson model to see if
;; there is any time trend: 

 (def m (poissonreg-model months-before event-counts))
")
(list "
;; Residuals for the fit can be obtained using the :residuals message: 

 (send m :residuals)
")
(list "
;; A residual plot can be obtained using 

 (send m :plot-residuals)
")
(list "
;; The :fit-values message returns , the linear predictor without any
;; offset. The :fit-means message returns fitted mean response values.
;; Thus the expression 

(let ((p (plot-points months-before event-counts)))
  (send p :add-lines months-before (send m :fit-means)))

;; constructs a plot of raw counts and fitted means against time. 
")
(list "
;; To illustrate fitting binomial models, we can use the leukemia survival data
;; of Feigl and Zelen [4, Section 2.8.3,] with the survival time converted to a
;; one-year survival indicator: 

 (load-data \"leukemia\")
 (def surv-1 (if-else (> times-pos 52) 1 0))

 surv-1
")
(list "
;; The dependent variable is the base 10 logarithm of the white blood cell counts
;; divided by 10,000: 

 transformed-wbc-pos
")
(list "
;; A binomial model for these data can be constructed by 

 (def lk (binomialreg-model transformed-wbc-pos surv-1 1))
")
(list "
;; This model uses the logit link, the canonical link for the binomial
;; distribution. As an alternative, the expression 

 (binomialreg-model transformed-wbc-pos surv-1 1 :link probit-link)

;; returns a model using a probit link. 
")
(list "
;; The :cooks-distances message helps to highlight the last observation
;; for possible further examination: 

 (send lk :cooks-distances)
")
(list "
;; This observation also stands out in the plot produced by 

 (send lk :plot-bayes-residuals)
")
(list "
;; Four functions are provided to help construct indicator vectors for
;; categorical variables. As an illustration, a data set used by Bishop,
;; Fienberg, and Holland examines the relationship between occupational
;; classifications of fathers and sons. The classes are

;; a - professional, high-administrative
;; s - managarial,executive, high supervisory
;; i - low inspectional, supervisory
;; n - routine nonmanual, skilled manual
;; u - semi- and unskilled manual

;; We can set up the occupation codes as 

 (def occupation '(a s i n u))
")
(list "
;; and construct the son's and father's code vectors for entering the data row by
;; row as

 (def son (repeat occupation 5))
 (def father (repeat occupation (repeat 5 5)))

;; The counts can then be entered as 

 (def counts '(50  45   8  18   8 
               28 174  84 154  55 
               11  78 110 223  96
               14 150 185 714 447
                3  42  72 320 411))
")
(list "
;; To fit an additive log-linear model, we need to construct level indicators.
;; This can be done using the function indicators: 

 (indicators son)

;; the five levels of the variable son. By default, the first level is
;; dropped. To obtain indicators for all five levels, we can supply the
;; :drop-first keyword with value nil:

 (indicators son :drop-first nil)
")
(list "
;; To produce a readable summary of the fit, we also need some labels: 

 (level-names son :prefix 'son)

;; By default, this function also drops the first level. This can again be
;; changed by supplying the :drop-first keyword argument as nil: 

 (level-names son :prefix 'son :drop-first nil)
")
(list "
;; Using indicator variables and level labels, we can now fit an additive model
;; as

 (def mob-add
       (poissonreg-model
        (append (indicators son) (indicators father)) counts
        :predictor-names (append (level-names son :prefix 'son)
                                 (level-names father :prefix 'father))))

")
(list "
;; Examining the residuals using 

 (send mob-add :plot-residuals)

;; shows that the first cell is an outlier -- the model does not fit this cell
;; well.
")
(list "
;; To fit a saturated model to these data, we need the cross products of the
;; indicator variables and also a corresponding set of labels. The indicators are
;; produced with the cross-terms function 

 (cross-terms (indicators son) (indicators father))
")
(list "
;; and the names with the cross-names function: 

 (cross-names (level-names son :prefix 'son)
               (level-names father :prefix 'father))
")
(list "
;; The saturated model can now be fit by 

 (let ((s (indicators son))
        (f (indicators father))
        (sn (level-names son :prefix 'son))
        (fn (level-names father :prefix 'father)))
    (def mob-sat
         (poissonreg-model (append s f (cross-terms s f)) counts 
                           :predictor-names
                           (append sn fn (cross-names sn fn)))))

")

) :title "g.l.m.")
