(tutorial-dialog (combine 
(list "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                             Integration
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;      You can find the actual values of certain integrals using the function
;; integration. This function will integrate simple functions of a single
;; variable, which by default is assumed to be x.

;; See Also: simplifier.

;; Examples:
        (integration (3*x+2)^2)
        (integration (x*sin(x^2)))
        (integration ((3*x^3)-1/(3*x^3)))
        (integration (sin(x)^2*cos(x)))
        (integration (sin(x)/(1+cos(x))))
        (integration ((2*x+1)/(x^2+x-1)))
        (integration (8*x^2/(x^3+2)^3))
")
(list "
;;      You can use other variables of integration (even t!). Just be sure to
;; use the :var key to indicate the variable of integration:

        (integration t^2 :var t)
        (integration 2+alpha^2 :var alpha)
")
(list "
;;      Left- and right-rectangle rules, and the midpoint rule, let you
;; approximate definite integrals. Rectangle rules weight all points the same,
;; and then take an average value of those:

        (func f(x) x ^ 3 - 2 * x + 3) ; f(x)=x^3-2*x+3
        (lrr f 0 1 100) ; (left) integral of f from 0 to 1 with 100 steps:
        (rrr f 0 1 100) ; (right rectangle rule) 
        (mid f 0 1 100) ; midpoint rule
")
(list "
;;      Refinements on rectangle rules include the trapezoidal rule, which is
;; the average of the left and right-rectangle rules; and Simpson's rule, which
;; is a weighted average of the midpoint and trapezoidal rules:

;;                          Simpson's = (2*mid+trap)/3.

;;      This particular combination of methods allows the errors to cancel, to
;; give a better estimate than either the midpoint or trapezoidal gives.

;;      Another useful rule is gaussian integration, but it is more complicated
;; and better left to numerical analysis class! Suffice it to say that gaussian
;; integration, like the midpoint rule, is good for improper integrals where the
;; function has vertical asymptotes.

        (trap f 0 1 100) ; trapezoidal rule
        (gauss f 0 1 100) ; gaussian integration 
        (simp f 0 1 100) ; simpson's rule
")
(list "
;;      Simpson's rule does better than the rectangle rules (with a given
;; tolerance: the default is tolerance 1e-6, which equals .000001 (the decimal
;; place has moved left six places)):

        (simpson f 0 1)

;; This procedure is iterative, and starts with a relatively crude subdivision
;; of the interval, refining it as long as improvement is seen. Simpson's dance
;; is the lovely

;;             1 4 2 4 2 4 2 4 2 4 2 4 ... 4 2 4 2 4 2 4 2 4 2 4 1

;; weighting scheme. That is, it weights the endpoints the least, and then
;; appears to weight certain interior points twice as much as it does the other
;; points. It also requires an odd number of points to weight (the basic step
;; of Simpson's rule is \"1 4 1\", and when you butt them together you get the
;; dance above).
")
(list "
;;      How about a picture of the anti-derivative of a function? This is
;; actually a numerical approximation of the integral, and so is clumsy: it
;; creates a function which evaluates the integral at each point using
;; Simpson's rule.

;;      Integral defines a function, here called f.int, which represents that
;; anti-derivative of f which is zero at 0. The .001 represents a tolerance for
;; the numerical integration: a large tolerance helps speed things up!

        (integral f.int f 0 x .001)
        (title \"Numerical anti-derivative\")
        (plot f.int f 0 pi)
")
(list "
;;      Integration is also available in higher dimensions: one routine (simp2)
;; provided does a form of simpson's rule in one direction, then uses another
;; good method (hermite integration) to do the integral of these
;; integrals. Hermite's dance is given by

;;                      9 28 23 24 24 24 ... 24 24 24 23 28 9

        (simp2 (func f(x y) x * y) 0 1 0 1) ;; the true answer is .25
")
(list "
;;     A completely different technique for either single-variable or
;; multi-variable integration is monte-carlo integration. It involves throwing
;; darts at the domain of the function, and is easily generalized to functions
;; of even more than two variables.

;;                             Monte-carlo integration:

;;     In monte-carlo integration of a function f, we rely on a good sampling
;; of the domain of f, and on the mean-value theorem for integrals: that the
;; average value of the function multiplied by the
;; length/area/volume/etc. gives a good approximation to the value of the
;; integral.

;;     For a univariate function, the calculation of the integral of the
;; following function is carried out from 0 to 1. Note that we're 

;; (a) using rand.u to provide us with 100 values on the interval [0,1],
;; (b) evaluating f at those values, and
;; (c) generating the mean of those values. Since we're on the interval [0,1],
;;     we don't need to divide by the interval length: it's one.

        (func f(x) sqrt (1 - x ^ 2))
        (calc (mean (f (rand.u 100))))
        (simpson f 0 1) ;; compare to the \"correct\" numerical value:
")
(list "
;;      If we wanted to do the same problem, but on the interval [.4,.7], say,
;; then we'd need to take our random numbers from [.4,.7]:

        (def xvals (calc .4+(.7-.4)*(rand.u 100)))
        (calc (mean (f xvals))*(.7-.4))
        (simpson f .4 .7) ;; compare to the \"correct\" numerical value:
")
(list "
;;      Here's a little simulation, in which we compute the monte-carlo
;; integral for f from 0 to 1 20 times, and look at the histogram of the results.
;; (The mean of the trials gives us the value of a monte-carlo integration with
;; 2000 samples!)

        (setq trials (calc (mean (f (rand.u 100)))))
        (dotimes (i 20)
         (setq trials (combine trials (calc (mean (f (rand.u 100))))))
         )
        (mean trials)
        (sd trials)
        (histogram trials)
")
(list "
;;      The situation is essentially the same for multivariate function
;; integration: here's an example: integration of f over [0,1]x[0,1] (of area
;; 1): 

        (func f(x y ) exp(-(x^2+y^2)))
        (calc (mean (f (rand.u 100) (rand.u 100))))
        (integrate f 0 1 0 1) ;; compare to Simpson's rule-type integral

;; or even three (or more) variables

        (func f(x y z) x*y*z) ;; true value 1/8 = .125
        (calc (mean (f (rand.u 100) (rand.u 100) (rand.u 100))))
        .125
")

) :title "Integration")
