(def x (rand.u 200))
(def y (rand.u 200))
	      (setq c (first (rand.u 1)))
	      (setq d (first (rand.u 1)))
(def svd (svd (bind-columns x y (+ (* c x) (* d y)))))
(tutorial-dialog (combine 
(list ";;                         pca versus svd

;;         Principal component analysis is a standard multivariate technique for
;; analyzing the relationship between variables. It is essentially the singular
;; value decomposition of a scaled matrix. But sometimes the svd may be more
;; enlightening, and here are a couple of examples to demonstrate that.
;; 
;;         It is the zero singular values that tell you what the dependency is
;; between variables: and for that, the singular value decomposition gets it just
;; right (rather than principal component analysis, which suggests ... what?).
;; 
;;         In the following examples I look at several kinds of dependencies: in
;; the first case, three variables including the product of the first two; in the
;; next cases, three variables including the weighted sum of the first two. With
;; the svd, you can read the weights of the sum off of the last column of the
;; factors. Notice that transformations of the variables in appropriate ways may
;; lead to zero singular values, which describe dependencies (or near
;; dependencies, more generally).
;; 
")
(list "
;;       We start with a pair of random uniform variables x and y:
;; (def x (rand.u 200))
;; (def y (rand.u 200))
")
(list "
;;       Here is the pca versus svd for x, y, and x*y:

(def pca (pca (bind-columns x y (* x y))))
(pca-factors pca)
(def svd (svd (bind-columns x y (* x y))))
(pca-factors svd)

;;       Notice that there are no zero eigenvalues, as the variables are not
;; linearly dependent.
")
(list "
;;       Here is the pca versus svd for ln(x), ln(y), and ln(x*y)=ln(x)+ln(y):

(def pca (pca (bind-columns (ln x) (ln y) (ln (* x y)))))
(pca-factors pca)
(def svd (svd (bind-columns (ln x) (ln y) (ln (* x y)))))
(pca-factors svd)

;; There is now one zero eigenvalue, as the variables are linearly
;; dependent.
")
(list "
;;       Here is the pca versus svd for x, y, and x+y:

(def pca (pca (bind-columns x y (+ x y))))
(pca-factors pca)
(def svd (svd (bind-columns x y (+ x y))))
(pca-factors svd)

;; There is a zero eigenvalue, as the variables are linearly
;; dependent.
")
(list "
;;       Here is the pca versus svd for ln(x), ln(y), and ln(x+y):

(def pca (pca (bind-columns (ln x) (ln y) (ln (+ x y)))))
(pca-factors pca)
(def svd (svd (bind-columns (ln x) (ln y) (ln (+ x y)))))
(pca-factors svd)

;; Notice that the ln transform just messed things up! Only make
;; transformations expected to help make relationships linear.
")
(list (format nil "
;;       Here is the pca versus svd for x, y, and c*x+d*y, where
;; c is ~f, and d is ~f.

(def pca (pca (bind-columns x y (+ (* c x) (* d y)))))
(pca-factors pca)
(def svd (svd (bind-columns x y (+ (* c x) (* d y)))))
(pca-factors svd)
" 
	      c
	      d
	      )
      )
(list
 (format nil "
;;      I can use the svd to discover exactly the constants c and d: they are
;; essentially contained in the last column of the factors, as ratios of the
;; first two entries to the (negative of the) third:
;; c was ~f,
;; d was ~f; (c = ~f, d = ~f)
"
	 (/ (first (column 3 (third svd))) (- (third (column 3 (third svd)))))
	 (/ (second (column 3 (third svd))) (- (third (column 3 (third svd)))))
	 c d))

(list "
;;      The same information is given in the pca, only it relates to the scaled
;; variables, rather than the original variables: the conversion is as follows,
;; and involves scaling by the standard deviations of the variables:

   (def z (+ (* c x) (* d y)))
   (format t \"c and d: ~g and ~g.\"
   (/ (* (sd z) (first (column 3 (third pca))))
       (* (sd x) (- (third (column 3 (third pca))))))
   (/ (* (sd z) (second (column 3 (third pca))))
       (* (sd y) (- (third (column 3 (third pca)))))))
")

(list (format nil "
;;       Here is the pca versus svd for x, y, and c*x + d*y + e, where
;; c is ~f,
;; d is ~f,
;; and
;; e is ~f.

(def z (+ e (* c x) (* d y)))
(def pca (pca (bind-columns x y z)))
(pca-factors pca)
(def svd (svd (bind-columns x y z)))
(pca-factors svd)
" 
		 (setq c (first (rand.u 1)))
		 (setq d (first (rand.u 1)))
		 (setq e (first (rand.u 1)))
		 )
	 )
(list "
;;       The pca will still give the coefficients of the linear relationship
;; (not the constant), since the variables are first mean-centered (which
;; removes the constant e). Here are the results using pca (with e provided by
;; subtracting, i.e.
;;                        e-est = z - c-est * x - d-est * y
;; ):

(def c-est (/ (* (sd z) (first (column 3 (third pca))))
	      (* (sd x) (- (third (column 3 (third pca)))))))
(def d-est (/ (* (sd z) (second (column 3 (third pca))))
	      (* (sd y) (- (third (column 3 (third pca)))))))
(format t \"c was ~g,~%d was ~g,~%e was ~g.~%\" c-est d-est
	(- (mean z) (* c-est (mean x)) (* d-est (mean y))))
(format t \"Compare these with the real values:~%~g, ~g, and ~g.\"
	c d e)
")

(list "
;;      However, if a fourth column is added, of a constant, then the svd and
;; pca will both give the proper result (look for the values of c, d, and e in
;; the last column of the svd):

(def svd (svd (bind-columns x y z (repeat 1 (length z)))))
(pca-factors svd)
(format t \"c was ~g.\"
	(/ (first (column 4 (third svd))) (- (third (column 4 (third svd))))))
(format t \"d was ~g.\"
	(/ (second (column 4 (third svd))) (- (third (column 4 (third svd))))))
(format t \"e was ~g.\"
	(/ (fourth (column 4 (third svd))) (- (third (column 4 (third svd))))))
(format t \"Compare with the real values:~%~g, ~g, ~g.\"
        c d e)

")
) :title "PCA vs SVD")
