;;########################################################################
;; generic.lsp
;; Copyright (c) 1993-95 by Forrest W. Young
;; This file contains generic functions for menu items 
;; and guidemap and authoring load-on-demand functions.
;;########################################################################

(defun open-data (&optional file)
"Args: (&optional file)
Opens a data object contained in a file, and displays the datasheet. 
The file's name must end with .lsp. If the optional string argument FILE is included, the data object is loaded from FILE, otherwise a dialog is presented to select the file. The string need not end with .lsp.  Returns the object-id of the data object."
  (let ((result (load-data file))) ;(break)
    (when result (browse-data result))
    result))

(defun edit-datasheet (&rest args)
  (show-datasheet args))

(defun load-edit ()
"Args: none
Loads a Lisp Program, including ViSta Scripts and Examples."
  (let ((f (open-file-dialog t)))
    (when f (load f) (format t "; finished loading ~s~%" f))))

(defun save-data (&optional name)
"Args: (&optional name)
Saves the current data to a file named NAME"
  (if name (send current-data :save-data name)
      (send current-data :save-data)))

(defun create-data (&rest args)
"Args: (&rest args)
Makes the current object create output data.
Arguments depend on which object is current." 
  (if args (apply #'send current-object :create-data args)
      (send current-object :create-data)))

(defun delete-data ()
"Args: none
Deletes the current data object and all its child objects."
  (send *workmap* :delete-data))

(defun delete-model ()
"Args: none
Deletes the current model object and all its child objects."
  (send *workmap* :delete-model))

(defun analyze-data ()
  (cond
    ((send *current-data* :matrices)
     (multidimensional-scaling :dialog t))
    ((send *current-data* :ways)
     (analysis-of-variance :dialog t))
    (t
     (let ((choice (choose-item-dialog "Analyze Data ..."
                       '("Multiple Regression" 
                         "Principal Components"
                         "Correspondence Analysis" ))))
       (case choice
         (0 (Multiple-Regression :dialog t))
         (1 (Principal-components :dialog t))
         (2 (Correspondence-analysis :dialog t)))))))

(defun visualize ()
"Args: none
Displays a visualization of the current object."
  (send current-object :visualize))

(defun report ()
"Args: none
Makes a numeric report of the current object."
  (send current-object :report))

(defun summary 
  (&key (moments t) (quartiles t) ranges correlations covariances)
"Args: (&key (moments t) (quartiles t) ranges correlations covariances)
Presents univariate summary statistics for each variable
in the current data object."
  (send current-data :summary :moments moments :quartiles quartiles
        :ranges ranges :correlations correlations :covariances covariances))

(defun merge-vars (&optional name)
"Args: &optional name
Merges the variables of the current and previous data objects."
  (if name (send current-data :merge-variables name)
      (send current-data :merge-variables)))

(defun merge-variables (&optional name)
"Args: &optional name
Merges the variables of the current and previous data objects as object NAME."
  (if name (send current-data :merge-variables name)
      (send current-data :merge-variables)))

(defun merge-obs (&optional name)
"Args: &optional name
Merges the observations of the current and previous data objects."
  (if name (send current-data :merge-observations name)
      (send current-data :merge-observations)))

(defun merge-observations (&optional name)
"Args: &optional name
Merges the observations of the current and previous data objects."
  (if name (send current-data :merge-observations name)
      (send current-data :merge-observations)))

(defun select-variables (var-name-list)
"Args: VAR-NAME-LIST
Selects variables and displays the selection in the variable window, if open."
  (send current-data :select-variables var-name-list))

(defun select-observations (obs-label-list)
"Args: OBS-LABEL-LIST
Selects observations and displays the selection in the observations window, if open."
  (send current-data :select-observations obs-label-list))

(defun show-bug-list () (send *vista* :show-bug-list))

(defun interpret-model () (send current-model :interpret-model))

(defun show-workmap () (send *workmap* :gui t))

(defun hide-workmap () (send *workmap* :close))

(defun hide-guidemap () (send *guidemap* :close))

#+macintosh(defun show-xlisp-stat () (send *listener* :show-window))
#-macintosh(defun show-xlisp-stat () )

#+macintosh(defun hide-xlisp-stat () (send *listener* :close))

(defun preferences () (send *vista* :preferences))

(defun toplevel () (top-level))

(defun show-varobs ()
"Args: none
Shows both the variable and the observation windows."
  (list-variables)
  (list-observations))

(defun list-var ()
"Args: none
Lists the variable names of the current data object."
  (send *vista* :list-variables)
  t)

(defun list-vars ()
"Args: none
Lists the variable names of the current data object."
  (send *vista* :list-variables)
  t)

(defun list-variables ()
"Args: none
Lists the variable names of the current data object."
  (send *vista* :list-variables)
  t)

(defun list-obs ()
"Args: none
Lists the observation labels of the current data object."
  (send *vista* :list-observations)
  t)

(defun list-observations ()
"Args: none
Lists the observation labels of the current data object."
  (send *vista* :list-observations)
  t)

(defun list-mat ()
"Args: none
Lists the matrix names of the current dissimilarity data object."
  (send *vista* :list-matrices)
  t)

(defun list-mats ()
"Args: none
Lists the matrix names of the current dissimilarity data object."
  (send *vista* :list-matrices)
  t)

(defun list-matrices ()
"Args: none
Lists the variable names of the current dissimilarity data object."
  (send *vista* :list-matrices)
  t)

(defun list-cells ()
"Args: none
Lists the cell names of the current table data object."
  (send *vista* :list-cells)
  t)

(defun close-windows ()
"Args: none
Closes the data object windows."
  (send *var-window* :close)
  (send *obs-window* :close))

;;########################################################################
;;guidemap load-on-demand functions
;;########################################################################

(defun show-guidemap () 
"Args: none
Function to show the guidemap. Uses the Guidance function." 
  (if current-object 
      (let ((current-icon-name 
             (send (select (send *workmap* :icon-list) 
                           (send *workmap* :selected-icon))
                   :slot-value 'proto-name))
            ) 
        (if (or (equal current-icon-name
                       (send dob-icon-proto :slot-value 'proto-name))
                (equal current-icon-name
                       (send dib-icon-proto :slot-value 'proto-name))
                (equal current-icon-name
                       (send tab-icon-proto :slot-value 'proto-name)))
            (guidance "data")
            (guidance "model")))
      (guidance)))

(defun guidance (&optional string)
  "Args: string
Load-on-demand version of the function which provides guidance. For model guidance STRING is MODEL, DATA for data guidance.  Otherwise, general guidance."
;Complete function is in the guidemap file 
  (error-message "WARNING: GuideMaps Are Experimental and Incomplete!")
  (load (strcat *vista-dir-name* "guidemap"))
        (guidance string))

(defun author (&optional (logical nil set) &key new)
"Args: (&optional logical &key new)
Load-on-demand version of the function which enables authoring guidemaps."
;Complete function is in guidemap file.
(when set 
      (load (strcat *vista-dir-name* "author"))
      (author logical))
(when (not set) nil))

(provide "generic")