;;########################################################################
;; mmrmob.lsp
;; multivariate multiple regression ViSta model object
;; Copyright (c) 1991-95 by Forrest W. Young
;; Modified for X11 by Anthony J. Rossini (rossini@stat.sc.edu)
;;########################################################################

(require "vista")

;;---------------------------------------------------------------------------
;;constructor function for multivariate multiple regression model object
;;----------------------------------------------------------------------------

(defun multivariate-regression 
  (&key 
   (data current-data)  
   (title "Multivariate Regression")
   (name nil)
   (dialog nil)
   (responses nil)
   (predictors nil)
   (intercept t) )
"ViSta function to perform Multivariate Regression analysis.  
With no arguments the current data are analyzed, with the first active numeric variable as the response, and the remaining active numeric variables as the predictors. Keyword arguments are
:RESPONSES followed by a list of the response variables 
      (if there is more than one response variable then several separate
       multiple regressions are performed, each with the same predictors);
:PREDICTORS followed by a list of the predictor variables;
:INTERCEPT followed by t if an intercept is to be computed (default) or 
       nil if an intercept is not to be computed;
:DATA followed by the name of the data to be analyzed (default: current-data);
:TITLE followed by a character string (default: Multivariate Regression);
:DIALOG followed by t (to display parameters dialog box) or nil (default)."
  (if (not (eq current-object data)) (setcd data))
  (if (not name) (setf name (strcat "MulReg-" (send current-data :name))))
  (when (= 1 (send current-data :active-nvar '(numeric)))
        (error (format nil "Cannot do Regression with data that has only one active numeric variable.")))
  (send mmr-model-object-proto 
        :new responses predictors intercept 5 data title name dialog ))

;;---------------------------------------------------------------------------
;;define multivariate multiple regression model object
;;----------------------------------------------------------------------------

(defproto mmr-model-object-proto 
  '(reg-models num-iv num-dv iv dv intercept predictors responses
               weights scores coefs) () 
  mv-model-object-proto)

(defmeth mmr-model-object-proto :isnew 
             (responses predictors intercept number data title name dialog)
  (when (not dialog) 
        (when (not responses)
              (setf responses (list (first 
                      (send current-data :active-variables '(numeric))))))
        (when (not predictors)
              (setf predictors 
                    (rest (send current-data :active-variables '(numeric)))))
        (send self :responses responses)
        (send self :predictors predictors)
        (send self :intercept intercept))
  (send self :model-abbrev "MulReg")
  (call-next-method number data title name dialog)
  (send self :check-vars (send self :responses) (send self :predictors)))

(defmeth mmr-model-object-proto :reg-models (&optional (names nil set))
    (if set (setf (slot-value 'reg-models) names))
    (slot-value 'reg-models))

(defmeth mmr-model-object-proto :num-iv (&optional (names nil set))
    (if set (setf (slot-value 'num-iv) names))
    (slot-value 'num-iv))

(defmeth mmr-model-object-proto :num-dv (&optional (names nil set))
    (if set (setf (slot-value 'num-dv) names))
    (slot-value 'num-dv))

(defmeth mmr-model-object-proto :iv (&optional (numbers nil set))
    (if set (setf (slot-value 'iv) numbers))
    (slot-value 'iv))

(defmeth mmr-model-object-proto :dv (&optional (numbers nil set))
    (if set (setf (slot-value 'dv) numbers))
    (slot-value 'dv))
(defmeth mmr-model-object-proto :predictors (&optional (names nil set))
    (if set (setf (slot-value 'predictors) names))
    (slot-value 'predictors))

(defmeth mmr-model-object-proto :responses (&optional (names nil set))
    (if set (setf (slot-value 'responses) names))
    (slot-value 'responses))

(defmeth mmr-model-object-proto :intercept (&optional (val nil set))
    (if set (setf (slot-value 'intercept) val))
    (slot-value 'intercept))

(defmeth mmr-model-object-proto :weights (&optional (val nil set))
    (if set (setf (slot-value 'weights) val))
    (slot-value 'weights))

(defmeth mmr-model-object-proto :scores (&optional (names nil set))
  (if set (setf (slot-value 'scores) names))
  (slot-value 'scores))

(defmeth mmr-model-object-proto :coefs (&optional (names nil set))
    (if set (setf (slot-value 'coefs) names))
    (slot-value 'coefs))

(defmeth mmr-model-object-proto :check-vars (responses predictors)
  (let ((vars (send current-data :variables))
        )
    (dotimes (i (length responses))
             (when (not (member (select responses i) vars :test #'equal))
                   (error "Bad Response Variable List.")))
    (dotimes (i (length predictors))
             (when (not (member (select predictors i) vars :test #'equal))
                   (error "Bad Predictor Variable List.")))))

(defmeth mmr-model-object-proto :create-data 
  (&key (dialog nil)
        (scores t)
        (coefs  t)
        (input  nil))
  (if (not (eq current-object self)) (setcm self)) 
  (let ((creator (send *desktop* :selected-icon))
        (desires (list (list (if scores 0) (if coefs 1) (if input 2))))
        )
    (if dialog
        (setf desires 
              (choose-subset-dialog "Choose Desired Data Objects"
                  '("Predicted Values (Scores)" 
                    "Regression Coefficients"
                    "Analyzed Input Data")
                   :initial (select desires 0))))
    (when desires
          (when (member '0 (select desires 0))
                (send self :mmr-scores-data-object creator))
          (when (member '1 (select desires 0))
                (send self :mmr-coefs-data-object creator))
          (when (member '2 (select desires 0))
                (send self :create-input-data-object "MulReg" creator)))
    t))

(defmeth mmr-model-object-proto :mmr-scores-data-object (creator)
  (let ((n (send self :num-dv)))
  (data (concatenate 'string "Scores-" (send self :name))
   :created creator
   :creator-object self
   :title (concatenate 'string "MulReg Scores for " 
                       (send self :title))
   :data (combine (send self :scores))
   :variables (mapcar #'concatenate (repeat 'string n) (select (send self :variables) (send self :dv)) (repeat "-hat" n))
   :labels (send self :labels)
   :types (select (send self :types) (send self :dv)))
  ))

(defmeth mmr-model-object-proto :mmr-coefs-data-object (creator)
  (data (concatenate 'string "Coefs-" (send self :name))
   :created creator
   :creator-object self
   :title (concatenate 'string "MulReg Coefficients for " 
                       (send self :title))
   :data (combine (send self :coefs))
   :variables (select (send self :variables) (send self :iv))
   :labels (select (send self :variables) (send self :dv))
   :types (select (send self :types) (send self :iv))
  ))

(defmeth mmr-model-object-proto :create-input-data-object (mmr creator)
  (let ((used-vars (combine (send self :dv)
                            (send self :iv)))
        )
  (data  (concatenate 'string "Input-" (send self :name))
   :title (concatenate 'string mmr " Input: " (send self :title))
   :created creator
   :creator-object self
   :labels (send self :labels)
   :data (combine (select (send self :data-matrix) 
                          (iseq (send self :nobs)) used-vars))
   :variables (select (send self :variables) used-vars)                      
   :types (select (send self :types) used-vars)
  )))

(defmeth mmr-model-object-proto :save-model-template (data-object)
"Args: (data-object)
DATA-OBJECT is the object-identification information of a data object. 
The method contains a template for saving the model-object."
  `(multivariate-regression    
    :title       ,(send self :title)
    :name        ,(send self :name) 
    :dialog       nil
    :responses  ',(send self :responses)
    :predictors ',(send self :predictors)
    :intercept    t
    :data (data  ,(send data-object :name)
                 :title      ,(send data-object :title)
                 :variables ',(send self :variables)
                 :types     ',(send self :types)
                 :labels    ',(send self :labels)
                 :data      ',(send self :data))))

(defmeth mmr-model-object-proto :report (&key (dialog nil))
  (if (not (eq current-object self)) (setcm self))
  (let* ((labels (send self :labels))
         (dvars  (select (send self :variables) 
                         (send self :dv)))
         (ivars  (select (send self :variables) 
                         (send self :iv)))
         (scores (send self :scores))
         (coefs  (send self :coefs ))
         (lc-names (mapcar #'(lambda (x) (format nil "MulReg~a" x)) 
                      (iseq (send self :nvar))))
         (w nil)
         )
    (setf w (report-header (send self :title)))
    (display-string (format nil "Multivariate Multiple Regression Analysis~%") w)
    (display-string (format nil "(Programming for this report is unfinished: ~%No multivariate tests are performed)~2%") w)
    (display-string (format nil "Model:               ~a~%" 
                            (send self :name )) w)
    (display-string (format nil "Response  Variables: ~a~%" dvars)   w)
    (display-string (format nil "Predictor Variables: ~a~%" ivars)   w)
    (dotimes (i (length dvars))
             (display-string
              (format nil "~%For Response Variable ~a" (select dvars i)) w)
             (send (select (send self :reg-models) i) :display w)
             )
    (display-string 
     (format nil "~%Regression Coefficients (Regression Weights):~%") w)
    (print-matrix-to-window (transpose coefs) w :labels ivars)
    (display-string 
     (format nil "~%Regression Scores (Predicted Values):~%") w)
    (print-matrix-to-window scores w :labels labels)
    w
    ))

(defmeth mmr-model-object-proto :options ()
"Args: none
Constructs and displays the options dialog window for regression models.
Returns nil or a four element list.  Returns nil when dialog canceled or when no response or predictor variables selected, returns four element list otherwise.  The first element of the list is a list of response variable indices.  The second element is a list of predictor variable indices. The third element is the index of the weight variable, or nil for unweighted analysis.  The fourth element is T for intercept models, nil for non-intercept models."
  (when (not (send self :dialog))
        (send self :iv ($position (send self :predictors)
                                  (send self :variables)))
        (send self :dv ($position (send self :responses)
                                  (send self :variables))))
  (when (send self :dialog)
  (let* ((mob self)
         (box-text-item (send text-item-proto :new 
                              "Multivariate Multiple Regression Options"
                              :location (list 112 5)))
         (select-toggle (send choice-item-proto :new (list
                              "Select Response  Variables"
                              "Select Predictor Variables"
                              "Select Weighting Variable") 
                              :value 0
                              :location (list 10 25)))
         (intercept     (send toggle-item-proto :new "Intercept" 
                              :value t
                              ;;#-X11 :location (list 235 28)
	                      ;;#+X11 :location (list 250 28) ;AJR (X11)
			      :location (list 290 28)
			      ))
         (weight-var-text (send text-item-proto :new "Weight Variable"
				;;#-X11 :location (list 235 47)
	                        ;;#+X11 :location (list 250 47) ;AJR (X11)
				:location (list 290 47) 
				))
         (weight-var    (send list-item-proto :new (list "")
                              :size (list 242 18)
                              :location (list 290 66)))
         (var-text-item (send text-item-proto :new "Selectable Variables"
                              :location (list 15 90)))
         (rsp-text-item (send text-item-proto :new "Selected Responses"
                              :location (list 175 90)))
         (prd-text-item (send text-item-proto :new "Selected Predictors"
                              :location (list 335 90)))
         (var-list (send self :variables))
         (rsp-list (repeat " " (length var-list)))
         (prd-list (repeat " " (length var-list)))
         (rsp-nums nil)
         (prd-nums nil)
         (weight-num nil)
         (var-list-item (send list-item-proto :new var-list
                              :action #'(lambda () (move-vars &optional dc))
                              :size (list 150 192)))
         (rsp-list-item (send list-item-proto :new rsp-list
                              :size (list 150 192)))
         (prd-list-item (send list-item-proto :new prd-list
                              :size (list 150 192)))
         (ok        (send modal-button-proto :new "OK"
                          :location (list 150 315)))
         (cancel    (send modal-button-proto :new "Cancel"
                          :location (list 250 315)))
         (reg-dialog (send modal-dialog-proto :new
                     (list box-text-item
                           (list select-toggle intercept 
                                 weight-var-text weight-var)
                           (list var-text-item rsp-text-item prd-text-item)
                           (list var-list-item rsp-list-item prd-list-item)
                           (list ok cancel))
                           :default-button ok)))

    (defmeth ok :do-action ()
      (let ((dialog (send ok :dialog))
            )
        (send mob :intercept (send intercept :value))
        (send mob :weights weight-num)
        (send mob :iv prd-nums)
        (send mob :dv rsp-nums)
        (cond 
          ((or (> 1 (length prd-nums))
               (> 1 (length rsp-nums)))
           (error-message "You must select at least one response and one predictor variable.")
           (send dialog :modal-dialog-return nil))
          (t (send dialog :modal-dialog-return t)))
        ))

    (defmeth cancel :do-action ()
      (let ((dialog (send cancel :dialog)))
        (send dialog :modal-dialog-return nil)))

    (defmeth var-list-item :do-action (&optional dbl-clk)
      (let* ((n (send self :selection))
             (s (select (send self :slot-value 'list-data) n)))
        (when (and n (not (equal s " ")))
              (when (= 0 (send select-toggle :value))
                    (send self :set-text n " ")
                    (send rsp-list-item :set-text n s)
                    (setf rsp-nums (concatenate 'list rsp-nums (list n))))
              (when (= 1 (send select-toggle :value))
                    (send self :set-text n " ")
                    (send prd-list-item :set-text n s)
                    (setf prd-nums (concatenate 'list prd-nums (list n))))
              (when (and (not weight-num) (= 2 (send select-toggle :value)))
                    (send self :set-text n " ")
                    (send weight-var :set-text 0 s)
                    (setf weight-num n))
              (send self :selection nil))))

    (defmeth rsp-list-item :do-action (&optional dbl-clk)
      (let* ((n (send self :selection))
             (s (select (send self :slot-value 'list-data) n)))
        (when (and n (not (equal s " ")))
              (send self :set-text n " ")
              (send var-list-item :set-text n s)
              (send self :selection nil)
              (setf rsp-nums (remove n rsp-nums)))))

    (defmeth prd-list-item :do-action (&optional dbl-clk)
      (let* ((n (send self :selection))
             (s (select (send self :slot-value 'list-data) n)))
        (when (and n (not (equal s " ")))
              (send self :set-text n " ")
              (send var-list-item :set-text n s)
              (send self :selection nil)
              (setf prd-nums (remove n prd-nums)))))

    (defmeth weight-var :do-action (&optional dbl-clk)
      (let* ((s (select (send self :slot-value 'list-data) 0))
             )
        (send self :set-text 0 " ")
        (send var-list-item :set-text weight-num s)
        (send self :selection nil)
        (setf weight-num nil)))

    (if (send reg-dialog :modal-dialog)
        (list rsp-nums prd-nums weight-num (send intercept :value))
        nil))))

(defmeth mmr-model-object-proto :analysis ()
  (let* ((dv (send self :dv))
         (iv (send self :iv))
         (intercept (send self :intercept))
         (weights (send self :weights))
         (all-coefs nil)
         (all-coefs-mat nil)
         (all-scores nil)
         (all-scores-mat nil)
         (rm ())
         (num-dv (length dv))
         (num-iv (length iv))
         (numobs (send self :nobs)) 
         )
    (when (>= num-iv numobs) 
          (error-message (format nil "Cannot analyze data that does not have more observations (~d) than predictor variables (~d)." numobs num-iv)))
    (dolist (i dv)
            (setf rm (concatenate 'list rm (list  
               (regression-model 
                (col (send self :data-matrix) iv)
                (col (send self :data-matrix) i )
                :intercept intercept
                :predictor-names (select (send self :variables) iv)
                :response-name   (select (send self :variables) i )
                :weights (if weights 
                             (col (send self :data-matrix) weights)
                             nil)
                :print nil
                )))))
   (dotimes (i num-dv)
            (setf all-coefs (combine all-coefs 
                (if intercept 
                    (rest (send (select rm i) :coef-estimates))
                    (send (select rm i) :coef-estimates))))
            (setf all-scores 
                 (combine all-scores 
                          (send (select rm i) :fit-values)))
            )
   (setf all-coefs  (rest all-coefs))
   (setf all-scores (rest all-scores))
   (setf all-coefs-mat  (matrix (list num-dv num-iv) all-coefs))
   (setf all-scores-mat (transpose (matrix (list num-dv numobs) all-scores)))
   (send self :num-iv num-iv)
   (send self :num-dv num-dv)
   (send self :coefs all-coefs-mat)
   (send self :scores all-scores-mat)
   (send self :reg-models rm)
   ))
 
(defmeth mmr-model-object-proto :visualize ()
  (if (not (eq current-object self)) (setcm self))
   (when (> (send self :num-iv) 2)
         (spreadplot 
          (select 
           (column-list (send self :data-matrix)) 
           (send self :iv)) 
          (send self :labels) 
          (select (send self :variables) 
                  (send self :iv))
          :scale-type 'fixed)
         (let* ((scale-factor (/ (max (abs (send self :scores))) 
                                 (max (abs (send self :coefs)))))
                (spin-dim (send spin-plot :num-variables))
                (scaled-coefs (* scale-factor
                                 (select (column-list (send self :coefs))
                                         (iseq spin-dim))))
                (ray-labels (select (send self :variables) (send self :dv))))
           (send spin-plot   :depth-cuing nil)
           (send spin-plot   :scale-type nil)
           (send spin-plot   :add-rays scaled-coefs :ray-labels ray-labels)
           (send scatterplot :add-rays scaled-coefs :ray-labels ray-labels)))
  (when (= (send self :num-iv) 2)
        (setf pp (plot-points 
         (select 
          (column-list (send self :data-matrix)) 
          (send self :iv))
                 :title (send self :title)
                 :point-labels (send self :labels)
                 :variable-labels 
                  (select (send self :variables) (send self :iv))
                 :show nil))
        (send pp :size 489 438)
        (send pp :add-rays (column-list (send self :coefs))
              :ray-labels (select (send self :variables) (send self :dv)))
        (send pp :show-window))
  t)

