;;########################################################################
;; unimob.lsp
;; contains code to implement univariate analysis model object
;; Copyright (c) 1995 by Forrest W. Young
;;########################################################################


;1 define the prototype inheriting from mv-model-object-proto

(defproto univar-model-object-proto 
  '(var var1 var2 mu sigma t-p-level direction ci-level n-ways 
    t-value t-df t-ci-limits paired std-err-diff pooled-variance
    nonpar nonpar-z nonpar-p mann-whitney-sum)
  () mv-model-object-proto)

;2 define the slot accessor methods

(defmeth univar-model-object-proto :var (&optional (list nil set))
"Message args: (&optional list)
Sets or retrieves the data that are analyzed.  These are the sample values (when there is one sample), sample differences (for two paired samples) or values for both samples (for two independent samples)." 
  (if set (setf (slot-value 'var) list))
  (slot-value 'var))

(defmeth univar-model-object-proto :var1 (&optional (list nil set))
"Message args: (&optional list)
Sets or retrieves values for the first sample." 
  (if set (setf (slot-value 'var1) list))
  (slot-value 'var1))

(defmeth univar-model-object-proto :var2 (&optional (list nil set))
"Message args: (&optional list)
Sets or retrieves values for the second sample, if there is one." 
  (if set (setf (slot-value 'var2) list))
  (slot-value 'var2))

(defmeth univar-model-object-proto :mu (&optional (number nil set))
"Message args: (&optional number)
Sets or retrieves the population mean." 
  (if set (setf (slot-value 'mu) number))
  (slot-value 'mu))

(defmeth univar-model-object-proto :sigma (&optional (number nil set))
"Message args: (&optional number)
Sets or retrieves the population standard-deviation." 
  (if set (setf (slot-value 'sigma) number))
  (slot-value 'sigma))

(defmeth univar-model-object-proto :t-df (&optional (number nil set))
"Message args: (&optional number)
Sets or retrieves the degrees of freedom." 
  (if set (setf (slot-value 't-df) number))
  (slot-value 't-df))

(defmeth univar-model-object-proto :t-value (&optional (number nil set))
"Message args: (&optional number)
Sets or retrieves the t-value." 
  (if set (setf (slot-value 't-value) number))
  (slot-value 't-value))

(defmeth univar-model-object-proto :t-p-level (&optional (number nil set))
"Message args: (&optional number)
Sets or retrieves the p-level for the t-value." 
  (if set (setf (slot-value 't-p-level) number))
  (slot-value 't-p-level))

(defmeth univar-model-object-proto :direction (&optional (number nil set))
"Message args: (&optional number)
Sets or retrieves the direction. Negative or positive for a one-tailed directional test, zero or nil for a two-tailed test." 
  (if set (setf (slot-value 'direction) number))
  (slot-value 'direction))

(defmeth univar-model-object-proto :ci-level (&optional (number nil set))
"Message args: (&optional number)
Sets or retrieves the t value confidence interval level." 
  (if set (setf (slot-value 'ci-level) number))
  (slot-value 'ci-level)) 

(defmeth univar-model-object-proto :n-ways (&optional (number nil set))
"Message args: (&optional number)
Sets or retrieves the number of ways of table data (0 for mv data)." 
  (if set (setf (slot-value 'n-ways) number))
  (slot-value 'n-ways)) 

(defmeth univar-model-object-proto :t-ci-limits (&optional (number nil set))
"Message args: (&optional number)
Sets or retrieves the t value confidence interval limits." 
  (if set (setf (slot-value 't-ci-limits) number))
  (slot-value 't-ci-limits)) 

(defmeth univar-model-object-proto :std-err-diff (&optional (number nil set))
"Message args: (&optional number)
Sets or retrieves the standard error of the difference of two independent samples." 
  (if set (setf (slot-value 'std-err-diff) number))
  (slot-value 'std-err-diff))

(defmeth univar-model-object-proto :pooled-variance 
  (&optional (number nil set))
"Message args: (&optional number)
Sets or retrieves the pooled variance for two independent samples." 
  (if set (setf (slot-value 'pooled-variance) number))
  (slot-value 'pooled-variance))

(defmeth univar-model-object-proto :paired (&optional (logical nil set))
"Message args: (&optional logical)
Sets or retrieves whether situation is paired samples." 
  (if set (setf (slot-value 'paired) logical))
  (slot-value 'paired))

(defmeth univar-model-object-proto :nonpar (&optional (number nil set))
"Message args: (&optional number)
Sets or retrieves the value of the nonparametric test statistic (Wilcoxon T or Mann-Whitney U)." 
  (if set (setf (slot-value 'nonpar) number))
  (slot-value 'nonpar))

(defmeth univar-model-object-proto :nonpar-z (&optional (number nil set))
"Message args: (&optional number)
Sets or retrieves the value of the large sample normal approximation to the nonparametric statistic." 
  (if set (setf (slot-value 'nonpar-z) number))
  (slot-value 'nonpar-z))

(defmeth univar-model-object-proto :nonpar-p (&optional (number nil set))
"Message args: (&optional number)
Sets or retrieves the significance level of the large sample normal approximation to nonparametric statistic." 
  (if set (setf (slot-value 'nonpar-p) number))
  (slot-value 'nonpar-p))

(defmeth univar-model-object-proto :mann-whitney-sum 
  (&optional (number nil set))
"Message args: (&optional number)
Sets or retrieves the mann-whitney sum statistic." 
  (if set (setf (slot-value 'mann-whitney-sum) number))
  (slot-value 'mann-whitney-sum))

;3 Vista System Methods

(defmeth univar-model-object-proto :options ()
  (if (= (send self :n-ways) 0)
      (send self :dependent-dialog)
      (send self :independent-dialog)))

(defmeth univar-model-object-proto :independent-dialog ()
"Message args: none
For a two independent samples, presents dialog box to get univariate analysis options if :dialog t is used on the model constructor statement. Returns nil if :dialog nil, if dialog values are bad, or if dialog is canceled. Returns dialog values otherwise. Dialog values are checked for accuracy and error messages are produced if values are bad."
  (when (send self :dialog)
        (let* ((n-ways (send self :n-ways))
               (title (send text-item-proto :new "Univariate Analysis: Two Independent Samples"))
               (text1 (send text-item-proto :new "Ho: Difference in Population Means ="))
               (mu    (send edit-text-item-proto :new "        0.00"))
               (text2 (send text-item-proto :new "H1: (alternative hypothesis)"))
               (direc (send choice-item-proto :new 
                            (list "Difference in Sample Means LT Difference in Pop Means (1 tail)"
                                  "Difference in Sample Means NE Difference in Pop Means (2 tails)"
                                  "Difference in Sample Means GT Difference in Pop Means (1 tail)")
                            :value 1))
               (text3 (send text-item-proto :new 
                            "Confidence Interval Level ="))
               (cilev (send edit-text-item-proto :new ".95"))
               (ok (send modal-button-proto :new "OK"
                         :action #'(lambda () 
                                     (list (- (send direc :value) 1)
                                           (read-from-string 
                                            (send mu    :text) nil)
                                           (read-from-string 
                                            (send cilev :text) nil)
                                           ))))
               (cancel (send modal-button-proto :new "Cancel"))
               (dialog (send modal-dialog-proto :new
                             (list title 
                                   (list text1 mu) 
                                   text2 direc
                                   (list text3 cilev)
                                   (list ok cancel)) :default-button ok))
               (result (send dialog :modal-dialog))
               (error nil))
          (when result
                (when (not (numberp (second result)))
                      (error-message "The Population Mean must be a number.")
                      (setf error t))
                (when (or (not (numberp (third result)))
                          (>= (third result) 1) (<= (third result) 0))
                      (error-message "The Confidence Level must be a number between 0 and 1, but not including 0 or 1.")
                      (setf error t))
                (when error (setf result nil)))
          (when result
                (send self :direction (first result))
                (send self :mu (second result))
                (send self :ci-level (third result))
                (send self :var (send current-data :data))
                (send self :var1 (first (send current-data :data)))
                (send self :var2 (second (send current-data :data))))
          )))

(defmeth univar-model-object-proto :dependent-dialog ()
"Message args: none
For a single sample, or for two dependent samples, presents dialog box to get univariate analysis options if :dialog t is used on the model constructor statement. Returns nil if :dialog nil, if dialog values are bad, or if dialog is canceled. Returns dialog values otherwise. Dialog values are checked for accuracy and error messages are produced if values are bad."
  (when (send self :dialog)
        (let* ((title (send text-item-proto :new "Univariate Analysis: One Sample or Two Paired Samples" :location '(55 5)))
               (title2 (send text-item-proto :new "1) Provide details for t (or Z) test:"))
               (text1 (send text-item-proto :new "Ho: Population Mean ="))
               (mu    (send edit-text-item-proto :new "        0.00"))
               (text2 (send text-item-proto :new "H1: (alternative hypothesis)"))
               (direc (send choice-item-proto :new 
                            (list "Sample Mean LT Pop Mean (1 tail)"
                                  "Sample Mean NE Pop Mean (2 tails)"
                                  "Sample Mean GT Pop Mean (1 tail)")
                            :value 1))
               (text2a (send text-item-proto :new (format nil 
                       "Population Std. Dev. =~% (only if known)")))
               (popvar (send edit-text-item-proto :new "              "))
               (text3 (send text-item-proto :new "Confidence Interval Level ="))
               (cilev (send edit-text-item-proto :new ".95"))
               (text4 (send text-item-proto :new 
                            "2) Select a variable:"))
               
               (var1  (send list-item-proto :new
                            (send current-data :active-variables '(numeric))
                            :size (list 170 180))) ; changed 80 to 180  TAC
               (text5 (send text-item-proto :new (format nil 
                      "3) Select 2nd variable:~%(only for paired samples)")))
               (var2  (send list-item-proto :new
                            (send current-data :active-variables '(numeric))
                            :size (list 170 180))) ; changed 80 to 180  TAC
               (ok (send modal-button-proto :new "OK"
                         :action #'(lambda () 
                                     (list (- (send direc :value) 1)
                                           (read-from-string 
                                            (send mu    :text) nil)
                                           (read-from-string 
                                            (send cilev :text) nil)
                                           (send var1 :selection)
                                           (send var2 :selection)
                                           (read-from-string
                                            (send popvar :text) nil)
                                           ))))
               (cancel (send modal-button-proto :new "Cancel"))
               (dialog (send modal-dialog-proto :new
                             (list title 
                                  (list (list title2
                                              (list text1 mu) 
                                              text2 direc
                                              (list text2a popvar)
                                              (list text3 cilev)
                                              (list ok cancel))
                                        (list text4 var1 text5 var2))
                                   )
                             :default-button ok))
               (var0 (send var1 :selection 0))
               (result (send dialog :modal-dialog))
               (error nil)
               ) 
          (when result
                (when (not (numberp (second result)))
                      (error-message "The Population Mean must be a number.")
                      (setf error t))
                (when (or (not (numberp (third result)))
                          (>= (third result) 1) (<= (third result) 0))
                      (error-message "The Confidence Level must be a number between 0 and 1, but not including 0 or 1.")
                      (setf error t))
                (when (not (fourth result)) 
                      (error-message "You must select a variable from the first variable list")
                      (setf error t))
                (when (fifth result)
                      (when (equal (fifth result) (fourth result))
                            (error-message "If you select a second variable, it must be different than the first.")
                            (setf error t)))
                (when (sixth result)
                      (when (not (numberp (sixth result)))
                            (error-message "If you specify a Population Variance, it must be a number.")
                            (setf error t)))
                (when error (setf result nil)))
          (when result
                (send self :direction (first result))
                (send self :mu (second result))
                (send self :ci-level (third result))
                (send self :var1 (col (send current-data :data-matrix)
                                      (fourth result)))
                (cond 
                  ((fifth result)
                   (send self :var2 (col (send current-data :data-matrix)
                                         (fifth result)))
                   (send self :var (- (send self :var1) (send self :var2)))
                   (send self :paired t))
                  (t (send self :var (send self :var1))))
                (when (sixth result)
                      (send self :sigma (sixth result)))
                )
          result)))

(defmeth univar-model-object-proto :analysis ()
"Args: none
Does analysis for the univariate analysis model."
  (if (= (send self :n-ways) 0)
      (send self :dependent-analysis)
      (send self :independent-analysis)))

(defmeth univar-model-object-proto :dependent-analysis ()
"Args: none
Does analysis for the univariate analysis model. Computes, for single sample or paired sample situations, the t-test or z-test value, p-level and confidence limits. When appropriate, also computes wilcoxon test."
  (send self :t-df (- (length (send self :var)) 1))
  (send self :t-value (send self :compute-t-or-z-value))
  (send self :t-p-level (send self :compute-t-or-z-p-level))
  (send self :t-ci-limits (send self :compute-t-or-z-ci-limits)) 
  (when (and (>= (send self :t-df) 7) (send self :paired)) 
        (send self :compute-wilcoxon))
  )

(defmeth univar-model-object-proto :independent-analysis ()
"Args: none
Does analysis for the univariate analysis model. Computes, for two independent sample situation, the t-test or z-test value, p-level and confidence limits. When appropriate, also computes Mann-Whitney test." 
  (send self :t-df (- (sum (mapcar #'length (send self :var))) 2))
  (send self :t-value (send self :compute-independent-t))
  (send self :t-p-level (send self :compute-t-or-z-p-level))
  (send self :t-ci-limits (send self :compute-t-or-z-ci-limits)) 
  (when (>= (min (mapcar #'length (send self :var))) 7) 
        (send self :compute-mann-whitney))
  )

(defmeth univar-model-object-proto :report (&key (dialog nil))
  (let* ((var (send self :var))
         (var1 (send self :var1))
         (var2 (send self :var2))
         (n-ways (send self :n-ways))
         (mu  (send self :mu))
         (direction (send self :direction))
         (ci-level  (send self :ci-level))
         (ci-limits (send self :t-ci-limits))
         (df    (send self :t-df))
         (t-val (send self :t-value))
         (p-lev (send self :t-p-level))
         (n1 (length var1))
         (mean1  (mean var1))
         (stdev1 (standard-deviation var1))
         (n2 nil)
         (mean2 nil)
         (stdev2 nil)
         (meandif nil)
         (stdevdif nil)
         (sterrdif nil)
         (pooled-variance nil)
         (compare "one")
         (sign "_")
         (paired-st "a Single Sample")
         (zt "T")
         (sigma (send self :sigma))
         (w nil)
         (mus "Mu")
         (nonpar-st "Wilcoxon's Signed Rank Test for")
         (nonpar-au "Wilcoxon's   T")
         )
    (when var2
          (setf n2 (length var2))
          (setf mean2  (mean var2))
          (setf stdev2 (standard-deviation var2)))
    (cond
      ((= direction 0)
       (setf compare "two"))
      ((> direction 0) (setf sign">"))
      (t (setf sign "<")))
    (when (send self :sigma) (setf zt "Z"))
    (when (send self :paired) 
          (setf meandif  (mean var))
          (setf stdevdif (standard-deviation var))
          (setf sterrdif (/ stdevdif (sqrt (1+ df))))
          (setf paired-st "Two Paired (Dependent) Samples"))
    (when (> n-ways 0) 
          (setf paired-st "Two Independent Samples")
          (setf nonpar-st "Mann-Whitney Test for")
          (setf nonpar-au "Mann-Whitney U")
          (setf mus "Mu1 - Mu2")
          (setf sterrdif (send self :std-err-diff))
          (setf pooled-variance (send self :pooled-variance))
          )
    (setf w (report-header (send self :title)))
    (display-string (format nil "Univariate Analysis of the ~a Data."
                            (send current-data :name)) w)
    (display-string (format nil "~2%~a-Test for ~a:" zt paired-st) w)
    (display-string 
     (format nil "~2%Null hypothesis:        ~a = ~7,3f" mus mu) w)
    (display-string
     (format nil "~%Alternative hypothesis: ~a ~a ~7,3f (~a tailed)" 
             mus sign mu compare) w)
    (when sigma (display-string 
       (format nil "~%Known Population Standard Deviation:~7,3f" sigma) w))


    (display-string (format nil "~2%Sample Statistics:") w)
    (when var2 (display-string (format nil "~%Sample 1:") w))
    (display-string (format nil "  N = ~7d Mean = ~8,3f StDev = ~7,3f " 
                            n1 mean1 stdev1) w)
    (when var2 
          (display-string (format nil "~%Sample 2:") w)
          (display-string (format nil "  N = ~7d Mean = ~8,3f StDev = ~7,3f " 
                            n2 mean2 stdev2) w))
    (when (send self :paired)
          (display-string (format nil "~%Sample Differences:    Mean = ~8,3f StDev =~8,3f StErr =~8,3f" meandif stdevdif sterrdif) w))
    (when (> n-ways 0)
          (display-string 
           (format nil "~%Pooled Variance =~8,3f Std. Err. =~8,3f" 
                   pooled-variance sterrdif) w))
    (display-string (format nil "~2%~a-Test Results:" zt) w)
    (display-string (format nil "  ~a = ~7,3f " zt t-val) w)
    (when (not sigma) 
          (display-string (format nil "df   = ~7f " (round df)) w))
    (display-string (format nil "p = ~6,4f" p-lev) w)
    (display-string
     (format nil "~2%The ~2d% confidence interval is: (~8,3f,~8,3f)~%" 
             (round (* 100 ci-level)) (first ci-limits) 
             (second ci-limits)) w)
    (when (send self :nonpar)
          (display-string
           (format nil "~2%~a ~a:~2%Null Hypothesis:        Both Populations Identically Distributed " nonpar-st paired-st) w)
          (display-string
           (format nil "~%Alternative Hypothesis: Both Populations Differently Distributed (two tailed)") w)
          (when (> n-ways 0) (display-string
                (format nil "~%Mann-Whitney Sum of Ranks for each group = ~8,3f" 
                        (send self :mann-whitney-sum)) w))
          (display-string
           (format nil "~%~a =~5d~%Approximate  Z = ~8,3f   p = ~6,4f~%"
                   nonpar-au (send self :nonpar) (send self :nonpar-z)
                   (send self :nonpar-p)) w))
    )
  t)

(defmeth univar-model-object-proto :create-data (&rest args)
  (error-message "Not available for Univariate Analysis."))

(defmeth univar-model-object-proto :visualize-model (&rest args)
  (error-message "Not yet available for Univariate Analysis."))

;4 Object Specific Methods

(defmeth univar-model-object-proto :compute-t-or-z-value ()
"Args: none
Computes t-test for one sample or two paired samples"
  (let ((var (send self :var))
        (mu  (send self :mu))
        (s   (send self :sigma)))
    (when (not s) (setf s (standard-deviation var)))
    (/ (- (mean var) mu) (/ s (sqrt (length var))))))

(defmeth univar-model-object-proto :compute-independent-t ()
"Args: none
Computes t-test for independent samples"
  (let* ((var (send self :var))
         (mu  (send self :mu))
         (mean-dif (apply #'- (mapcar #'mean var)))
         (pooled-var (/ (apply #'sum  (* (- (mapcar #'length var) 1) 
                                         (mapcar #'variance var)))
                        (sum (- (mapcar #'length var) 1))))
         (std-err (sqrt (* pooled-var (sum (/ (mapcar #'length var)))))))
    (send self :pooled-variance pooled-var)
    (send self :std-err-diff std-err)
    (/ mean-dif std-err)))

(defmeth univar-model-object-proto :compute-t-or-z-p-level ()
"Args: none
Computes significance of t or z statistic"
  (let ((t-value (send self :t-value))
        (df (send self :t-df))
        (direction (send self :direction)))
    (when (send self :sigma) (setf df 1000))
    (cond 
      ((not direction) (* 2 (- 1 (t-cdf (abs t-value) df))))
      ((= direction 0) (* 2 (- 1 (t-cdf (abs t-value) df))))
      ((> direction 0) (- 1 (t-cdf t-value df)))
      ((< direction 0) (t-cdf t-value df)))))

(defmeth univar-model-object-proto :compute-t-or-z-ci-limits ()
"Args: none
Computes confidence limits based on t or z statistic"
  (let* ((var   (send self :var))
         (level (send self :ci-level))
         (mean  (mean var))
         (n     (length var))
         (sterr (/ (standard-deviation var) (- n 1)))
         (df    (send self :t-df))
         (t-crit nil)
         (width nil))
    (when (send self :sigma) (setf df 1000))
    (when (> (send self :n-ways) 0) 
          (setf mean (apply #'- (mapcar #'mean (send self :var))))
          (setf sterr (send self :std-err-diff)))
    (setf t-crit (t-quant (- 1 (/ (- 1 level) 2)) df))
    (setf width (* t-crit sterr))
    (list (- mean width) (+ mean width))))

(defmeth univar-model-object-proto :compute-wilcoxon () 
"Args: none
Computes the wilcoxon Signed-Rank test for matched samples."
  (let* ((sign-dif  (remove 0 (send self :var)))
         (n (length sign-dif))
         (ranks nil)
         (WT nil))
    (if (= n (length (remove-duplicates sign-dif)))
        (setf ranks (1+ (rank (abs sign-dif))))
        (setf ranks (1+ (rank-with-ties (abs sign-dif)))))
    (if (< (length (which (< sign-dif 0)))
           (length (which (> sign-dif 0))))
        (setf WT (sum (select ranks (which (< sign-dif 0)))))
        (setf WT (sum (select ranks (which (> sign-dif 0))))))
    (when (numberp wt)
          (send self :nonpar wt)
          (send self :nonpar-z
                (/ (- wt (/ (* n (1+ n)) 4)) 
                   (sqrt (/ (* n (1+ n) (1+ (* 2 n))) 24))))
          (send self :nonpar-p
                (* 2 (- 1 (normal-cdf (abs (send self :nonpar-z)) )))))
    t))

(defmeth univar-model-object-proto :compute-mann-whitney ()
  (let* ((vars (send self :var))
         (ranks (1+ (rank-with-ties (combine vars))))
         (nandm (mapcar #'length vars))
         (m (first nandm))
         (n (second nandm))
         (mn1 (+ m n 1))
         (wx (sum (select ranks (iseq m))))
         (wx2 (sum (select ranks (iseq m (1- (+ m n))))))
         (u (- (+ (* m n) (/ (* m (1+ m)) 2)) wx))
         (z (abs (/ (- wx (/ (* m mn1) 2)) (sqrt (/ (* m n mn1) 12)))))
         (p (* 2 (- 1 (normal-cdf z))))
         ) 
    (when (> u (/ (* m n) 2)) (setf u (- (* m n) u))) 
    (send self :mann-whitney-sum (list wx wx2))
    (send self :nonpar u)
    (send self :nonpar-z z)
    (send self :nonpar-p p)
    t))

;5 Isnew Method

(defmeth univar-model-object-proto :isnew 
  (variable mu sigma direction ci-level &rest args) 
  (let ((n-ways (length (send current-data :ways))))
    (cond
      ((< (send current-data :active-nvar '(numeric)) 1)
       (error-message "There are no active numeric variables in the data.")
       (send *toolbox* :reset-button (first args)))
      ((> n-ways 1) (error-message "Multi-Way table data are not appropriate for Univariate Analysis. Try Analysis of Variance instead.")
       (send *toolbox* :reset-button (first args)))
      
      (t
       (when (= n-ways 1) 
             (when (> (length (first (send current-data :classes))) 2)
                   (send *toolbox* :reset-button (first args))
                   (error "One-Way table data may only have 2 levels (classes) for Univariate Analysis. Try Analysis of Variance.")))
       (send self :model-abbrev "UNI")
       (send self :var (send current-data :data))
       (send self :mu mu)
       (send self :sigma sigma)
       (send self :direction direction)
       (send self :ci-level ci-level)
       (send self :n-ways n-ways)
       (cond 
         ((> n-ways 0)
          (send self :var  (send current-data :data))
          (send self :var1 (first (send self :var)))
          (send self :var2 (second (send self :var))))
         ((stringp variable)
          (send self :var  (combine (send current-data :variable variable)))
          (send self :var1 (send self :var)) )
         ((and (stringp (first variable)) (stringp (second variable)))
          (send self :paired t)
          (send self :var (- (send current-data :variable (first variable))
                             (send current-data :variable (second variable))))
          (send self :var1 (send current-data :variable (first variable)))
          (send self :var2 (send current-data :variable (second variable))))
         (t (error "Analysis variable(s) not specified correctly.")))
       (apply #'call-next-method args)))))

;6 Constructor Function

(defun univariate-analysis  
  (&key
   (data current-data)
   (title "Univariate Analysis")
   (name (strcat "UNI-" (send current-data :name)))
   (dialog nil)
   (variable (first (send current-data :variables)))
   (mu 0)
   (sigma nil)
   (direction 0)
   (ci-level .95)
   ) 
"Args:(dialog nil) (data current-data) (title Univariate Analysis)
      (name (strcat UNI- (send current-data :name)))
      (variable (first (send current-data :variables)))
      (mu 0) (sigma nil)  (direction 0)  (ci-level .95) 
Performs Univariate-Analysis. When DIALOG is t a dialog box is presented to obtain argument values. When DIALOG is false (the default) the indicated argument defaults are used unless overridden. Single sample analysis of the first variable in the dataset is default for multivariate data. Independent samples analysis is default for table data. For one sample or two independent samples, VARIABLE must be a variable name string (the default is the name of the first variable in the dataset). VARIABLE must be a list of two variable name strings to specify two paired samples. MU is the hypothesized population mean value for a single sample, mean difference for two samples. SIGMA is the population standard deviation (not used for independent samples). DIRECTION must be -1 for negative one-tailed tests, 0 for two-tailed, or 1 for positive two-tailed tests. CI-LEVEL specifies the confidence level."
  (if (not (eq current-object data)) (setcd data)) 
  (send univar-model-object-proto
        :new variable mu sigma direction ci-level 
        8 data title name dialog))

